/*************************************************************************
 *
 *  OpenOffice.org - a multi-platform office productivity suite
 *
 *  $RCSfile: QName.cxx,v $
 *
 *  $Revision: 1.2 $
 *
 *  last change: $Author: hbrinkm $ $Date: 2006/11/01 09:14:35 $
 *
 *  The Contents of this file are made available subject to
 *  the terms of GNU Lesser General Public License Version 2.1.
 *
 *
 *    GNU Lesser General Public License Version 2.1
 *    =============================================
 *    Copyright 2005 by Sun Microsystems, Inc.
 *    901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License version 2.1, as published by the Free Software Foundation.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public
 *    License along with this library; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *    MA  02111-1307  USA
 *
 ************************************************************************/

/* Copyright 2006 SUN Microsystems, Inc. */

#include <odiapi/qname/QName.hxx>
#include <rtl/string.h>
#include <hash_set>
#include <hash_map>
#include <stack>
#include <assert.h>
#include <stdexcept>
#include <string>
#include <stdio.h>

namespace writerfilter
{
	const char QName::EMPTY_STRING[] = "";


	static const char _LN_item[] = "item";
	static const char _LN_time_value[] = "time-value";
	static const char _LN_rslthtml[] = "rslthtml";
	static const char _LN_manager[] = "manager";
	static const char _LN_text_underline_style[] = "text-underline-style";
	static const char _LN_DisplayIn[] = "DisplayIn";
	static const char _LN_UICat[] = "UICat";
	static const char _LN_nvCxnSpPr[] = "nvCxnSpPr";
	static const char _LN_viewPr[] = "viewPr";
	static const char _LN_wbitmap[] = "wbitmap";
	static const char _LN_engine[] = "engine";
	static const char _LN_end_intensity[] = "end-intensity";
	static const char _LN_symbol_height[] = "symbol-height";
	static const char _LN_aftnnganada[] = "aftnnganada";
	static const char _LN_pivotTableDefinition[] = "pivotTableDefinition";
	static const char _LN_EffortDriven[] = "EffortDriven";
	static const char _LN_listbox[] = "listbox";
	static const char _LN_OLEObject[] = "OLEObject";
	static const char _LN_cell_range_address[] = "cell-range-address";
	static const char _LN_rsltmerge[] = "rsltmerge";
	static const char _LN_margb[] = "margb";
	static const char _LN_time_adjust[] = "time-adjust";
	static const char _LN_UICod[] = "UICod";
	static const char _LN_GroupNumber[] = "GroupNumber";
	static const char _LN_wbmbitspixel[] = "wbmbitspixel";
	static const char _LN_anvGraphicFramePr[] = "anvGraphicFramePr";
	static const char _LN_useWord2002TableStyleRules[] = "useWord2002TableStyleRules";
	static const char _LN_multiple[] = "multiple";
	static const char _LN_aftnngbnum[] = "aftnngbnum";
	static const char _LN_angle[] = "angle";
	static const char _LN_vertical[] = "vertical";
	static const char _LN_enhanced_geometry[] = "enhanced-geometry";
	static const char _LN_complex[] = "complex";
	static const char _LN_Recurring[] = "Recurring";
	static const char _LN_text_underline_width[] = "text-underline-width";
	static const char _LN_margbsxn[] = "margbsxn";
	static const char _LN_UIFmt[] = "UIFmt";
	static const char _LN_rsltpict[] = "rsltpict";
	static const char _LN_page_number[] = "page-number";
	static const char _LN_cacheSource[] = "cacheSource";
	static const char _LN_ParentName[] = "ParentName";
	static const char _LN_direction[] = "direction";
	static const char _LN_border[] = "border";
	static const char _LN_wbmplanes[] = "wbmplanes";
	static const char _LN_xforms_list_source[] = "xforms-list-source";
	static const char _LN_connect_bars[] = "connect-bars";
	static const char _LN_ObjectKind[] = "ObjectKind";
	static const char _LN_DocumentProperties[] = "DocumentProperties";
	static const char _LN_aftnngbnumd[] = "aftnngbnumd";
	static const char _LN_margl[] = "margl";
	static const char _LN_OverAllocated[] = "OverAllocated";
	static const char _LN_nvGraphicFramePr[] = "nvGraphicFramePr";
	static const char _LN_SmartTagType[] = "SmartTagType";
	static const char _LN_rsltrtf[] = "rsltrtf";
	static const char _LN_mirror_vertical[] = "mirror-vertical";
	static const char _LN_page_adjust[] = "page-adjust";
	static const char _LN_linearGradient[] = "linearGradient";
	static const char _LN_fields[] = "fields";
	static const char _LN_dontGrowAutofit[] = "dontGrowAutofit";
	static const char _LN_contains_error[] = "contains-error";
	static const char _LN_ParentIsOther[] = "ParentIsOther";
	static const char _LN_text_underline_color[] = "text-underline-color";
	static const char _LN_wbmwidthbytes[] = "wbmwidthbytes";
	static const char _LN_marglsxn[] = "marglsxn";
	static const char _LN_XDyn[] = "XDyn";
	static const char _LN_gap_width[] = "gap-width";
	static const char _LN_rslttxt[] = "rslttxt";
	static const char _LN_aftnngbnumk[] = "aftnngbnumk";
	static const char _LN_schemaRef[] = "schemaRef";
	static const char _LN_Estimated[] = "Estimated";
	static const char _LN_option[] = "option";
	static const char _LN_hierarchies[] = "hierarchies";
	static const char _LN_anvGspPr[] = "anvGspPr";
	static const char _LN_select_page[] = "select-page";
	static const char _LN_extent[] = "extent";
	static const char _LN_YDyn[] = "YDyn";
	static const char _LN_margmirror[] = "margmirror";
	static const char _LN_mirror_horizontal[] = "mirror-horizontal";
	static const char _LN_radialGradient[] = "radialGradient";
	static const char _LN_GroupMember[] = "GroupMember";
	static const char _LN_marked_invalid[] = "marked-invalid";
	static const char _LN_useFELayout[] = "useFELayout";
	static const char _LN_webhidden[] = "webhidden";
	static const char _LN_rtf[] = "rtf";
	static const char _LN_overlap[] = "overlap";
	static const char _LN_button[] = "button";
	static const char _LN_aftnngbnuml[] = "aftnngbnuml";
	static const char _LN_Milestone[] = "Milestone";
	static const char _LN_nvGspPr[] = "nvGspPr";
	static const char _LN_XCon[] = "XCon";
	static const char _LN_kpis[] = "kpis";
	static const char _LN_effectExtent[] = "effectExtent";
	static const char _LN_rtlch[] = "rtlch";
	static const char _LN_font_weight_asian[] = "font-weight-asian";
	static const char _LN_page_continuation[] = "page-continuation";
	static const char _LN_margr[] = "margr";
	static const char _LN_default_button[] = "default-button";
	static const char _LN_japanese_candle_stick[] = "japanese-candle-stick";
	static const char _LN_widctlpar[] = "widctlpar";
	static const char _LN_IsNotFiltered[] = "IsNotFiltered";
	static const char _LN_useNormalStyleForList[] = "useNormalStyleForList";
	static const char _LN_fx[] = "fx";
	static const char _LN_YCon[] = "YCon";
	static const char _LN_structure_protected[] = "structure-protected";
	static const char _LN_text_rotate_angle[] = "text-rotate-angle";
	static const char _LN_sdTupleCache[] = "sdTupleCache";
	static const char _LN_Summary[] = "Summary";
	static const char _LN_gspPr[] = "gspPr";
	static const char _LN_aftnnrlc[] = "aftnnrlc";
	static const char _LN_rtldoc[] = "rtldoc";
	static const char _LN_margrsxn[] = "margrsxn";
	static const char _LN_docPr[] = "docPr";
	static const char _LN_fy[] = "fy";
	static const char _LN_CanGlue[] = "CanGlue";
	static const char _LN_widowctrl[] = "widowctrl";
	static const char _LN_sp[] = "sp";
	static const char _LN_Critical[] = "Critical";
	static const char _LN_font_weight_complex[] = "font-weight-complex";
	static const char _LN_TotalAllMembers[] = "TotalAllMembers";
	static const char _LN_string_value[] = "string-value";
	static const char _LN_rtlgutter[] = "rtlgutter";
	static const char _LN_aftnnruc[] = "aftnnruc";
	static const char _LN_stop[] = "stop";
	static const char _LN_ciFmlas[] = "ciFmlas";
	static const char _LN_margt[] = "margt";
	static const char _LN_interpolation[] = "interpolation";
	static const char _LN_extrusion_allowed[] = "extrusion-allowed";
	static const char _LN_cxnsp[] = "cxnsp";
	static const char _LN_dontUseIndentAsNumberingTabStop[] = "dontUseIndentAsNumberingTabStop";
	static const char _LN_calculation_settings[] = "calculation-settings";
	static const char _LN_NoFill[] = "NoFill";
	static const char _LN_windowcaption[] = "windowcaption";
	static const char _LN_IsSubproject[] = "IsSubproject";
	static const char _LN_aftnnzodiac[] = "aftnnzodiac";
	static const char _LN_rtlmark[] = "rtlmark";
	static const char _LN_calcMbrs[] = "calcMbrs";
	static const char _LN_margtsxn[] = "margtsxn";
	static const char _LN_nor[] = "nor";
	static const char _LN_sender_firstname[] = "sender-firstname";
	static const char _LN_graphicFrame[] = "graphicFrame";
	static const char _LN_toggle[] = "toggle";
	static const char _LN_stop_color[] = "stop-color";
	static const char _LN_DetailMaxHeight[] = "DetailMaxHeight";
	static const char _LN_NoLine[] = "NoLine";
	static const char _LN_text_underline_mode[] = "text-underline-mode";
	static const char _LN_schemaLibrary[] = "schemaLibrary";
	static const char _LN_wrapPolygon[] = "wrapPolygon";
	static const char _LN_spline_order[] = "spline-order";
	static const char _LN_lit[] = "lit";
	static const char _LN_text_path_allowed[] = "text-path-allowed";
	static const char _LN_rtlpar[] = "rtlpar";
	static const char _LN_wmetafile[] = "wmetafile";
	static const char _LN_mhtmltag[] = "mhtmltag";
	static const char _LN_IsSubprojectReadOnly[] = "IsSubprojectReadOnly";
	static const char _LN_focus_on_click[] = "focus-on-click";
	static const char _LN_aftnnzodiacd[] = "aftnnzodiacd";
	static const char _LN_case_sensitive[] = "case-sensitive";
	static const char _LN_NoShow[] = "NoShow";
	static const char _LN_ElementType[] = "ElementType";
	static const char _LN_stop_opacity[] = "stop-opacity";
	static const char _LN_gsp[] = "gsp";
	static const char _LN_scr[] = "scr";
	static const char _LN_sender_lastname[] = "sender-lastname";
	static const char _LN_DetailMaxWidth[] = "DetailMaxWidth";
	static const char _LN_wrapNone[] = "wrapNone";
	static const char _LN_dontUseWord11FELineBreak[] = "dontUseWord11FELineBreak";
	static const char _LN_rtlrow[] = "rtlrow";
	static const char _LN_spline_resolution[] = "spline-resolution";
	static const char _LN_shItems[] = "shItems";
	static const char _LN_wpeqn[] = "wpeqn";
	static const char _LN_min[] = "min";
	static const char _LN_NoSnap[] = "NoSnap";
	static const char _LN_text_line_through_mode[] = "text-line-through-mode";
	static const char _LN_sty[] = "sty";
	static const char _LN_aftnnzodiacl[] = "aftnnzodiacl";
	static const char _LN_concentric_gradient_fill_allowed[] = "concentric-gradient-fill-allowed";
	static const char _LN_xforms_submission[] = "xforms-submission";
	static const char _LN_precision_as_shown[] = "precision-as-shown";
	static const char _LN_gradientUnits[] = "gradientUnits";
	static const char _LN_rtlsect[] = "rtlsect";
	static const char _LN_wrapSquare[] = "wrapSquare";
	static const char _LN_mo[] = "mo";
	static const char _LN_SubprojectName[] = "SubprojectName";
	static const char _LN_wpjst[] = "wpjst";
	static const char _LN_brk[] = "brk";
	static const char _LN_extends[] = "extends";
	static const char _LN_MoveTo[] = "MoveTo";
	static const char _LN_sender_initials[] = "sender-initials";
	static const char _LN_NoDetailAutoFit[] = "NoDetailAutoFit";
	static const char _LN_pie_offset[] = "pie-offset";
	static const char _LN_wksht[] = "wksht";
	static const char _LN_msmcap[] = "msmcap";
	static const char _LN_aftnrestart[] = "aftnrestart";
	static const char _LN_aln[] = "aln";
	static const char _LN_rxe[] = "rxe";
	static const char _LN_allowSpaceOfSameStyleInTable[] = "allowSpaceOfSameStyleInTable";
	static const char _LN_wpsp[] = "wpsp";
	static const char _LN_wrapTight[] = "wrapTight";
	static const char _LN_AttributeType[] = "AttributeType";
	static const char _LN_LineTo[] = "LineTo";
	static const char _LN_gradientTransform[] = "gradientTransform";
	static const char _LN_letter_kerning[] = "letter-kerning";
	static const char _LN_search_criteria_must_apply_to_whole_cell[] = "search-criteria-must-apply-to-whole-cell";
	static const char _LN_ExternalTask[] = "ExternalTask";
	static const char _LN_consol[] = "consol";
	static const char _LN_nestcell[] = "nestcell";
	static const char _LN_sender_title[] = "sender-title";
	static const char _LN_checkbox[] = "checkbox";
	static const char _LN_wraptrsp[] = "wraptrsp";
	static const char _LN_DetailRowHeight[] = "DetailRowHeight";
	static const char _LN_aftnrstcont[] = "aftnrstcont";
	static const char _LN_ArcTo[] = "ArcTo";
	static const char _LN_sa[] = "sa";
	static const char _LN_extrusion[] = "extrusion";
	static const char _LN_VisioDocument[] = "VisioDocument";
	static const char _LN_wrapThrough[] = "wrapThrough";
	static const char _LN_solid_type[] = "solid-type";
	static const char _LN_datatype[] = "datatype";
	static const char _LN_nestrow[] = "nestrow";
	static const char _LN_text_blinking[] = "text-blinking";
	static const char _LN_ExternalTaskProject[] = "ExternalTaskProject";
	static const char _LN_current_state[] = "current-state";
	static const char _LN_spreadMethod[] = "spreadMethod";
	static const char _LN_wrppunct[] = "wrppunct";
	static const char _LN_saauto[] = "saauto";
	static const char _LN_rngSets[] = "rngSets";
	static const char _LN_InfiniteLine[] = "InfiniteLine";
	static const char _LN_sender_position[] = "sender-position";
	static const char _LN_spid[] = "spid";
	static const char _LN_ww11IndentRules[] = "ww11IndentRules";
	static const char _LN_TotalFormat[] = "TotalFormat";
	static const char _LN_extrusion_brightness[] = "extrusion-brightness";
	static const char _LN_aftnsep[] = "aftnsep";
	static const char _LN_stacked[] = "stacked";
	static const char _LN_wrapTopAndBottom[] = "wrapTopAndBottom";
	static const char _LN_SolutionXML[] = "SolutionXML";
	static const char _LN_nesttableprops[] = "nesttableprops";
	static const char _LN_saftnnalc[] = "saftnnalc";
	static const char _LN_xe[] = "xe";
	static const char _LN_text_combine[] = "text-combine";
	static const char _LN_hatch[] = "hatch";
	static const char _LN_is_tristate[] = "is-tristate";
	static const char _LN_Ellipse[] = "Ellipse";
	static const char _LN_aftnsepc[] = "aftnsepc";
	static const char _LN_descr[] = "descr";
	static const char _LN_automatic_find_labels[] = "automatic-find-labels";
	static const char _LN_EarlyStart[] = "EarlyStart";
	static const char _LN_percentage[] = "percentage";
	static const char _LN_MemberFormat[] = "MemberFormat";
	static const char _LN_sender_email[] = "sender-email";
	static const char _LN_xef[] = "xef";
	static const char _LN_dontAutofitConstrainedTables[] = "dontAutofitConstrainedTables";
	static const char _LN_cp[] = "cp";
	static const char _LN_saftnnar[] = "saftnnar";
	static const char _LN_rngSet[] = "rngSet";
	static const char _LN_nextfile[] = "nextfile";
	static const char _LN_extrusion_depth[] = "extrusion-depth";
	static const char _LN_opEmu[] = "opEmu";
	static const char _LN_EllipticalArcTo[] = "EllipticalArcTo";
	static const char _LN_yr[] = "yr";
	static const char _LN_text_combine_start_char[] = "text-combine-start-char";
	static const char _LN_aftnstart[] = "aftnstart";
	static const char _LN_state[] = "state";
	static const char _LN_rotation[] = "rotation";
	static const char _LN_pp[] = "pp";
	static const char _LN_dobypara[] = "dobypara";
	static const char _LN_EarlyFinish[] = "EarlyFinish";
	static const char _LN_link_data_style_to_source[] = "link-data-style-to-source";
	static const char _LN_FieldLabelFormat[] = "FieldLabelFormat";
	static const char _LN_sender_phone_private[] = "sender-phone-private";
	static const char _LN_saftnnauc[] = "saftnnauc";
	static const char _LN_noBreak[] = "noBreak";
	static const char _LN_yts[] = "yts";
	static const char _LN_simplePos[] = "simplePos";
	static const char _LN_nobrkwrptbl[] = "nobrkwrptbl";
	static const char _LN_tp[] = "tp";
	static const char _LN_use_regular_expressions[] = "use-regular-expressions";
	static const char _LN_radio[] = "radio";
	static const char _LN_n[] = "n";
	static const char _LN_aftntj[] = "aftntj";
	static const char _LN_fill_image[] = "fill-image";
	static const char _LN_extrusion_diffusion[] = "extrusion-diffusion";
	static const char _LN_doccomm[] = "doccomm";
	static const char _LN_yxe[] = "yxe";
	static const char _LN_SplineStart[] = "SplineStart";
	static const char _LN_LateStart[] = "LateStart";
	static const char _LN_boxPr[] = "boxPr";
	static const char _LN_autofitLikeWW11[] = "autofitLikeWW11";
	static const char _LN_saftnnchi[] = "saftnnchi";
	static const char _LN_positionH[] = "positionH";
	static const char _LN_text_combine_end_char[] = "text-combine-end-char";
	static const char _LN_nocolbal[] = "nocolbal";
	static const char _LN_DetailSortOrder[] = "DetailSortOrder";
	static const char _LN_XProp[] = "XProp";
	static const char _LN_ai[] = "ai";
	static const char _LN_zwbo[] = "zwbo";
	static const char _LN_doctemp[] = "doctemp";
	static const char _LN_sender_fax[] = "sender-fax";
	static const char _LN_image_frame[] = "image-frame";
	static const char _LN_opacity[] = "opacity";
	static const char _LN_visible[] = "visible";
	static const char _LN_hideTop[] = "hideTop";
	static const char _LN_LateFinish[] = "LateFinish";
	static const char _LN_SplineKnot[] = "SplineKnot";
	static const char _LN_alang[] = "alang";
	static const char _LN_saftnnchosung[] = "saftnnchosung";
	static const char _LN_null_year[] = "null-year";
	static const char _LN_Text[] = "Text";
	static const char _LN_positionV[] = "positionV";
	static const char _LN_zwj[] = "zwj";
	static const char _LN_extrusion_number_of_line_segments[] = "extrusion-number-of-line-segments";
	static const char _LN_nocompatoptions[] = "nocompatoptions";
	static const char _LN_underlineTabInNumList[] = "underlineTabInNumList";
	static const char _LN_doctype[] = "doctype";
	static const char _LN_logarithmic[] = "logarithmic";
	static const char _LN_sender_company[] = "sender-company";
	static const char _LN_hideBot[] = "hideBot";
	static const char _LN_CellsExpanded[] = "CellsExpanded";
	static const char _LN_hidden[] = "hidden";
	static const char _LN_rot[] = "rot";
	static const char _LN_mbrs[] = "mbrs";
	static const char _LN_allowfieldendsel[] = "allowfieldendsel";
	static const char _LN_zwnbo[] = "zwnbo";
	static const char _LN_XForm[] = "XForm";
	static const char _LN_StartVariance[] = "StartVariance";
	static const char _LN_PolylineTo[] = "PolylineTo";
	static const char _LN_text_emphasize[] = "text-emphasize";
	static const char _LN_null_date[] = "null-date";
	static const char _LN_saftnncnum[] = "saftnncnum";
	static const char _LN_docvar[] = "docvar";
	static const char _LN_marker[] = "marker";
	static const char _LN_mpm[] = "mpm";
	static const char _LN_hideLeft[] = "hideLeft";
	static const char _LN_zwnj[] = "zwnj";
	static const char _LN_nocwrap[] = "nocwrap";
	static const char _LN_Line[] = "Line";
	static const char _LN_maximum[] = "maximum";
	static const char _LN_CellsNotExpanded[] = "CellsNotExpanded";
	static const char _LN_sender_phone_work[] = "sender-phone-work";
	static const char _LN_NURBSTo[] = "NURBSTo";
	static const char _LN_docVar[] = "docVar";
	static const char _LN_mpn[] = "mpn";
	static const char _LN_dodhgt[] = "dodhgt";
	static const char _LN_allprot[] = "allprot";
	static const char _LN_FinishVariance[] = "FinishVariance";
	static const char _LN_stroke_dash[] = "stroke-dash";
	static const char _LN_text_scale[] = "text-scale";
	static const char _LN_saftnndbar[] = "saftnndbar";
	static const char _LN_Fill[] = "Fill";
	static const char _LN_DataSource[] = "DataSource";
	static const char _LN_hideRight[] = "hideRight";
	static const char _LN_noextrasprl[] = "noextrasprl";
	static const char _LN_extrusion_light_face[] = "extrusion-light-face";
	static const char _LN_date_value_type[] = "date-value-type";
	static const char _LN_minimum[] = "minimum";
	static const char _LN_mpb[] = "mpb";
	static const char _LN_value_range[] = "value-range";
	static const char _LN_rsidRoot[] = "rsidRoot";
	static const char _LN_E[] = "E";
	static const char _LN_dolock[] = "dolock";
	static const char _LN_alntblind[] = "alntblind";
	static const char _LN_XForm1D[] = "XForm1D";
	static const char _LN_CellsExpandedSeqNum[] = "CellsExpandedSeqNum";
	static const char _LN_text_rotation_angle[] = "text-rotation-angle";
	static const char _LN_sender_street[] = "sender-street";
	static const char _LN_dots1[] = "dots1";
	static const char _LN_saftnndbnum[] = "saftnndbnum";
	static const char _LN_WorkVariance[] = "WorkVariance";
	static const char _LN_Menu[] = "Menu";
	static const char _LN_strikeH[] = "strikeH";
	static const char _LN_mps[] = "mps";
	static const char _LN_nofchars[] = "nofchars";
	static const char _LN_rsid[] = "rsid";
	static const char _LN_donotshowcomments[] = "donotshowcomments";
	static const char _LN_SubType[] = "SubType";
	static const char _LN_iteration[] = "iteration";
	static const char _LN_interval_major[] = "interval-major";
	static const char _LN_Event[] = "Event";
	static const char _LN_alt[] = "alt";
	static const char _LN_extrusion_first_light_harsh[] = "extrusion-first-light-harsh";
	static const char _LN_dots1_length[] = "dots1-length";
	static const char _LN_mpd[] = "mpd";
	static const char _LN_Action[] = "Action";
	static const char _LN_strikeV[] = "strikeV";
	static const char _LN_saftnndbnumd[] = "saftnndbnumd";
	static const char _LN_rPrDefault[] = "rPrDefault";
	static const char _LN_nofcharsws[] = "nofcharsws";
	static const char _LN_sender_city[] = "sender-city";
	static const char _LN_FreeSlack[] = "FreeSlack";
	static const char _LN_animtext[] = "animtext";
	static const char _LN_LayerMem[] = "LayerMem";
	static const char _LN_rangeDesc[] = "rangeDesc";
	static const char _LN_Operator[] = "Operator";
	static const char _LN_text_rotation_scale[] = "text-rotation-scale";
	static const char _LN_status[] = "status";
	static const char _LN_Checked[] = "Checked";
	static const char _LN_step_size[] = "step-size";
	static const char _LN_interval_minor[] = "interval-minor";
	static const char _LN_ConnectionInfo[] = "ConnectionInfo";
	static const char _LN_strikeBLTR[] = "strikeBLTR";
	static const char _LN_donotshowinsdel[] = "donotshowinsdel";
	static const char _LN_dots2[] = "dots2";
	static const char _LN_pPrDefault[] = "pPrDefault";
	static const char _LN_nofpages[] = "nofpages";
	static const char _LN_TotalSlack[] = "TotalSlack";
	static const char _LN_saftnndbnumk[] = "saftnndbnumk";
	static const char _LN_sender_postal_code[] = "sender-postal-code";
	static const char _LN_StyleProp[] = "StyleProp";
	static const char _LN_steps[] = "steps";
	static const char _LN_discMapping[] = "discMapping";
	static const char _LN_Disabled[] = "Disabled";
	static const char _LN_page_step_size[] = "page-step-size";
	static const char _LN_dots2_length[] = "dots2-length";
	static const char _LN_annotprot[] = "annotprot";
	static const char _LN_extrusion_second_light_harsh[] = "extrusion-second-light-harsh";
	static const char _LN_CaptionAlignment[] = "CaptionAlignment";
	static const char _LN_strikeTLBR[] = "strikeTLBR";
	static const char _LN_ilvl[] = "ilvl";
	static const char _LN_hyphenate[] = "hyphenate";
	static const char _LN_tick_marks_major_inner[] = "tick-marks-major-inner";
	static const char _LN_maximum_difference[] = "maximum-difference";
	static const char _LN_nofwords[] = "nofwords";
	static const char _LN_donotshowmarkup[] = "donotshowmarkup";
	static const char _LN_view[] = "view";
	static const char _LN_FixedCost[] = "FixedCost";
	static const char _LN_Foreign[] = "Foreign";
	static const char _LN_saftnndbnumt[] = "saftnndbnumt";
	static const char _LN_ReadOnly[] = "ReadOnly";
	static const char _LN_grItems[] = "grItems";
	static const char _LN_ilfo[] = "ilfo";
	static const char _LN_ansi[] = "ansi";
	static const char _LN_sender_country[] = "sender-country";
	static const char _LN_presentation_page_layout[] = "presentation-page-layout";
	static const char _LN_boxFormPr[] = "boxFormPr";
	static const char _LN_delay_for_repeat[] = "delay-for-repeat";
	static const char _LN_TooltipInfo[] = "TooltipInfo";
	static const char _LN_hyphenation_remain_char_count[] = "hyphenation-remain-char-count";
	static const char _LN_nolead[] = "nolead";
	static const char _LN_PageProps[] = "PageProps";
	static const char _LN_FitWidth[] = "FitWidth";
	static const char _LN_FixedCostAccrual[] = "FixedCostAccrual";
	static const char _LN_Invisible[] = "Invisible";
	static const char _LN_removePersonalInformation[] = "removePersonalInformation";
	static const char _LN_ansicpg[] = "ansicpg";
	static const char _LN_donotshowprops[] = "donotshowprops";
	static const char _LN_saftnnganada[] = "saftnnganada";
	static const char _LN_content_validations[] = "content-validations";
	static const char _LN_camera[] = "camera";
	static const char _LN_extrusion_first_light_level[] = "extrusion-first-light-level";
	static const char _LN_tick_marks_major_outer[] = "tick-marks-major-outer";
	static const char _LN_noline[] = "noline";
	static const char _LN_sender_state_or_province[] = "sender-state-or-province";
	static const char _LN_ClientParameterBindings[] = "ClientParameterBindings";
	static const char _LN_FitHeight[] = "FitHeight";
	static const char _LN_TextBlock[] = "TextBlock";
	static const char _LN_kpi[] = "kpi";
	static const char _LN_aoutl[] = "aoutl";
	static const char _LN_BeginGroup[] = "BeginGroup";
	static const char _LN_lightRig[] = "lightRig";
	static const char _LN_PercentComplete[] = "PercentComplete";
	static const char _LN_saftnngbnum[] = "saftnngbnum";
	static const char _LN_nolnhtadjtbl[] = "nolnhtadjtbl";
	static const char _LN_generic_control[] = "generic-control";
	static const char _LN_dpaendhol[] = "dpaendhol";
	static const char _LN_hierarchy[] = "hierarchy";
	static const char _LN_ApplyBrkRules[] = "ApplyBrkRules";
	static const char _LN_page_layout_properties[] = "page-layout-properties";
	static const char _LN_hyphenation_push_char_count[] = "hyphenation-push-char-count";
	static const char _LN_LeftToRight[] = "LeftToRight";
	static const char _LN_content_validation[] = "content-validation";
	static const char _LN_TextXForm[] = "TextXForm";
	static const char _LN_between[] = "between";
	static const char _LN_removeDateAndTime[] = "removeDateAndTime";
	static const char _LN_TagName[] = "TagName";
	static const char _LN_backdrop[] = "backdrop";
	static const char _LN_tick_marks_minor_inner[] = "tick-marks-minor-inner";
	static const char _LN_extrusion_second_light_level[] = "extrusion-second-light-level";
	static const char _LN_dpaendl[] = "dpaendl";
	static const char _LN_saftnngbnumd[] = "saftnngbnumd";
	static const char _LN_ClientParameterBinding[] = "ClientParameterBinding";
	static const char _LN_author_name[] = "author-name";
	static const char _LN_nonesttables[] = "nonesttables";
	static const char _LN_flds[] = "flds";
	static const char _LN_PercentWorkComplete[] = "PercentWorkComplete";
	static const char _LN_Align[] = "Align";
	static const char _LN_BlackAndWhite[] = "BlackAndWhite";
	static const char _LN_ascaps[] = "ascaps";
	static const char _LN_ButtonFace[] = "ButtonFace";
	static const char _LN_control_implementation[] = "control-implementation";
	static const char _LN_grLvls[] = "grLvls";
	static const char _LN_dpaendsol[] = "dpaendsol";
	static const char _LN_dontDisplayPageBoundaries[] = "dontDisplayPageBoundaries";
	static const char _LN_page_width[] = "page-width";
	static const char _LN_Protection[] = "Protection";
	static const char _LN_pStyle[] = "pStyle";
	static const char _LN_author_initials[] = "author-initials";
	static const char _LN_saftnngbnumk[] = "saftnngbnumk";
	static const char _LN_nonshppict[] = "nonshppict";
	static const char _LN_base_cell_address[] = "base-cell-address";
	static const char _LN_paragraph_properties[] = "paragraph-properties";
	static const char _LN_ashad[] = "ashad";
	static const char _LN_tick_marks_minor_outer[] = "tick-marks-minor-outer";
	static const char _LN_SortKey[] = "SortKey";
	static const char _LN_NoPrinterInfo[] = "NoPrinterInfo";
	static const char _LN_fld[] = "fld";
	static const char _LN_Cost[] = "Cost";
	static const char _LN_CredentialBinding[] = "CredentialBinding";
	static const char _LN_dpaendw[] = "dpaendw";
	static const char _LN_keepNext[] = "keepNext";
	static const char _LN_extrusion_first_light_direction[] = "extrusion-first-light-direction";
	static const char _LN_page_height[] = "page-height";
	static const char _LN_asianbrkrule[] = "asianbrkrule";
	static const char _LN_rSpRule[] = "rSpRule";
	static const char _LN_Help[] = "Help";
	static const char _LN_nooverflow[] = "nooverflow";
	static const char _LN_grLvl[] = "grLvl";
	static const char _LN_saftnngbnuml[] = "saftnngbnuml";
	static const char _LN_OvertimeCost[] = "OvertimeCost";
	static const char _LN_bind[] = "bind";
	static const char _LN_chapter[] = "chapter";
	static const char _LN_allow_empty_cell[] = "allow-empty-cell";
	static const char _LN_dparc[] = "dparc";
	static const char _LN_displayBackgroundShape[] = "displayBackgroundShape";
	static const char _LN_DraftQuality[] = "DraftQuality";
	static const char _LN_Status[] = "Status";
	static const char _LN_line_height[] = "line-height";
	static const char _LN_Misc[] = "Misc";
	static const char _LN_keepLines[] = "keepLines";
	static const char _LN_rSp[] = "rSp";
	static const char _LN_display_label[] = "display-label";
	static const char _LN_grps[] = "grps";
	static const char _LN_button_type[] = "button-type";
	static const char _LN_paper_tray_name[] = "paper-tray-name";
	static const char _LN_Credential[] = "Credential";
	static const char _LN_aspalpha[] = "aspalpha";
	static const char _LN_noproof[] = "noproof";
	static const char _LN_file_name[] = "file-name";
	static const char _LN_saftnnrlc[] = "saftnnrlc";
	static const char _LN_dparcflipx[] = "dparcflipx";
	static const char _LN_OvertimeWork[] = "OvertimeWork";
	static const char _LN_RulerGrid[] = "RulerGrid";
	static const char _LN_Visible[] = "Visible";
	static const char _LN_grp[] = "grp";
	static const char _LN_anchor[] = "anchor";
	static const char _LN_pageBreakBefore[] = "pageBreakBefore";
	static const char _LN_line_height_at_least[] = "line-height-at-least";
	static const char _LN_CommentsLayout[] = "CommentsLayout";
	static const char _LN_display_list[] = "display-list";
	static const char _LN_aspnum[] = "aspnum";
	static const char _LN_extrusion_second_light_direction[] = "extrusion-second-light-direction";
	static const char _LN_nosectexpand[] = "nosectexpand";
	static const char _LN_Attribute[] = "Attribute";
	static const char _LN_text_overlap[] = "text-overlap";
	static const char _LN_template_name[] = "template-name";
	static const char _LN_print_orientation[] = "print-orientation";
	static const char _LN_grMbrs[] = "grMbrs";
	static const char _LN_printPostScriptOverText[] = "printPostScriptOverText";
	static const char _LN_saftnnruc[] = "saftnnruc";
	static const char _LN_dparcflipy[] = "dparcflipy";
	static const char _LN_Print[] = "Print";
	static const char _LN_DocProps[] = "DocProps";
	static const char _LN_normal[] = "normal";
	static const char _LN_current_selected[] = "current-selected";
	static const char _LN_ActualStart[] = "ActualStart";
	static const char _LN_astrike[] = "astrike";
	static const char _LN_framePr[] = "framePr";
	static const char _LN_If[] = "If";
	static const char _LN_help_message[] = "help-message";
	static const char _LN_Scale[] = "Scale";
	static const char _LN_grMbr[] = "grMbr";
	static const char _LN_Active[] = "Active";
	static const char _LN_nosnaplinegrid[] = "nosnaplinegrid";
	static const char _LN_label_arrangement[] = "label-arrangement";
	static const char _LN_saftnnzodiac[] = "saftnnzodiac";
	static const char _LN_up[] = "up";
	static const char _LN_Image[] = "Image";
	static const char _LN_sheet_name[] = "sheet-name";
	static const char _LN_line_spacing[] = "line-spacing";
	static const char _LN_dpastarthol[] = "dpastarthol";
	static const char _LN_ClientParameterValue[] = "ClientParameterValue";
	static const char _LN_ActualFinish[] = "ActualFinish";
	static const char _LN_atnauthor[] = "atnauthor";
	static const char _LN_register_truth_ref_style_name[] = "register-truth-ref-style-name";
	static const char _LN_PrintErrors[] = "PrintErrors";
	static const char _LN_disabled[] = "disabled";
	static const char _LN_entries[] = "entries";
	static const char _LN_widowControl[] = "widowControl";
	static const char _LN_Lock[] = "Lock";
	static const char _LN_printFractionalCharacterWidth[] = "printFractionalCharacterWidth";
	static const char _LN_Group[] = "Group";
	static const char _LN_extrusion_metal[] = "extrusion-metal";
	static const char _LN_variable_decl[] = "variable-decl";
	static const char _LN_saftnnzodiacd[] = "saftnnzodiacd";
	static const char _LN_nospaceforul[] = "nospaceforul";
	static const char _LN_dpastartl[] = "dpastartl";
	static const char _LN_font_independent_line_spacing[] = "font-independent-line-spacing";
	static const char _LN_error_message[] = "error-message";
	static const char _LN_atndate[] = "atndate";
	static const char _LN_Snap[] = "Snap";
	static const char _LN_data_label_number[] = "data-label-number";
	static const char _LN_sets[] = "sets";
	static const char _LN_ActualDuration[] = "ActualDuration";
	static const char _LN_dropdown[] = "dropdown";
	static const char _LN_Layout[] = "Layout";
	static const char _LN_ValidPrinterInfo[] = "ValidPrinterInfo";
	static const char _LN_listPr[] = "listPr";
	static const char _LN_CredentialValue[] = "CredentialValue";
	static const char _LN_Glue[] = "Glue";
	static const char _LN_qryCache[] = "qryCache";
	static const char _LN_dpastartsol[] = "dpastartsol";
	static const char _LN_atnicn[] = "atnicn";
	static const char _LN_extrusion_rotation_angle[] = "extrusion-rotation-angle";
	static const char _LN_nosupersub[] = "nosupersub";
	static const char _LN_variable_set[] = "variable-set";
	static const char _LN_saftnnzodiacl[] = "saftnnzodiacl";
	static const char _LN_PageLayout[] = "PageLayout";
	static const char _LN_for[] = "for";
	static const char _LN_message_type[] = "message-type";
	static const char _LN_supressLineNumbers[] = "supressLineNumbers";
	static const char _LN_ActualCost[] = "ActualCost";
	static const char _LN_NameUniv[] = "NameUniv";
	static const char _LN_data_label_text[] = "data-label-text";
	static const char _LN_printFormsData[] = "printFormsData";
	static const char _LN_print_page_order[] = "print-page-order";
	static const char _LN_PaperSizeIndex[] = "PaperSizeIndex";
	static const char _LN_atnid[] = "atnid";
	static const char _LN_image_data[] = "image-data";
	static const char _LN_mbr[] = "mbr";
	static const char _LN_dpastartw[] = "dpastartw";
	static const char _LN_text_align[] = "text-align";
	static const char _LN_notabind[] = "notabind";
	static const char _LN_PrintProps[] = "PrintProps";
	static const char _LN_variable_get[] = "variable-get";
	static const char _LN_saftnrestart[] = "saftnrestart";
	static const char _LN_error_macro[] = "error-macro";
	static const char _LN_Label[] = "Label";
	static const char _LN_ActualOvertimeCost[] = "ActualOvertimeCost";
	static const char _LN_atnparent[] = "atnparent";
	static const char _LN_bevelT[] = "bevelT";
	static const char _LN_pBdr[] = "pBdr";
	static const char _LN_extrusion_rotation_center[] = "extrusion-rotation-center";
	static const char _LN_embedTrueTypeFonts[] = "embedTrueTypeFonts";
	static const char _LN_data_label_symbol[] = "data-label-symbol";
	static const char _LN_tuples[] = "tuples";
	static const char _LN_noultrlspc[] = "noultrlspc";
	static const char _LN_dpcallout[] = "dpcallout";
	static const char _LN_HorizontalResolution[] = "HorizontalResolution";
	static const char _LN_first_page_number[] = "first-page-number";
	static const char _LN_text_align_last[] = "text-align-last";
	static const char _LN_Char[] = "Char";
	static const char _LN_Verify[] = "Verify";
	static const char _LN_max_length[] = "max-length";
	static const char _LN_variable_input[] = "variable-input";
	static const char _LN_saftnrstcont[] = "saftnrstcont";
	static const char _LN_bevelB[] = "bevelB";
	static const char _LN_cGpRule[] = "cGpRule";
	static const char _LN_shd[] = "shd";
	static const char _LN_execute[] = "execute";
	static const char _LN_atnref[] = "atnref";
	static const char _LN_sortByTuple[] = "sortByTuple";
	static const char _LN_Para[] = "Para";
	static const char _LN_dpcoa[] = "dpcoa";
	static const char _LN_ActualWork[] = "ActualWork";
	static const char _LN_nowidctlpar[] = "nowidctlpar";
	static const char _LN_tabs[] = "tabs";
	static const char _LN_extrusionClr[] = "extrusionClr";
	static const char _LN_error_category[] = "error-category";
	static const char _LN_doNotEmbedSystemFonts[] = "doNotEmbedSystemFonts";
	static const char _LN_cSp[] = "cSp";
	static const char _LN_printable[] = "printable";
	static const char _LN_atntime[] = "atntime";
	static const char _LN_label_ranges[] = "label-ranges";
	static const char _LN_justify_single_word[] = "justify-single-word";
	static const char _LN_scale_to[] = "scale-to";
	static const char _LN_Address[] = "Address";
	static const char _LN_saftnstart[] = "saftnstart";
	static const char _LN_VerticalResolution[] = "VerticalResolution";
	static const char _LN_user_field_decl[] = "user-field-decl";
	static const char _LN_Tabs[] = "Tabs";
	static const char _LN_extrusion_shininess[] = "extrusion-shininess";
	static const char _LN_dpcoaccent[] = "dpcoaccent";
	static const char _LN_Proxy[] = "Proxy";
	static const char _LN_qry[] = "qry";
	static const char _LN_suppressAutoHyphens[] = "suppressAutoHyphens";
	static const char _LN_cGp[] = "cGp";
	static const char _LN_ActualOvertimeWork[] = "ActualOvertimeWork";
	static const char _LN_nowrap[] = "nowrap";
	static const char _LN_atrfend[] = "atrfend";
	static const char _LN_readonly[] = "readonly";
	static const char _LN_Scratch[] = "Scratch";
	static const char _LN_SubAddress[] = "SubAddress";
	static const char _LN_scale_to_pages[] = "scale-to-pages";
	static const char _LN_contourClr[] = "contourClr";
	static const char _LN_ciFmla[] = "ciFmla";
	static const char _LN_Authentication[] = "Authentication";
	static const char _LN_label_range[] = "label-range";
	static const char _LN_error_percentage[] = "error-percentage";
	static const char _LN_sautoupd[] = "sautoupd";
	static const char _LN_dpcobestfit[] = "dpcobestfit";
	static const char _LN_user_field_get[] = "user-field-get";
	static const char _LN_keep_together[] = "keep-together";
	static const char _LN_saveSubsetFonts[] = "saveSubsetFonts";
	static const char _LN_nowwrap[] = "nowwrap";
	static const char _LN_Gridlines[] = "Gridlines";
	static const char _LN_atrfstart[] = "atrfstart";
	static const char _LN_extrusion_skew[] = "extrusion-skew";
	static const char _LN_selected[] = "selected";
	static const char _LN_rule[] = "rule";
	static const char _LN_kinsoku[] = "kinsoku";
	static const char _LN_ExtraInfo[] = "ExtraInfo";
	static const char _LN_flatTx[] = "flatTx";
	static const char _LN_RegularWork[] = "RegularWork";
	static const char _LN_sb[] = "sb";
	static const char _LN_label_cell_range_address[] = "label-cell-range-address";
	static const char _LN_table_centering[] = "table-centering";
	static const char _LN_ConnectionABCD[] = "ConnectionABCD";
	static const char _LN_noxlattoyen[] = "noxlattoyen";
	static const char _LN_dpcoborder[] = "dpcoborder";
	static const char _LN_error_margin[] = "error-margin";
	static const char _LN_calcMbr[] = "calcMbr";
	static const char _LN_NumberOfCopies[] = "NumberOfCopies";
	static const char _LN_aul[] = "aul";
	static const char _LN_widows[] = "widows";
	static const char _LN_sbasedon[] = "sbasedon";
	static const char _LN_user_field_input[] = "user-field-input";
	static const char _LN_Basic[] = "Basic";
	static const char _LN_wordWrap[] = "wordWrap";
	static const char _LN_saveFormsData[] = "saveFormsData";
	static const char _LN_Frame[] = "Frame";
	static const char _LN_extrusion_specularity[] = "extrusion-specularity";
	static const char _LN_RemainingDuration[] = "RemainingDuration";
	static const char _LN_tab_index[] = "tab-index";
	static const char _LN_auld[] = "auld";
	static const char _LN_Field[] = "Field";
	static const char _LN_filters[] = "filters";
	static const char _LN_objalias[] = "objalias";
	static const char _LN_dpcodabs[] = "dpcodabs";
	static const char _LN_footnote_max_height[] = "footnote-max-height";
	static const char _LN_orphans[] = "orphans";
	static const char _LN_useExplicit[] = "useExplicit";
	static const char _LN_error_lower_limit[] = "error-lower-limit";
	static const char _LN_sbauto[] = "sbauto";
	static const char _LN_NewWindow[] = "NewWindow";
	static const char _LN_overflowPunct[] = "overflowPunct";
	static const char _LN_RowColHeadings[] = "RowColHeadings";
	static const char _LN_data_cell_range_address[] = "data-cell-range-address";
	static const char _LN_mirrorMargins[] = "mirrorMargins";
	static const char _LN_auldb[] = "auldb";
	static const char _LN_sequence_decl[] = "sequence-decl";
	static const char _LN_tab_stop[] = "tab-stop";
	static const char _LN_Control[] = "Control";
	static const char _LN_tab_stops[] = "tab-stops";
	static const char _LN_objalign[] = "objalign";
	static const char _LN_dpcodbottom[] = "dpcodbottom";
	static const char _LN_sbkcol[] = "sbkcol";
	static const char _LN_RemainingCost[] = "RemainingCost";
	static const char _LN_userId[] = "userId";
	static const char _LN_extrusion_viewpoint[] = "extrusion-viewpoint";
	static const char _LN_location[] = "location";
	static const char _LN_aulnone[] = "aulnone";
	static const char _LN_topLinePunct[] = "topLinePunct";
	static const char _LN_footnote_sep[] = "footnote-sep";
	static const char _LN_error_upper_limit[] = "error-upper-limit";
	static const char _LN_Geom[] = "Geom";
	static const char _LN_PTRule[] = "PTRule";
	static const char _LN_Initials[] = "Initials";
	static const char _LN_alignBordersAndEdges[] = "alignBordersAndEdges";
	static const char _LN_sbkeven[] = "sbkeven";
	static const char _LN_display_outline_level[] = "display-outline-level";
	static const char _LN_objattph[] = "objattph";
	static const char _LN_dpcodcenter[] = "dpcodcenter";
	static const char _LN_visual_effect[] = "visual-effect";
	static const char _LN_zeroWid[] = "zeroWid";
	static const char _LN_aulw[] = "aulw";
	static const char _LN_rwFlds[] = "rwFlds";
	static const char _LN_orientation[] = "orientation";
	static const char _LN_RemainingWork[] = "RemainingWork";
	static const char _LN_char[] = "char";
	static const char _LN_Act[] = "Act";
	static const char _LN_SOAPAction[] = "SOAPAction";
	static const char _LN_FormatType[] = "FormatType";
	static const char _LN_ReviewerID[] = "ReviewerID";
	static const char _LN_autoSpaceDE[] = "autoSpaceDE";
	static const char _LN_clrFrom[] = "clrFrom";
	static const char _LN_sbknone[] = "sbknone";
	static const char _LN_extrusion_origin[] = "extrusion-origin";
	static const char _LN_objautlink[] = "objautlink";
	static const char _LN_aup[] = "aup";
	static const char _LN_zeroAsc[] = "zeroAsc";
	static const char _LN_error_upper_indicator[] = "error-upper-indicator";
	static const char _LN_Layer[] = "Layer";
	static const char _LN_leader_type[] = "leader-type";
	static const char _LN_rwItms[] = "rwItms";
	static const char _LN_dpcodescent[] = "dpcodescent";
	static const char _LN_image_position[] = "image-position";
	static const char _LN_bordersDontSurroundHeader[] = "bordersDontSurroundHeader";
	static const char _LN_named_expressions[] = "named-expressions";
	static const char _LN_separation_character[] = "separation-character";
	static const char _LN_IndividualCellBorders[] = "IndividualCellBorders";
	static const char _LN_RemainingOvertimeCost[] = "RemainingOvertimeCost";
	static const char _LN_clrTo[] = "clrTo";
	static const char _LN_sbkodd[] = "sbkodd";
	static const char _LN_author[] = "author";
	static const char _LN_line_style[] = "line-style";
	static const char _LN_CurrentIndex[] = "CurrentIndex";
	static const char _LN_autoSpaceDN[] = "autoSpaceDN";
	static const char _LN_User[] = "User";
	static const char _LN_objclass[] = "objclass";
	static const char _LN_zeroDesc[] = "zeroDesc";
	static const char _LN_rwDrills[] = "rwDrills";
	static const char _LN_extrusion_color[] = "extrusion-color";
	static const char _LN_leader_style[] = "leader-style";
	static const char _LN_dpcodtop[] = "dpcodtop";
	static const char _LN_fillToRect[] = "fillToRect";
	static const char _LN_sbkpage[] = "sbkpage";
	static const char _LN_Prop[] = "Prop";
	static const char _LN_image_align[] = "image-align";
	static const char _LN_adjustment[] = "adjustment";
	static const char _LN_error_lower_indicator[] = "error-lower-indicator";
	static const char _LN_objcropb[] = "objcropb";
	static const char _LN_MarkerIndex[] = "MarkerIndex";
	static const char _LN_named_range[] = "named-range";
	static const char _LN_transp[] = "transp";
	static const char _LN_colFlds[] = "colFlds";
	static const char _LN_bidi[] = "bidi";
	static const char _LN_sequence[] = "sequence";
	static const char _LN_dpcolength[] = "dpcolength";
	static const char _LN_ItemType[] = "ItemType";
	static const char _LN_RemainingOvertimeWork[] = "RemainingOvertimeWork";
	static const char _LN_Hyperlink[] = "Hyperlink";
	static const char _LN_bordersDontSurroundFooter[] = "bordersDontSurroundFooter";
	static const char _LN_sbys[] = "sbys";
	static const char _LN_bdbfhdr[] = "bdbfhdr";
	static const char _LN_leader_width[] = "leader-width";
	static const char _LN_gs[] = "gs";
	static const char _LN_enhanced_path[] = "enhanced-path";
	static const char _LN_adjustRightInd[] = "adjustRightInd";
	static const char _LN_bound_column[] = "bound-column";
	static const char _LN_distance_before_sep[] = "distance-before-sep";
	static const char _LN_colItms[] = "colItms";
	static const char _LN_objcropl[] = "objcropl";
	static const char _LN_Date[] = "Date";
	static const char _LN_range_usable_as[] = "range-usable-as";
	static const char _LN_gsList[] = "gsList";
	static const char _LN_scaps[] = "scaps";
	static const char _LN_ref_name[] = "ref-name";
	static const char _LN_BlockTotal[] = "BlockTotal";
	static const char _LN_bdrrlswsix[] = "bdrrlswsix";
	static const char _LN_Reviewer[] = "Reviewer";
	static const char _LN_dpcominusx[] = "dpcominusx";
	static const char _LN_series_source[] = "series-source";
	static const char _LN_leader_color[] = "leader-color";
	static const char _LN_colDrills[] = "colDrills";
	static const char _LN_XJustify[] = "XJustify";
	static const char _LN_path_stretchpoint_x[] = "path-stretchpoint-x";
	static const char _LN_objcropr[] = "objcropr";
	static const char _LN_scompose[] = "scompose";
	static const char _LN_lin[] = "lin";
	static const char _LN_convert_empty_to_null[] = "convert-empty-to-null";
	static const char _LN_snapToGrid[] = "snapToGrid";
	static const char _LN_ACWP[] = "ACWP";
	static const char _LN_expression[] = "expression";
	static const char _LN_gutterAtTop[] = "gutterAtTop";
	static const char _LN_Annotation[] = "Annotation";
	static const char _LN_CountOfSameItems[] = "CountOfSameItems";
	static const char _LN_bgbdiag[] = "bgbdiag";
	static const char _LN_distance_after_sep[] = "distance-after-sep";
	static const char _LN_dpcominusy[] = "dpcominusy";
	static const char _LN_regression_type[] = "regression-type";
	static const char _LN_CV[] = "CV";
	static const char _LN_YJustify[] = "YJustify";
	static const char _LN_pgFields[] = "pgFields";
	static const char _LN_objcropt[] = "objcropt";
	static const char _LN_leader_text[] = "leader-text";
	static const char _LN_named_expression[] = "named-expression";
	static const char _LN_sec[] = "sec";
	static const char _LN_spacing[] = "spacing";
	static const char _LN_tileRect[] = "tileRect";
	static const char _LN_text_input[] = "text-input";
	static const char _LN_bgcross[] = "bgcross";
	static const char _LN_shape[] = "shape";
	static const char _LN_ConstraintType[] = "ConstraintType";
	static const char _LN_path_stretchpoint_y[] = "path-stretchpoint-y";
	static const char _LN_SmartTagDef[] = "SmartTagDef";
	static const char _LN_hideSpellingErrors[] = "hideSpellingErrors";
	static const char _LN_dpcooffset[] = "dpcooffset";
	static const char _LN_datFields[] = "datFields";
	static const char _LN_objdata[] = "objdata";
	static const char _LN_PTLineItem[] = "PTLineItem";
	static const char _LN_boxForm[] = "boxForm";
	static const char _LN_layout_grid_mode[] = "layout-grid-mode";
	static const char _LN_leader_text_style[] = "leader-text-style";
	static const char _LN_ind[] = "ind";
	static const char _LN_sectd[] = "sectd";
	static const char _LN_fillRect[] = "fillRect";
	static const char _LN_transition_type[] = "transition-type";
	static const char _LN_list_source[] = "list-source";
	static const char _LN_bgdcross[] = "bgdcross";
	static const char _LN_Data1[] = "Data1";
	static const char _LN_shapetype[] = "shapetype";
	static const char _LN_dpcosmarta[] = "dpcosmarta";
	static const char _LN_contextualSpacing[] = "contextualSpacing";
	static const char _LN_ConstraintDate[] = "ConstraintDate";
	static const char _LN_database_ranges[] = "database-ranges";
	static const char _LN_fmts[] = "fmts";
	static const char _LN_sectdefaultcl[] = "sectdefaultcl";
	static const char _LN_RuleType[] = "RuleType";
	static const char _LN_hideGrammaticalErrors[] = "hideGrammaticalErrors";
	static const char _LN_text_areas[] = "text-areas";
	static const char _LN_alphaBiLevel[] = "alphaBiLevel";
	static const char _LN_Manager[] = "Manager";
	static const char _LN_objemb[] = "objemb";
	static const char _LN_bgdkbdiag[] = "bgdkbdiag";
	static const char _LN_creation_time[] = "creation-time";
	static const char _LN_list_source_type[] = "list-source-type";
	static const char _LN_eqArr[] = "eqArr";
	static const char _LN_Data2[] = "Data2";
	static const char _LN_layout_grid_base_height[] = "layout-grid-base-height";
	static const char _LN_tab_stop_distance[] = "tab-stop-distance";
	static const char _LN_transition_style[] = "transition-style";
	static const char _LN_condfmts[] = "condfmts";
	static const char _LN_VMLFrame[] = "VMLFrame";
	static const char _LN_dpcotdouble[] = "dpcotdouble";
	static const char _LN_Rule[] = "Rule";
	static const char _LN_objh[] = "objh";
	static const char _LN_Deadline[] = "Deadline";
	static const char _LN_Company[] = "Company";
	static const char _LN_glue_points[] = "glue-points";
	static const char _LN_sectexpand[] = "sectexpand";
	static const char _LN_database_range[] = "database-range";
	static const char _LN_Data3[] = "Data3";
	static const char _LN_mirrorIndents[] = "mirrorIndents";
	static const char _LN_bgdkcross[] = "bgdkcross";
	static const char _LN_alphaCeiling[] = "alphaCeiling";
	static const char _LN_crtfmts[] = "crtfmts";
	static const char _LN_RuleV10[] = "RuleV10";
	static const char _LN_activeWritingStyle[] = "activeWritingStyle";
	static const char _LN_properties[] = "properties";
	static const char _LN_objhtml[] = "objhtml";
	static const char _LN_dpcotright[] = "dpcotright";
	static const char _LN_transition_speed[] = "transition-speed";
	static const char _LN_hyphenation_keep[] = "hyphenation-keep";
	static const char _LN_ForeignData[] = "ForeignData";
	static const char _LN_LevelAssignments[] = "LevelAssignments";
	static const char _LN_sectlinegrid[] = "sectlinegrid";
	static const char _LN_glue_point_type[] = "glue-point-type";
	static const char _LN_layout_grid_ruby_height[] = "layout-grid-ruby-height";
	static const char _LN_bgdkdcross[] = "bgdkdcross";
	static const char _LN_drillSaved[] = "drillSaved";
	static const char _LN_textboxTightWrap[] = "textboxTightWrap";
	static const char _LN_is_selection[] = "is-selection";
	static const char _LN_alphaFloor[] = "alphaFloor";
	static const char _LN_formulas[] = "formulas";
	static const char _LN_ParseRuleAsV10[] = "ParseRuleAsV10";
	static const char _LN_limLow[] = "limLow";
	static const char _LN_objicemb[] = "objicemb";
	static const char _LN_property[] = "property";
	static const char _LN_Desc[] = "Desc";
	static const char _LN_dpcotsingle[] = "dpcotsingle";
	static const char _LN_PinX[] = "PinX";
	static const char _LN_proofState[] = "proofState";
	static const char _LN_bgdkfdiag[] = "bgdkfdiag";
	static const char _LN_limUpp[] = "limUpp";
	static const char _LN_hyphenation_ladder_count[] = "hyphenation-ladder-count";
	static const char _LN_sectnum[] = "sectnum";
	static const char _LN_LevelingCanSplit[] = "LevelingCanSplit";
	static const char _LN_strSelects[] = "strSelects";
	static const char _LN_alphaInv[] = "alphaInv";
	static const char _LN_HyperlinkBase[] = "HyperlinkBase";
	static const char _LN_glue_point_leaving_directions[] = "glue-point-leaving-directions";
	static const char _LN_objlink[] = "objlink";
	static const char _LN_on_update_keep_styles[] = "on-update-keep-styles";
	static const char _LN_boolean_value[] = "boolean-value";
	static const char _LN_property_name[] = "property-name";
	static const char _LN_Area[] = "Area";
	static const char _LN_PinY[] = "PinY";
	static const char _LN_suppressOverlap[] = "suppressOverlap";
	static const char _LN_handles[] = "handles";
	static const char _LN_layout_grid_lines[] = "layout-grid-lines";
	static const char _LN_dpcottriple[] = "dpcottriple";
	static const char _LN_background_size[] = "background-size";
	static const char _LN_formsDesign[] = "formsDesign";
	static const char _LN_sectrsid[] = "sectrsid";
	static const char _LN_bgdkhoriz[] = "bgdkhoriz";
	static const char _LN_alphaModFix[] = "alphaModFix";
	static const char _LN_objlock[] = "objlock";
	static const char _LN_Width[] = "Width";
	static const char _LN_dims[] = "dims";
	static const char _LN_LevelingDelay[] = "LevelingDelay";
	static const char _LN_AlternateNames[] = "AlternateNames";
	static const char _LN_MissingItemsLimit[] = "MissingItemsLimit";
	static const char _LN_list_property[] = "list-property";
	static const char _LN_dpcount[] = "dpcount";
	static const char _LN_drop_cap[] = "drop-cap";
	static const char _LN_sectspecifycl[] = "sectspecifycl";
	static const char _LN_mgs[] = "mgs";
	static const char _LN_on_update_keep_size[] = "on-update-keep-size";
	static const char _LN_Height[] = "Height";
	static const char _LN_bgdkvert[] = "bgdkvert";
	static const char _LN_attachedTemplate[] = "attachedTemplate";
	static const char _LN_objname[] = "objname";
	static const char _LN_print_time[] = "print-time";
	static const char _LN_layout_grid_color[] = "layout-grid-color";
	static const char _LN_background_objects_visible[] = "background-objects-visible";
	static const char _LN_alphaRepl[] = "alphaRepl";
	static const char _LN_textDirection[] = "textDirection";
	static const char _LN_text_path[] = "text-path";
	static const char _LN_LevelingDelayFormat[] = "LevelingDelayFormat";
	static const char _LN_dpellipse[] = "dpellipse";
	static const char _LN_Template[] = "Template";
	static const char _LN_length[] = "length";
	static const char _LN_LocPinX[] = "LocPinX";
	static const char _LN_list_value[] = "list-value";
	static const char _LN_objocx[] = "objocx";
	static const char _LN_bgfdiag[] = "bgfdiag";
	static const char _LN_sxfilters[] = "sxfilters";
	static const char _LN_VersionLastRefresh[] = "VersionLastRefresh";
	static const char _LN_sectspecifygen[] = "sectspecifygen";
	static const char _LN_textbox[] = "textbox";
	static const char _LN_biLevel[] = "biLevel";
	static const char _LN_text_path_mode[] = "text-path-mode";
	static const char _LN_linkStyles[] = "linkStyles";
	static const char _LN_textAlignment[] = "textAlignment";
	static const char _LN_has_persistent_data[] = "has-persistent-data";
	static const char _LN_layout_grid_ruby_below[] = "layout-grid-ruby-below";
	static const char _LN_lines[] = "lines";
	static const char _LN_BuildNumberCreated[] = "BuildNumberCreated";
	static const char _LN_LocPinY[] = "LocPinY";
	static const char _LN_dpendgroup[] = "dpendgroup";
	static const char _LN_objpub[] = "objpub";
	static const char _LN_bghoriz[] = "bghoriz";
	static const char _LN_annotation[] = "annotation";
	static const char _LN_items[] = "items";
	static const char _LN_textpath[] = "textpath";
	static const char _LN_blur[] = "blur";
	static const char _LN_background_visible[] = "background-visible";
	static const char _LN_PreLeveledStart[] = "PreLeveledStart";
	static const char _LN_sectspecifyl[] = "sectspecifyl";
	static const char _LN_stylePaneFormatFilter[] = "stylePaneFormatFilter";
	static const char _LN_RefreshName[] = "RefreshName";
	static const char _LN_Angle[] = "Angle";
	static const char _LN_objscalex[] = "objscalex";
	static const char _LN_text_path_scale[] = "text-path-scale";
	static const char _LN_outlineLvl[] = "outlineLvl";
	static const char _LN_bgvert[] = "bgvert";
	static const char _LN_dpfillbgcb[] = "dpfillbgcb";
	static const char _LN_cdls[] = "cdls";
	static const char _LN_clrBal[] = "clrBal";
	static const char _LN_argSz[] = "argSz";
	static const char _LN_keywords[] = "keywords";
	static const char _LN_imagedata[] = "imagedata";
	static const char _LN_contains_header[] = "contains-header";
	static const char _LN_BuildNumberEdited[] = "BuildNumberEdited";
	static const char _LN_layout_grid_print[] = "layout-grid-print";
	static const char _LN_register_true[] = "register-true";
	static const char _LN_FlipX[] = "FlipX";
	static const char _LN_date_string[] = "date-string";
	static const char _LN_autoSortScope[] = "autoSortScope";
	static const char _LN_sectunlocked[] = "sectunlocked";
	static const char _LN_PreLeveledFinish[] = "PreLeveledFinish";
	static const char _LN_objscaley[] = "objscaley";
	static const char _LN_argPr[] = "argPr";
	static const char _LN_display_header[] = "display-header";
	static const char _LN_RefreshDate[] = "RefreshDate";
	static const char _LN_clrChange[] = "clrChange";
	static const char _LN_divId[] = "divId";
	static const char _LN_dpfillbgcg[] = "dpfillbgcg";
	static const char _LN_text_path_same_letter_heights[] = "text-path-same-letter-heights";
	static const char _LN_binfsxn[] = "binfsxn";
	static const char _LN_FlipY[] = "FlipY";
	static const char _LN_stylePaneSortMethod[] = "stylePaneSortMethod";
	static const char _LN_display_filter_buttons[] = "display-filter-buttons";
	static const char _LN_jc[] = "jc";
	static const char _LN_Behavior[] = "Behavior";
	static const char _LN_num_prefix[] = "num-prefix";
	static const char _LN_margin_left[] = "margin-left";
	static const char _LN_cnfStyle[] = "cnfStyle";
	static const char _LN_objsect[] = "objsect";
	static const char _LN_PreviewPicture[] = "PreviewPicture";
	static const char _LN_clrRepl[] = "clrRepl";
	static const char _LN_sftnbj[] = "sftnbj";
	static const char _LN_layout_grid_display[] = "layout-grid-display";
	static const char _LN_cdl[] = "cdl";
	static const char _LN_duration[] = "duration";
	static const char _LN_RefreshDateCopy[] = "RefreshDateCopy";
	static const char _LN_dpfillbgcr[] = "dpfillbgcr";
	static const char _LN_binsxn[] = "binsxn";
	static const char _LN_display_footer[] = "display-footer";
	static const char _LN_ResizeMode[] = "ResizeMode";
	static const char _LN_mathFont[] = "mathFont";
	static const char _LN_HyperlinkAddress[] = "HyperlinkAddress";
	static const char _LN_pgField[] = "pgField";
	static const char _LN_Sequence[] = "Sequence";
	static const char _LN_objsetsize[] = "objsetsize";
	static const char _LN_sftnnalc[] = "sftnnalc";
	static const char _LN_duotone[] = "duotone";
	static const char _LN_num_suffix[] = "num-suffix";
	static const char _LN_margin_right[] = "margin-right";
	static const char _LN_documentType[] = "documentType";
	static const char _LN_modification_time[] = "modification-time";
	static const char _LN_bkmkcolf[] = "bkmkcolf";
	static const char _LN_CustomProps[] = "CustomProps";
	static const char _LN_sectPr[] = "sectPr";
	static const char _LN_dpfillbggray[] = "dpfillbggray";
	static const char _LN_defSz[] = "defSz";
	static const char _LN_LineWeight[] = "LineWeight";
	static const char _LN_modifiers[] = "modifiers";
	static const char _LN_ConsolidationReference[] = "ConsolidationReference";
	static const char _LN_target_range_address[] = "target-range-address";
	static const char _LN_display_page_number[] = "display-page-number";
	static const char _LN_header_footer_properties[] = "header-footer-properties";
	static const char _LN_datField[] = "datField";
	static const char _LN_fillOverlay[] = "fillOverlay";
	static const char _LN_sftnnar[] = "sftnnar";
	static const char _LN_objsub[] = "objsub";
	static const char _LN_num_format[] = "num-format";
	static const char _LN_bkmkcoll[] = "bkmkcoll";
	static const char _LN_brkBin[] = "brkBin";
	static const char _LN_HyperlinkSubAddress[] = "HyperlinkSubAddress";
	static const char _LN_text_indent[] = "text-indent";
	static const char _LN_mailMerge[] = "mailMerge";
	static const char _LN_TimeCreated[] = "TimeCreated";
	static const char _LN_LineColor[] = "LineColor";
	static const char _LN_equation[] = "equation";
	static const char _LN_dpfillbgpal[] = "dpfillbgpal";
	static const char _LN_modification_date[] = "modification-date";
	static const char _LN_curve[] = "curve";
	static const char _LN_sftnnauc[] = "sftnnauc";
	static const char _LN_objtime[] = "objtime";
	static const char _LN_grayscl[] = "grayscl";
	static const char _LN_brkBinSub[] = "brkBinSub";
	static const char _LN_bkmkend[] = "bkmkend";
	static const char _LN_num_letter_sync[] = "num-letter-sync";
	static const char _LN_fmt[] = "fmt";
	static const char _LN_database_source_sql[] = "database-source-sql";
	static const char _LN_display_date_time[] = "display-date-time";
	static const char _LN_NoSaveData[] = "NoSaveData";
	static const char _LN_revisionView[] = "revisionView";
	static const char _LN_auto_text_indent[] = "auto-text-indent";
	static const char _LN_LinePattern[] = "LinePattern";
	static const char _LN_TimeSaved[] = "TimeSaved";
	static const char _LN_dynamic_spacing[] = "dynamic-spacing";
	static const char _LN_movie[] = "movie";
	static const char _LN_dpfillfgcb[] = "dpfillfgcb";
	static const char _LN_objtransy[] = "objtransy";
	static const char _LN_sftnnchi[] = "sftnnchi";
	static const char _LN_IgnoreResourceCalendar[] = "IgnoreResourceCalendar";
	static const char _LN_cfmt[] = "cfmt";
	static const char _LN_handle[] = "handle";
	static const char _LN_hsl[] = "hsl";
	static const char _LN_bkmkpub[] = "bkmkpub";
	static const char _LN_smallFrac[] = "smallFrac";
	static const char _LN_SourceConsolidation[] = "SourceConsolidation";
	static const char _LN_background[] = "background";
	static const char _LN_roundrect[] = "roundrect";
	static const char _LN_change_info[] = "change-info";
	static const char _LN_crtfmt[] = "crtfmt";
	static const char _LN_TimeEdited[] = "TimeEdited";
	static const char _LN_Rounding[] = "Rounding";
	static const char _LN_sftnnchosung[] = "sftnnchosung";
	static const char _LN_objupdate[] = "objupdate";
	static const char _LN_dpfillfgcg[] = "dpfillfgcg";
	static const char _LN_handle_mirror_vertical[] = "handle-mirror-vertical";
	static const char _LN_margin_top[] = "margin-top";
	static const char _LN_bkmkstart[] = "bkmkstart";
	static const char _LN_text_properties[] = "text-properties";
	static const char _LN_database_name[] = "database-name";
	static const char _LN_sharpen[] = "sharpen";
	static const char _LN_dispDef[] = "dispDef";
	static const char _LN_sql_statement[] = "sql-statement";
	static const char _LN_drill[] = "drill";
	static const char _LN_oval[] = "oval";
	static const char _LN_Notes[] = "Notes";
	static const char _LN_EndArrowSize[] = "EndArrowSize";
	static const char _LN_event_listeners[] = "event-listeners";
	static const char _LN_TimePrinted[] = "TimePrinted";
	static const char _LN_objw[] = "objw";
	static const char _LN_dpfillfgcr[] = "dpfillfgcr";
	static const char _LN_margin_bottom[] = "margin-bottom";
	static const char _LN_bliptag[] = "bliptag";
	static const char _LN_msAccessHTML[] = "msAccessHTML";
	static const char _LN_sftnncnum[] = "sftnncnum";
	static const char _LN_BackgroundQuery[] = "BackgroundQuery";
	static const char _LN_blip[] = "blip";
	static const char _LN_lMargin[] = "lMargin";
	static const char _LN_strSelect[] = "strSelect";
	static const char _LN_table_name[] = "table-name";
	static const char _LN_HideBar[] = "HideBar";
	static const char _LN_parse_sql_statement[] = "parse-sql-statement";
	static const char _LN_oldas[] = "oldas";
	static const char _LN_handle_mirror_horizontal[] = "handle-mirror-horizontal";
	static const char _LN_srcRect[] = "srcRect";
	static const char _LN_text_transform[] = "text-transform";
	static const char _LN_BeginArrow[] = "BeginArrow";
	static const char _LN_blipuid[] = "blipuid";
	static const char _LN_CustomProp[] = "CustomProp";
	static const char _LN_rMargin[] = "rMargin";
	static const char _LN_dpfillfggray[] = "dpfillfggray";
	static const char _LN_sftnndbar[] = "sftnndbar";
	static const char _LN_margin[] = "margin";
	static const char _LN_schema[] = "schema";
	static const char _LN_mbrProps[] = "mbrProps";
	static const char _LN_Rollup[] = "Rollup";
	static const char _LN_table_type[] = "table-type";
	static const char _LN_oldcprops[] = "oldcprops";
	static const char _LN_CubeSource[] = "CubeSource";
	static const char _LN_macro_name[] = "macro-name";
	static const char _LN_inline[] = "inline";
	static const char _LN_tile[] = "tile";
	static const char _LN_blipupi[] = "blipupi";
	static const char _LN_defJc[] = "defJc";
	static const char _LN_EndArrow[] = "EndArrow";
	static const char _LN_GlueSettings[] = "GlueSettings";
	static const char _LN_break_before[] = "break-before";
	static const char _LN_sftnndbnum[] = "sftnndbnum";
	static const char _LN_database_source_table[] = "database-source-table";
	static const char _LN_dpfillfgpal[] = "dpfillfgpal";
	static const char _LN_BCWS[] = "BCWS";
	static const char _LN_itms[] = "itms";
	static const char _LN_oldpprops[] = "oldpprops";
	static const char _LN_stretch[] = "stretch";
	static const char _LN_database_display[] = "database-display";
	static const char _LN_DisableRefresh[] = "DisableRefresh";
	static const char _LN_handle_switched[] = "handle-switched";
	static const char _LN_args[] = "args";
	static const char _LN_math[] = "math";
	static const char _LN_use_window_font_color[] = "use-window-font-color";
	static const char _LN_preSp[] = "preSp";
	static const char _LN_partFragment[] = "partFragment";
	static const char _LN_LineCap[] = "LineCap";
	static const char _LN_BCWP[] = "BCWP";
	static const char _LN_selctPg[] = "selctPg";
	static const char _LN_sftnndbnumd[] = "sftnndbnumd";
	static const char _LN_bookfold[] = "bookfold";
	static const char _LN_SnapSettings[] = "SnapSettings";
	static const char _LN_break_after[] = "break-after";
	static const char _LN_appletText[] = "appletText";
	static const char _LN_dpfillpat[] = "dpfillpat";
	static const char _LN_anyName[] = "anyName";
	static const char _LN_fgClr[] = "fgClr";
	static const char _LN_oldsprops[] = "oldsprops";
	static const char _LN_postSp[] = "postSp";
	static const char _LN_PhysicalPercentComplete[] = "PhysicalPercentComplete";
	static const char _LN_HasNoRecords[] = "HasNoRecords";
	static const char _LN_BeginArrowSize[] = "BeginArrowSize";
	static const char _LN_column_name[] = "column-name";
	static const char _LN_database_table_name[] = "database-table-name";
	static const char _LN_handle_position[] = "handle-position";
	static const char _LN_mp[] = "mp";
	static const char _LN_xmlFileAdapter[] = "xmlFileAdapter";
	static const char _LN_bookfoldrev[] = "bookfoldrev";
	static const char _LN_bgClr[] = "bgClr";
	static const char _LN_dde_connection_decl[] = "dde-connection-decl";
	static const char _LN_sftnndbnumk[] = "sftnndbnumk";
	static const char _LN_text_outline[] = "text-outline";
	static const char _LN_fldData[] = "fldData";
	static const char _LN_intraSp[] = "intraSp";
	static const char _LN_dpgroup[] = "dpgroup";
	static const char _LN_SnapExtensions[] = "SnapExtensions";
	static const char _LN_background_color[] = "background-color";
	static const char _LN_oldtprops[] = "oldtprops";
	static const char _LN_noFill[] = "noFill";
	static const char _LN_OptimizeCache[] = "OptimizeCache";
	static const char _LN_mg[] = "mg";
	static const char _LN_database_next[] = "database-next";
	static const char _LN_LineColorTrans[] = "LineColorTrans";
	static const char _LN_handle_range_x_minimum[] = "handle-range-x-minimum";
	static const char _LN_interSp[] = "interSp";
	static const char _LN_dpline[] = "dpline";
	static const char _LN_bookfoldsheets[] = "bookfoldsheets";
	static const char _LN_sharepointListAdapter[] = "sharepointListAdapter";
	static const char _LN_sftnndbnumt[] = "sftnndbnumt";
	static const char _LN_database_source_query[] = "database-source-query";
	static const char _LN_oldlinewrap[] = "oldlinewrap";
	static const char _LN_PredecessorLink[] = "PredecessorLink";
	static const char _LN_text_line_through_type[] = "text-line-through-type";
	static const char _LN_SnapAngles[] = "SnapAngles";
	static const char _LN_fldSimple[] = "fldSimple";
	static const char _LN_background_image[] = "background-image";
	static const char _LN_solidFill[] = "solidFill";
	static const char _LN_dde_application[] = "dde-application";
	static const char _LN_sxfilter[] = "sxfilter";
	static const char _LN_dplinecob[] = "dplinecob";
	static const char _LN_wrapRight[] = "wrapRight";
	static const char _LN_RefreshOnFileOpen[] = "RefreshOnFileOpen";
	static const char _LN_database_row_select[] = "database-row-select";
	static const char _LN_FillForegnd[] = "FillForegnd";
	static const char _LN_sftnnganada[] = "sftnnganada";
	static const char _LN_hyperlink[] = "hyperlink";
	static const char _LN_DynamicGridEnabled[] = "DynamicGridEnabled";
	static const char _LN_gradFill[] = "gradFill";
	static const char _LN_PredecessorUID[] = "PredecessorUID";
	static const char _LN_handle_range_x_maximum[] = "handle-range-x-maximum";
	static const char _LN_brdrart[] = "brdrart";
	static const char _LN_autofilter[] = "autofilter";
	static const char _LN_otblrul[] = "otblrul";
	static const char _LN_dplinecog[] = "dplinecog";
	static const char _LN_query_name[] = "query-name";
	static const char _LN_wrapIndent[] = "wrapIndent";
	static const char _LN_dde_topic[] = "dde-topic";
	static const char _LN_repeat[] = "repeat";
	static const char _LN_text_line_through_style[] = "text-line-through-style";
	static const char _LN_davAdapter[] = "davAdapter";
	static const char _LN_FillBkgnd[] = "FillBkgnd";
	static const char _LN_sftnngbnum[] = "sftnngbnum";
	static const char _LN_PublishObject[] = "PublishObject";
	static const char _LN_ffData[] = "ffData";
	static const char _LN_blipFill[] = "blipFill";
	static const char _LN_row_number[] = "row-number";
	static const char _LN_brdrb[] = "brdrb";
	static const char _LN_outl[] = "outl";
	static const char _LN_dplinecor[] = "dplinecor";
	static const char _LN_CrossProject[] = "CrossProject";
	static const char _LN_sort[] = "sort";
	static const char _LN_naryGrow[] = "naryGrow";
	static const char _LN_dde_item[] = "dde-item";
	static const char _LN_ProtectStyles[] = "ProtectStyles";
	static const char _LN_folderURL[] = "folderURL";
	static const char _LN_FillPattern[] = "FillPattern";
	static const char _LN_outlinelevel[] = "outlinelevel";
	static const char _LN_brdrbar[] = "brdrbar";
	static const char _LN_handle_range_y_minimum[] = "handle-range-y-minimum";
	static const char _LN_pattFill[] = "pattFill";
	static const char _LN_sftnngbnumd[] = "sftnngbnumd";
	static const char _LN_database_row_number[] = "database-row-number";
	static const char _LN_DivID[] = "DivID";
	static const char _LN_border_top[] = "border-top";
	static const char _LN_bind_styles_to_content[] = "bind-styles-to-content";
	static const char _LN_text_line_through_width[] = "text-line-through-width";
	static const char _LN_dplinedado[] = "dplinedado";
	static const char _LN_intLim[] = "intLim";
	static const char _LN_automatic_update[] = "automatic-update";
	static const char _LN_bordertop[] = "bordertop";
	static const char _LN_CrossProjectName[] = "CrossProjectName";
	static const char _LN_calcOnExit[] = "calcOnExit";
	static const char _LN_fileName[] = "fileName";
	static const char _LN_ProtectShapes[] = "ProtectShapes";
	static const char _LN_brdrbtw[] = "brdrbtw";
	static const char _LN_ShdwForegnd[] = "ShdwForegnd";
	static const char _LN_grpFill[] = "grpFill";
	static const char _LN_SourceType[] = "SourceType";
	static const char _LN_sftnngbnumk[] = "sftnngbnumk";
	static const char _LN_overlay[] = "overlay";
	static const char _LN_border_bottom[] = "border-bottom";
	static const char _LN_naryLim[] = "naryLim";
	static const char _LN_dplinedadodo[] = "dplinedadodo";
	static const char _LN_borderleft[] = "borderleft";
	static const char _LN_emailAdapter[] = "emailAdapter";
	static const char _LN_handle_range_y_maximum[] = "handle-range-y-maximum";
	static const char _LN_brdrcf[] = "brdrcf";
	static const char _LN_entryMacro[] = "entryMacro";
	static const char _LN_cont[] = "cont";
	static const char _LN_dde_link[] = "dde-link";
	static const char _LN_LinkLag[] = "LinkLag";
	static const char _LN_ProtectMasters[] = "ProtectMasters";
	static const char _LN_HtmlType[] = "HtmlType";
	static const char _LN_ShdwBkgnd[] = "ShdwBkgnd";
	static const char _LN_text_line_through_color[] = "text-line-through-color";
	static const char _LN_page_variable_set[] = "page-variable-set";
	static const char _LN_mathPr[] = "mathPr";
	static const char _LN_sftnngbnuml[] = "sftnngbnuml";
	static const char _LN_trackRevisions[] = "trackRevisions";
	static const char _LN_pagebb[] = "pagebb";
	static const char _LN_border_left[] = "border-left";
	static const char _LN_brdrdash[] = "brdrdash";
	static const char _LN_borderright[] = "borderright";
	static const char _LN_dplinedash[] = "dplinedash";
	static const char _LN_querytable[] = "querytable";
	static const char _LN_dde_source[] = "dde-source";
	static const char _LN_exitMacro[] = "exitMacro";
	static const char _LN_algorithm[] = "algorithm";
	static const char _LN_LagFormat[] = "LagFormat";
	static const char _LN_AutoRepublish[] = "AutoRepublish";
	static const char _LN_alphaOutset[] = "alphaOutset";
	static const char _LN_oMathParaPr[] = "oMathParaPr";
	static const char _LN_ShdwPattern[] = "ShdwPattern";
	static const char _LN_ProtectBkgnds[] = "ProtectBkgnds";
	static const char _LN_panose[] = "panose";
	static const char _LN_cc[] = "cc";
	static const char _LN_sftnnrlc[] = "sftnnrlc";
	static const char _LN_handle_polar[] = "handle-polar";
	static const char _LN_brdrdashd[] = "brdrdashd";
	static const char _LN_trackMoves[] = "trackMoves";
	static const char _LN_active[] = "active";
	static const char _LN_border_right[] = "border-right";
	static const char _LN_dplinedot[] = "dplinedot";
	static const char _LN_borderbottom[] = "borderbottom";
	static const char _LN_qtRefr[] = "qtRefr";
	static const char _LN_helpText[] = "helpText";
	static const char _LN_sort_by[] = "sort-by";
	static const char _LN_conversion_mode[] = "conversion-mode";
	static const char _LN_ActualWorkProtected[] = "ActualWorkProtected";
	static const char _LN_text_line_through_text[] = "text-line-through-text";
	static const char _LN_bcc[] = "bcc";
	static const char _LN_blend[] = "blend";
	static const char _LN_oMath[] = "oMath";
	static const char _LN_paperh[] = "paperh";
	static const char _LN_FillForegndTrans[] = "FillForegndTrans";
	static const char _LN_QueryType[] = "QueryType";
	static const char _LN_sftnnruc[] = "sftnnruc";
	static const char _LN_page_variable_get[] = "page-variable-get";
	static const char _LN_CustomMenusFile[] = "CustomMenusFile";
	static const char _LN_brdrdashdd[] = "brdrdashdd";
	static const char _LN_handle_radius_range_minimum[] = "handle-radius-range-minimum";
	static const char _LN_trackFormatting[] = "trackFormatting";
	static const char _LN_dplinegray[] = "dplinegray";
	static const char _LN_field_number[] = "field-number";
	static const char _LN_statusText[] = "statusText";
	static const char _LN_border_line_width[] = "border-line-width";
	static const char _LN_oMathPara[] = "oMathPara";
	static const char _LN_paperw[] = "paperw";
	static const char _LN_intro[] = "intro";
	static const char _LN_delNms[] = "delNms";
	static const char _LN_animate[] = "animate";
	static const char _LN_sftnnzodiac[] = "sftnnzodiac";
	static const char _LN_anchorlock[] = "anchorlock";
	static const char _LN_glow[] = "glow";
	static const char _LN_ActualOvertimeWorkProtected[] = "ActualOvertimeWorkProtected";
	static const char _LN_field[] = "field";
	static const char _LN_FillBkgndTrans[] = "FillBkgndTrans";
	static const char _LN_brdrdashdotstr[] = "brdrdashdotstr";
	static const char _LN_text_line_through_text_style[] = "text-line-through-text-style";
	static const char _LN_dplinehollow[] = "dplinehollow";
	static const char _LN_attachmentFileName[] = "attachmentFileName";
	static const char _LN_CustomToolbarsFile[] = "CustomToolbarsFile";
	static const char _LN_checkBox[] = "checkBox";
	static const char _LN_qtflds[] = "qtflds";
	static const char _LN_placeholder[] = "placeholder";
	static const char _LN_data_type[] = "data-type";
	static const char _LN_documentProtection[] = "documentProtection";
	static const char _LN_innerShdw[] = "innerShdw";
	static const char _LN_set[] = "set";
	static const char _LN_Maintain[] = "Maintain";
	static const char _LN_border_line_width_top[] = "border-line-width-top";
	static const char _LN_pararsid[] = "pararsid";
	static const char _LN_sftnnzodiacd[] = "sftnnzodiacd";
	static const char _LN_handle_radius_range_maximum[] = "handle-radius-range-maximum";
	static const char _LN_animateMotion[] = "animateMotion";
	static const char _LN_delNm[] = "delNm";
	static const char _LN_ddList[] = "ddList";
	static const char _LN_ShdwForegndTrans[] = "ShdwForegndTrans";
	static const char _LN_brdrdashsm[] = "brdrdashsm";
	static const char _LN_order[] = "order";
	static const char _LN_dplinepal[] = "dplinepal";
	static const char _LN_placeholder_type[] = "placeholder-type";
	static const char _LN_outerShdw[] = "outerShdw";
	static const char _LN_pard[] = "pard";
	static const char _LN_dataAdapters[] = "dataAdapters";
	static const char _LN_AttachedToolbars[] = "AttachedToolbars";
	static const char _LN_Baseline[] = "Baseline";
	static const char _LN_autoFormatOverride[] = "autoFormatOverride";
	static const char _LN_qtfld[] = "qtfld";
	static const char _LN_sftnnzodiacl[] = "sftnnzodiacl";
	static const char _LN_textInput[] = "textInput";
	static const char _LN_subtotal_rules[] = "subtotal-rules";
	static const char _LN_text_position[] = "text-position";
	static const char _LN_border_line_width_bottom[] = "border-line-width-bottom";
	static const char _LN_pc[] = "pc";
	static const char _LN_dplinesolid[] = "dplinesolid";
	static const char _LN_brdrdb[] = "brdrdb";
	static const char _LN_CommandTextOrignal[] = "CommandTextOrignal";
	static const char _LN_prstShdw[] = "prstShdw";
	static const char _LN_ShdwBkgndTrans[] = "ShdwBkgndTrans";
	static const char _LN_TimephasedData[] = "TimephasedData";
	static const char _LN_conditional_text[] = "conditional-text";
	static const char _LN_origin[] = "origin";
	static const char _LN_documentSchemas[] = "documentSchemas";
	static const char _LN_pca[] = "pca";
	static const char _LN_class[] = "class";
	static const char _LN_sftnrestart[] = "sftnrestart";
	static const char _LN_SnapAngle[] = "SnapAngle";
	static const char _LN_brdrdot[] = "brdrdot";
	static const char _LN_font_name[] = "font-name";
	static const char _LN_page_breaks_on_group_change[] = "page-breaks-on-group-change";
	static const char _LN_reflection[] = "reflection";
	static const char _LN_styleLockTheme[] = "styleLockTheme";
	static const char _LN_dplinew[] = "dplinew";
	static const char _LN_sizeAuto[] = "sizeAuto";
	static const char _LN_pgbrdrb[] = "pgbrdrb";
	static const char _LN_calcMode[] = "calcMode";
	static const char _LN_Types[] = "Types";
	static const char _LN_SourceConnectionFile[] = "SourceConnectionFile";
	static const char _LN_ShapeShdwType[] = "ShapeShdwType";
	static const char _LN_brdremboss[] = "brdremboss";
	static const char _LN_Number[] = "Number";
	static const char _LN_ColorEntry[] = "ColorEntry";
	static const char _LN_border_line_width_left[] = "border-line-width-left";
	static const char _LN_string_value_if_true[] = "string-value-if-true";
	static const char _LN_sftnrstcont[] = "sftnrstcont";
	static const char _LN_documentSchema[] = "documentSchema";
	static const char _LN_user_transformed[] = "user-transformed";
	static const char _LN_font_name_asian[] = "font-name-asian";
	static const char _LN_dppolycount[] = "dppolycount";
	static const char _LN_relOff[] = "relOff";
	static const char _LN_pgbrdrfoot[] = "pgbrdrfoot";
	static const char _LN_Default[] = "Default";
	static const char _LN_animateColor[] = "animateColor";
	static const char _LN_styleLockQFSet[] = "styleLockQFSet";
	static const char _LN_listEntry[] = "listEntry";
	static const char _LN_Interim[] = "Interim";
	static const char _LN_brdrengrave[] = "brdrengrave";
	static const char _LN_ShapeShdwOffsetX[] = "ShapeShdwOffsetX";
	static const char _LN_PaperSize[] = "PaperSize";
	static const char _LN_sftEdg[] = "sftEdg";
	static const char _LN_sftnrstpg[] = "sftnrstpg";
	static const char _LN_sort_groups[] = "sort-groups";
	static const char _LN_dppolygon[] = "dppolygon";
	static const char _LN_SourceDataFile[] = "SourceDataFile";
	static const char _LN_Override[] = "Override";
	static const char _LN_customValidation[] = "customValidation";
	static const char _LN_pgbrdrhead[] = "pgbrdrhead";
	static const char _LN_animations[] = "animations";
	static const char _LN_font_name_complex[] = "font-name-complex";
	static const char _LN_color_interpolation[] = "color-interpolation";
	static const char _LN_EstimatedDuration[] = "EstimatedDuration";
	static const char _LN_string_value_if_false[] = "string-value-if-false";
	static const char _LN_border_line_width_right[] = "border-line-width-right";
	static const char _LN_defaultTabStop[] = "defaultTabStop";
	static const char _LN_workSheet[] = "workSheet";
	static const char _LN_effectLst[] = "effectLst";
	static const char _LN_PaperWidth[] = "PaperWidth";
	static const char _LN_brdrframe[] = "brdrframe";
	static const char _LN_maxLengh[] = "maxLengh";
	static const char _LN_sftnstart[] = "sftnstart";
	static const char _LN_dppolyline[] = "dppolyline";
	static const char _LN_subtotal_rule[] = "subtotal-rule";
	static const char _LN_ShapeShdwOffsetY[] = "ShapeShdwOffsetY";
	static const char _LN_pgbrdrl[] = "pgbrdrl";
	static const char _LN_sound[] = "sound";
	static const char _LN_RobustConnect[] = "RobustConnect";
	static const char _LN_errorCondition[] = "errorCondition";
	static const char _LN_macroSheet[] = "macroSheet";
	static const char _LN_effectDag[] = "effectDag";
	static const char _LN_brdrhair[] = "brdrhair";
	static const char _LN_Resources[] = "Resources";
	static const char _LN_PaperHeight[] = "PaperHeight";
	static const char _LN_autoHyphenation[] = "autoHyphenation";
	static const char _LN_format[] = "format";
	static const char _LN_color_interpolation_direction[] = "color-interpolation-direction";
	static const char _LN_sftntj[] = "sftntj";
	static const char _LN_current_value[] = "current-value";
	static const char _LN_play_full[] = "play-full";
	static const char _LN_pgbrdropt[] = "pgbrdropt";
	static const char _LN_dpptx[] = "dpptx";
	static const char _LN_font_family_asian[] = "font-family-asian";
	static const char _LN_extension[] = "extension";
	static const char _LN_padding[] = "padding";
	static const char _LN_group_by_field_number[] = "group-by-field-number";
	static const char _LN_ShapeShdwObliqueAngle[] = "ShapeShdwObliqueAngle";
	static const char _LN_RefreshTimeSpan[] = "RefreshTimeSpan";
	static const char _LN_dialogSheet[] = "dialogSheet";
	static const char _LN_col[] = "col";
	static const char _LN_shad[] = "shad";
	static const char _LN_brdrinset[] = "brdrinset";
	static const char _LN_errorMessage[] = "errorMessage";
	static const char _LN_Resource[] = "Resource";
	static const char _LN_dppty[] = "dppty";
	static const char _LN_PrintCenteredH[] = "PrintCenteredH";
	static const char _LN_show_shape[] = "show-shape";
	static const char _LN_pgbrdrr[] = "pgbrdrr";
	static const char _LN_padding_top[] = "padding-top";
	static const char _LN_consecutiveHyphenLimit[] = "consecutiveHyphenLimit";
	static const char _LN_hidden_text[] = "hidden-text";
	static const char _LN_hdr[] = "hdr";
	static const char _LN_shading[] = "shading";
	static const char _LN_font_family_complex[] = "font-family-complex";
	static const char _LN_dprect[] = "dprect";
	static const char _LN_chartSheet[] = "chartSheet";
	static const char _LN_brdrl[] = "brdrl";
	static const char _LN_LocalConnection[] = "LocalConnection";
	static const char _LN_domEventHandlers[] = "domEventHandlers";
	static const char _LN_subtotal_field[] = "subtotal-field";
	static const char _LN_pgbrdrsnap[] = "pgbrdrsnap";
	static const char _LN_animateTransform[] = "animateTransform";
	static const char _LN_ftr[] = "ftr";
	static const char _LN_shape_id[] = "shape-id";
	static const char _LN_ShapeShdwScaleFactor[] = "ShapeShdwScaleFactor";
	static const char _LN_Phonetics[] = "Phonetics";
	static const char _LN_PrintCenteredV[] = "PrintCenteredV";
	static const char _LN_padding_bottom[] = "padding-bottom";
	static const char _LN_shidden[] = "shidden";
	static const char _LN_is_hidden[] = "is-hidden";
	static const char _LN_dproundr[] = "dproundr";
	static const char _LN_brdrnil[] = "brdrnil";
	static const char _LN_headerReference[] = "headerReference";
	static const char _LN_wsPr[] = "wsPr";
	static const char _LN_hyphenationZone[] = "hyphenationZone";
	static const char _LN_pgbrdrt[] = "pgbrdrt";
	static const char _LN_font_family_generic_asian[] = "font-family-generic-asian";
	static const char _LN_NoRefreshCache[] = "NoRefreshCache";
	static const char _LN_NTAccount[] = "NTAccount";
	static const char _LN_function[] = "function";
	static const char _LN_shift[] = "shift";
	static const char _LN_domEventHandler[] = "domEventHandler";
	static const char _LN_note_ref[] = "note-ref";
	static const char _LN_dpshadow[] = "dpshadow";
	static const char _LN_brdroutset[] = "brdroutset";
	static const char _LN_arc[] = "arc";
	static const char _LN_PrintFitOnPages[] = "PrintFitOnPages";
	static const char _LN_padding_left[] = "padding-left";
	static const char _LN_BeginX[] = "BeginX";
	static const char _LN_transitionFilter[] = "transitionFilter";
	static const char _LN_pghsxn[] = "pghsxn";
	static const char _LN_wsViews[] = "wsViews";
	static const char _LN_footerReference[] = "footerReference";
	static const char _LN_shpbottom[] = "shpbottom";
	static const char _LN_filter[] = "filter";
	static const char _LN_MaterialLabel[] = "MaterialLabel";
	static const char _LN_doNotHyphenateCaps[] = "doNotHyphenateCaps";
	static const char _LN_sequence_ref[] = "sequence-ref";
	static const char _LN_dpshadx[] = "dpshadx";
	static const char _LN_brdrr[] = "brdrr";
	static const char _LN_UseLocalConnection[] = "UseLocalConnection";
	static const char _LN_BeginY[] = "BeginY";
	static const char _LN_padding_right[] = "padding-right";
	static const char _LN_pgnbidia[] = "pgnbidia";
	static const char _LN_PrintLandscape[] = "PrintLandscape";
	static const char _LN_userShViews[] = "userShViews";
	static const char _LN_condition_source[] = "condition-source";
	static const char _LN_font_family_generic_complex[] = "font-family-generic-complex";
	static const char _LN_shpbxcolumn[] = "shpbxcolumn";
	static const char _LN_CoreProperties[] = "CoreProperties";
	static const char _LN_dpshady[] = "dpshady";
	static const char _LN_brdrs[] = "brdrs";
	static const char _LN_footnotePr[] = "footnotePr";
	static const char _LN_Code[] = "Code";
	static const char _LN_EndX[] = "EndX";
	static const char _LN_reference_format[] = "reference-format";
	static const char _LN_subtype[] = "subtype";
	static const char _LN_pgnbidib[] = "pgnbidib";
	static const char _LN_showEnvelope[] = "showEnvelope";
	static const char _LN_shadow[] = "shadow";
	static const char _LN_wsFmtInfo[] = "wsFmtInfo";
	static const char _LN_VersionLastEdit[] = "VersionLastEdit";
	static const char _LN_brdrsh[] = "brdrsh";
	static const char _LN_PrintPagesAcross[] = "PrintPagesAcross";
	static const char _LN_shpbxignore[] = "shpbxignore";
	static const char _LN_dptxbtlr[] = "dptxbtlr";
	static const char _LN_EndY[] = "EndY";
	static const char _LN_importParameters[] = "importParameters";
	static const char _LN_condition_source_range_address[] = "condition-source-range-address";
	static const char _LN_all[] = "all";
	static const char _LN_endnotePr[] = "endnotePr";
	static const char _LN_base[] = "base";
	static const char _LN_WorkGroup[] = "WorkGroup";
	static const char _LN_pgnchosung[] = "pgnchosung";
	static const char _LN_keep_with_next[] = "keep-with-next";
	static const char _LN_brdrt[] = "brdrt";
	static const char _LN_TheData[] = "TheData";
	static const char _LN_summaryLength[] = "summaryLength";
	static const char _LN_colInfos[] = "colInfos";
	static const char _LN_fadeColor[] = "fadeColor";
	static const char _LN_font_style_name[] = "font-style-name";
	static const char _LN_Creator[] = "Creator";
	static const char _LN_dptxbx[] = "dptxbx";
	static const char _LN_default[] = "default";
	static const char _LN_shpbxmargin[] = "shpbxmargin";
	static const char _LN_DoNotJoinDelimiters[] = "DoNotJoinDelimiters";
	static const char _LN_execute_macro[] = "execute-macro";
	static const char _LN_gd[] = "gd";
	static const char _LN_pgSz[] = "pgSz";
	static const char _LN_PrintPagesDown[] = "PrintPagesDown";
	static const char _LN_EmailAddress[] = "EmailAddress";
	static const char _LN_brdrtbl[] = "brdrtbl";
	static const char _LN_importSource[] = "importSource";
	static const char _LN_pgncnum[] = "pgncnum";
	static const char _LN_TheText[] = "TheText";
	static const char _LN_sheetData[] = "sheetData";
	static const char _LN_dptxbxmar[] = "dptxbxmar";
	static const char _LN_Title[] = "Title";
	static const char _LN_pos[] = "pos";
	static const char _LN_line_number[] = "line-number";
	static const char _LN_content[] = "content";
	static const char _LN_clickAndTypeStyle[] = "clickAndTypeStyle";
	static const char _LN_pgMar[] = "pgMar";
	static const char _LN_font_style_name_asian[] = "font-style-name-asian";
	static const char _LN_shpbxpage[] = "shpbxpage";
	static const char _LN_brdrth[] = "brdrth";
	static const char _LN_hidden_paragraph[] = "hidden-paragraph";
	static const char _LN_display_duplicates[] = "display-duplicates";
	static const char _LN_pgncont[] = "pgncont";
	static const char _LN_ahXY[] = "ahXY";
	static const char _LN_EventDblClick[] = "EventDblClick";
	static const char _LN_Subject[] = "Subject";
	static const char _LN_MaxUnits[] = "MaxUnits";
	static const char _LN_PrintScale[] = "PrintScale";
	static const char _LN_listProperties[] = "listProperties";
	static const char _LN_NoTextToColumns[] = "NoTextToColumns";
	static const char _LN_targetElement[] = "targetElement";
	static const char _LN_calcChains[] = "calcChains";
	static const char _LN_paperSrc[] = "paperSrc";
	static const char _LN_dptxbxtext[] = "dptxbxtext";
	static const char _LN_context[] = "context";
	static const char _LN_text_autospace[] = "text-autospace";
	static const char _LN_shpbyignore[] = "shpbyignore";
	static const char _LN_brdrthtnlg[] = "brdrthtnlg";
	static const char _LN_ahPolar[] = "ahPolar";
	static const char _LN_pgndbnum[] = "pgndbnum";
	static const char _LN_defaultTableStyle[] = "defaultTableStyle";
	static const char _LN_Category[] = "Category";
	static const char _LN_PeakUnits[] = "PeakUnits";
	static const char _LN_FontEntry[] = "FontEntry";
	static const char _LN_pgBorders[] = "pgBorders";
	static const char _LN_EventXFMod[] = "EventXFMod";
	static const char _LN_dde_connection[] = "dde-connection";
	static const char _LN_dptxlrtb[] = "dptxlrtb";
	static const char _LN_font_style_name_complex[] = "font-style-name-complex";
	static const char _LN_filter_and[] = "filter-and";
	static const char _LN_attributeName[] = "attributeName";
	static const char _LN_DisableDateRecognition[] = "DisableDateRecognition";
	static const char _LN_cxn[] = "cxn";
	static const char _LN_punctuation_wrap[] = "punctuation-wrap";
	static const char _LN_sortState[] = "sortState";
	static const char _LN_pgndbnumd[] = "pgndbnumd";
	static const char _LN_brdrthtnmg[] = "brdrthtnmg";
	static const char _LN_shpbymargin[] = "shpbymargin";
	static const char _LN_submit[] = "submit";
	static const char _LN_Keywords[] = "Keywords";
	static const char _LN_AvailableFrom[] = "AvailableFrom";
	static const char _LN_FaceName[] = "FaceName";
	static const char _LN_lnNumType[] = "lnNumType";
	static const char _LN_dptxlrtbv[] = "dptxlrtbv";
	static const char _LN_EventDrop[] = "EventDrop";
	static const char _LN_evenAndOddHeaders[] = "evenAndOddHeaders";
	static const char _LN_filter_or[] = "filter-or";
	static const char _LN_connection_name[] = "connection-name";
	static const char _LN_sub_item[] = "sub-item";
	static const char _LN_submitAction[] = "submitAction";
	static const char _LN_pt[] = "pt";
	static const char _LN_rwBrks[] = "rwBrks";
	static const char _LN_pgndbnumk[] = "pgndbnumk";
	static const char _LN_brdrthtnsg[] = "brdrthtnsg";
	static const char _LN_Description[] = "Description";
	static const char _LN_shpbypage[] = "shpbypage";
	static const char _LN_StyleSheet[] = "StyleSheet";
	static const char _LN_font_pitch_asian[] = "font-pitch-asian";
	static const char _LN_pgNumType[] = "pgNumType";
	static const char _LN_dptxtbrl[] = "dptxtbrl";
	static const char _LN_EnableRedirections[] = "EnableRedirections";
	static const char _LN_AvailableTo[] = "AvailableTo";
	static const char _LN_LayerMember[] = "LayerMember";
	static const char _LN_close[] = "close";
	static const char _LN_filter_condition[] = "filter-condition";
	static const char _LN_vertical_align[] = "vertical-align";
	static const char _LN_colBrks[] = "colBrks";
	static const char _LN_values[] = "values";
	static const char _LN_pgndbnumt[] = "pgndbnumt";
	static const char _LN_bookFoldRevPrinting[] = "bookFoldRevPrinting";
	static const char _LN_brdrtnthlg[] = "brdrtnthlg";
	static const char _LN_successMessage[] = "successMessage";
	static const char _LN_measure[] = "measure";
	static const char _LN_shpbypara[] = "shpbypara";
	static const char _LN_ContentType[] = "ContentType";
	static const char _LN_moveTo[] = "moveTo";
	static const char _LN_Master[] = "Master";
	static const char _LN_cols[] = "cols";
	static const char _LN_dptxtbrlv[] = "dptxtbrlv";
	static const char _LN_CanLevel[] = "CanLevel";
	static const char _LN_EnableLineProps[] = "EnableLineProps";
	static const char _LN_keyTimes[] = "keyTimes";
	static const char _LN_pageMargins[] = "pageMargins";
	static const char _LN_font_pitch_complex[] = "font-pitch-complex";
	static const char _LN_pgndec[] = "pgndec";
	static const char _LN_EditWebPage[] = "EditWebPage";
	static const char _LN_kind[] = "kind";
	static const char _LN_writing_mode[] = "writing-mode";
	static const char _LN_formProt[] = "formProt";
	static const char _LN_shpfblwtxt[] = "shpfblwtxt";
	static const char _LN_brdrtnthmg[] = "brdrtnthmg";
	static const char _LN_lineTo[] = "lineTo";
	static const char _LN_MasterShortcut[] = "MasterShortcut";
	static const char _LN_ContentStatus[] = "ContentStatus";
	static const char _LN_useHttpHandler[] = "useHttpHandler";
	static const char _LN_dpx[] = "dpx";
	static const char _LN_operator[] = "operator";
	static const char _LN_AccrueAt[] = "AccrueAt";
	static const char _LN_bookFoldPrinting[] = "bookFoldPrinting";
	static const char _LN_keySplines[] = "keySplines";
	static const char _LN_table_formula[] = "table-formula";
	static const char _LN_printOptions[] = "printOptions";
	static const char _LN_pgndecd[] = "pgndecd";
	static const char _LN_EnableFillProps[] = "EnableFillProps";
	static const char _LN_arcTo[] = "arcTo";
	static const char _LN_vAlign[] = "vAlign";
	static const char _LN_dpxsize[] = "dpxsize";
	static const char _LN_RefreshedInXl9[] = "RefreshedInXl9";
	static const char _LN_shpfhdr[] = "shpfhdr";
	static const char _LN_brdrtnthsg[] = "brdrtnthsg";
	static const char _LN_writing_mode_automatic[] = "writing-mode-automatic";
	static const char _LN_data_pilot_tables[] = "data-pilot-tables";
	static const char _LN_StandardRate[] = "StandardRate";
	static const char _LN_PageSheet[] = "PageSheet";
	static const char _LN_Version[] = "Version";
	static const char _LN_useScriptHandler[] = "useScriptHandler";
	static const char _LN_accumulate[] = "accumulate";
	static const char _LN_pgnganada[] = "pgnganada";
	static const char _LN_quadBezTo[] = "quadBezTo";
	static const char _LN_noEndnote[] = "noEndnote";
	static const char _LN_bookFoldPrintingSheets[] = "bookFoldPrintingSheets";
	static const char _LN_pageSetup[] = "pageSetup";
	static const char _LN_dpy[] = "dpy";
	static const char _LN_shpgrp[] = "shpgrp";
	static const char _LN_brdrtnthtnlg[] = "brdrtnthtnlg";
	static const char _LN_EnableTextProps[] = "EnableTextProps";
	static const char _LN_EntirePage[] = "EntirePage";
	static const char _LN_currency[] = "currency";
	static const char _LN_Revision[] = "Revision";
	static const char _LN_Shapes[] = "Shapes";
	static const char _LN_dpysize[] = "dpysize";
	static const char _LN_StandardRateFormat[] = "StandardRateFormat";
	static const char _LN_pgngbnum[] = "pgngbnum";
	static const char _LN_additive[] = "additive";
	static const char _LN_cubicBezTo[] = "cubicBezTo";
	static const char _LN_data_pilot_table[] = "data-pilot-table";
	static const char _LN_titlePg[] = "titlePg";
	static const char _LN_shpleft[] = "shpleft";
	static const char _LN_snap_to_layout_grid[] = "snap-to-layout-grid";
	static const char _LN_headerFooter[] = "headerFooter";
	static const char _LN_useQueryAdapter[] = "useQueryAdapter";
	static const char _LN_Icon[] = "Icon";
	static const char _LN_fixed[] = "fixed";
	static const char _LN_DateCreated[] = "DateCreated";
	static const char _LN_brdrtnthtnmg[] = "brdrtnthtnmg";
	static const char _LN_HTMLFormat[] = "HTMLFormat";
	static const char _LN_dropcapli[] = "dropcapli";
	static const char _LN_HideForApply[] = "HideForApply";
	static const char _LN_pgngbnumd[] = "pgngbnumd";
	static const char _LN_drawingGridHorizontalSpacing[] = "drawingGridHorizontalSpacing";
	static const char _LN_scriptAnchor[] = "scriptAnchor";
	static const char _LN_textFlow[] = "textFlow";
	static const char _LN_shplid[] = "shplid";
	static const char _LN_avLst[] = "avLst";
	static const char _LN_Connects[] = "Connects";
	static const char _LN_formula[] = "formula";
	static const char _LN_to[] = "to";
	static const char _LN_OvertimeRate[] = "OvertimeRate";
	static const char _LN_application_data[] = "application-data";
	static const char _LN_onLoad[] = "onLoad";
	static const char _LN_DateModified[] = "DateModified";
	static const char _LN_URLString[] = "URLString";
	static const char _LN_dropcapt[] = "dropcapt";
	static const char _LN_brdrtnthtnsg[] = "brdrtnthtnsg";
	static const char _LN_gdLst[] = "gdLst";
	static const char _LN_shplockanchor[] = "shplockanchor";
	static const char _LN_pgngbnumk[] = "pgngbnumk";
	static const char _LN_from[] = "from";
	static const char _LN_rtlGutter[] = "rtlGutter";
	static const char _LN_ImgOffsetX[] = "ImgOffsetX";
	static const char _LN_background_transparency[] = "background-transparency";
	static const char _LN_Shape[] = "Shape";
	static const char _LN_data_style_name[] = "data-style-name";
	static const char _LN_phonetic[] = "phonetic";
	static const char _LN_roles[] = "roles";
	static const char _LN_OvertimeRateFormat[] = "OvertimeRateFormat";
	static const char _LN_ahLst[] = "ahLst";
	static const char _LN_by[] = "by";
	static const char _LN_WebPostString[] = "WebPostString";
	static const char _LN_LastModifiedBy[] = "LastModifiedBy";
	static const char _LN_pgngbnuml[] = "pgngbnuml";
	static const char _LN_grand_total[] = "grand-total";
	static const char _LN_drawingGridVerticalSpacing[] = "drawingGridVerticalSpacing";
	static const char _LN_docGrid[] = "docGrid";
	static const char _LN_brdrtriple[] = "brdrtriple";
	static const char _LN_Connect[] = "Connect";
	static const char _LN_ImgOffsetY[] = "ImgOffsetY";
	static const char _LN_shppict[] = "shppict";
	static const char _LN_dxfrtext[] = "dxfrtext";
	static const char _LN_dataConsolidation[] = "dataConsolidation";
	static const char _LN_role[] = "role";
	static const char _LN_begin[] = "begin";
	static const char _LN_cxnLst[] = "cxnLst";
	static const char _LN_scriptId[] = "scriptId";
	static const char _LN_toc_mark_start[] = "toc-mark-start";
	static const char _LN_pgnhindia[] = "pgnhindia";
	static const char _LN_Page[] = "Page";
	static const char _LN_RefreshInfo[] = "RefreshInfo";
	static const char _LN_membership[] = "membership";
	static const char _LN_ruby_properties[] = "ruby-properties";
	static const char _LN_shpright[] = "shpright";
	static const char _LN_ignore_empty_rows[] = "ignore-empty-rows";
	static const char _LN_brdrw[] = "brdrw";
	static const char _LN_CostPerUse[] = "CostPerUse";
	static const char _LN_dy[] = "dy";
	static const char _LN_LastPrinted[] = "LastPrinted";
	static const char _LN_ImgWidth[] = "ImgWidth";
	static const char _LN_rectLst[] = "rectLst";
	static const char _LN_dur[] = "dur";
	static const char _LN_Window[] = "Window";
	static const char _LN_mergeCells[] = "mergeCells";
	static const char _LN_edmins[] = "edmins";
	static const char _LN_delText[] = "delText";
	static const char _LN_brdrwavy[] = "brdrwavy";
	static const char _LN_toc_mark_end[] = "toc-mark-end";
	static const char _LN_pgnhindib[] = "pgnhindib";
	static const char _LN_scriptText[] = "scriptText";
	static const char _LN_displayHorizontalDrawingGridEvery[] = "displayHorizontalDrawingGridEvery";
	static const char _LN_shprslt[] = "shprslt";
	static const char _LN_getUserNameFromData[] = "getUserNameFromData";
	static const char _LN_TextWizardSettings[] = "TextWizardSettings";
	static const char _LN_ImgHeight[] = "ImgHeight";
	static const char _LN_CostVariance[] = "CostVariance";
	static const char _LN_pathLst[] = "pathLst";
	static const char _LN_endsync[] = "endsync";
	static const char _LN_Language[] = "Language";
	static const char _LN_ruby_position[] = "ruby-position";
	static const char _LN_StencilGroup[] = "StencilGroup";
	static const char _LN_identify_categories[] = "identify-categories";
	static const char _LN_embo[] = "embo";
	static const char _LN_instrText[] = "instrText";
	static const char _LN_dVals[] = "dVals";
	static const char _LN_brdrwavydb[] = "brdrwavydb";
	static const char _LN_shptop[] = "shptop";
	static const char _LN_pgnhindic[] = "pgnhindic";
	static const char _LN_toc_mark[] = "toc-mark";
	static const char _LN_custGeom[] = "custGeom";
	static const char _LN_repeatDur[] = "repeatDur";
	static const char _LN_PageWidth[] = "PageWidth";
	static const char _LN_Identifier[] = "Identifier";
	static const char _LN_emdash[] = "emdash";
	static const char _LN_SV[] = "SV";
	static const char _LN_smartTags[] = "smartTags";
	static const char _LN_ruby_align[] = "ruby-align";
	static const char _LN_HTMLTables[] = "HTMLTables";
	static const char _LN_userName[] = "userName";
	static const char _LN_StencilGroupPos[] = "StencilGroupPos";
	static const char _LN_delInstrText[] = "delInstrText";
	static const char _LN_shptxt[] = "shptxt";
	static const char _LN_brkfrm[] = "brkfrm";
	static const char _LN_pgnhindid[] = "pgnhindid";
	static const char _LN_IsGeneric[] = "IsGeneric";
	static const char _LN_user_index_mark_start[] = "user-index-mark-start";
	static const char _LN_prstGeom[] = "prstGeom";
	static const char _LN_buttons[] = "buttons";
	static const char _LN_repeatCount[] = "repeatCount";
	static const char _LN_PageHeight[] = "PageHeight";
	static const char _LN_emfblip[] = "emfblip";
	static const char _LN_displayVerticalDrawingGridEvery[] = "displayVerticalDrawingGridEvery";
	static const char _LN_hyperlinks[] = "hyperlinks";
	static const char _LN_shpwrk[] = "shpwrk";
	static const char _LN_section_properties[] = "section-properties";
	static const char _LN_UseSameSettings[] = "UseSameSettings";
	static const char _LN_brsp[] = "brsp";
	static const char _LN_noBreakHyphen[] = "noBreakHyphen";
	static const char _LN_pgnhn[] = "pgnhn";
	static const char _LN_IsInactive[] = "IsInactive";
	static const char _LN_show_filter_button[] = "show-filter-button";
	static const char _LN_hwsWorkflow[] = "hwsWorkflow";
	static const char _LN_ShowRulers[] = "ShowRulers";
	static const char _LN_emspace[] = "emspace";
	static const char _LN_ShdwOffsetX[] = "ShdwOffsetX";
	static const char _LN_fill[] = "fill";
	static const char _LN_shpwr[] = "shpwr";
	static const char _LN_bullet[] = "bullet";
	static const char _LN_conditionalFormatting[] = "conditionalFormatting";
	static const char _LN_user_index_mark_end[] = "user-index-mark-end";
	static const char _LN_pgnhnsc[] = "pgnhnsc";
	static const char _LN_endash[] = "endash";
	static const char _LN_fillDefault[] = "fillDefault";
	static const char _LN_Query97[] = "Query97";
	static const char _LN_IsEnterprise[] = "IsEnterprise";
	static const char _LN_ShowGrid[] = "ShowGrid";
	static const char _LN_softHyphen[] = "softHyphen";
	static const char _LN_columns[] = "columns";
	static const char _LN_shpz[] = "shpz";
	static const char _LN_Symbol[] = "Symbol";
	static const char _LN_ShdwOffsetY[] = "ShdwOffsetY";
	static const char _LN_drill_down_on_double_click[] = "drill-down-on-double-click";
	static const char _LN_useMarginsForDrawingGridOrigin[] = "useMarginsForDrawingGridOrigin";
	static const char _LN_allowedActions[] = "allowedActions";
	static const char _LN_pgnhnsh[] = "pgnhnsh";
	static const char _LN_enddoc[] = "enddoc";
	static const char _LN_sl[] = "sl";
	static const char _LN_DoNotPromptForFile[] = "DoNotPromptForFile";
	static const char _LN_ShowPageBreaks[] = "ShowPageBreaks";
	static const char _LN_column_count[] = "column-count";
	static const char _LN_restart[] = "restart";
	static const char _LN_ocx[] = "ocx";
	static const char _LN_buptim[] = "buptim";
	static const char _LN_dayShort[] = "dayShort";
	static const char _LN_BookingType[] = "BookingType";
	static const char _LN_user_index_mark[] = "user-index-mark";
	static const char _LN_legacyDrawing[] = "legacyDrawing";
	static const char _LN_PageScale[] = "PageScale";
	static const char _LN_slmult[] = "slmult";
	static const char _LN_pgnhnsm[] = "pgnhnsm";
	static const char _LN_endnhere[] = "endnhere";
	static const char _LN_Relationships[] = "Relationships";
	static const char _LN_bxe[] = "bxe";
	static const char _LN_restartDefault[] = "restartDefault";
	static const char _LN_ocxPr[] = "ocxPr";
	static const char _LN_monthShort[] = "monthShort";
	static const char _LN_column_gap[] = "column-gap";
	static const char _LN_ActiveDirectoryGUID[] = "ActiveDirectoryGUID";
	static const char _LN_allowedTasks[] = "allowedTasks";
	static const char _LN_ShowGuides[] = "ShowGuides";
	static const char _LN_round[] = "round";
	static const char _LN_snaptogridincell[] = "snaptogridincell";
	static const char _LN_pgnhnsn[] = "pgnhnsn";
	static const char _LN_AutoFormatWidth[] = "AutoFormatWidth";
	static const char _LN_source_cell_range[] = "source-cell-range";
	static const char _LN_DrawingScale[] = "DrawingScale";
	static const char _LN_scenarios[] = "scenarios";
	static const char _LN_index_name[] = "index-name";
	static const char _LN_drawingGridHorizontalOrigin[] = "drawingGridHorizontalOrigin";
	static const char _LN_ocxMBed[] = "ocxMBed";
	static const char _LN_Relationship[] = "Relationship";
	static const char _LN_enspace[] = "enspace";
	static const char _LN_bevel[] = "bevel";
	static const char _LN_yearShort[] = "yearShort";
	static const char _LN_accelerate[] = "accelerate";
	static const char _LN_pgnhnsp[] = "pgnhnsp";
	static const char _LN_ShowConnectionPoints[] = "ShowConnectionPoints";
	static const char _LN_task[] = "task";
	static const char _LN_cellWatches[] = "cellWatches";
	static const char _LN_cb[] = "cb";
	static const char _LN_DrawingSizeType[] = "DrawingSizeType";
	static const char _LN_alphabetical_index_mark_start[] = "alphabetical-index-mark-start";
	static const char _LN_start_indent[] = "start-indent";
	static const char _LN_snext[] = "snext";
	static const char _LN_DisableEdit[] = "DisableEdit";
	static const char _LN_AvailabilityPeriods[] = "AvailabilityPeriods";
	static const char _LN_source_service[] = "source-service";
	static const char _LN_miter[] = "miter";
	static const char _LN_expnd[] = "expnd";
	static const char _LN_cbpat[] = "cbpat";
	static const char _LN_dayLong[] = "dayLong";
	static const char _LN_pgnid[] = "pgnid";
	static const char _LN_fileNew[] = "fileNew";
	static const char _LN_decelerate[] = "decelerate";
	static const char _LN_softcol[] = "softcol";
	static const char _LN_sheetProtection[] = "sheetProtection";
	static const char _LN_prstDash[] = "prstDash";
	static const char _LN_expndtw[] = "expndtw";
	static const char _LN_drawingGridVerticalOrigin[] = "drawingGridVerticalOrigin";
	static const char _LN_Filled[] = "Filled";
	static const char _LN_end_indent[] = "end-indent";
	static const char _LN_cchs[] = "cchs";
	static const char _LN_DrawingScaleType[] = "DrawingScaleType";
	static const char _LN_pgnlcltr[] = "pgnlcltr";
	static const char _LN_TabSplitterPos[] = "TabSplitterPos";
	static const char _LN_monthLong[] = "monthLong";
	static const char _LN_initialXmlDocument[] = "initialXmlDocument";
	static const char _LN_source_name[] = "source-name";
	static const char _LN_autoReverse[] = "autoReverse";
	static const char _LN_softlheight[] = "softlheight";
	static const char _LN_AvailabilityPeriod[] = "AvailabilityPeriod";
	static const char _LN_expshrtn[] = "expshrtn";
	static const char _LN_custDash[] = "custDash";
	static const char _LN_InsertEntireRows[] = "InsertEntireRows";
	static const char _LN_alphabetical_index_mark_end[] = "alphabetical-index-mark-end";
	static const char _LN_pgnlcrm[] = "pgnlcrm";
	static const char _LN_space_before[] = "space-before";
	static const char _LN_protectedRanges[] = "protectedRanges";
	static const char _LN_yearLong[] = "yearLong";
	static const char _LN_cellx[] = "cellx";
	static const char _LN_EventItem[] = "EventItem";
	static const char _LN_object_name[] = "object-name";
	static const char _LN_InhibitSnap[] = "InhibitSnap";
	static const char _LN_par[] = "par";
	static const char _LN_softline[] = "softline";
	static const char _LN_headEnd[] = "headEnd";
	static const char _LN_customCategory[] = "customCategory";
	static const char _LN_doNotShadeFormData[] = "doNotShadeFormData";
	static const char _LN_pgnrestart[] = "pgnrestart";
	static const char _LN_cf[] = "cf";
	static const char _LN_AvailableUnits[] = "AvailableUnits";
	static const char _LN_faauto[] = "faauto";
	static const char _LN_include[] = "include";
	static const char _LN_annotationRef[] = "annotationRef";
	static const char _LN_NewAsync[] = "NewAsync";
	static const char _LN_space_after[] = "space-after";
	static const char _LN_seq[] = "seq";
	static const char _LN_HeaderMargin[] = "HeaderMargin";
	static const char _LN_user_name[] = "user-name";
	static const char _LN_tailEnd[] = "tailEnd";
	static const char _LN_UIVisibility[] = "UIVisibility";
	static const char _LN_softpage[] = "softpage";
	static const char _LN_cfpat[] = "cfpat";
	static const char _LN_snd[] = "snd";
	static const char _LN_webPubItems[] = "webPubItems";
	static const char _LN_alphabetical_index_mark[] = "alphabetical-index-mark";
	static const char _LN_facenter7_0[] = "facenter7.0";
	static const char _LN_iterate[] = "iterate";
	static const char _LN_pgnstart[] = "pgnstart";
	static const char _LN_package[] = "package";
	static const char _LN_NoAutofit[] = "NoAutofit";
	static const char _LN_punctuationKerning[] = "punctuationKerning";
	static const char _LN_stSnd[] = "stSnd";
	static const char _LN_Rates[] = "Rates";
	static const char _LN_column_sep[] = "column-sep";
	static const char _LN_footnoteRef[] = "footnoteRef";
	static const char _LN_cgrid[] = "cgrid";
	static const char _LN_password[] = "password";
	static const char _LN_spersonal[] = "spersonal";
	static const char _LN_FooterMargin[] = "FooterMargin";
	static const char _LN_ShdwType[] = "ShdwType";
	static const char _LN_iterate_type[] = "iterate-type";
	static const char _LN_files[] = "files";
	static const char _LN_pgnstarts[] = "pgnstarts";
	static const char _LN_facingp[] = "facingp";
	static const char _LN_endSnd[] = "endSnd";
	static const char _LN_Rate[] = "Rate";
	static const char _LN_charrsid[] = "charrsid";
	static const char _LN_NoTitles[] = "NoTitles";
	static const char _LN_st[] = "st";
	static const char _LN_chain[] = "chain";
	static const char _LN_data_pilot_field[] = "data-pilot-field";
	static const char _LN_endnoteRef[] = "endnoteRef";
	static const char _LN_splytwnine[] = "splytwnine";
	static const char _LN_key1[] = "key1";
	static const char _LN_ShdwObliqueAngle[] = "ShdwObliqueAngle";
	static const char _LN_HeaderLeft[] = "HeaderLeft";
	static const char _LN_characterSpacingControl[] = "characterSpacingControl";
	static const char _LN_RatesFrom[] = "RatesFrom";
	static const char _LN_blinds[] = "blinds";
	static const char _LN_pgnthaia[] = "pgnthaia";
	static const char _LN_fahang[] = "fahang";
	static const char _LN_iterate_interval[] = "iterate-interval";
	static const char _LN_charscalex[] = "charscalex";
	static const char _LN_OverwriteCells[] = "OverwriteCells";
	static const char _LN_fileProperties[] = "fileProperties";
	static const char _LN_dont_balance_text_columns[] = "dont-balance-text-columns";
	static const char _LN_key2[] = "key2";
	static const char _LN_separator[] = "separator";
	static const char _LN_sprsbsp[] = "sprsbsp";
	static const char _LN_colInfo[] = "colInfo";
	static const char _LN_checker[] = "checker";
	static const char _LN_falt[] = "falt";
	static const char _LN_HeaderCenter[] = "HeaderCenter";
	static const char _LN_audioCd[] = "audioCd";
	static const char _LN_RatesTo[] = "RatesTo";
	static const char _LN_source_field_name[] = "source-field-name";
	static const char _LN_pgnthaib[] = "pgnthaib";
	static const char _LN_ShdwScaleFactor[] = "ShdwScaleFactor";
	static const char _LN_string_value_phonetic[] = "string-value-phonetic";
	static const char _LN_chatn[] = "chatn";
	static const char _LN_Synchronous[] = "Synchronous";
	static const char _LN_audio[] = "audio";
	static const char _LN_sprslnsp[] = "sprslnsp";
	static const char _LN_faroman[] = "faroman";
	static const char _LN_printTwoOnOne[] = "printTwoOnOne";
	static const char _LN_footnote[] = "footnote";
	static const char _LN_audioFile[] = "audioFile";
	static const char _LN_RateTable[] = "RateTable";
	static const char _LN_ds[] = "ds";
	static const char _LN_pgnthaic[] = "pgnthaic";
	static const char _LN_permissions[] = "permissions";
	static const char _LN_chbgbdiag[] = "chbgbdiag";
	static const char _LN_HeaderRight[] = "HeaderRight";
	static const char _LN_v[] = "v";
	static const char _LN_dissolve[] = "dissolve";
	static const char _LN_audio_level[] = "audio-level";
	static const char _LN_table_properties[] = "table-properties";
	static const char _LN_selected_page[] = "selected-page";
	static const char _LN_favar[] = "favar";
	static const char _LN_noLine[] = "noLine";
	static const char _LN_LeftMargin[] = "LeftMargin";
	static const char _LN_sprsspbf[] = "sprsspbf";
	static const char _LN_endnote[] = "endnote";
	static const char _LN_NextId[] = "NextId";
	static const char _LN_rst[] = "rst";
	static const char _LN_videoFile[] = "videoFile";
	static const char _LN_Assignments[] = "Assignments";
	static const char _LN_strictFirstAndLastChars[] = "strictFirstAndLastChars";
	static const char _LN_pgnucltr[] = "pgnucltr";
	static const char _LN_key1_phonetic[] = "key1-phonetic";
	static const char _LN_chbgcross[] = "chbgcross";
	static const char _LN_allowedControl[] = "allowedControl";
	static const char _LN_comb[] = "comb";
	static const char _LN_FooterLeft[] = "FooterLeft";
	static const char _LN_fbias[] = "fbias";
	static const char _LN_tabColor[] = "tabColor";
	static const char _LN_solidLine[] = "solidLine";
	static const char _LN_ColumnInfo[] = "ColumnInfo";
	static const char _LN_continuationSeparator[] = "continuationSeparator";
	static const char _LN_sprstsm[] = "sprstsm";
	static const char _LN_RightMargin[] = "RightMargin";
	static const char _LN_is_data_layout_field[] = "is-data-layout-field";
	static const char _LN_pgnucrm[] = "pgnucrm";
	static const char _LN_cover[] = "cover";
	static const char _LN_Assignment[] = "Assignment";
	static const char _LN_fbidi[] = "fbidi";
	static const char _LN_chbgdcross[] = "chbgdcross";
	static const char _LN_key2_phonetic[] = "key2-phonetic";
	static const char _LN_may_break_between_rows[] = "may-break-between-rows";
	static const char _LN_FooterCenter[] = "FooterCenter";
	static const char _LN_wsView[] = "wsView";
	static const char _LN_externalViews[] = "externalViews";
	static const char _LN_gradLine[] = "gradLine";
	static const char _LN_sprstsp[] = "sprstsp";
	static const char _LN_DoNotPersist[] = "DoNotPersist";
	static const char _LN_cut[] = "cut";
	static const char _LN_noLineBreaksAfter[] = "noLineBreaksAfter";
	static const char _LN_TopMargin[] = "TopMargin";
	static const char _LN_pgnvieta[] = "pgnvieta";
	static const char _LN_fchars[] = "fchars";
	static const char _LN_TaskUID[] = "TaskUID";
	static const char _LN_pane[] = "pane";
	static const char _LN_chbgdkbdiag[] = "chbgdkbdiag";
	static const char _LN_style[] = "style";
	static const char _LN_ParameterValue[] = "ParameterValue";
	static const char _LN_used_hierarchy[] = "used-hierarchy";
	static const char _LN_spv[] = "spv";
	static const char _LN_pattLine[] = "pattLine";
	static const char _LN_diamond[] = "diamond";
	static const char _LN_main_entry[] = "main-entry";
	static const char _LN_FooterRight[] = "FooterRight";
	static const char _LN_externalView[] = "externalView";
	static const char _LN_pgNum[] = "pgNum";
	static const char _LN_pgnx[] = "pgnx";
	static const char _LN_fcharset[] = "fcharset";
	static const char _LN_DoNotPersistSort[] = "DoNotPersistSort";
	static const char _LN_BottomMargin[] = "BottomMargin";
	static const char _LN_selection[] = "selection";
	static const char _LN_border_model[] = "border-model";
	static const char _LN_ResourceUID[] = "ResourceUID";
	static const char _LN_sreply[] = "sreply";
	static const char _LN_noLineBreaksBefore[] = "noLineBreaksBefore";
	static const char _LN_fade[] = "fade";
	static const char _LN_chbgdkcross[] = "chbgdkcross";
	static const char _LN_cr[] = "cr";
	static const char _LN_pgny[] = "pgny";
	static const char _LN_bibliography_mark[] = "bibliography-mark";
	static const char _LN_display_name[] = "display-name";
	static const char _LN_PromptString[] = "PromptString";
	static const char _LN_HeaderFooterFont[] = "HeaderFooterFont";
	static const char _LN_data_pilot_level[] = "data-pilot-level";
	static const char _LN_fdecor[] = "fdecor";
	static const char _LN_attributeData[] = "attributeData";
	static const char _LN_ssemihidden[] = "ssemihidden";
	static const char _LN_VerticalAlign[] = "VerticalAlign";
	static const char _LN_newsflash[] = "newsflash";
	static const char _LN_Confirmed[] = "Confirmed";
	static const char _LN_table_column_properties[] = "table-column-properties";
	static const char _LN_DoNotPersstAF[] = "DoNotPersstAF";
	static const char _LN_pgnzodiac[] = "pgnzodiac";
	static const char _LN_dark1[] = "dark1";
	static const char _LN_afdd[] = "afdd";
	static const char _LN_chbgdkdcross[] = "chbgdkdcross";
	static const char _LN_pict[] = "pict";
	static const char _LN_fet[] = "fet";
	static const char _LN_parent_style_name[] = "parent-style-name";
	static const char _LN_webPageEncoding[] = "webPageEncoding";
	static const char _LN_NonDefaultName[] = "NonDefaultName";
	static const char _LN_bibliography_type[] = "bibliography-type";
	static const char _LN_staticval[] = "staticval";
	static const char _LN_show_empty[] = "show-empty";
	static const char _LN_plus[] = "plus";
	static const char _LN_light1[] = "light1";
	static const char _LN_CostRateTable[] = "CostRateTable";
	static const char _LN_TextBkgnd[] = "TextBkgnd";
	static const char _LN_pgnzodiacd[] = "pgnzodiacd";
	static const char _LN_fetch[] = "fetch";
	static const char _LN_fldChar[] = "fldChar";
	static const char _LN_b_cs[] = "b-cs";
	static const char _LN_Sort[] = "Sort";
	static const char _LN_DocumentSettings[] = "DocumentSettings";
	static const char _LN_chooseFragment[] = "chooseFragment";
	static const char _LN_DateGroupItem[] = "DateGroupItem";
	static const char _LN_chbgdkfdiag[] = "chbgdkfdiag";
	static const char _LN_pull[] = "pull";
	static const char _LN_dark2[] = "dark2";
	static const char _LN_column_width[] = "column-width";
	static const char _LN_stextflow[] = "stextflow";
	static const char _LN_ffdefres[] = "ffdefres";
	static const char _LN_optimizeForBrowser[] = "optimizeForBrowser";
	static const char _LN_SQLType[] = "SQLType";
	static const char _LN_data_pilot_subtotals[] = "data-pilot-subtotals";
	static const char _LN_CaseSensitive[] = "CaseSensitive";
	static const char _LN_next_style_name[] = "next-style-name";
	static const char _LN_DefaultTabStop[] = "DefaultTabStop";
	static const char _LN_pgnzodiacl[] = "pgnzodiacl";
	static const char _LN_Delay[] = "Delay";
	static const char _LN_index_body[] = "index-body";
	static const char _LN_i_cs[] = "i-cs";
	static const char _LN_push[] = "push";
	static const char _LN_footnoteReference[] = "footnoteReference";
	static const char _LN_light2[] = "light2";
	static const char _LN_chbgdkhoriz[] = "chbgdkhoriz";
	static const char _LN_criteria[] = "criteria";
	static const char _LN_editWith[] = "editWith";
	static const char _LN_Colors[] = "Colors";
	static const char _LN_ffdeftext[] = "ffdeftext";
	static const char _LN_RefreshOnChange[] = "RefreshOnChange";
	static const char _LN_rel_column_width[] = "rel-column-width";
	static const char _LN_HasFixedRateUnits[] = "HasFixedRateUnits";
	static const char _LN_random[] = "random";
	static const char _LN_pgp[] = "pgp";
	static const char _LN_FuturePersist[] = "FuturePersist";
	static const char _LN_striked1[] = "striked1";
	static const char _LN_index_title[] = "index-title";
	static const char _LN_list_style_name[] = "list-style-name";
	static const char _LN_accent1[] = "accent1";
	static const char _LN_TextDirection[] = "TextDirection";
	static const char _LN_relyOnVML[] = "relyOnVML";
	static const char _LN_PrintSetup[] = "PrintSetup";
	static const char _LN_data_pilot_subtotal[] = "data-pilot-subtotal";
	static const char _LN_criteriaColor[] = "criteriaColor";
	static const char _LN_unboundControls[] = "unboundControls";
	static const char _LN_chbgdkvert[] = "chbgdkvert";
	static const char _LN_ffentrymcr[] = "ffentrymcr";
	static const char _LN_pgptbl[] = "pgptbl";
	static const char _LN_randomBar[] = "randomBar";
	static const char _LN_endnoteReference[] = "endnoteReference";
	static const char _LN_DisplayEmptyMembers[] = "DisplayEmptyMembers";
	static const char _LN_accent2[] = "accent2";
	static const char _LN_stshfbi[] = "stshfbi";
	static const char _LN_use_optimal_column_width[] = "use-optimal-column-width";
	static const char _LN_allowPNG[] = "allowPNG";
	static const char _LN_HeaderRow[] = "HeaderRow";
	static const char _LN_FixedMaterial[] = "FixedMaterial";
	static const char _LN_table_of_content[] = "table-of-content";
	static const char _LN_TextBkgndTrans[] = "TextBkgndTrans";
	static const char _LN_Fonts[] = "Fonts";
	static const char _LN_pgwsxn[] = "pgwsxn";
	static const char _LN_chbgfdiag[] = "chbgfdiag";
	static const char _LN_criteriaCellIcon[] = "criteriaCellIcon";
	static const char _LN_ffexitmcr[] = "ffexitmcr";
	static const char _LN_split[] = "split";
	static const char _LN_editing[] = "editing";
	static const char _LN_data_pilot_members[] = "data-pilot-members";
	static const char _LN_auto_update[] = "auto-update";
	static const char _LN_accent3[] = "accent3";
	static const char _LN_stshfdbch[] = "stshfdbch";
	static const char _LN_oArt[] = "oArt";
	static const char _LN_doNotRelyOnCSS[] = "doNotRelyOnCSS";
	static const char _LN_FaceNames[] = "FaceNames";
	static const char _LN_IdWrapped[] = "IdWrapped";
	static const char _LN_phcol[] = "phcol";
	static const char _LN_LinkedFields[] = "LinkedFields";
	static const char _LN_strips[] = "strips";
	static const char _LN_chbghoriz[] = "chbghoriz";
	static const char _LN_masterDetail[] = "masterDetail";
	static const char _LN_table_of_content_source[] = "table-of-content-source";
	static const char _LN_TxtPinX[] = "TxtPinX";
	static const char _LN_ffformat[] = "ffformat";
	static const char _LN_accent4[] = "accent4";
	static const char _LN_ptab[] = "ptab";
	static const char _LN_stshfhich[] = "stshfhich";
	static const char _LN_table_row_properties[] = "table-row-properties";
	static const char _LN_dataRefs[] = "dataRefs";
	static const char _LN_phmrg[] = "phmrg";
	static const char _LN_StyleSheets[] = "StyleSheets";
	static const char _LN_AlertVersion[] = "AlertVersion";
	static const char _LN_wedge[] = "wedge";
	static const char _LN_NotVisible[] = "NotVisible";
	static const char _LN_data_pilot_member[] = "data-pilot-member";
	static const char _LN_doNotSaveWebPagesAsSingleFile[] = "doNotSaveWebPagesAsSingleFile";
	static const char _LN_default_outline_level[] = "default-outline-level";
	static const char _LN_TxtPinY[] = "TxtPinY";
	static const char _LN_rStyle[] = "rStyle";
	static const char _LN_chbgvert[] = "chbgvert";
	static const char _LN_ffhaslistbox[] = "ffhaslistbox";
	static const char _LN_Overallocated[] = "Overallocated";
	static const char _LN_accent5[] = "accent5";
	static const char _LN_fragmentToInsert[] = "fragmentToInsert";
	static const char _LN_phpg[] = "phpg";
	static const char _LN_dataRef[] = "dataRef";
	static const char _LN_stshfloch[] = "stshfloch";
	static const char _LN_wheel[] = "wheel";
	static const char _LN_DocumentSheet[] = "DocumentSheet";
	static const char _LN_rFonts[] = "rFonts";
	static const char _LN_SortOrder[] = "SortOrder";
	static const char _LN_use_outline_level[] = "use-outline-level";
	static const char _LN_TxtWidth[] = "TxtWidth";
	static const char _LN_Orientation[] = "Orientation";
	static const char _LN_chbrdr[] = "chbrdr";
	static const char _LN_row_height[] = "row-height";
	static const char _LN_accent6[] = "accent6";
	static const char _LN_picbmp[] = "picbmp";
	static const char _LN_ffhelptext[] = "ffhelptext";
	static const char _LN_mergeCell[] = "mergeCell";
	static const char _LN_ResponsePending[] = "ResponsePending";
	static const char _LN_show_details[] = "show-details";
	static const char _LN_wipe[] = "wipe";
	static const char _LN_stylesheet[] = "stylesheet";
	static const char _LN_map[] = "map";
	static const char _LN_mainpane[] = "mainpane";
	static const char _LN_chcbpat[] = "chcbpat";
	static const char _LN_TxtHeight[] = "TxtHeight";
	static const char _LN_ExtraLeftColumns[] = "ExtraLeftColumns";
	static const char _LN_picbpp[] = "picbpp";
	static const char _LN_hlink[] = "hlink";
	static const char _LN_Masters[] = "Masters";
	static const char _LN_doNotOrganizeInFolder[] = "doNotOrganizeInFolder";
	static const char _LN_PivotCache[] = "PivotCache";
	static const char _LN_zoom[] = "zoom";
	static const char _LN_min_row_height[] = "min-row-height";
	static const char _LN_bCs[] = "bCs";
	static const char _LN_cellSmartTags[] = "cellSmartTags";
	static const char _LN_ffhps[] = "ffhps";
	static const char _LN_use_index_marks[] = "use-index-marks";
	static const char _LN_styrsid[] = "styrsid";
	static const char _LN_data_pilot_display_info[] = "data-pilot-display-info";
	static const char _LN_font[] = "font";
	static const char _LN_printSettings[] = "printSettings";
	static const char _LN_Units[] = "Units";
	static const char _LN_chcfpat[] = "chcfpat";
	static const char _LN_apply_style_name[] = "apply-style-name";
	static const char _LN_folHlink[] = "folHlink";
	static const char _LN_piccropb[] = "piccropb";
	static const char _LN_sndAc[] = "sndAc";
	static const char _LN_TxtLocPinX[] = "TxtLocPinX";
	static const char _LN_Pages[] = "Pages";
	static const char _LN_ffl[] = "ffl";
	static const char _LN_CacheIndex[] = "CacheIndex";
	static const char _LN_iCs[] = "iCs";
	static const char _LN_ExtraRightColumns[] = "ExtraRightColumns";
	static const char _LN_use_optimal_row_height[] = "use-optimal-row-height";
	static const char _LN_sz_cs[] = "sz-cs";
	static const char _LN_UpdateNeeded[] = "UpdateNeeded";
	static const char _LN_cellSmartTag[] = "cellSmartTag";
	static const char _LN_sldRef[] = "sldRef";
	static const char _LN_chdate[] = "chdate";
	static const char _LN_Windows[] = "Windows";
	static const char _LN_use_index_source_styles[] = "use-index-source-styles";
	static const char _LN_doNotUseLongFileNames[] = "doNotUseLongFileNames";
	static const char _LN_piccropl[] = "piccropl";
	static const char _LN_ffmaxlen[] = "ffmaxlen";
	static const char _LN_subdocument[] = "subdocument";
	static const char _LN_TxtLocPinY[] = "TxtLocPinY";
	static const char _LN_caps[] = "caps";
	static const char _LN_custClr[] = "custClr";
	static const char _LN_PTPivotData[] = "PTPivotData";
	static const char _LN_enabled[] = "enabled";
	static const char _LN_ssJumpAc[] = "ssJumpAc";
	static const char _LN_chdpa[] = "chdpa";
	static const char _LN_EventList[] = "EventList";
	static const char _LN_default_style[] = "default-style";
	static const char _LN_VAC[] = "VAC";
	static const char _LN_smallCaps[] = "smallCaps";
	static const char _LN_piccropr[] = "piccropr";
	static const char _LN_cellSmartTagPr[] = "cellSmartTagPr";
	static const char _LN_ffname[] = "ffname";
	static const char _LN_DeletedTitle[] = "DeletedTitle";
	static const char _LN_toolbar[] = "toolbar";
	static const char _LN_subfontbysize[] = "subfontbysize";
	static const char _LN_TxtAngle[] = "TxtAngle";
	static const char _LN_latin[] = "latin";
	static const char _LN_table_cell_properties[] = "table-cell-properties";
	static const char _LN_data_field[] = "data-field";
	static const char _LN_chdpl[] = "chdpl";
	static const char _LN_InvertedRowMember[] = "InvertedRowMember";
	static const char _LN_WorkContour[] = "WorkContour";
	static const char _LN_sldJumpAc[] = "sldJumpAc";
	static const char _LN_pixelsPerInch[] = "pixelsPerInch";
	static const char _LN_index_scope[] = "index-scope";
	static const char _LN_ffownhelp[] = "ffownhelp";
	static const char _LN_HeaderFooter[] = "HeaderFooter";
	static const char _LN_piccropt[] = "piccropt";
	static const char _LN_menu[] = "menu";
	static const char _LN_strike[] = "strike";
	static const char _LN_page_layout[] = "page-layout";
	static const char _LN_highAnsi[] = "highAnsi";
	static const char _LN_NoToggleDataHeader[] = "NoToggleDataHeader";
	static const char _LN_AlignLeft[] = "AlignLeft";
	static const char _LN_userShView[] = "userShView";
	static const char _LN_chftn[] = "chftn";
	static const char _LN_member_count[] = "member-count";
	static const char _LN_menuArea[] = "menuArea";
	static const char _LN_urlAc[] = "urlAc";
	static const char _LN_dstrike[] = "dstrike";
	static const char _LN_pich[] = "pich";
	static const char _LN_ffownstat[] = "ffownstat";
	static const char _LN_relative_tab_stop_position[] = "relative-tab-stop-position";
	static const char _LN_txbPr[] = "txbPr";
	static const char _LN_targetScreenSz[] = "targetScreenSz";
	static const char _LN_super[] = "super";
	static const char _LN_InvertedColumnMember[] = "InvertedColumnMember";
	static const char _LN_VBProjectData[] = "VBProjectData";
	static const char _LN_complexScript[] = "complexScript";
	static const char _LN_chftnsep[] = "chftnsep";
	static const char _LN_text_align_source[] = "text-align-source";
	static const char _LN_page_usage[] = "page-usage";
	static const char _LN_AlignCenter[] = "AlignCenter";
	static const char _LN_dVal[] = "dVal";
	static const char _LN_fileAc[] = "fileAc";
	static const char _LN_pichgoal[] = "pichgoal";
	static const char _LN_taskpane[] = "taskpane";
	static const char _LN_txLstStyle[] = "txLstStyle";
	static const char _LN_swpbdr[] = "swpbdr";
	static const char _LN_AlternateMethod[] = "AlternateMethod";
	static const char _LN_display_member_mode[] = "display-member-mode";
	static const char _LN_ffprot[] = "ffprot";
	static const char _LN_chftnsepc[] = "chftnsepc";
	static const char _LN_formula1[] = "formula1";
	static const char _LN_savePreviewPicture[] = "savePreviewPicture";
	static const char _LN_EmailRoutingData[] = "EmailRoutingData";
	static const char _LN_presAc[] = "presAc";
	static const char _LN_header_style[] = "header-style";
	static const char _LN_eastAsian[] = "eastAsian";
	static const char _LN_AlignRight[] = "AlignRight";
	static const char _LN_picprop[] = "picprop";
	static const char _LN_views[] = "views";
	static const char _LN_LeafRowMember[] = "LeafRowMember";
	static const char _LN_ffrecalc[] = "ffrecalc";
	static const char _LN_spDef[] = "spDef";
	static const char _LN_table_of_content_entry_template[] = "table-of-content-entry-template";
	static const char _LN_tabsnoovrlp[] = "tabsnoovrlp";
	static const char _LN_glyph_orientation_vertical[] = "glyph-orientation-vertical";
	static const char _LN_formula2[] = "formula2";
	static const char _LN_imprint[] = "imprint";
	static const char _LN_chpgn[] = "chpgn";
	static const char _LN_NoAutoPage[] = "NoAutoPage";
	static const char _LN_tooltip[] = "tooltip";
	static const char _LN_data_pilot_sort_info[] = "data-pilot-sort-info";
	static const char _LN_majorFont[] = "majorFont";
	static const char _LN_AlignTop[] = "AlignTop";
	static const char _LN_picscaled[] = "picscaled";
	static const char _LN_footer_style[] = "footer-style";
	static const char _LN_lnDef[] = "lnDef";
	static const char _LN_ffres[] = "ffres";
	static const char _LN_validateAgainstSchema[] = "validateAgainstSchema";
	static const char _LN_xmlToEdit[] = "xmlToEdit";
	static const char _LN_chshdng[] = "chshdng";
	static const char _LN_LeafColumnMember[] = "LeafColumnMember";
	static const char _LN_noProof[] = "noProof";
	static const char _LN_OWCVersion[] = "OWCVersion";
	static const char _LN_cfRule[] = "cfRule";
	static const char _LN_custShow[] = "custShow";
	static const char _LN_taprtl[] = "taprtl";
	static const char _LN_SheetName[] = "SheetName";
	static const char _LN_txbxDef[] = "txbxDef";
	static const char _LN_minorFont[] = "minorFont";
	static const char _LN_AlignMiddle[] = "AlignMiddle";
	static const char _LN_picscalex[] = "picscalex";
	static const char _LN_ffsize[] = "ffsize";
	static const char _LN_master_page[] = "master-page";
	static const char _LN_chtime[] = "chtime";
	static const char _LN_documentSignatures[] = "documentSignatures";
	static const char _LN_vanish[] = "vanish";
	static const char _LN_sort_mode[] = "sort-mode";
	static const char _LN_tb[] = "tb";
	static const char _LN_programAc[] = "programAc";
	static const char _LN_diagonal_tl_br[] = "diagonal-tl-br";
	static const char _LN_worksheetoptions[] = "worksheetoptions";
	static const char _LN_illustration_index[] = "illustration-index";
	static const char _LN_ExternName[] = "ExternName";
	static const char _LN_colorScale[] = "colorScale";
	static const char _LN_ffstattext[] = "ffstattext";
	static const char _LN_Item[] = "Item";
	static const char _LN_fillStyle[] = "fillStyle";
	static const char _LN_saveInvalidXML[] = "saveInvalidXML";
	static const char _LN_picscaley[] = "picscaley";
	static const char _LN_AlignBottom[] = "AlignBottom";
	static const char _LN_tbllkbestfit[] = "tbllkbestfit";
	static const char _LN_clbgbdiag[] = "clbgbdiag";
	static const char _LN_webHidden[] = "webHidden";
	static const char _LN_data_pilot_layout_info[] = "data-pilot-layout-info";
	static const char _LN_macroAc[] = "macroAc";
	static const char _LN_SourceName[] = "SourceName";
	static const char _LN_Xct[] = "Xct";
	static const char _LN_diagonal_tl_br_widths[] = "diagonal-tl-br-widths";
	static const char _LN_signedDataBlock[] = "signedDataBlock";
	static const char _LN_databar[] = "databar";
	static const char _LN_fftype[] = "fftype";
	static const char _LN_displaycustomheaders[] = "displaycustomheaders";
	static const char _LN_clbgcross[] = "clbgcross";
	static const char _LN_illustration_index_source[] = "illustration-index-source";
	static const char _LN_LockWidth[] = "LockWidth";
	static const char _LN_header_left[] = "header-left";
	static const char _LN_ignoreMixedContent[] = "ignoreMixedContent";
	static const char _LN_tbllkborder[] = "tbllkborder";
	static const char _LN_lineStyle[] = "lineStyle";
	static const char _LN_FormatSettings[] = "FormatSettings";
	static const char _LN_hlinkAc[] = "hlinkAc";
	static const char _LN_picw[] = "picw";
	static const char _LN_BoundField[] = "BoundField";
	static const char _LN_w[] = "w";
	static const char _LN_fftypetxt[] = "fftypetxt";
	static const char _LN_iconset[] = "iconset";
	static const char _LN_clbgdcross[] = "clbgdcross";
	static const char _LN_LockHeight[] = "LockHeight";
	static const char _LN_picwgoal[] = "picwgoal";
	static const char _LN_message[] = "message";
	static const char _LN_layout_mode[] = "layout-mode";
	static const char _LN_kern[] = "kern";
	static const char _LN_oleVerbAc[] = "oleVerbAc";
	static const char _LN_footer_left[] = "footer-left";
	static const char _LN_scene3d[] = "scene3d";
	static const char _LN_tbllkcolor[] = "tbllkcolor";
	static const char _LN_diagonal_bl_tr[] = "diagonal-bl-tr";
	static const char _LN_ComponentOptions[] = "ComponentOptions";
	static const char _LN_Delimiters[] = "Delimiters";
	static const char _LN_alwaysShowPlaceholderText[] = "alwaysShowPlaceholderText";
	static const char _LN_fi[] = "fi";
	static const char _LN_use_caption[] = "use-caption";
	static const char _LN_documentVersionUpgrade[] = "documentVersionUpgrade";
	static const char _LN_masterClrMapping[] = "masterClrMapping";
	static const char _LN_cfvo[] = "cfvo";
	static const char _LN_plain[] = "plain";
	static const char _LN_FilterCaption[] = "FilterCaption";
	static const char _LN_clbgdkbdiag[] = "clbgdkbdiag";
	static const char _LN_sp3d[] = "sp3d";
	static const char _LN_LockMoveX[] = "LockMoveX";
	static const char _LN_xDocumentClass[] = "xDocumentClass";
	static const char _LN_add_empty_lines[] = "add-empty-lines";
	static const char _LN_mediaAc[] = "mediaAc";
	static const char _LN_fid[] = "fid";
	static const char _LN_sz[] = "sz";
	static const char _LN_tbllkfont[] = "tbllkfont";
	static const char _LN_Comma[] = "Comma";
	static const char _LN_diagonal_bl_tr_widths[] = "diagonal-bl-tr-widths";
	static const char _LN_region_left[] = "region-left";
	static const char _LN_pmmetafile[] = "pmmetafile";
	static const char _LN_effectStyle[] = "effectStyle";
	static const char _LN_Toolbar[] = "Toolbar";
	static const char _LN_szCs[] = "szCs";
	static const char _LN_caption_sequence_name[] = "caption-sequence-name";
	static const char _LN_custShowAc[] = "custShowAc";
	static const char _LN_LockMoveY[] = "LockMoveY";
	static const char _LN_oddHeader[] = "oddHeader";
	static const char _LN_clbgdkcross[] = "clbgdkcross";
	static const char _LN_PLDataOrientation[] = "PLDataOrientation";
	static const char _LN_tbllkhdrcols[] = "tbllkhdrcols";
	static const char _LN_overrideClrMapping[] = "overrideClrMapping";
	static const char _LN_Consecutive[] = "Consecutive";
	static const char _LN_doNotUnderlineInvalidXML[] = "doNotUnderlineInvalidXML";
	static const char _LN_data_pilot_field_reference[] = "data-pilot-field-reference";
	static const char _LN_useTransform[] = "useTransform";
	static const char _LN_HideOfficeLogo[] = "HideOfficeLogo";
	static const char _LN_filetbl[] = "filetbl";
	static const char _LN_pn[] = "pn";
	static const char _LN_region_center[] = "region-center";
	static const char _LN_schemaErrorMessages[] = "schemaErrorMessages";
	static const char _LN_bgFillStyle[] = "bgFillStyle";
	static const char _LN_LockAspect[] = "LockAspect";
	static const char _LN_onClick[] = "onClick";
	static const char _LN_oddFooter[] = "oddFooter";
	static const char _LN_clbgdkdcross[] = "clbgdkdcross";
	static const char _LN_wrap_option[] = "wrap-option";
	static const char _LN_pnacross[] = "pnacross";
	static const char _LN_tbllkhdrrows[] = "tbllkhdrrows";
	static const char _LN_Custom[] = "Custom";
	static const char _LN_fittext[] = "fittext";
	static const char _LN_PLPosition[] = "PLPosition";
	static const char _LN_caption_sequence_format[] = "caption-sequence-format";
	static const char _LN_extensions[] = "extensions";
	static const char _LN_baseStyles[] = "baseStyles";
	static const char _LN_bdr[] = "bdr";
	static const char _LN_SpreadsheetAutoFit[] = "SpreadsheetAutoFit";
	static const char _LN_onMouseOver[] = "onMouseOver";
	static const char _LN_fillStyleLst[] = "fillStyleLst";
	static const char _LN_region_right[] = "region-right";
	static const char _LN_LockDelete[] = "LockDelete";
	static const char _LN_evenHeader[] = "evenHeader";
	static const char _LN_removeWordSchemaOnSave[] = "removeWordSchemaOnSave";
	static const char _LN_SemiColon[] = "SemiColon";
	static const char _LN_pnaiu[] = "pnaiu";
	static const char _LN_rotation_angle[] = "rotation-angle";
	static const char _LN_clbgdkfdiag[] = "clbgdkfdiag";
	static const char _LN_override[] = "override";
	static const char _LN_field_name[] = "field-name";
	static const char _LN_fldalt[] = "fldalt";
	static const char _LN_fitText[] = "fitText";
	static const char _LN_tbllklastcol[] = "tbllklastcol";
	static const char _LN_objectDefaults[] = "objectDefaults";
	static const char _LN_pnaiud[] = "pnaiud";
	static const char _LN_ShowAllItems[] = "ShowAllItems";
	static const char _LN_lineStyleLst[] = "lineStyleLst";
	static const char _LN_LockBegin[] = "LockBegin";
	static const char _LN_evenFooter[] = "evenFooter";
	static const char _LN_notes[] = "notes";
	static const char _LN_Space[] = "Space";
	static const char _LN_ruleSetAction[] = "ruleSetAction";
	static const char _LN_flddirty[] = "flddirty";
	static const char _LN_applicationParameters[] = "applicationParameters";
	static const char _LN_illustration_index_entry_template[] = "illustration-index-entry-template";
	static const char _LN_vertAlign[] = "vertAlign";
	static const char _LN_member_type[] = "member-type";
	static const char _LN_clbgdkhor[] = "clbgdkhor";
	static const char _LN_percent[] = "percent";
	static const char _LN_rotation_align[] = "rotation-align";
	static const char _LN_tbllklastrow[] = "tbllklastrow";
	static const char _LN_Caption[] = "Caption";
	static const char _LN_pnaiueo[] = "pnaiueo";
	static const char _LN_table_template[] = "table-template";
	static const char _LN_useXSLTWhenSaving[] = "useXSLTWhenSaving";
	static const char _LN_masterClrScheme[] = "masterClrScheme";
	static const char _LN_LockEnd[] = "LockEnd";
	static const char _LN_firstHeader[] = "firstHeader";
	static const char _LN_fldedit[] = "fldedit";
	static const char _LN_NumberFormat[] = "NumberFormat";
	static const char _LN_effectStyleLst[] = "effectStyleLst";
	static const char _LN_chRg[] = "chRg";
	static const char _LN_rtl[] = "rtl";
	static const char _LN_clbgdkvert[] = "clbgdkvert";
	static const char _LN_TextQualifier[] = "TextQualifier";
	static const char _LN_member_name[] = "member-name";
	static const char _LN_MaxHeight[] = "MaxHeight";
	static const char _LN_pnaiueod[] = "pnaiueod";
	static const char _LN_cell_protect[] = "cell-protect";
	static const char _LN_tbllkshading[] = "tbllkshading";
	static const char _LN_exitRuleSet[] = "exitRuleSet";
	static const char _LN_solutionProperties[] = "solutionProperties";
	static const char _LN_LockRotate[] = "LockRotate";
	static const char _LN_pRg[] = "pRg";
	static const char _LN_fldinst[] = "fldinst";
	static const char _LN_first_row_start_column[] = "first-row-start-column";
	static const char _LN_firstFooter[] = "firstFooter";
	static const char _LN_saveThroughXSLT[] = "saveThroughXSLT";
	static const char _LN_CurrentPage[] = "CurrentPage";
	static const char _LN_overrideClrScheme[] = "overrideClrScheme";
	static const char _LN_em[] = "em";
	static const char _LN_clbgfdiag[] = "clbgfdiag";
	static const char _LN_bgFillStyleLst[] = "bgFillStyleLst";
	static const char _LN_StartRow[] = "StartRow";
	static const char _LN_table_index[] = "table-index";
	static const char _LN_pnb[] = "pnb";
	static const char _LN_MaxWidth[] = "MaxWidth";
	static const char _LN_bg[] = "bg";
	static const char _LN_fldlock[] = "fldlock";
	static const char _LN_dialogBoxMessageAction[] = "dialogBoxMessageAction";
	static const char _LN_print_content[] = "print-content";
	static const char _LN_LockCrop[] = "LockCrop";
	static const char _LN_tblrsid[] = "tblrsid";
	static const char _LN_data_pilot_groups[] = "data-pilot-groups";
	static const char _LN_hyphen[] = "hyphen";
	static const char _LN_subSp[] = "subSp";
	static const char _LN_sortCondition[] = "sortCondition";
	static const char _LN_pnbidia[] = "pnbidia";
	static const char _LN_featureRestrictions[] = "featureRestrictions";
	static const char _LN_clbghoriz[] = "clbghoriz";
	static const char _LN_NewItemsHidden[] = "NewItemsHidden";
	static const char _LN_Decimal[] = "Decimal";
	static const char _LN_NextSheetNumber[] = "NextSheetNumber";
	static const char _LN_showXMLTags[] = "showXMLTags";
	static const char _LN_table_index_source[] = "table-index-source";
	static const char _LN_clrScheme[] = "clrScheme";
	static const char _LN_masterFontScheme[] = "masterFontScheme";
	static const char _LN_fldpriv[] = "fldpriv";
	static const char _LN_first_row_end_column[] = "first-row-end-column";
	static const char _LN_lang[] = "lang";
	static const char _LN_LockVtxEdit[] = "LockVtxEdit";
	static const char _LN_oleChartEl[] = "oleChartEl";
	static const char _LN_pnbidib[] = "pnbidib";
	static const char _LN_tcelld[] = "tcelld";
	static const char _LN_clbgvert[] = "clbgvert";
	static const char _LN_ThousandSeparator[] = "ThousandSeparator";
	static const char _LN_date_start[] = "date-start";
	static const char _LN_sct[] = "sct";
	static const char _LN_asianLayout[] = "asianLayout";
	static const char _LN_repeat_content[] = "repeat-content";
	static const char _LN_dialogBoxExpressionAction[] = "dialogBoxExpressionAction";
	static const char _LN_fldrslt[] = "fldrslt";
	static const char _LN_Subtotal[] = "Subtotal";
	static const char _LN_custClrLst[] = "custClrLst";
	static const char _LN_alwaysMergeEmptyNamespace[] = "alwaysMergeEmptyNamespace";
	static const char _LN_save[] = "save";
	static const char _LN_LockTextEdit[] = "LockTextEdit";
	static const char _LN_pncaps[] = "pncaps";
	static const char _LN_tx[] = "tx";
	static const char _LN_tcf[] = "tcf";
	static const char _LN_overrideFontScheme[] = "overrideFontScheme";
	static const char _LN_rangeProtection[] = "rangeProtection";
	static const char _LN_table_index_entry_template[] = "table-index-entry-template";
	static const char _LN_clbrdrb[] = "clbrdrb";
	static const char _LN_last_row_start_column[] = "last-row-start-column";
	static const char _LN_fldtype[] = "fldtype";
	static const char _LN_PLSubtotal[] = "PLSubtotal";
	static const char _LN_exportToWeb[] = "exportToWeb";
	static const char _LN_specVanish[] = "specVanish";
	static const char _LN_e2o[] = "e2o";
	static const char _LN_fontScheme[] = "fontScheme";
	static const char _LN_tcl[] = "tcl";
	static const char _LN_FitToPage[] = "FitToPage";
	static const char _LN_shrink_to_fit[] = "shrink-to-fit";
	static const char _LN_pncard[] = "pncard";
	static const char _LN_date_end[] = "date-end";
	static const char _LN_LockFormat[] = "LockFormat";
	static const char _LN_clbrdrl[] = "clbrdrl";
	static const char _LN_sldTgt[] = "sldTgt";
	static const char _LN_tcn[] = "tcn";
	static const char _LN_fmodern[] = "fmodern";
	static const char _LN_slc[] = "slc";
	static const char _LN_switchViewAction[] = "switchViewAction";
	static const char _LN_doNotEmbedLinguisticData[] = "doNotEmbedLinguisticData";
	static const char _LN_masterStyleMatrix[] = "masterStyleMatrix";
	static const char _LN_DetailFormat[] = "DetailFormat";
	static const char _LN_pncf[] = "pncf";
	static const char _LN_exportToExcel[] = "exportToExcel";
	static const char _LN_effectScheme[] = "effectScheme";
	static const char _LN_DoNotDisplayColHeaders[] = "DoNotDisplayColHeaders";
	static const char _LN_end[] = "end";
	static const char _LN_last_row_end_column[] = "last-row-end-column";
	static const char _LN_list_level_properties[] = "list-level-properties";
	static const char _LN_tdfrmtxtBottom[] = "tdfrmtxtBottom";
	static const char _LN_clbrdrr[] = "clbrdrr";
	static const char _LN_object_index[] = "object-index";
	static const char _LN_cellWatch[] = "cellWatch";
	static const char _LN_lists[] = "lists";
	static const char _LN_sndTgt[] = "sndTgt";
	static const char _LN_LockGroup[] = "LockGroup";
	static const char _LN_fn[] = "fn";
	static const char _LN_pnchosung[] = "pnchosung";
	static const char _LN_step[] = "step";
	static const char _LN_SubtotalFormat[] = "SubtotalFormat";
	static const char _LN_assignmentAction[] = "assignmentAction";
	static const char _LN_fname[] = "fname";
	static const char _LN_clbrdrt[] = "clbrdrt";
	static const char _LN_spTgt[] = "spTgt";
	static const char _LN_overrideStyleMatrix[] = "overrideStyleMatrix";
	static const char _LN_LockCalcWH[] = "LockCalcWH";
	static const char _LN_grouped_by[] = "grouped-by";
	static const char _LN_tdfrmtxtLeft[] = "tdfrmtxtLeft";
	static const char _LN_sendMail[] = "sendMail";
	static const char _LN_hdrShapeDefaults[] = "hdrShapeDefaults";
	static const char _LN_pncnum[] = "pncnum";
	static const char _LN_DoNotDisplayRowHeaders[] = "DoNotDisplayRowHeaders";
	static const char _LN_first_row[] = "first-row";
	static const char _LN_fnetwork[] = "fnetwork";
	static const char _LN_tgtEl[] = "tgtEl";
	static const char _LN_rubyAlign[] = "rubyAlign";
	static const char _LN_clcbpat[] = "clcbpat";
	static const char _LN_DetailWidth[] = "DetailWidth";
	static const char _LN_min_label_width[] = "min-label-width";
	static const char _LN_LockSelect[] = "LockSelect";
	static const char _LN_pndbnum[] = "pndbnum";
	static const char _LN_queryAction[] = "queryAction";
	static const char _LN_data_pilot_group[] = "data-pilot-group";
	static const char _LN_autoRecovery[] = "autoRecovery";
	static const char _LN_tdfrmtxtRight[] = "tdfrmtxtRight";
	static const char _LN_tn[] = "tn";
	static const char _LN_clrMap[] = "clrMap";
	static const char _LN_last_row[] = "last-row";
	static const char _LN_fnil[] = "fnil";
	static const char _LN_clcbpatraw[] = "clcbpatraw";
	static const char _LN_compat[] = "compat";
	static const char _LN_hps[] = "hps";
	static const char _LN_pndbnumd[] = "pndbnumd";
	static const char _LN_rtn[] = "rtn";
	static const char _LN_GroupedWidth[] = "GroupedWidth";
	static const char _LN_ViewableRange[] = "ViewableRange";
	static const char _LN_LockCustProp[] = "LockCustProp";
	static const char _LN_fnonfilesys[] = "fnonfilesys";
	static const char _LN_openNewDocumentAction[] = "openNewDocumentAction";
	static const char _LN_hpsRaise[] = "hpsRaise";
	static const char _LN_min_label_distance[] = "min-label-distance";
	static const char _LN_first_column[] = "first-column";
	static const char _LN_docVars[] = "docVars";
	static const char _LN_data_pilot_group_member[] = "data-pilot-group-member";
	static const char _LN_tdfrmtxtTop[] = "tdfrmtxtTop";
	static const char _LN_clcfpat[] = "clcfpat";
	static const char _LN_cond[] = "cond";
	static const char _LN_pndbnumk[] = "pndbnumk";
	static const char _LN_TotalWidth[] = "TotalWidth";
	static const char _LN_hpsBaseText[] = "hpsBaseText";
	static const char _LN_HelpTopic[] = "HelpTopic";
	static const char _LN_fontemb[] = "fontemb";
	static const char _LN_GridlineColor[] = "GridlineColor";
	static const char _LN_rsids[] = "rsids";
	static const char _LN_clcfpatraw[] = "clcfpatraw";
	static const char _LN_DataTable[] = "DataTable";
	static const char _LN_last_column[] = "last-column";
	static const char _LN_dataObjects[] = "dataObjects";
	static const char _LN_pndbnuml[] = "pndbnuml";
	static const char _LN_closeDocumentAction[] = "closeDocumentAction";
	static const char _LN_uiCompat97To2003[] = "uiCompat97To2003";
	static const char _LN_fontfile[] = "fontfile";
	static const char _LN_PivotItem[] = "PivotItem";
	static const char _LN_consolidation[] = "consolidation";
	static const char _LN_excl[] = "excl";
	static const char _LN_Copyright[] = "Copyright";
	static const char _LN_lid[] = "lid";
	static const char _LN_ExcelWorksheetType[] = "ExcelWorksheetType";
	static const char _LN_Range[] = "Range";
	static const char _LN_cldgll[] = "cldgll";
	static const char _LN_stroke[] = "stroke";
	static const char _LN_dataObject[] = "dataObject";
	static const char _LN_pndbnumt[] = "pndbnumt";
	static const char _LN_rubyPr[] = "rubyPr";
	static const char _LN_titlepg[] = "titlepg";
	static const char _LN_even_rows[] = "even-rows";
	static const char _LN_fonttbl[] = "fonttbl";
	static const char _LN_DataField[] = "DataField";
	static const char _LN_NoObjHandles[] = "NoObjHandles";
	static const char _LN_RowInputCell[] = "RowInputCell";
	static const char _LN_cldglu[] = "cldglu";
	static const char _LN_ruleSet[] = "ruleSet";
	static const char _LN_source_cell_range_addresses[] = "source-cell-range-addresses";
	static const char _LN_binData[] = "binData";
	static const char _LN_stroke_dash_names[] = "stroke-dash-names";
	static const char _LN_lnL[] = "lnL";
	static const char _LN_condense[] = "condense";
	static const char _LN_pndec[] = "pndec";
	static const char _LN_adoAdapter[] = "adoAdapter";
	static const char _LN_rt[] = "rt";
	static const char _LN_animateEffect[] = "animateEffect";
	static const char _LN_tldot[] = "tldot";
	static const char _LN_grammar[] = "grammar";
	static const char _LN_IntlMacro[] = "IntlMacro";
	static const char _LN_odd_rows[] = "odd-rows";
	static const char _LN_clFitText[] = "clFitText";
	static const char _LN_MapChildItems[] = "MapChildItems";
	static const char _LN_lnR[] = "lnR";
	static const char _LN_ruleSets[] = "ruleSets";
	static const char _LN_sourceFileName[] = "sourceFileName";
	static const char _LN_NonPrinting[] = "NonPrinting";
	static const char _LN_rubyBase[] = "rubyBase";
	static const char _LN_footerf[] = "footerf";
	static const char _LN_ColumnInputCell[] = "ColumnInputCell";
	static const char _LN_pndecd[] = "pndecd";
	static const char _LN_extend[] = "extend";
	static const char _LN_tleq[] = "tleq";
	static const char _LN_webServiceAdapter[] = "webServiceAdapter";
	static const char _LN_datatypeLibrary[] = "datatypeLibrary";
	static const char _LN_lnT[] = "lnT";
	static const char _LN_stroke_width[] = "stroke-width";
	static const char _LN_Unsynced[] = "Unsynced";
	static const char _LN_even_columns[] = "even-columns";
	static const char _LN_clftsWidth[] = "clftsWidth";
	static const char _LN_calculations[] = "calculations";
	static const char _LN_pnf[] = "pnf";
	static const char _LN_footerl[] = "footerl";
	static const char _LN_dateFormat[] = "dateFormat";
	static const char _LN_tlhyph[] = "tlhyph";
	static const char _LN_AutoShowCount[] = "AutoShowCount";
	static const char _LN_target_cell_address[] = "target-cell-address";
	static const char _LN_animateRotation[] = "animateRotation";
	static const char _LN_NoCtlHandles[] = "NoCtlHandles";
	static const char _LN_lnB[] = "lnB";
	static const char _LN_marW[] = "marW";
	static const char _LN_sx[] = "sx";
	static const char _LN_ExcelWorkbook[] = "ExcelWorkbook";
	static const char _LN_Selected[] = "Selected";
	static const char _LN_pnfs[] = "pnfs";
	static const char _LN_stroke_color[] = "stroke-color";
	static const char _LN_odd_columns[] = "odd-columns";
	static const char _LN_tlmdot[] = "tlmdot";
	static const char _LN_footery[] = "footery";
	static const char _LN_define[] = "define";
	static const char _LN_lnTlToBr[] = "lnTlToBr";
	static const char _LN_clmgf[] = "clmgf";
	static const char _LN_hwsAdapter[] = "hwsAdapter";
	static const char _LN_sy[] = "sy";
	static const char _LN_marH[] = "marH";
	static const char _LN_calculatedField[] = "calculatedField";
	static const char _LN_listItem[] = "listItem";
	static const char _LN_AutoShowField[] = "AutoShowField";
	static const char _LN_NoAlignBox[] = "NoAlignBox";
	static const char _LN_pnganada[] = "pnganada";
	static const char _LN_animateScale[] = "animateScale";
	static const char _LN_CodeName[] = "CodeName";
	static const char _LN_off[] = "off";
	static const char _LN_PublishObjects[] = "PublishObjects";
	static const char _LN_use_labels[] = "use-labels";
	static const char _LN_tlth[] = "tlth";
	static const char _LN_clmrg[] = "clmrg";
	static const char _LN_name[] = "name";
	static const char _LN_scrollbar[] = "scrollbar";
	static const char _LN_lnBlToTr[] = "lnBlToTr";
	static const char _LN_sstItem[] = "sstItem";
	static const char _LN_font_face[] = "font-face";
	static const char _LN_marker_start[] = "marker-start";
	static const char _LN_docPartType[] = "docPartType";
	static const char _LN_formdisp[] = "formdisp";
	static const char _LN_ext[] = "ext";
	static const char _LN_pngblip[] = "pngblip";
	static const char _LN_UpdateAlignBox[] = "UpdateAlignBox";
	static const char _LN_tlul[] = "tlul";
	static const char _LN_optional[] = "optional";
	static const char _LN_hwsOperation[] = "hwsOperation";
	static const char _LN_cloWrap[] = "cloWrap";
	static const char _LN_PageSetup[] = "PageSetup";
	static const char _LN_AutoShowRange[] = "AutoShowRange";
	static const char _LN_link_to_source_data[] = "link-to-source-data";
	static const char _LN_noResizeAllowed[] = "noResizeAllowed";
	static const char _LN_cell3D[] = "cell3D";
	static const char _LN_webPubItem[] = "webPubItem";
	static const char _LN_SupBook[] = "SupBook";
	static const char _LN_font_family[] = "font-family";
	static const char _LN_chOff[] = "chOff";
	static const char _LN_marker_end[] = "marker-end";
	static const char _LN_docPartCategory[] = "docPartCategory";
	static const char _LN_toplinepunct[] = "toplinepunct";
	static const char _LN_formfield[] = "formfield";
	static const char _LN_pngbnum[] = "pngbnum";
	static const char _LN_clpadb[] = "clpadb";
	static const char _LN_attribute[] = "attribute";
	static const char _LN_DisplayPageBreak[] = "DisplayPageBreak";
	static const char _LN_input[] = "input";
	static const char _LN_gridCol[] = "gridCol";
	static const char _LN_HideText[] = "HideText";
	static const char _LN_chExt[] = "chExt";
	static const char _LN_AutoShowType[] = "AutoShowType";
	static const char _LN_SelectedSheets[] = "SelectedSheets";
	static const char _LN_csViews[] = "csViews";
	static const char _LN_video[] = "video";
	static const char _LN_font_style[] = "font-style";
	static const char _LN_linkedToFile[] = "linkedToFile";
	static const char _LN_pngbnumd[] = "pngbnumd";
	static const char _LN_formprot[] = "formprot";
	static const char _LN_marker_start_width[] = "marker-start-width";
	static const char _LN_clpadfb[] = "clpadfb";
	static const char _LN_dde_links[] = "dde-links";
	static const char _LN_tphcol[] = "tphcol";
	static const char _LN_docPartUnique[] = "docPartUnique";
	static const char _LN_txbx[] = "txbx";
	static const char _LN_pattern[] = "pattern";
	static const char _LN_DynFeedback[] = "DynFeedback";
	static const char _LN_csView[] = "csView";
	static const char _LN_beginCondLst[] = "beginCondLst";
	static const char _LN_TransitionExpressionEvaluation[] = "TransitionExpressionEvaluation";
	static const char _LN_AutoSortOrder[] = "AutoSortOrder";
	static const char _LN_pngbnumk[] = "pngbnumk";
	static const char _LN_formshade[] = "formshade";
	static const char _LN_clpadfl[] = "clpadfl";
	static const char _LN_WindowHidden[] = "WindowHidden";
	static const char _LN_font_variant[] = "font-variant";
	static const char _LN_tphmrg[] = "tphmrg";
	static const char _LN_tcPr[] = "tcPr";
	static const char _LN_noBorder[] = "noBorder";
	static const char _LN_tint[] = "tint";
	static const char _LN_Import[] = "Import";
	static const char _LN_process_content[] = "process-content";
	static const char _LN_marker_end_width[] = "marker-end-width";
	static const char _LN_pictureStorageType[] = "pictureStorageType";
	static const char _LN_GlueType[] = "GlueType";
	static const char _LN_tc[] = "tc";
	static const char _LN_endCondLst[] = "endCondLst";
	static const char _LN_tphpg[] = "tphpg";
	static const char _LN_pngbnuml[] = "pngbnuml";
	static const char _LN_clpadfr[] = "clpadfr";
	static const char _LN_fosnum[] = "fosnum";
	static const char _LN_shade[] = "shade";
	static const char _LN_AutoSortField[] = "AutoSortField";
	static const char _LN_flatBorders[] = "flatBorders";
	static const char _LN_HideHorizontalScrollBar[] = "HideHorizontalScrollBar";
	static const char _LN_font_weight[] = "font-weight";
	static const char _LN_tableStyle[] = "tableStyle";
	static const char _LN_WalkPreference[] = "WalkPreference";
	static const char _LN_tposnegx[] = "tposnegx";
	static const char _LN_EnsurePage[] = "EnsurePage";
	static const char _LN_clpadft[] = "clpadft";
	static const char _LN_ref[] = "ref";
	static const char _LN_comp[] = "comp";
	static const char _LN_pnhang[] = "pnhang";
	static const char _LN_fprq[] = "fprq";
	static const char _LN_endSync[] = "endSync";
	static const char _LN_TransitionFormulaEntry[] = "TransitionFormulaEntry";
	static const char _LN_count[] = "count";
	static const char _LN_marker_start_center[] = "marker-start-center";
	static const char _LN_docPart[] = "docPart";
	static const char _LN_framesetSplitbar[] = "framesetSplitbar";
	static const char _LN_BaseField[] = "BaseField";
	static const char _LN_font_stretch[] = "font-stretch";
	static const char _LN_start[] = "start";
	static const char _LN_tableStyleId[] = "tableStyleId";
	static const char _LN_inv[] = "inv";
	static const char _LN_fracwidth[] = "fracwidth";
	static const char _LN_tposnegy[] = "tposnegy";
	static const char _LN_pni[] = "pni";
	static const char _LN_clpadl[] = "clpadl";
	static const char _LN_PlaceObjects[] = "PlaceObjects";
	static const char _LN_dependencies[] = "dependencies";
	static const char _LN_BegTrigger[] = "BegTrigger";
	static const char _LN_HideVerticalScrollBar[] = "HideVerticalScrollBar";
	static const char _LN_alias[] = "alias";
	static const char _LN_gray[] = "gray";
	static const char _LN_choice[] = "choice";
	static const char _LN_childLst[] = "childLst";
	static const char _LN_BaseItem[] = "BaseItem";
	static const char _LN_pnindent[] = "pnindent";
	static const char _LN_font_size[] = "font-size";
	static const char _LN_clpadr[] = "clpadr";
	static const char _LN_marker_end_center[] = "marker-end-center";
	static const char _LN_tposxc[] = "tposxc";
	static const char _LN_frelative[] = "frelative";
	static const char _LN_frameLayout[] = "frameLayout";
	static const char _LN_tblPr[] = "tblPr";
	static const char _LN_invalidIfBlank[] = "invalidIfBlank";
	static const char _LN_Object[] = "Object";
	static const char _LN_dependency[] = "dependency";
	static const char _LN_alpha[] = "alpha";
	static const char _LN_EndTrigger[] = "EndTrigger";
	static const char _LN_element[] = "element";
	static const char _LN_Zoom[] = "Zoom";
	static const char _LN_subLst[] = "subLst";
	static const char _LN_BlankLineAfterItems[] = "BlankLineAfterItems";
	static const char _LN_attrName[] = "attrName";
	static const char _LN_pniroha[] = "pniroha";
	static const char _LN_clpadt[] = "clpadt";
	static const char _LN_tposxi[] = "tposxi";
	static const char _LN_tblGrid[] = "tblGrid";
	static const char _LN_unicode_range[] = "unicode-range";
	static const char _LN_revisions[] = "revisions";
	static const char _LN_frmtxbtlr[] = "frmtxbtlr";
	static const char _LN_HideWorkbookTabs[] = "HideWorkbookTabs";
	static const char _LN_alphaOff[] = "alphaOff";
	static const char _LN_frameset[] = "frameset";
	static const char _LN_Delete[] = "Delete";
	static const char _LN_PageBreakZoom[] = "PageBreakZoom";
	static const char _LN_document[] = "document";
	static const char _LN_stroke_opacity[] = "stroke-opacity";
	static const char _LN_ObjType[] = "ObjType";
	static const char _LN_timeNodeBase[] = "timeNodeBase";
	static const char _LN_lock[] = "lock";
	static const char _LN_pnirohad[] = "pnirohad";
	static const char _LN_clshdng[] = "clshdng";
	static const char _LN_attrNameLst[] = "attrNameLst";
	static const char _LN_tposxl[] = "tposxl";
	static const char _LN_tr[] = "tr";
	static const char _LN_deletions[] = "deletions";
	static const char _LN_info[] = "info";
	static const char _LN_frmtxlrtb[] = "frmtxlrtb";
	static const char _LN_alphaMod[] = "alphaMod";
	static const char _LN_units_per_em[] = "units-per-em";
	static const char _LN_CantGetUniqueItems[] = "CantGetUniqueItems";
	static const char _LN_Comment[] = "Comment";
	static const char _LN_document_content[] = "document-content";
	static const char _LN_WindowHeight[] = "WindowHeight";
	static const char _LN_tposx[] = "tposx";
	static const char _LN_clshdngraw[] = "clshdngraw";
	static const char _LN_ShowPageBreakZoom[] = "ShowPageBreakZoom";
	static const char _LN_pnlcltr[] = "pnlcltr";
	static const char _LN_prevCondLst[] = "prevCondLst";
	static const char _LN_showingPlcHdr[] = "showingPlcHdr";
	static const char _LN_stroke_linejoin[] = "stroke-linejoin";
	static const char _LN_boolVal[] = "boolVal";
	static const char _LN_cell_content_deletion[] = "cell-content-deletion";
	static const char _LN_Ink[] = "Ink";
	static const char _LN_nfc[] = "nfc";
	static const char _LN_frmtxlrtbv[] = "frmtxlrtbv";
	static const char _LN_hue[] = "hue";
	static const char _LN_rwCol[] = "rwCol";
	static const char _LN_IsDropSource[] = "IsDropSource";
	static const char _LN_tposxo[] = "tposxo";
	static const char _LN_panose_1[] = "panose-1";
	static const char _LN_pnlcrm[] = "pnlcrm";
	static const char _LN_clshdrawnil[] = "clshdrawnil";
	static const char _LN_Outline[] = "Outline";
	static const char _LN_WindowWidth[] = "WindowWidth";
	static const char _LN_lvlRestart[] = "lvlRestart";
	static const char _LN_intVal[] = "intVal";
	static const char _LN_hueOff[] = "hueOff";
	static const char _LN_GroupBy[] = "GroupBy";
	static const char _LN_document_styles[] = "document-styles";
	static const char _LN_nextCondLst[] = "nextCondLst";
	static const char _LN_move[] = "move";
	static const char _LN_edittingPlcHdr[] = "edittingPlcHdr";
	static const char _LN_DefaultRowHeight[] = "DefaultRowHeight";
	static const char _LN_frmtxtbrl[] = "frmtxtbrl";
	static const char _LN_tposxr[] = "tposxr";
	static const char _LN_stemv[] = "stemv";
	static const char _LN_pnlvl[] = "pnlvl";
	static const char _LN_NoLiveDynamics[] = "NoLiveDynamics";
	static const char _LN_Html[] = "Html";
	static const char _LN_fill_color[] = "fill-color";
	static const char _LN_fltVal[] = "fltVal";
	static const char _LN_change_deletion[] = "change-deletion";
	static const char _LN_hueMod[] = "hueMod";
	static const char _LN_userView[] = "userView";
	static const char _LN_GroupStart[] = "GroupStart";
	static const char _LN_cltxbtlr[] = "cltxbtlr";
	static const char _LN_isLgl[] = "isLgl";
	static const char _LN_WindowTopX[] = "WindowTopX";
	static const char _LN_tposy[] = "tposy";
	static const char _LN_stemh[] = "stemh";
	static const char _LN_pnlvlblt[] = "pnlvlblt";
	static const char _LN_frmtxtbrlv[] = "frmtxtbrlv";
	static const char _LN_document_meta[] = "document-meta";
	static const char _LN_dataBinding[] = "dataBinding";
	static const char _LN_File[] = "File";
	static const char _LN_strVal[] = "strVal";
	static const char _LN_DefaultColumnWidth[] = "DefaultColumnWidth";
	static const char _LN_sat[] = "sat";
	static const char _LN_suff[] = "suff";
	static const char _LN_secondary_fill_color[] = "secondary-fill-color";
	static const char _LN_sheetName[] = "sheetName";
	static const char _LN_cltxlrtb[] = "cltxlrtb";
	static const char _LN_LocalizeMerge[] = "LocalizeMerge";
	static const char _LN_GroupEnd[] = "GroupEnd";
	static const char _LN_tposyb[] = "tposyb";
	static const char _LN_unthemedFill[] = "unthemedFill";
	static const char _LN_WindowTopY[] = "WindowTopY";
	static const char _LN_slope[] = "slope";
	static const char _LN_Data[] = "Data";
	static const char _LN_pnlvlbody[] = "pnlvlbody";
	static const char _LN_satOff[] = "satOff";
	static const char _LN_froman[] = "froman";
	static const char _LN_val[] = "val";
	static const char _LN_lvlText[] = "lvlText";
	static const char _LN_comboBox[] = "comboBox";
	static const char _LN_document_settings[] = "document-settings";
	static const char _LN_multi_deletion_spanned[] = "multi-deletion-spanned";
	static const char _LN_sheet[] = "sheet";
	static const char _LN_tposyc[] = "tposyc";
	static const char _LN_cltxlrtbv[] = "cltxlrtbv";
	static const char _LN_cap_height[] = "cap-height";
	static const char _LN_GroupStartAuto[] = "GroupStartAuto";
	static const char _LN_tav[] = "tav";
	static const char _LN_StandardWidth[] = "StandardWidth";
	static const char _LN_ActiveSheet[] = "ActiveSheet";
	static const char _LN_Calendar[] = "Calendar";
	static const char _LN_satMod[] = "satMod";
	static const char _LN_themedFill[] = "themedFill";
	static const char _LN_fromhtml[] = "fromhtml";
	static const char _LN_pnlvlcont[] = "pnlvlcont";
	static const char _LN_lvlPicBulletId[] = "lvlPicBulletId";
	static const char _LN_fill_gradient_name[] = "fill-gradient-name";
	static const char _LN_cell[] = "cell";
	static const char _LN_tposyil[] = "tposyil";
	static const char _LN_behaviorBase[] = "behaviorBase";
	static const char _LN_cltxtbrl[] = "cltxtbrl";
	static const char _LN_x_height[] = "x-height";
	static const char _LN_docPartObj[] = "docPartObj";
	static const char _LN_lum[] = "lum";
	static const char _LN_combine[] = "combine";
	static const char _LN_LangID[] = "LangID";
	static const char _LN_fromtext[] = "fromtext";
	static const char _LN_GroupEndAuto[] = "GroupEndAuto";
	static const char _LN_ActiveChart[] = "ActiveChart";
	static const char _LN_ln[] = "ln";
	static const char _LN_pnnumonce[] = "pnnumonce";
	static const char _LN_cut_offs[] = "cut-offs";
	static const char _LN_tposyin[] = "tposyin";
	static const char _LN_lumOff[] = "lumOff";
	static const char _LN_legacy[] = "legacy";
	static const char _LN_autofmt[] = "autofmt";
	static const char _LN_cltxtbrlv[] = "cltxtbrlv";
	static const char _LN_LeftColumnVisible[] = "LeftColumnVisible";
	static const char _LN_unthemedLine[] = "unthemedLine";
	static const char _LN_accent_height[] = "accent-height";
	static const char _LN_ShapeKeywords[] = "ShapeKeywords";
	static const char _LN_version[] = "version";
	static const char _LN_gradient_step_count[] = "gradient-step-count";
	static const char _LN_tavLst[] = "tavLst";
	static const char _LN_docPartList[] = "docPartList";
	static const char _LN_fs[] = "fs";
	static const char _LN_insertion_cut_off[] = "insertion-cut-off";
	static const char _LN_pnord[] = "pnord";
	static const char _LN_GroupType[] = "GroupType";
	static const char _LN_FirstVisibleSheet[] = "FirstVisibleSheet";
	static const char _LN_UID[] = "UID";
	static const char _LN_tposyoutv[] = "tposyoutv";
	static const char _LN_lumMod[] = "lumMod";
	static const char _LN_lvlJc[] = "lvlJc";
	static const char _LN_fscript[] = "fscript";
	static const char _LN_rgb[] = "rgb";
	static const char _LN_clvertalb[] = "clvertalb";
	static const char _LN_mimetype[] = "mimetype";
	static const char _LN_themedLine[] = "themedLine";
	static const char _LN_pnordt[] = "pnordt";
	static const char _LN_Start[] = "Start";
	static const char _LN_ascent[] = "ascent";
	static const char _LN_dropDownList[] = "dropDownList";
	static const char _LN_DropOnPageScale[] = "DropOnPageScale";
	static const char _LN_DisplayRightToLeft[] = "DisplayRightToLeft";
	static const char _LN_lsid[] = "lsid";
	static const char _LN_LayoutPageBreak[] = "LayoutPageBreak";
	static const char _LN_tposyt[] = "tposyt";
	static const char _LN_fill_hatch_name[] = "fill-hatch-name";
	static const char _LN_qtField[] = "qtField";
	static const char _LN_fswiss[] = "fswiss";
	static const char _LN_movement_cut_off[] = "movement-cut-off";
	static const char _LN_meta[] = "meta";
	static const char _LN_redOff[] = "redOff";
	static const char _LN_clvertalc[] = "clvertalc";
	static const char _LN_HidePivotTableFieldList[] = "HidePivotTableFieldList";
	static const char _LN_Finish[] = "Finish";
	static const char _LN_pnprev[] = "pnprev";
	static const char _LN_descent[] = "descent";
	static const char _LN_plt[] = "plt";
	static const char _LN_unthemedFonts[] = "unthemedFonts";
	static const char _LN_picture[] = "picture";
	static const char _LN_tpvmrg[] = "tpvmrg";
	static const char _LN_conflict[] = "conflict";
	static const char _LN_ftnalt[] = "ftnalt";
	static const char _LN_zeroOrMore[] = "zeroOrMore";
	static const char _LN_XRulerDensity[] = "XRulerDensity";
	static const char _LN_redMod[] = "redMod";
	static const char _LN_ParentField[] = "ParentField";
	static const char _LN_Unit[] = "Unit";
	static const char _LN_pnqc[] = "pnqc";
	static const char _LN_GridlineColorIndex[] = "GridlineColorIndex";
	static const char _LN_clvertalt[] = "clvertalt";
	static const char _LN_widths[] = "widths";
	static const char _LN_fill_hatch_solid[] = "fill-hatch-solid";
	static const char _LN_start_position[] = "start-position";
	static const char _LN_styleLink[] = "styleLink";
	static const char _LN_tpvpara[] = "tpvpara";
	static const char _LN_richText[] = "richText";
	static const char _LN_progress[] = "progress";
	static const char _LN_ftnbj[] = "ftnbj";
	static const char _LN_themedFonts[] = "themedFonts";
	static const char _LN_revdata[] = "revdata";
	static const char _LN_pnql[] = "pnql";
	static const char _LN_body[] = "body";
	static const char _LN_ProtectStructure[] = "ProtectStructure";
	static const char _LN_bbox[] = "bbox";
	static const char _LN_YRulerDensity[] = "YRulerDensity";
	static const char _LN_clvmgf[] = "clvmgf";
	static const char _LN_Function[] = "Function";
	static const char _LN_Project[] = "Project";
	static const char _LN_greenOff[] = "greenOff";
	static const char _LN_tpvpg[] = "tpvpg";
	static const char _LN_ftncn[] = "ftncn";
	static const char _LN_listStyleLink[] = "listStyleLink";
	static const char _LN_rCenter[] = "rCenter";
	static const char _LN_pnqr[] = "pnqr";
	static const char _LN_fill_image_name[] = "fill-image-name";
	static const char _LN_DisplayFormulas[] = "DisplayFormulas";
	static const char _LN_text[] = "text";
	static const char _LN_tabidMap[] = "tabidMap";
	static const char _LN_end_position[] = "end-position";
	static const char _LN_ideographic[] = "ideographic";
	static const char _LN_fonts[] = "fonts";
	static const char _LN_clvmrg[] = "clvmrg";
	static const char _LN_LayoutForm[] = "LayoutForm";
	static const char _LN_tqc[] = "tqc";
	static const char _LN_greenMod[] = "greenMod";
	static const char _LN_ftnil[] = "ftnil";
	static const char _LN_pnrauth[] = "pnrauth";
	static const char _LN_XRulerOrigin[] = "XRulerOrigin";
	static const char _LN_ProtectWindows[] = "ProtectWindows";
	static const char _LN_group[] = "group";
	static const char _LN_tbl[] = "tbl";
	static const char _LN_mediaNodeBase[] = "mediaNodeBase";
	static const char _LN_b[] = "b";
	static const char _LN_tabId[] = "tabId";
	static const char _LN_lvl[] = "lvl";
	static const char _LN_tqdec[] = "tqdec";
	static const char _LN_clwWidth[] = "clwWidth";
	static const char _LN_alphabetic[] = "alphabetic";
	static const char _LN_movement[] = "movement";
	static const char _LN_fill_image_width[] = "fill-image-width";
	static const char _LN_DoNotDisplayGridlines[] = "DoNotDisplayGridlines";
	static const char _LN_ftnlytwnine[] = "ftnlytwnine";
	static const char _LN_global[] = "global";
	static const char _LN_i[] = "i";
	static const char _LN_pnrdate[] = "pnrdate";
	static const char _LN_LayoutSubtotalLocation[] = "LayoutSubtotalLocation";
	static const char _LN_startOverride[] = "startOverride";
	static const char _LN_undoinfo[] = "undoinfo";
	static const char _LN_YRulerOrigin[] = "YRulerOrigin";
	static const char _LN_blueOff[] = "blueOff";
	static const char _LN_tqr[] = "tqr";
	static const char _LN_DisplayInkNotes[] = "DisplayInkNotes";
	static const char _LN_timeNodeLst[] = "timeNodeLst";
	static const char _LN_left[] = "left";
	static const char _LN_collapsed[] = "collapsed";
	static const char _LN_source_range_address[] = "source-range-address";
	static const char _LN_mathematical[] = "mathematical";
	static const char _LN_sdtPr[] = "sdtPr";
	static const char _LN_drawing[] = "drawing";
	static const char _LN_pnrestart[] = "pnrestart";
	static const char _LN_ftnnalc[] = "ftnnalc";
	static const char _LN_transmf[] = "transmf";
	static const char _LN_fill_image_height[] = "fill-image-height";
	static const char _LN_cellOld[] = "cellOld";
	static const char _LN_right[] = "right";
	static const char _LN_blueMod[] = "blueMod";
	static const char _LN_ilst[] = "ilst";
	static const char _LN_DoNotDisplayHeadings[] = "DoNotDisplayHeadings";
	static const char _LN_XGridDensity[] = "XGridDensity";
	static const char _LN_sdtContent[] = "sdtContent";
	static const char _LN_tmpl[] = "tmpl";
	static const char _LN_colno[] = "colno";
	static const char _LN_Author[] = "Author";
	static const char _LN_empty[] = "empty";
	static const char _LN_NoDragToRow[] = "NoDragToRow";
	static const char _LN_EmbedSaveSmartTags[] = "EmbedSaveSmartTags";
	static const char _LN_ftnnar[] = "ftnnar";
	static const char _LN_pnrnfc[] = "pnrnfc";
	static const char _LN_hanging[] = "hanging";
	static const char _LN_trauth[] = "trauth";
	static const char _LN_top[] = "top";
	static const char _LN_cellNew[] = "cellNew";
	static const char _LN_lvlOverride[] = "lvlOverride";
	static const char _LN_column[] = "column";
	static const char _LN_tmplLst[] = "tmplLst";
	static const char _LN_colortbl[] = "colortbl";
	static const char _LN_CreationDate[] = "CreationDate";
	static const char _LN_presentation[] = "presentation";
	static const char _LN_fill_image_ref_point_x[] = "fill-image-ref-point-x";
	static const char _LN_YGridDensity[] = "YGridDensity";
	static const char _LN_scrgbClr[] = "scrgbClr";
	static const char _LN_ftnnauc[] = "ftnnauc";
	static const char _LN_bottom[] = "bottom";
	static const char _LN_pnrnot[] = "pnrnot";
	static const char _LN_v_ideographic[] = "v-ideographic";
	static const char _LN_trautofit[] = "trautofit";
	static const char _LN_NoDragToColumn[] = "NoDragToColumn";
	static const char _LN_dxfOld[] = "dxfOld";
	static const char _LN_sdt[] = "sdt";
	static const char _LN_DoNotDisplayOutline[] = "DoNotDisplayOutline";
	static const char _LN_row[] = "row";
	static const char _LN_bldAsOne[] = "bldAsOne";
	static const char _LN_listPicBullet[] = "listPicBullet";
	static const char _LN_FutureVer[] = "FutureVer";
	static const char _LN_insideH[] = "insideH";
	static const char _LN_pnrpnbr[] = "pnrpnbr";
	static const char _LN_ftnnchi[] = "ftnnchi";
	static const char _LN_srgbClr[] = "srgbClr";
	static const char _LN_spreadsheet[] = "spreadsheet";
	static const char _LN_subDoc[] = "subDoc";
	static const char _LN_dxfNew[] = "dxfNew";
	static const char _LN_colsr[] = "colsr";
	static const char _LN_XGridSpacing[] = "XGridSpacing";
	static const char _LN_start_column[] = "start-column";
	static const char _LN_trbgbdiag[] = "trbgbdiag";
	static const char _LN_v_alphabetic[] = "v-alphabetic";
	static const char _LN_LastSaved[] = "LastSaved";
	static const char _LN_NoDragToData[] = "NoDragToData";
	static const char _LN_bldSub[] = "bldSub";
	static const char _LN_TabRatio[] = "TabRatio";
	static const char _LN_fill_image_ref_point_y[] = "fill-image-ref-point-y";
	static const char _LN_insideV[] = "insideV";
	static const char _LN_pnrrgb[] = "pnrrgb";
	static const char _LN_ftnnchosung[] = "ftnnchosung";
	static const char _LN_hslClr[] = "hslClr";
	static const char _LN_listDef[] = "listDef";
	static const char _LN_colsx[] = "colsx";
	static const char _LN_dxfFmt[] = "dxfFmt";
	static const char _LN_ApplyAutomaticOutlineStyles[] = "ApplyAutomaticOutlineStyles";
	static const char _LN_chart[] = "chart";
	static const char _LN_trbgcross[] = "trbgcross";
	static const char _LN_YGridSpacing[] = "YGridSpacing";
	static const char _LN_start_row[] = "start-row";
	static const char _LN_bldP[] = "bldP";
	static const char _LN_ScheduleFromStart[] = "ScheduleFromStart";
	static const char _LN_v_mathematical[] = "v-mathematical";
	static const char _LN_WindowIconic[] = "WindowIconic";
	static const char _LN_pnrstart[] = "pnrstart";
	static const char _LN_NoDragToHide[] = "NoDragToHide";
	static const char _LN_tl2br[] = "tl2br";
	static const char _LN_sysClr[] = "sysClr";
	static const char _LN_image[] = "image";
	static const char _LN_ftnncnum[] = "ftnncnum";
	static const char _LN_bldDgm[] = "bldDgm";
	static const char _LN_colw[] = "colw";
	static const char _LN_ilfoMacAtCleanup[] = "ilfoMacAtCleanup";
	static const char _LN_trbgdcross[] = "trbgdcross";
	static const char _LN_formulaOld[] = "formulaOld";
	static const char _LN_start_table[] = "start-table";
	static const char _LN_fill_image_ref_point[] = "fill-image-ref-point";
	static const char _LN_tr2bl[] = "tr2bl";
	static const char _LN_XGridOrigin[] = "XGridOrigin";
	static const char _LN_schemeClr[] = "schemeClr";
	static const char _LN_pnrstop[] = "pnrstop";
	static const char _LN_settings[] = "settings";
	static const char _LN_DisplayDrawingObjects[] = "DisplayDrawingObjects";
	static const char _LN_NoDragToPage[] = "NoDragToPage";
	static const char _LN_v_hanging[] = "v-hanging";
	static const char _LN_ftnndbar[] = "ftnndbar";
	static const char _LN_StartDate[] = "StartDate";
	static const char _LN_bldOleChart[] = "bldOleChart";
	static const char _LN_NoSummaryRowsBelowDetail[] = "NoSummaryRowsBelowDetail";
	static const char _LN_tcW[] = "tcW";
	static const char _LN_tcBorders[] = "tcBorders";
	static const char _LN_trbgdkbdiag[] = "trbgdkbdiag";
	static const char _LN_company[] = "company";
	static const char _LN_end_column[] = "end-column";
	static const char _LN_pnrxst[] = "pnrxst";
	static const char _LN_prstClr[] = "prstClr";
	static const char _LN_aliases[] = "aliases";
	static const char _LN_oneOrMore[] = "oneOrMore";
	static const char _LN_YGridOrigin[] = "YGridOrigin";
	static const char _LN_gridSpan[] = "gridSpan";
	static const char _LN_ftnndbnum[] = "ftnndbnum";
	static const char _LN_underline_position[] = "underline-position";
	static const char _LN_FinishDate[] = "FinishDate";
	static const char _LN_MemberProperty[] = "MemberProperty";
	static const char _LN_tile_repeat_offset[] = "tile-repeat-offset";
	static const char _LN_bldE2o[] = "bldE2o";
	static const char _LN_cpg[] = "cpg";
	static const char _LN_rPr[] = "rPr";
	static const char _LN_pnscaps[] = "pnscaps";
	static const char _LN_trbgdkcross[] = "trbgdkcross";
	static const char _LN_CreateBackup[] = "CreateBackup";
	static const char _LN_sti[] = "sti";
	static const char _LN_end_row[] = "end-row";
	static const char _LN_config_item_set[] = "config-item-set";
	static const char _LN_OutputFormat[] = "OutputFormat";
	static const char _LN_hmerge[] = "hmerge";
	static const char _LN_ftnndbnumd[] = "ftnndbnumd";
	static const char _LN_NoSummaryColumnsRightDetail[] = "NoSummaryColumnsRightDetail";
	static const char _LN_buildLst[] = "buildLst";
	static const char _LN_crauth[] = "crauth";
	static const char _LN_tableBg[] = "tableBg";
	static const char _LN_FYStartDate[] = "FYStartDate";
	static const char _LN_basedOn[] = "basedOn";
	static const char _LN_pnseclvl[] = "pnseclvl";
	static const char _LN_MemberPropertiesOrder[] = "MemberPropertiesOrder";
	static const char _LN_underline_thickness[] = "underline-thickness";
	static const char _LN_end_table[] = "end-table";
	static const char _LN_trbgdkdcross[] = "trbgdkdcross";
	static const char _LN_opacity_name[] = "opacity-name";
	static const char _LN_vmerge[] = "vmerge";
	static const char _LN_Calculation[] = "Calculation";
	static const char _LN_crdate[] = "crdate";
	static const char _LN_wholeTable[] = "wholeTable";
	static const char _LN_ftnndbnumk[] = "ftnndbnumk";
	static const char _LN_LockPreview[] = "LockPreview";
	static const char _LN_next[] = "next";
	static const char _LN_CriticalSlackLimit[] = "CriticalSlackLimit";
	static const char _LN_pnsp[] = "pnsp";
	static const char _LN_config_item[] = "config-item";
	static const char _LN_change_track_table_cell[] = "change-track-table-cell";
	static const char _LN_creatim[] = "creatim";
	static const char _LN_trbgdkfdiag[] = "trbgdkfdiag";
	static const char _LN_fill_rule[] = "fill-rule";
	static const char _LN_DoNotCalculateBeforeSave[] = "DoNotCalculateBeforeSave";
	static const char _LN_DoNotDisplayZeros[] = "DoNotDisplayZeros";
	static const char _LN_band1Horz[] = "band1Horz";
	static const char _LN_noWrap[] = "noWrap";
	static const char _LN_pnstart[] = "pnstart";
	static const char _LN_strikethrough_position[] = "strikethrough-position";
	static const char _LN_ftnndbnumt[] = "ftnndbnumt";
	static const char _LN_HideDropDowns[] = "HideDropDowns";
	static const char _LN_AddMarkup[] = "AddMarkup";
	static const char _LN_users[] = "users";
	static const char _LN_priority[] = "priority";
	static const char _LN_CurrencyDigits[] = "CurrencyDigits";
	static const char _LN_symbol_color[] = "symbol-color";
	static const char _LN_tcMar[] = "tcMar";
	static const char _LN_trbgdkhor[] = "trbgdkhor";
	static const char _LN_pnstrike[] = "pnstrike";
	static const char _LN_band2Horz[] = "band2Horz";
	static const char _LN_ctrl[] = "ctrl";
	static const char _LN_object_index_source[] = "object-index-source";
	static const char _LN_usrinfo[] = "usrinfo";
	static const char _LN_type[] = "type";
	static const char _LN_ftnnganada[] = "ftnnganada";
	static const char _LN_ViewMarkup[] = "ViewMarkup";
	static const char _LN_autoRedefine[] = "autoRedefine";
	static const char _LN_DrilledLevel[] = "DrilledLevel";
	static const char _LN_cell_address[] = "cell-address";
	static const char _LN_ActiveColumn[] = "ActiveColumn";
	static const char _LN_tcFitText[] = "tcFitText";
	static const char _LN_Date1904[] = "Date1904";
	static const char _LN_cts[] = "cts";
	static const char _LN_strikethrough_thickness[] = "strikethrough-thickness";
	static const char _LN_value[] = "value";
	static const char _LN_pntext[] = "pntext";
	static const char _LN_trbgdkvert[] = "trbgdkvert";
	static const char _LN_CurrencySymbol[] = "CurrencySymbol";
	static const char _LN_band1Vert[] = "band1Vert";
	static const char _LN_animation[] = "animation";
	static const char _LN_sld[] = "sld";
	static const char _LN_ftnngbnum[] = "ftnngbnum";
	static const char _LN_PreviewQuality[] = "PreviewQuality";
	static const char _LN_cufi[] = "cufi";
	static const char _LN_ServerSortOrder[] = "ServerSortOrder";
	static const char _LN_matrix_covered[] = "matrix-covered";
	static const char _LN_FilterOn[] = "FilterOn";
	static const char _LN_config_item_map_indexed[] = "config-item-map-indexed";
	static const char _LN_use_spreadsheet_objects[] = "use-spreadsheet-objects";
	static const char _LN_hideMark[] = "hideMark";
	static const char _LN_RefModeR1C1[] = "RefModeR1C1";
	static const char _LN_pntxta[] = "pntxta";
	static const char _LN_semiHidden[] = "semiHidden";
	static const char _LN_trbgfdiag[] = "trbgfdiag";
	static const char _LN_band2Vert[] = "band2Vert";
	static const char _LN_animation_direction[] = "animation-direction";
	static const char _LN_culi[] = "culi";
	static const char _LN_ftnngbnumd[] = "ftnngbnumd";
	static const char _LN_CurrencySymbolPosition[] = "CurrencySymbolPosition";
	static const char _LN_range[] = "range";
	static const char _LN_PreviewScope[] = "PreviewScope";
	static const char _LN_pntxtb[] = "pntxtb";
	static const char _LN_TopRowVisible[] = "TopRowVisible";
	static const char _LN_overline_position[] = "overline-position";
	static const char _LN_Iteration[] = "Iteration";
	static const char _LN_cell_content_change[] = "cell-content-change";
	static const char _LN_IsMemberProperty[] = "IsMemberProperty";
	static const char _LN_curi[] = "curi";
	static const char _LN_locked[] = "locked";
	static const char _LN_trbghoriz[] = "trbghoriz";
	static const char _LN_lastCol[] = "lastCol";
	static const char _LN_gridBefore[] = "gridBefore";
	static const char _LN_custData[] = "custData";
	static const char _LN_ftnngbnumk[] = "ftnngbnumk";
	static const char _LN_pnucltr[] = "pnucltr";
	static const char _LN_config_item_map_entry[] = "config-item-map-entry";
	static const char _LN_use_math_objects[] = "use-math-objects";
	static const char _LN_animation_start_inside[] = "animation-start-inside";
	static const char _LN_cvmme[] = "cvmme";
	static const char _LN_DocLangID[] = "DocLangID";
	static const char _LN_TopRowBottomPane[] = "TopRowBottomPane";
	static const char _LN_MaxIterations[] = "MaxIterations";
	static const char _LN_unhideWhenUsed[] = "unhideWhenUsed";
	static const char _LN_firstCol[] = "firstCol";
	static const char _LN_trbgvert[] = "trbgvert";
	static const char _LN_CalendarUID[] = "CalendarUID";
	static const char _LN_overline_thickness[] = "overline-thickness";
	static const char _LN_gridAfter[] = "gridAfter";
	static const char _LN_FilterMember[] = "FilterMember";
	static const char _LN_pnucrm[] = "pnucrm";
	static const char _LN_ftnngbnuml[] = "ftnngbnuml";
	static const char _LN_previous[] = "previous";
	static const char _LN_datafield[] = "datafield";
	static const char _LN_singleCells[] = "singleCells";
	static const char _LN_Gamma[] = "Gamma";
	static const char _LN_use_draw_objects[] = "use-draw-objects";
	static const char _LN_lastRow[] = "lastRow";
	static const char _LN_trbrdrb[] = "trbrdrb";
	static const char _LN_MaxChange[] = "MaxChange";
	static const char _LN_config_item_map_named[] = "config-item-map-named";
	static const char _LN_qformat[] = "qformat";
	static const char _LN_wBefore[] = "wBefore";
	static const char _LN_pnul[] = "pnul";
	static const char _LN_DefaultStartTime[] = "DefaultStartTime";
	static const char _LN_LeftColumnRightPane[] = "LeftColumnRightPane";
	static const char _LN_acceptance_state[] = "acceptance-state";
	static const char _LN_animation_stop_inside[] = "animation-stop-inside";
	static const char _LN_ftnnrlc[] = "ftnnrlc";
	static const char _LN_UniqueName[] = "UniqueName";
	static const char _LN_Contrast[] = "Contrast";
	static const char _LN_font_face_src[] = "font-face-src";
	static const char _LN_seCell[] = "seCell";
	static const char _LN_trbrdrh[] = "trbrdrh";
	static const char _LN_pnuld[] = "pnuld";
	static const char _LN_dbch[] = "dbch";
	static const char _LN_personal[] = "personal";
	static const char _LN_wAfter[] = "wAfter";
	static const char _LN_PrecisionAsDisplayed[] = "PrecisionAsDisplayed";
	static const char _LN_use_chart_objects[] = "use-chart-objects";
	static const char _LN_ftnnruc[] = "ftnnruc";
	static const char _LN_Brightness[] = "Brightness";
	static const char _LN_DefaultFinishTime[] = "DefaultFinishTime";
	static const char _LN_deff[] = "deff";
	static const char _LN_swCell[] = "swCell";
	static const char _LN_pnuldash[] = "pnuldash";
	static const char _LN_rejecting_change_id[] = "rejecting-change-id";
	static const char _LN_trbrdrl[] = "trbrdrl";
	static const char _LN_ActivePane[] = "ActivePane";
	static const char _LN_scripts[] = "scripts";
	static const char _LN_font_face_uri[] = "font-face-uri";
	static const char _LN_cantSplit[] = "cantSplit";
	static const char _LN_ServerBased[] = "ServerBased";
	static const char _LN_personalCompose[] = "personalCompose";
	static const char _LN_animation_repeat[] = "animation-repeat";
	static const char _LN_ftnnzodiac[] = "ftnnzodiac";
	static const char _LN_commentAuthor[] = "commentAuthor";
	static const char _LN_defformat[] = "defformat";
	static const char _LN_firstRow[] = "firstRow";
	static const char _LN_Sharpen[] = "Sharpen";
	static const char _LN_pnuldashd[] = "pnuldashd";
	static const char _LN_trbrdrr[] = "trbrdrr";
	static const char _LN_DoNotSaveLinkValues[] = "DoNotSaveLinkValues";
	static const char _LN_script[] = "script";
	static const char _LN_use_other_objects[] = "use-other-objects";
	static const char _LN_SplitHorizontal[] = "SplitHorizontal";
	static const char _LN_trHeight[] = "trHeight";
	static const char _LN_MinutesPerDay[] = "MinutesPerDay";
	static const char _LN_font_face_format[] = "font-face-format";
	static const char _LN_SubtotalName[] = "SubtotalName";
	static const char _LN_ftnnzodiacd[] = "ftnnzodiacd";
	static const char _LN_handout_master[] = "handout-master";
	static const char _LN_personalReply[] = "personalReply";
	static const char _LN_deflang[] = "deflang";
	static const char _LN_Blur[] = "Blur";
	static const char _LN_neCell[] = "neCell";
	static const char _LN_animation_delay[] = "animation-delay";
	static const char _LN_mixed[] = "mixed";
	static const char _LN_trbrdrt[] = "trbrdrt";
	static const char _LN_pnuldashdd[] = "pnuldashdd";
	static const char _LN_tblHeader[] = "tblHeader";
	static const char _LN_Denoise[] = "Denoise";
	static const char _LN_object_index_entry_template[] = "object-index-entry-template";
	static const char _LN_SplitVertical[] = "SplitVertical";
	static const char _LN_Sst[] = "Sst";
	static const char _LN_deflangfe[] = "deflangfe";
	static const char _LN_PLTPivotItem[] = "PLTPivotItem";
	static const char _LN_NoAutoRecover[] = "NoAutoRecover";
	static const char _LN_MinutesPerWeek[] = "MinutesPerWeek";
	static const char _LN_nwCell[] = "nwCell";
	static const char _LN_language[] = "language";
	static const char _LN_ftnnzodiacl[] = "ftnnzodiacl";
	static const char _LN_string[] = "string";
	static const char _LN_cm[] = "cm";
	static const char _LN_trbrdrv[] = "trbrdrv";
	static const char _LN_tblStylePr[] = "tblStylePr";
	static const char _LN_presentation_page_layout_name[] = "presentation-page-layout-name";
	static const char _LN_pnuldb[] = "pnuldb";
	static const char _LN_animation_steps[] = "animation-steps";
	static const char _LN_tblCellSpacing[] = "tblCellSpacing";
	static const char _LN_Transparency[] = "Transparency";
	static const char _LN_defshp[] = "defshp";
	static const char _LN_font_face_decls[] = "font-face-decls";
	static const char _LN_font_face_name[] = "font-face-name";
	static const char _LN_trcbpat[] = "trcbpat";
	static const char _LN_FreezePanes[] = "FreezePanes";
	static const char _LN_FormulaV10[] = "FormulaV10";
	static const char _LN_AcceptLabelsInFormulas[] = "AcceptLabelsInFormulas";
	static const char _LN_pnulhair[] = "pnulhair";
	static const char _LN_ftnrestart[] = "ftnrestart";
	static const char _LN_DaysPerMonth[] = "DaysPerMonth";
	static const char _LN_lsdException[] = "lsdException";
	static const char _LN_deftab[] = "deftab";
	static const char _LN_auto_grow_width[] = "auto-grow-width";
	static const char _LN_tblPrEx[] = "tblPrEx";
	static const char _LN_user_index[] = "user-index";
	static const char _LN_SelectMode[] = "SelectMode";
	static const char _LN_trcfpat[] = "trcfpat";
	static const char _LN_pnulnone[] = "pnulnone";
	static const char _LN_ParseFormulaAsV10[] = "ParseFormulaAsV10";
	static const char _LN_FrozenNoSplit[] = "FrozenNoSplit";
	static const char _LN_ftnrstcont[] = "ftnrstcont";
	static const char _LN_styles[] = "styles";
	static const char _LN_DefaultTaskType[] = "DefaultTaskType";
	static const char _LN_page_layout_name[] = "page-layout-name";
	static const char _LN_versionOfBuiltInStylenames[] = "versionOfBuiltInStylenames";
	static const char _LN_deleted[] = "deleted";
	static const char _LN_definition_src[] = "definition-src";
	static const char _LN_trPr[] = "trPr";
	static const char _LN_trdate[] = "trdate";
	static const char _LN_Uncalced[] = "Uncalced";
	static const char _LN_user_index_source[] = "user-index-source";
	static const char _LN_DisplayMode[] = "DisplayMode";
	static const char _LN_pnulth[] = "pnulth";
	static const char _LN_interleave[] = "interleave";
	static const char _LN_auto_grow_height[] = "auto-grow-height";
	static const char _LN_ftnrstpg[] = "ftnrstpg";
	static const char _LN_tblStyle[] = "tblStyle";
	static const char _LN_TabColorIndex[] = "TabColorIndex";
	static const char _LN_delrsid[] = "delrsid";
	static const char _LN_PLCaption[] = "PLCaption";
	static const char _LN_trftsWidthA[] = "trftsWidthA";
	static const char _LN_effect[] = "effect";
	static const char _LN_Styles[] = "Styles";
	static const char _LN_DefaultFixedCostAccrual[] = "DefaultFixedCostAccrual";
	static const char _LN_pnulw[] = "pnulw";
	static const char _LN_font_adornments[] = "font-adornments";
	static const char _LN_IsDropTarget[] = "IsDropTarget";
	static const char _LN_topBaseline[] = "topBaseline";
	static const char _LN_automatic_styles[] = "automatic-styles";
	static const char _LN_layer_set[] = "layer-set";
	static const char _LN_ftnsep[] = "ftnsep";
	static const char _LN_DataValidation[] = "DataValidation";
	static const char _LN_use_graphics[] = "use-graphics";
	static const char _LN_tblpPr[] = "tblpPr";
	static const char _LN_dfrauth[] = "dfrauth";
	static const char _LN_latentStyles[] = "latentStyles";
	static const char _LN_fit_to_size[] = "fit-to-size";
	static const char _LN_PLGroupType[] = "PLGroupType";
	static const char _LN_Panes[] = "Panes";
	static const char _LN_pnulwave[] = "pnulwave";
	static const char _LN_trftsWidthB[] = "trftsWidthB";
	static const char _LN_speed[] = "speed";
	static const char _LN_ftnsepc[] = "ftnsepc";
	static const char _LN_tblOverlap[] = "tblOverlap";
	static const char _LN_IsSnapTarget[] = "IsSnapTarget";
	static const char _LN_layer[] = "layer";
	static const char _LN_bottomBaseline[] = "bottomBaseline";
	static const char _LN_dfrdate[] = "dfrdate";
	static const char _LN_font_family_generic[] = "font-family-generic";
	static const char _LN_fills[] = "fills";
	static const char _LN_ProtectContents[] = "ProtectContents";
	static const char _LN_DefaultStandardRate[] = "DefaultStandardRate";
	static const char _LN_use_tables[] = "use-tables";
	static const char _LN_AutoFilter[] = "AutoFilter";
	static const char _LN_master_styles[] = "master-styles";
	static const char _LN_fit_to_contour[] = "fit-to-contour";
	static const char _LN_pnzodiac[] = "pnzodiac";
	static const char _LN_PLName[] = "PLName";
	static const char _LN_trftsWidth[] = "trftsWidth";
	static const char _LN_ftnstart[] = "ftnstart";
	static const char _LN_altName[] = "altName";
	static const char _LN_dfrmtxtx[] = "dfrmtxtx";
	static const char _LN_start_scale[] = "start-scale";
	static const char _LN_tblRtl[] = "tblRtl";
	static const char _LN_IsTextEditTarget[] = "IsTextEditTarget";
	static const char _LN_pPr[] = "pPr";
	static const char _LN_SourceHierarchy[] = "SourceHierarchy";
	static const char _LN_use_floating_frames[] = "use-floating-frames";
	static const char _LN_pnzodiacd[] = "pnzodiacd";
	static const char _LN_xfs[] = "xfs";
	static const char _LN_ProtectObjects[] = "ProtectObjects";
	static const char _LN_generator[] = "generator";
	static const char _LN_font_pitch[] = "font-pitch";
	static const char _LN_textarea_vertical_align[] = "textarea-vertical-align";
	static const char _LN_panose1[] = "panose1";
	static const char _LN_trgaph[] = "trgaph";
	static const char _LN_ftntj[] = "ftntj";
	static const char _LN_AutoFilterCondition[] = "AutoFilterCondition";
	static const char _LN_DefaultOvertimeRate[] = "DefaultOvertimeRate";
	static const char _LN_bidiVisual[] = "bidiVisual";
	static const char _LN_dfrmtxty[] = "dfrmtxty";
	static const char _LN_endParaRPr[] = "endParaRPr";
	static const char _LN_path_id[] = "path-id";
	static const char _LN_pnzodiacl[] = "pnzodiacl";
	static const char _LN_fttruetype[] = "fttruetype";
	static const char _LN_dxfs[] = "dxfs";
	static const char _LN_trhdr[] = "trhdr";
	static const char _LN_DontMoveChildren[] = "DontMoveChildren";
	static const char _LN_title[] = "title";
	static const char _LN_charset[] = "charset";
	static const char _LN_embed[] = "embed";
	static const char _LN_MemberPropertyParent[] = "MemberPropertyParent";
	static const char _LN_font_charset[] = "font-charset";
	static const char _LN_dfrstart[] = "dfrstart";
	static const char _LN_ProtectScenarios[] = "ProtectScenarios";
	static const char _LN_use_objects[] = "use-objects";
	static const char _LN_tblStyleRowBandSize[] = "tblStyleRowBandSize";
	static const char _LN_show_text[] = "show-text";
	static const char _LN_use_header_name[] = "use-header-name";
	static const char _LN_defPPr[] = "defPPr";
	static const char _LN_tableStyles[] = "tableStyles";
	static const char _LN_trkeep[] = "trkeep";
	static const char _LN_description[] = "description";
	static const char _LN_posnegx[] = "posnegx";
	static const char _LN_AutoFilterOr[] = "AutoFilterOr";
	static const char _LN_DurationFormat[] = "DurationFormat";
	static const char _LN_textarea_horizontal_align[] = "textarea-horizontal-align";
	static const char _LN_fvaliddos[] = "fvaliddos";
	static const char _LN_dfrstop[] = "dfrstop";
	static const char _LN_outline1pPr[] = "outline1pPr";
	static const char _LN_number_style[] = "number-style";
	static const char _LN_notTrueType[] = "notTrueType";
	static const char _LN_ShapePermeableX[] = "ShapePermeableX";
	static const char _LN_hide_shape[] = "hide-shape";
	static const char _LN_copy_outline_levels[] = "copy-outline-levels";
	static const char _LN_trleft[] = "trleft";
	static const char _LN_EnableSelection[] = "EnableSelection";
	static const char _LN_posnegy[] = "posnegy";
	static const char _LN_SourceHierarchyLevel[] = "SourceHierarchyLevel";
	static const char _LN_use_footer_name[] = "use-footer-name";
	static const char _LN_colorpalette[] = "colorpalette";
	static const char _LN_subject[] = "subject";
	static const char _LN_tblStyleColBandSize[] = "tblStyleColBandSize";
	static const char _LN_fvalidhpfs[] = "fvalidhpfs";
	static const char _LN_AutoFilterAnd[] = "AutoFilterAnd";
	static const char _LN_dfrxst[] = "dfrxst";
	static const char _LN_WorkFormat[] = "WorkFormat";
	static const char _LN_outline2pPr[] = "outline2pPr";
	static const char _LN_trowd[] = "trowd";
	static const char _LN_hide_text[] = "hide-text";
	static const char _LN_pitch[] = "pitch";
	static const char _LN_posx[] = "posx";
	static const char _LN_ShapePermeableY[] = "ShapePermeableY";
	static const char _LN_keyword[] = "keyword";
	static const char _LN_color_mode[] = "color-mode";
	static const char _LN_decimal_replacement[] = "decimal-replacement";
	static const char _LN_dghorigin[] = "dghorigin";
	static const char _LN_AllowFormatCells[] = "AllowFormatCells";
	static const char _LN_user_index_entry_template[] = "user-index-entry-template";
	static const char _LN_fvalidmac[] = "fvalidmac";
	static const char _LN_use_date_time_name[] = "use-date-time-name";
	static const char _LN_EditableActualCosts[] = "EditableActualCosts";
	static const char _LN_trpaddb[] = "trpaddb";
	static const char _LN_Sorting[] = "Sorting";
	static const char _LN_sig[] = "sig";
	static const char _LN_posxc[] = "posxc";
	static const char _LN_TotalAlignment[] = "TotalAlignment";
	static const char _LN_dim[] = "dim";
	static const char _LN_tblW[] = "tblW";
	static const char _LN_lvl1pPr[] = "lvl1pPr";
	static const char _LN_initial_creator[] = "initial-creator";
	static const char _LN_color_inversion[] = "color-inversion";
	static const char _LN_defaultFonts[] = "defaultFonts";
	static const char _LN_dghshow[] = "dghshow";
	static const char _LN_fvalidntfs[] = "fvalidntfs";
	static const char _LN_ShapePermeablePlace[] = "ShapePermeablePlace";
	static const char _LN_trpaddfb[] = "trpaddfb";
	static const char _LN_posxi[] = "posxi";
	static const char _LN_ConditionalFormatting[] = "ConditionalFormatting";
	static const char _LN_color[] = "color";
	static const char _LN_tblInd[] = "tblInd";
	static const char _LN_AllowSizeCols[] = "AllowSizeCols";
	static const char _LN_lvl2pPr[] = "lvl2pPr";
	static const char _LN_display_factor[] = "display-factor";
	static const char _LN_sldId[] = "sldId";
	static const char _LN_TotalCaptionAlignment[] = "TotalCaptionAlignment";
	static const char _LN_HonorConstraints[] = "HonorConstraints";
	static const char _LN_dghspace[] = "dghspace";
	static const char _LN_posxl[] = "posxl";
	static const char _LN_alphabetical_index[] = "alphabetical-index";
	static const char _LN_play[] = "play";
	static const char _LN_creator[] = "creator";
	static const char _LN_tblBorders[] = "tblBorders";
	static const char _LN_trpaddfl[] = "trpaddfl";
	static const char _LN_rect[] = "rect";
	static const char _LN_luminance[] = "luminance";
	static const char _LN_sldMasterId[] = "sldMasterId";
	static const char _LN_lvl3pPr[] = "lvl3pPr";
	static const char _LN_blockQuote[] = "blockQuote";
	static const char _LN_gcw[] = "gcw";
	static const char _LN_AllowSizeRows[] = "AllowSizeRows";
	static const char _LN_ShapeFixedCode[] = "ShapeFixedCode";
	static const char _LN_animation_group[] = "animation-group";
	static const char _LN_diag[] = "diag";
	static const char _LN_posxo[] = "posxo";
	static const char _LN_embedded_text[] = "embedded-text";
	static const char _LN_dgmargin[] = "dgmargin";
	static const char _LN_QueryTable[] = "QueryTable";
	static const char _LN_corner_radius[] = "corner-radius";
	static const char _LN_printed_by[] = "printed-by";
	static const char _LN_trpaddfr[] = "trpaddfr";
	static const char _LN_tblLayout[] = "tblLayout";
	static const char _LN_EarnedValueMethod[] = "EarnedValueMethod";
	static const char _LN_lvl4pPr[] = "lvl4pPr";
	static const char _LN_contrast[] = "contrast";
	static const char _LN_FormulaIndex[] = "FormulaIndex";
	static const char _LN_notesMasterId[] = "notesMasterId";
	static const char _LN_alphabetical_index_source[] = "alphabetical-index-source";
	static const char _LN_posxr[] = "posxr";
	static const char _LN_bodyDiv[] = "bodyDiv";
	static const char _LN_dgsnap[] = "dgsnap";
	static const char _LN_horizontal[] = "horizontal";
	static const char _LN_AllowInsertCols[] = "AllowInsertCols";
	static const char _LN_PivotTable[] = "PivotTable";
	static const char _LN_ShapePlowCode[] = "ShapePlowCode";
	static const char _LN_trpaddft[] = "trpaddft";
	static const char _LN_node_type[] = "node-type";
	static const char _LN_creation_date[] = "creation-date";
	static const char _LN_line[] = "line";
	static const char _LN_lvl5pPr[] = "lvl5pPr";
	static const char _LN_tblCellMar[] = "tblCellMar";
	static const char _LN_gridtbl[] = "gridtbl";
	static const char _LN_gamma[] = "gamma";
	static const char _LN_posy[] = "posy";
	static const char _LN_scientific_number[] = "scientific-number";
	static const char _LN_Hidden[] = "Hidden";
	static const char _LN_marLeft[] = "marLeft";
	static const char _LN_dgvorigin[] = "dgvorigin";
	static const char _LN_InsertedProjectsLikeSummary[] = "InsertedProjectsLikeSummary";
	static const char _LN_handoutMasterId[] = "handoutMasterId";
	static const char _LN_x1[] = "x1";
	static const char _LN_fontPr[] = "fontPr";
	static const char _LN_trpaddl[] = "trpaddl";
	static const char _LN_WorksheetSource[] = "WorksheetSource";
	static const char _LN_red[] = "red";
	static const char _LN_posyb[] = "posyb";
	static const char _LN_lvl6pPr[] = "lvl6pPr";
	static const char _LN_preset_id[] = "preset-id";
	static const char _LN_gutter[] = "gutter";
	static const char _LN_ShapeRouteStyle[] = "ShapeRouteStyle";
	static const char _LN_tblLook[] = "tblLook";
	static const char _LN_AllowInsertRows[] = "AllowInsertRows";
	static const char _LN_date[] = "date";
	static const char _LN_HideDetail[] = "HideDetail";
	static const char _LN_ignore_case[] = "ignore-case";
	static const char _LN_y1[] = "y1";
	static const char _LN_marRight[] = "marRight";
	static const char _LN_dgvshow[] = "dgvshow";
	static const char _LN_green[] = "green";
	static const char _LN_posyc[] = "posyc";
	static const char _LN_trpaddr[] = "trpaddr";
	static const char _LN_gutterprl[] = "gutterprl";
	static const char _LN_lvl7pPr[] = "lvl7pPr";
	static const char _LN_min_exponent_digits[] = "min-exponent-digits";
	static const char _LN_sldLst[] = "sldLst";
	static const char _LN_print_date[] = "print-date";
	static const char _LN_x2[] = "x2";
	static const char _LN_preset_sub_type[] = "preset-sub-type";
	static const char _LN_WorksheetOptions[] = "WorksheetOptions";
	static const char _LN_marTop[] = "marTop";
	static const char _LN_CachePosition[] = "CachePosition";
	static const char _LN_blue[] = "blue";
	static const char _LN_main_entry_style_name[] = "main-entry-style-name";
	static const char _LN_dgvspace[] = "dgvspace";
	static const char _LN_ConFixedCode[] = "ConFixedCode";
	static const char _LN_posyil[] = "posyil";
	static const char _LN_AllowInsertHyperlinks[] = "AllowInsertHyperlinks";
	static const char _LN_MultipleCriticalPaths[] = "MultipleCriticalPaths";
	static const char _LN_y2[] = "y2";
	static const char _LN_trpaddt[] = "trpaddt";
	static const char _LN_lvl8pPr[] = "lvl8pPr";
	static const char _LN_sldMasterIdLst[] = "sldMasterIdLst";
	static const char _LN_guttersxn[] = "guttersxn";
	static const char _LN_template[] = "template";
	static const char _LN_image_opacity[] = "image-opacity";
	static const char _LN_marBottom[] = "marBottom";
	static const char _LN_polyline[] = "polyline";
	static const char _LN_preset_class[] = "preset-class";
	static const char _LN_posyin[] = "posyin";
	static const char _LN_numFmt[] = "numFmt";
	static const char _LN_dibitmap[] = "dibitmap";
	static const char _LN_fraction[] = "fraction";
	static const char _LN_fgColor[] = "fgColor";
	static const char _LN_trpat[] = "trpat";
	static const char _LN_Member[] = "Member";
	static const char _LN_Watches[] = "Watches";
	static const char _LN_ConLineJumpCode[] = "ConLineJumpCode";
	static const char _LN_lvl9pPr[] = "lvl9pPr";
	static const char _LN_href[] = "href";
	static const char _LN_sldIdLst[] = "sldIdLst";
	static const char _LN_alphabetical_separators[] = "alphabetical-separators";
	static const char _LN_points[] = "points";
	static const char _LN_divBdr[] = "divBdr";
	static const char _LN_posyout[] = "posyout";
	static const char _LN_AllowDeleteCols[] = "AllowDeleteCols";
	static const char _LN_numStart[] = "numStart";
	static const char _LN_NewTasksEffortDriven[] = "NewTasksEffortDriven";
	static const char _LN_headerf[] = "headerf";
	static const char _LN_trqc[] = "trqc";
	static const char _LN_dn[] = "dn";
	static const char _LN_min_numerator_digits[] = "min-numerator-digits";
	static const char _LN_bgColor[] = "bgColor";
	static const char _LN_DrilledMember[] = "DrilledMember";
	static const char _LN_master_element[] = "master-element";
	static const char _LN_Watch[] = "Watch";
	static const char _LN_noAutofit[] = "noAutofit";
	static const char _LN_shadow_offset_x[] = "shadow-offset-x";
	static const char _LN_dntblnsbdb[] = "dntblnsbdb";
	static const char _LN_ConLineJumpStyle[] = "ConLineJumpStyle";
	static const char _LN_polygon[] = "polygon";
	static const char _LN_divsChild[] = "divsChild";
	static const char _LN_notesMasterIdLst[] = "notesMasterIdLst";
	static const char _LN_trql[] = "trql";
	static const char _LN_posyt[] = "posyt";
	static const char _LN_actuate[] = "actuate";
	static const char _LN_numRestart[] = "numRestart";
	static const char _LN_headerl[] = "headerl";
	static const char _LN_Source[] = "Source";
	static const char _LN_AllowDeleteRows[] = "AllowDeleteRows";
	static const char _LN_normAutofit[] = "normAutofit";
	static const char _LN_DataValueEditing[] = "DataValueEditing";
	static const char _LN_group_id[] = "group-id";
	static const char _LN_trqr[] = "trqr";
	static const char _LN_prcolbl[] = "prcolbl";
	static const char _LN_NewTasksEstimated[] = "NewTasksEstimated";
	static const char _LN_combine_entries[] = "combine-entries";
	static const char _LN_regular_polygon[] = "regular-polygon";
	static const char _LN_min_denominator_digits[] = "min-denominator-digits";
	static const char _LN_div[] = "div";
	static const char _LN_do[] = "do";
	static const char _LN_headery[] = "headery";
	static const char _LN_shadow_offset_y[] = "shadow-offset-y";
	static const char _LN_PageBreaks[] = "PageBreaks";
	static const char _LN_xf[] = "xf";
	static const char _LN_ConLineJumpDirX[] = "ConLineJumpDirX";
	static const char _LN_handoutMasterIdLst[] = "handoutMasterIdLst";
	static const char _LN_trrh[] = "trrh";
	static const char _LN_dobxcolumn[] = "dobxcolumn";
	static const char _LN_shapeAutofit[] = "shapeAutofit";
	static const char _LN_txbxContent[] = "txbxContent";
	static const char _LN_hash[] = "hash";
	static const char _LN_event_listener[] = "event-listener";
	static const char _LN_printdata[] = "printdata";
	static const char _LN_alignment[] = "alignment";
	static const char _LN_AllowSort[] = "AllowSort";
	static const char _LN_auto_reload[] = "auto-reload";
	static const char _LN_hich[] = "hich";
	static const char _LN_QTSource[] = "QTSource";
	static const char _LN_ColBreaks[] = "ColBreaks";
	static const char _LN_combine_entries_with_dash[] = "combine-entries-with-dash";
	static const char _LN_concave[] = "concave";
	static const char _LN_trshdng[] = "trshdng";
	static const char _LN_SplitsInProgressTasks[] = "SplitsInProgressTasks";
	static const char _LN_shadow_color[] = "shadow-color";
	static const char _LN_dobxmargin[] = "dobxmargin";
	static const char _LN_ConLineJumpDirY[] = "ConLineJumpDirY";
	static const char _LN_denominator_value[] = "denominator-value";
	static const char _LN_printim[] = "printim";
	static const char _LN_cfChunk[] = "cfChunk";
	static const char _LN_protection[] = "protection";
	static const char _LN_textWarp[] = "textWarp";
	static const char _LN_uniqueTag[] = "uniqueTag";
	static const char _LN_AllowFilter[] = "AllowFilter";
	static const char _LN_slideSize[] = "slideSize";
	static const char _LN_ErrorString[] = "ErrorString";
	static const char _LN_event_name[] = "event-name";
	static const char _LN_show[] = "show";
	static const char _LN_corners[] = "corners";
	static const char _LN_trspdb[] = "trspdb";
	static const char _LN_hlfr[] = "hlfr";
	static const char _LN_ColBreak[] = "ColBreak";
	static const char _LN_shadow_opacity[] = "shadow-opacity";
	static const char _LN_private[] = "private";
	static const char _LN_proofErr[] = "proofErr";
	static const char _LN_delay[] = "delay";
	static const char _LN_dobxpage[] = "dobxpage";
	static const char _LN_prot[] = "prot";
	static const char _LN_styleInfo[] = "styleInfo";
	static const char _LN_mappedName[] = "mappedName";
	static const char _LN_hlinkbase[] = "hlinkbase";
	static const char _LN_notesSize[] = "notesSize";
	static const char _LN_trspdfb[] = "trspdfb";
	static const char _LN_AllowUsePivotTables[] = "AllowUsePivotTables";
	static const char _LN_SpreadActualCost[] = "SpreadActualCost";
	static const char _LN_sharpness[] = "sharpness";
	static const char _LN_ShapePlaceFlip[] = "ShapePlaceFlip";
	static const char _LN_action[] = "action";
	static const char _LN_NullString[] = "NullString";
	static const char _LN_combine_entries_with_pp[] = "combine-entries-with-pp";
	static const char _LN_Column[] = "Column";
	static const char _LN_currency_style[] = "currency-style";
	static const char _LN_bodyPr[] = "bodyPr";
	static const char _LN_hyperlink_behaviour[] = "hyperlink-behaviour";
	static const char _LN_propname[] = "propname";
	static const char _LN_permStart[] = "permStart";
	static const char _LN_dobymargin[] = "dobymargin";
	static const char _LN_trspdfl[] = "trspdfl";
	static const char _LN_hlloc[] = "hlloc";
	static const char _LN_verb[] = "verb";
	static const char _LN_start_line_spacing_horizontal[] = "start-line-spacing-horizontal";
	static const char _LN_custShowLst[] = "custShowLst";
	static const char _LN_dynamicAddress[] = "dynamicAddress";
	static const char _LN_RowBreaks[] = "RowBreaks";
	static const char _LN_path[] = "path";
	static const char _LN_PageFieldOrder[] = "PageFieldOrder";
	static const char _LN_dxf[] = "dxf";
	static const char _LN_ConLineRouteExt[] = "ConLineRouteExt";
	static const char _LN_lstStyle[] = "lstStyle";
	static const char _LN_SpreadPercentComplete[] = "SpreadPercentComplete";
	static const char _LN_currency_symbol[] = "currency-symbol";
	static const char _LN_proptype[] = "proptype";
	static const char _LN_SpaceAbove[] = "SpaceAbove";
	static const char _LN_header[] = "header";
	static const char _LN_permEnd[] = "permEnd";
	static const char _LN_hlsrc[] = "hlsrc";
	static const char _LN_d[] = "d";
	static const char _LN_trspdfr[] = "trspdfr";
	static const char _LN_dobypage[] = "dobypage";
	static const char _LN_use_keys_as_entries[] = "use-keys-as-entries";
	static const char _LN_RowBreak[] = "RowBreak";
	static const char _LN_photoAlbum[] = "photoAlbum";
	static const char _LN_target_frame_name[] = "target-frame-name";
	static const char _LN_udl[] = "udl";
	static const char _LN_circle[] = "circle";
	static const char _LN_indexedColors[] = "indexedColors";
	static const char _LN_horzdoc[] = "horzdoc";
	static const char _LN_PageFieldWrapCount[] = "PageFieldWrapCount";
	static const char _LN_footer[] = "footer";
	static const char _LN_comments[] = "comments";
	static const char _LN_SpaceBelow[] = "SpaceBelow";
	static const char _LN_trspdft[] = "trspdft";
	static const char _LN_ShapeSplit[] = "ShapeSplit";
	static const char _LN_Row[] = "Row";
	static const char _LN_start_line_spacing_vertical[] = "start-line-spacing-vertical";
	static const char _LN_TaskUpdatesResource[] = "TaskUpdatesResource";
	static const char _LN_commentAuthorLst[] = "commentAuthorLst";
	static const char _LN_cx[] = "cx";
	static const char _LN_date_time[] = "date-time";
	static const char _LN_src[] = "src";
	static const char _LN_horzsect[] = "horzsect";
	static const char _LN_footnotes[] = "footnotes";
	static const char _LN_capitalize_entries[] = "capitalize-entries";
	static const char _LN_trspdl[] = "trspdl";
	static const char _LN_ColFirst[] = "ColFirst";
	static const char _LN_themeColors[] = "themeColors";
	static const char _LN_ZeroHeight[] = "ZeroHeight";
	static const char _LN_percentage_style[] = "percentage-style";
	static const char _LN_ShapeSplittable[] = "ShapeSplittable";
	static const char _LN_cy[] = "cy";
	static const char _LN_DisableDrillDown[] = "DisableDrillDown";
	static const char _LN_editing_cycles[] = "editing-cycles";
	static const char _LN_hr[] = "hr";
	static const char _LN_header_decl[] = "header-decl";
	static const char _LN_r[] = "r";
	static const char _LN_trspdr[] = "trspdr";
	static const char _LN_endnotes[] = "endnotes";
	static const char _LN_custDataLst[] = "custDataLst";
	static const char _LN_ColLast[] = "ColLast";
	static const char _LN_FiscalYearStart[] = "FiscalYearStart";
	static const char _LN_mruColors[] = "mruColors";
	static const char _LN_fieldMapData[] = "fieldMapData";
	static const char _LN_htmautsp[] = "htmautsp";
	static const char _LN_comma_separated[] = "comma-separated";
	static const char _LN_end_line_spacing_horizontal[] = "end-line-spacing-horizontal";
	static const char _LN_ResizePage[] = "ResizePage";
	static const char _LN_date_style[] = "date-style";
	static const char _LN_Header[] = "Header";
	static const char _LN_trspdt[] = "trspdt";
	static const char _LN_editing_duration[] = "editing-duration";
	static const char _LN_DisableFieldDialog[] = "DisableFieldDialog";
	static const char _LN_start_angle[] = "start-angle";
	static const char _LN_category[] = "category";
	static const char _LN_creatingAppVersion[] = "creatingAppVersion";
	static const char _LN_rgbcolor[] = "rgbcolor";
	static const char _LN_htmlbase[] = "htmlbase";
	static const char _LN_footer_decl[] = "footer-decl";
	static const char _LN_WeekStartDay[] = "WeekStartDay";
	static const char _LN_Footer[] = "Footer";
	static const char _LN_colDelim[] = "colDelim";
	static const char _LN_truncatefontheight[] = "truncatefontheight";
	static const char _LN_buColorTx[] = "buColorTx";
	static const char _LN_EnableGrid[] = "EnableGrid";
	static const char _LN_day[] = "day";
	static const char _LN_Boolean[] = "Boolean";
	static const char _LN_types[] = "types";
	static const char _LN_end_angle[] = "end-angle";
	static const char _LN_document_statistic[] = "document-statistic";
	static const char _LN_country[] = "country";
	static const char _LN_htmlrtf[] = "htmlrtf";
	static const char _LN_PageMargins[] = "PageMargins";
	static const char _LN_DisableWizard[] = "DisableWizard";
	static const char _LN_jdsoType[] = "jdsoType";
	static const char _LN_date_time_decl[] = "date-time-decl";
	static const char _LN_end_line_spacing_vertical[] = "end-line-spacing-vertical";
	static const char _LN_editingAppVersion[] = "editingAppVersion";
	static const char _LN_buColor[] = "buColor";
	static const char _LN_MoveCompletedEndsBack[] = "MoveCompletedEndsBack";
	static const char _LN_tableStyleElement[] = "tableStyleElement";
	static const char _LN_DynamicsOff[] = "DynamicsOff";
	static const char _LN_Error[] = "Error";
	static const char _LN_month[] = "month";
	static const char _LN_ellipse[] = "ellipse";
	static const char _LN_trwWidthA[] = "trwWidthA";
	static const char _LN_guid[] = "guid";
	static const char _LN_sort_algorithm[] = "sort-algorithm";
	static const char _LN_htmltag[] = "htmltag";
	static const char _LN_fHdr[] = "fHdr";
	static const char _LN_Count[] = "Count";
	static const char _LN_buSzTx[] = "buSzTx";
	static const char _LN_Horizontal[] = "Horizontal";
	static const char _LN_textual[] = "textual";
	static const char _LN_DisplayErrorString[] = "DisplayErrorString";
	static const char _LN_page_count[] = "page-count";
	static const char _LN_source[] = "source";
	static const char _LN_CtrlAsInput[] = "CtrlAsInput";
	static const char _LN_docPartPr[] = "docPartPr";
	static const char _LN_rx[] = "rx";
	static const char _LN_trwWidthB[] = "trwWidthB";
	static const char _LN_hyphauto[] = "hyphauto";
	static const char _LN_recipientData[] = "recipientData";
	static const char _LN_SheetIndex[] = "SheetIndex";
	static const char _LN_MoveRemainingStartsBack[] = "MoveRemainingStartsBack";
	static const char _LN_buSzPct[] = "buSzPct";
	static const char _LN_line_distance[] = "line-distance";
	static const char _LN_alphabetical_index_auto_mark_file[] = "alphabetical-index-auto-mark-file";
	static const char _LN_possessive_form[] = "possessive-form";
	static const char _LN_ry[] = "ry";
	static const char _LN_Rotation[] = "Rotation";
	static const char _LN_table_count[] = "table-count";
	static const char _LN_docPartBody[] = "docPartBody";
	static const char _LN_PlaceStyle[] = "PlaceStyle";
	static const char _LN_hyphcaps[] = "hyphcaps";
	static const char _LN_start_page[] = "start-page";
	static const char _LN_trwWidth[] = "trwWidth";
	static const char _LN_NoPreserveFormatting[] = "NoPreserveFormatting";
	static const char _LN_connector[] = "connector";
	static const char _LN_buSzPts[] = "buSzPts";
	static const char _LN_Crn[] = "Crn";
	static const char _LN_mainDocumentType[] = "mainDocumentType";
	static const char _LN_Vertical[] = "Vertical";
	static const char _LN_guide_overhang[] = "guide-overhang";
	static const char _LN_year[] = "year";
	static const char _LN_draw_count[] = "draw-count";
	static const char _LN_hyphconsec[] = "hyphconsec";
	static const char _LN_Moper[] = "Moper";
	static const char _LN_RouteStyle[] = "RouteStyle";
	static const char _LN_ts[] = "ts";
	static const char _LN_MoveRemainingStartsForward[] = "MoveRemainingStartsForward";
	static const char _LN_buFontTx[] = "buFontTx";
	static const char _LN_docParts[] = "docParts";
	static const char _LN_ReadingOrder[] = "ReadingOrder";
	static const char _LN_era[] = "era";
	static const char _LN_start_shape[] = "start-shape";
	static const char _LN_full_screen[] = "full-screen";
	static const char _LN_tsbgbdiag[] = "tsbgbdiag";
	static const char _LN_HasNoAutoFormat[] = "HasNoAutoFormat";
	static const char _LN_ExcelName[] = "ExcelName";
	static const char _LN_linkToQuery[] = "linkToQuery";
	static const char _LN_alphabetical_index_entry_template[] = "alphabetical-index-entry-template";
	static const char _LN_image_count[] = "image-count";
	static const char _LN_guide_distance[] = "guide-distance";
	static const char _LN_hyphhotz[] = "hyphhotz";
	static const char _LN_PlaceDepth[] = "PlaceDepth";
	static const char _LN_buFont[] = "buFont";
	static const char _LN_day_of_week[] = "day-of-week";
	static const char _LN_ignoreSubtree[] = "ignoreSubtree";
	static const char _LN_tsbgcross[] = "tsbgcross";
	static const char _LN_Path[] = "Path";
	static const char _LN_start_glue_point[] = "start-glue-point";
	static const char _LN_endless[] = "endless";
	static const char _LN_pubWhat[] = "pubWhat";
	static const char _LN_FunctionGroup[] = "FunctionGroup";
	static const char _LN_dataType[] = "dataType";
	static const char _LN_buBlip[] = "buBlip";
	static const char _LN_hyphpar[] = "hyphpar";
	static const char _LN_ole_object_count[] = "ole-object-count";
	static const char _LN_SubtotalHiddenPageItems[] = "SubtotalHiddenPageItems";
	static const char _LN_MoveCompletedEndsForward[] = "MoveCompletedEndsForward";
	static const char _LN_PlowCode[] = "PlowCode";
	static const char _LN_start_guide[] = "start-guide";
	static const char _LN_week_of_year[] = "week-of-year";
	static const char _LN_Left[] = "Left";
	static const char _LN_ignoreElements[] = "ignoreElements";
	static const char _LN_pause[] = "pause";
	static const char _LN_pubAs[] = "pubAs";
	static const char _LN_tsbgdcross[] = "tsbgdcross";
	static const char _LN_noBullet[] = "noBullet";
	static const char _LN_defaultSQL[] = "defaultSQL";
	static const char _LN_FunctionGroupIndex[] = "FunctionGroupIndex";
	static const char _LN_LineJumpCode[] = "LineJumpCode";
	static const char _LN_end_shape[] = "end-shape";
	static const char _LN_Cell[] = "Cell";
	static const char _LN_bibliography[] = "bibliography";
	static const char _LN_show_logo[] = "show-logo";
	static const char _LN_end_guide[] = "end-guide";
	static const char _LN_present[] = "present";
	static const char _LN_paragraph_count[] = "paragraph-count";
	static const char _LN_quarter[] = "quarter";
	static const char _LN_autonumberBullet[] = "autonumberBullet";
	static const char _LN_tsbgdkbdiag[] = "tsbgdkbdiag";
	static const char _LN_NoAutoFormatWidth[] = "NoAutoFormatWidth";
	static const char _LN_Top[] = "Top";
	static const char _LN_BaselineForEarnedValue[] = "BaselineForEarnedValue";
	static const char _LN_connectString[] = "connectString";
	static const char _LN_end_glue_point[] = "end-glue-point";
	static const char _LN_impr[] = "impr";
	static const char _LN_LineJumpStyle[] = "LineJumpStyle";
	static const char _LN_browse[] = "browse";
	static const char _LN_bibliography_source[] = "bibliography-source";
	static const char _LN_wordDocument[] = "wordDocument";
	static const char _LN_force_manual[] = "force-manual";
	static const char _LN_time_style[] = "time-style";
	static const char _LN_placing[] = "placing";
	static const char _LN_KeyboardShortcut[] = "KeyboardShortcut";
	static const char _LN_NotInverted[] = "NotInverted";
	static const char _LN_word_count[] = "word-count";
	static const char _LN_tsbgdkcross[] = "tsbgdkcross";
	static const char _LN_charBullet[] = "charBullet";
	static const char _LN_kiosk[] = "kiosk";
	static const char _LN_query[] = "query";
	static const char _LN_parallel[] = "parallel";
	static const char _LN_NoRowGrand[] = "NoRowGrand";
	static const char _LN_line_skew[] = "line-skew";
	static const char _LN_insrsid[] = "insrsid";
	static const char _LN_truncate_on_overflow[] = "truncate-on-overflow";
	static const char _LN_PageLineJumpDirX[] = "PageLineJumpDirX";
	static const char _LN_wordTemplate[] = "wordTemplate";
	static const char _LN_mouse_visible[] = "mouse-visible";
	static const char _LN_Id[] = "Id";
	static const char _LN_AutoAddNewResourcesAndTasks[] = "AutoAddNewResourcesAndTasks";
	static const char _LN_character_count[] = "character-count";
	static const char _LN_bibliography_entry_template[] = "bibliography-entry-template";
	static const char _LN_Macro[] = "Macro";
	static const char _LN_showType[] = "showType";
	static const char _LN_dataSource[] = "dataSource";
	static const char _LN_tsbgdkdcross[] = "tsbgdkdcross";
	static const char _LN_blipBullet[] = "blipBullet";
	static const char _LN_measure_align[] = "measure-align";
	static const char _LN_Clipped[] = "Clipped";
	static const char _LN_intbl[] = "intbl";
	static const char _LN_NoColumnGrand[] = "NoColumnGrand";
	static const char _LN_caption[] = "caption";
	static const char _LN_glossaryDocument[] = "glossaryDocument";
	static const char _LN_phonetictext[] = "phonetictext";
	static const char _LN_mouse_as_pen[] = "mouse-as-pen";
	static const char _LN_PageLineJumpDirY[] = "PageLineJumpDirY";
	static const char _LN_showWhat[] = "showWhat";
	static const char _LN_headerSource[] = "headerSource";
	static const char _LN_hours[] = "hours";
	static const char _LN_ipgp[] = "ipgp";
	static const char _LN_supBook[] = "supBook";
	static const char _LN_tsbgdkfdiag[] = "tsbgdkfdiag";
	static const char _LN_Descending[] = "Descending";
	static const char _LN_frame_count[] = "frame-count";
	static const char _LN_caption_point_x[] = "caption-point-x";
	static const char _LN_measure_vertical_align[] = "measure-vertical-align";
	static const char _LN_NoDisplayNullString[] = "NoDisplayNullString";
	static const char _LN_StatusDate[] = "StatusDate";
	static const char _LN_irow[] = "irow";
	static const char _LN_minutes[] = "minutes";
	static const char _LN_penClr[] = "penClr";
	static const char _LN_AutoFilterColumn[] = "AutoFilterColumn";
	static const char _LN_index_source_styles[] = "index-source-styles";
	static const char _LN_start_with_navigator[] = "start-with-navigator";
	static const char _LN_sheetNames[] = "sheetNames";
	static const char _LN_doNotSuppressBlankLines[] = "doNotSuppressBlankLines";
	static const char _LN_FillDown[] = "FillDown";
	static const char _LN_tsbgdkhor[] = "tsbgdkhor";
	static const char _LN_LineToNodeX[] = "LineToNodeX";
	static const char _LN_sentence_count[] = "sentence-count";
	static const char _LN_irowband[] = "irowband";
	static const char _LN_caption_point_y[] = "caption-point-y";
	static const char _LN_htmlPubPr[] = "htmlPubPr";
	static const char _LN_CurrentDate[] = "CurrentDate";
	static const char _LN_seconds[] = "seconds";
	static const char _LN_namedLinks[] = "namedLinks";
	static const char _LN_NoPrintRepeatItems[] = "NoPrintRepeatItems";
	static const char _LN_RowNumbers[] = "RowNumbers";
	static const char _LN_unit[] = "unit";
	static const char _LN_tsbgdkvert[] = "tsbgdkvert";
	static const char _LN_WorkbookOptions[] = "WorkbookOptions";
	static const char _LN_LineToNodeY[] = "LineToNodeY";
	static const char _LN_index_source_style[] = "index-source-style";
	static const char _LN_itap[] = "itap";
	static const char _LN_syllable_count[] = "syllable-count";
	static const char _LN_transition_on_click[] = "transition-on-click";
	static const char _LN_decimal_places[] = "decimal-places";
	static const char _LN_webPr[] = "webPr";
	static const char _LN_MicrosoftProjectServerURL[] = "MicrosoftProjectServerURL";
	static const char _LN_show_unit[] = "show-unit";
	static const char _LN_destination[] = "destination";
	static const char _LN_supSheet[] = "supSheet";
	static const char _LN_ixe[] = "ixe";
	static const char _LN_tsbgfdiag[] = "tsbgfdiag";
	static const char _LN_control[] = "control";
	static const char _LN_prnPr[] = "prnPr";
	static const char _LN_BlockSizeX[] = "BlockSizeX";
	static const char _LN_MoveAfterReturn[] = "MoveAfterReturn";
	static const char _LN_PivotView[] = "PivotView";
	static const char _LN_non_whitespace_character_count[] = "non-whitespace-character-count";
	static const char _LN_jcompress[] = "jcompress";
	static const char _LN_am_pm[] = "am-pm";
	static const char _LN_index_title_template[] = "index-title-template";
	static const char _LN_outline[] = "outline";
	static const char _LN_sect[] = "sect";
	static const char _LN_addressFieldName[] = "addressFieldName";
	static const char _LN_Condition[] = "Condition";
	static const char _LN_stay_on_top[] = "stay-on-top";
	static const char _LN_showPr[] = "showPr";
	static const char _LN_page_thumbnail[] = "page-thumbnail";
	static const char _LN_tsbghoriz[] = "tsbghoriz";
	static const char _LN_caption_type[] = "caption-type";
	static const char _LN_xrow[] = "xrow";
	static const char _LN_BlockSizeY[] = "BlockSizeY";
	static const char _LN_AutoFormatAlignment[] = "AutoFormatAlignment";
	static const char _LN_boolean_style[] = "boolean-style";
	static const char _LN_Autolink[] = "Autolink";
	static const char _LN_sub_section[] = "sub-section";
	static const char _LN_jexpand[] = "jexpand";
	static const char _LN_engrave[] = "engrave";
	static const char _LN_presentationPr[] = "presentationPr";
	static const char _LN_Qualifier[] = "Qualifier";
	static const char _LN_pages[] = "pages";
	static const char _LN_xc[] = "xc";
	static const char _LN_tsbgvert[] = "tsbgvert";
	static const char _LN_mailSubject[] = "mailSubject";
	static const char _LN_caption_angle_type[] = "caption-angle-type";
	static const char _LN_index_entry_chapter[] = "index-entry-chapter";
	static const char _LN_AvenueSizeX[] = "AvenueSizeX";
	static const char _LN_NewTaskStartDate[] = "NewTaskStartDate";
	static const char _LN_jpegblip[] = "jpegblip";
	static const char _LN_emboss[] = "emboss";
	static const char _LN_xv[] = "xv";
	static const char _LN_boolean[] = "boolean";
	static const char _LN_allowEmptyCollapse[] = "allowEmptyCollapse";
	static const char _LN_row_count[] = "row-count";
	static const char _LN_g[] = "g";
	static const char _LN_Value1[] = "Value1";
	static const char _LN_AutoFormatBorder[] = "AutoFormatBorder";
	static const char _LN_tsbrdrb[] = "tsbrdrb";
	static const char _LN_namedRange[] = "namedRange";
	static const char _LN_y[] = "y";
	static const char _LN_mailAsAttachment[] = "mailAsAttachment";
	static const char _LN_highlight[] = "highlight";
	static const char _LN_jsksu[] = "jsksu";
	static const char _LN_AvenueSizeY[] = "AvenueSizeY";
	static const char _LN_text_style[] = "text-style";
	static const char _LN_column_mapping[] = "column-mapping";
	static const char _LN_Value2[] = "Value2";
	static const char _LN_cell_count[] = "cell-count";
	static const char _LN_caption_angle[] = "caption-angle";
	static const char _LN_DefaultTaskEVMethod[] = "DefaultTaskEVMethod";
	static const char _LN_index_entry_text[] = "index-entry-text";
	static const char _LN_x[] = "x";
	static const char _LN_tsbrdrdgl[] = "tsbrdrdgl";
	static const char _LN_keep[] = "keep";
	static const char _LN_DDELink[] = "DDELink";
	static const char _LN_AutoFormatFont[] = "AutoFormatFont";
	static const char _LN_formatting[] = "formatting";
	static const char _LN_width[] = "width";
	static const char _LN_LineToLineX[] = "LineToLineX";
	static const char _LN_MapInfo[] = "MapInfo";
	static const char _LN_viewMergedData[] = "viewMergedData";
	static const char _LN_object_count[] = "object-count";
	static const char _LN_tsbrdrdgr[] = "tsbrdrdgr";
	static const char _LN_row_mapping[] = "row-mapping";
	static const char _LN_keepn[] = "keepn";
	static const char _LN_FileName[] = "FileName";
	static const char _LN_caption_gap[] = "caption-gap";
	static const char _LN_text_content[] = "text-content";
	static const char _LN_OLELink[] = "OLELink";
	static const char _LN_height[] = "height";
	static const char _LN_index_entry_page_number[] = "index-entry-page-number";
	static const char _LN_ProjectExternallyEdited[] = "ProjectExternallyEdited";
	static const char _LN_u[] = "u";
	static const char _LN_Binding[] = "Binding";
	static const char _LN_AutoFormatName[] = "AutoFormatName";
	static const char _LN_LineToLineY[] = "LineToLineY";
	static const char _LN_kerning[] = "kerning";
	static const char _LN_tsbrdrh[] = "tsbrdrh";
	static const char _LN_Reference[] = "Reference";
	static const char _LN_user_defined[] = "user-defined";
	static const char _LN_ascii[] = "ascii";
	static const char _LN_activeRecord[] = "activeRecord";
	static const char _LN_transform[] = "transform";
	static const char _LN_caption_escape_direction[] = "caption-escape-direction";
	static const char _LN_str[] = "str";
	static const char _LN_MapID[] = "MapID";
	static const char _LN_keycode[] = "keycode";
	static const char _LN_volatile[] = "volatile";
	static const char _LN_ea[] = "ea";
	static const char _LN_tsbrdrl[] = "tsbrdrl";
	static const char _LN_LineJumpFactorX[] = "LineJumpFactorX";
	static const char _LN_AutoFormatNumber[] = "AutoFormatNumber";
	static const char _LN_cell_range[] = "cell-range";
	static const char _LN_bin[] = "bin";
	static const char _LN_index_entry_span[] = "index-entry-span";
	static const char _LN_viewBox[] = "viewBox";
	static const char _LN_ExtendedCreationDate[] = "ExtendedCreationDate";
	static const char _LN_checkErrors[] = "checkErrors";
	static const char _LN_Schema[] = "Schema";
	static const char _LN_CellRangeList[] = "CellRangeList";
	static const char _LN_cs[] = "cs";
	static const char _LN_value_type[] = "value-type";
	static const char _LN_automatic_order[] = "automatic-order";
	static const char _LN_tag[] = "tag";
	static const char _LN_tsbrdrr[] = "tsbrdrr";
	static const char _LN_caption_escape[] = "caption-escape";
	static const char _LN_ksulang[] = "ksulang";
	static const char _LN_hAnsi[] = "hAnsi";
	static const char _LN_subtitle[] = "subtitle";
	static const char _LN_Map[] = "Map";
	static const char _LN_LineJumpFactorY[] = "LineJumpFactorY";
	static const char _LN_odso[] = "odso";
	static const char _LN_AutoFormatPattern[] = "AutoFormatPattern";
	static const char _LN_index_entry_bibliography[] = "index-entry-bibliography";
	static const char _LN_tsbrdrt[] = "tsbrdrt";
	static const char _LN_UseBlank[] = "UseBlank";
	static const char _LN_h[] = "h";
	static const char _LN_ShowImportExportValidationErrors[] = "ShowImportExportValidationErrors";
	static const char _LN_ActualsInSync[] = "ActualsInSync";
	static const char _LN_sym[] = "sym";
	static const char _LN_landscape[] = "landscape";
	static const char _LN_format_source[] = "format-source";
	static const char _LN_origWordTableRules[] = "origWordTableRules";
	static const char _LN_vertFromText[] = "vertFromText";
	static const char _LN_outline_level[] = "outline-level";
	static const char _LN_caption_line_length[] = "caption-line-length";
	static const char _LN_text_style_name[] = "text-style-name";
	static const char _LN_legend[] = "legend";
	static const char _LN_link[] = "link";
	static const char _LN_tsbrdrv[] = "tsbrdrv";
	static const char _LN_LineAdjustFrom[] = "LineAdjustFrom";
	static const char _LN_Min[] = "Min";
	static const char _LN_GrandTotalString[] = "GrandTotalString";
	static const char _LN_RemoveFileProperties[] = "RemoveFileProperties";
	static const char _LN_transliteration_format[] = "transliteration-format";
	static const char _LN_Max[] = "Max";
	static const char _LN_legend_position[] = "legend-position";
	static const char _LN_horizFromText[] = "horizFromText";
	static const char _LN_bibliography_data_field[] = "bibliography-data-field";
	static const char _LN_langfe[] = "langfe";
	static const char _LN_tscbandhorzeven[] = "tscbandhorzeven";
	static const char _LN_restart_numbering[] = "restart-numbering";
	static const char _LN_pivotRefs[] = "pivotRefs";
	static const char _LN_wpJustification[] = "wpJustification";
	static const char _LN_z_index[] = "z-index";
	static const char _LN_LineAdjustTo[] = "LineAdjustTo";
	static const char _LN_caption_fit_line_length[] = "caption-fit-line-length";
	static const char _LN_Append[] = "Append";
	static const char _LN_langfenp[] = "langfenp";
	static const char _LN_PrintSetTitles[] = "PrintSetTitles";
	static const char _LN_ComboHide[] = "ComboHide";
	static const char _LN_borders[] = "borders";
	static const char _LN_pivotRef[] = "pivotRef";
	static const char _LN_end_cell_address[] = "end-cell-address";
	static const char _LN_legend_align[] = "legend-align";
	static const char _LN_ph[] = "ph";
	static const char _LN_AdminProject[] = "AdminProject";
	static const char _LN_tscbandhorzodd[] = "tscbandhorzodd";
	static const char _LN_Entry[] = "Entry";
	static const char _LN_noTabHangInd[] = "noTabHangInd";
	static const char _LN_start_value[] = "start-value";
	static const char _LN_transliteration_language[] = "transliteration-language";
	static const char _LN_PlaceFlip[] = "PlaceFlip";
	static const char _LN_index_entry_tab_stop[] = "index-entry-tab-stop";
	static const char _LN_langnp[] = "langnp";
	static const char _LN_media[] = "media";
	static const char _LN_IMEMode[] = "IMEMode";
	static const char _LN_MergeLabels[] = "MergeLabels";
	static const char _LN_horizontal_segments[] = "horizontal-segments";
	static const char _LN_Mapdata[] = "Mapdata";
	static const char _LN_legend_expansion[] = "legend-expansion";
	static const char _LN_OutlineCodes[] = "OutlineCodes";
	static const char _LN_pivotRefInternal[] = "pivotRefInternal";
	static const char _LN_end_x[] = "end-x";
	static const char _LN_lastrow[] = "lastrow";
	static const char _LN_LineRouteExt[] = "LineRouteExt";
	static const char _LN_actions[] = "actions";
	static const char _LN_tscbandsh[] = "tscbandsh";
	static const char _LN_is_list_header[] = "is-list-header";
	static const char _LN_noLeading[] = "noLeading";
	static const char _LN_InputHide[] = "InputHide";
	static const char _LN_apo[] = "apo";
	static const char _LN_acc[] = "acc";
	static const char _LN_PageFieldStyle[] = "PageFieldStyle";
	static const char _LN_leader_char[] = "leader-char";
	static const char _LN_end_y[] = "end-y";
	static const char _LN_transliteration_country[] = "transliteration-country";
	static const char _LN_lbr[] = "lbr";
	static const char _LN_HeaderRange[] = "HeaderRange";
	static const char _LN_OutlineCode[] = "OutlineCode";
	static const char _LN_tscbandsv[] = "tscbandsv";
	static const char _LN_spaceForUL[] = "spaceForUL";
	static const char _LN_PageShapeSplit[] = "PageShapeSplit";
	static const char _LN_legend_expansion_aspect_ratio[] = "legend-expansion-aspect-ratio";
	static const char _LN_vertical_segments[] = "vertical-segments";
	static const char _LN_bar[] = "bar";
	static const char _LN_InputTitle[] = "InputTitle";
	static const char _LN_flags[] = "flags";
	static const char _LN_number[] = "number";
	static const char _LN_pBdrGroup[] = "pBdrGroup";
	static const char _LN_table_background[] = "table-background";
	static const char _LN_lchars[] = "lchars";
	static const char _LN_pnvPr[] = "pnvPr";
	static const char _LN_PLExport[] = "PLExport";
	static const char _LN_box[] = "box";
	static const char _LN_tscbandverteven[] = "tscbandverteven";
	static const char _LN_slicers[] = "slicers";
	static const char _LN_XPath[] = "XPath";
	static const char _LN_FieldID[] = "FieldID";
	static const char _LN_noColumnBalance[] = "noColumnBalance";
	static const char _LN_p[] = "p";
	static const char _LN_position[] = "position";
	static const char _LN_InputMessage[] = "InputMessage";
	static const char _LN_ldblquote[] = "ldblquote";
	static const char _LN_PageLeftMargin[] = "PageLeftMargin";
	static const char _LN_transliteration_style[] = "transliteration-style";
	static const char _LN_edge_rounding[] = "edge-rounding";
	static const char _LN_style_name[] = "style-name";
	static const char _LN_ImmediateItemsOnDrop[] = "ImmediateItemsOnDrop";
	static const char _LN_uiName[] = "uiName";
	static const char _LN_anchor_type[] = "anchor-type";
	static const char _LN_slicer[] = "slicer";
	static const char _LN_FieldName[] = "FieldName";
	static const char _LN_eqArray[] = "eqArray";
	static const char _LN_index_entry_link_start[] = "index-entry-link-start";
	static const char _LN_plot_area[] = "plot-area";
	static const char _LN_ActiveRows[] = "ActiveRows";
	static const char _LN_tscbandvertodd[] = "tscbandvertodd";
	static const char _LN_balanceSingleByteDoubleByteWidth[] = "balanceSingleByteDoubleByteWidth";
	static const char _LN_ErrorHide[] = "ErrorHide";
	static const char _LN_PageRightMargin[] = "PageRightMargin";
	static const char _LN_class_names[] = "class-names";
	static const char _LN_elements[] = "elements";
	static const char _LN_levelfollow[] = "levelfollow";
	static const char _LN_edge_rounding_mode[] = "edge-rounding-mode";
	static const char _LN_f[] = "f";
	static const char _LN_Alias[] = "Alias";
	static const char _LN_anchor_page_number[] = "anchor-page-number";
	static const char _LN_min_integer_digits[] = "min-integer-digits";
	static const char _LN_data_source_has_labels[] = "data-source-has-labels";
	static const char _LN_NoInserts[] = "NoInserts";
	static const char _LN_ShowPageMultipleItemLabel[] = "ShowPageMultipleItemLabel";
	static const char _LN_func[] = "func";
	static const char _LN_ErrorStyle[] = "ErrorStyle";
	static const char _LN_tscellcbpat[] = "tscellcbpat";
	static const char _LN_PhoneticAlias[] = "PhoneticAlias";
	static const char _LN_cond_style_name[] = "cond-style-name";
	static const char _LN_levelindent[] = "levelindent";
	static const char _LN_index_entry_link_end[] = "index-entry-link-end";
	static const char _LN_PageTopMargin[] = "PageTopMargin";
	static const char _LN_groupChr[] = "groupChr";
	static const char _LN_back_scale[] = "back-scale";
	static const char _LN_glue_point[] = "glue-point";
	static const char _LN_ErrorMessage[] = "ErrorMessage";
	static const char _LN_grouping[] = "grouping";
	static const char _LN_transparentMetafiles[] = "transparentMetafiles";
	static const char _LN_tscellcfpat[] = "tscellcfpat";
	static const char _LN_InstanceShape[] = "InstanceShape";
	static const char _LN_Values[] = "Values";
	static const char _LN_wall[] = "wall";
	static const char _LN_leveljc[] = "leveljc";
	static const char _LN_lLow[] = "lLow";
	static const char _LN_page_sequence[] = "page-sequence";
	static const char _LN_PageBottomMargin[] = "PageBottomMargin";
	static const char _LN_SmallGrid[] = "SmallGrid";
	static const char _LN_depth[] = "depth";
	static const char _LN_table[] = "table";
	static const char _LN_align[] = "align";
	static const char _LN_calendar[] = "calendar";
	static const char _LN_ErrorTitle[] = "ErrorTitle";
	static const char _LN_floor[] = "floor";
	static const char _LN_ValueID[] = "ValueID";
	static const char _LN_lUp[] = "lUp";
	static const char _LN_tscellpaddb[] = "tscellpaddb";
	static const char _LN_leveljcn[] = "leveljcn";
	static const char _LN_backface_culling[] = "backface-culling";
	static const char _LN_TableStyle[] = "TableStyle";
	static const char _LN_escape_direction[] = "escape-direction";
	static const char _LN_noExtraLineSpacing[] = "noExtraLineSpacing";
	static const char _LN_page[] = "page";
	static const char _LN_XSDType[] = "XSDType";
	static const char _LN_m[] = "m";
	static const char _LN_axis[] = "axis";
	static const char _LN_family[] = "family";
	static const char _LN_ScaleX[] = "ScaleX";
	static const char _LN_ParentValueID[] = "ParentValueID";
	static const char _LN_levellegal[] = "levellegal";
	static const char _LN_tscellpaddfb[] = "tscellpaddfb";
	static const char _LN_nary[] = "nary";
	static const char _LN_master_page_name[] = "master-page-name";
	static const char _LN_OleLink[] = "OleLink";
	static const char _LN_dimension[] = "dimension";
	static const char _LN_Aggregate[] = "Aggregate";
	static const char _LN_Tag[] = "Tag";
	static const char _LN_linenumbering_configuration[] = "linenumbering-configuration";
	static const char _LN_ScaleY[] = "ScaleY";
	static const char _LN_frame[] = "frame";
	static const char _LN_phant[] = "phant";
	static const char _LN_doNotLeaveBackslashAlone[] = "doNotLeaveBackslashAlone";
	static const char _LN_levelnfc[] = "levelnfc";
	static const char _LN_close_front[] = "close-front";
	static const char _LN_VacatedStyle[] = "VacatedStyle";
	static const char _LN_WantAdvise[] = "WantAdvise";
	static const char _LN_print[] = "print";
	static const char _LN_tscellpaddfl[] = "tscellpaddfl";
	static const char _LN_PagesX[] = "PagesX";
	static const char _LN_Enterprise[] = "Enterprise";
	static const char _LN_rel_width[] = "rel-width";
	static const char _LN_list[] = "list";
	static const char _LN_rad[] = "rad";
	static const char _LN_grid[] = "grid";
	static const char _LN_levelnfcn[] = "levelnfcn";
	static const char _LN_print_ranges[] = "print-ranges";
	static const char _LN_close_back[] = "close-back";
	static const char _LN_WantPict[] = "WantPict";
	static const char _LN_PagesY[] = "PagesY";
	static const char _LN_HideTotalsAnnotation[] = "HideTotalsAnnotation";
	static const char _LN_sPre[] = "sPre";
	static const char _LN_continue_numbering[] = "continue-numbering";
	static const char _LN_tscellpaddfr[] = "tscellpaddfr";
	static const char _LN_series[] = "series";
	static const char _LN_divs[] = "divs";
	static const char _LN_EnterpriseOutlineCodeAlias[] = "EnterpriseOutlineCodeAlias";
	static const char _LN_number_lines[] = "number-lines";
	static const char _LN_rel_height[] = "rel-height";
	static const char _LN_clrMapOvr[] = "clrMapOvr";
	static const char _LN_ulTrailSpace[] = "ulTrailSpace";
	static const char _LN_levelnorestart[] = "levelnorestart";
	static const char _LN_sSub[] = "sSub";
	static const char _LN_CenterX[] = "CenterX";
	static const char _LN_table_row[] = "table-row";
	static const char _LN_Cf[] = "Cf";
	static const char _LN_docOleData[] = "docOleData";
	static const char _LN_normals_kind[] = "normals-kind";
	static const char _LN_values_cell_range_address[] = "values-cell-range-address";
	static const char _LN_tscellpaddft[] = "tscellpaddft";
	static const char _LN_copy_of[] = "copy-of";
	static const char _LN_increment[] = "increment";
	static const char _LN_sSubSup[] = "sSubSup";
	static const char _LN_FieldType[] = "FieldType";
	static const char _LN_list_item[] = "list-item";
	static const char _LN_ShowCellBackgroundFromOLAP[] = "ShowCellBackgroundFromOLAP";
	static const char _LN_doNotExpandShiftReturn[] = "doNotExpandShiftReturn";
	static const char _LN_CenterY[] = "CenterY";
	static const char _LN_bgPr[] = "bgPr";
	static const char _LN_number_rows_repeated[] = "number-rows-repeated";
	static const char _LN_levelnumbers[] = "levelnumbers";
	static const char _LN_docSuppData[] = "docSuppData";
	static const char _LN_normals_direction[] = "normals-direction";
	static const char _LN_ResourceSubstitutionEnabled[] = "ResourceSubstitutionEnabled";
	static const char _LN_text_box[] = "text-box";
	static const char _LN_sSup[] = "sSup";
	static const char _LN_number_position[] = "number-position";
	static const char _LN_tscellpaddl[] = "tscellpaddl";
	static const char _LN_FieldStart[] = "FieldStart";
	static const char _LN_bgLink[] = "bgLink";
	static const char _LN_OnPage[] = "OnPage";
	static const char _LN_list_header[] = "list-header";
	static const char _LN_label_cell_address[] = "label-cell-address";
	static const char _LN_levelold[] = "levelold";
	static const char _LN_shapeDefaults[] = "shapeDefaults";
	static const char _LN_chain_next_name[] = "chain-next-name";
	static const char _LN_bgMaster[] = "bgMaster";
	static const char _LN_spacingInWholePoints[] = "spacingInWholePoints";
	static const char _LN_PrintGrid[] = "PrintGrid";
	static const char _LN_cdrElemPr[] = "cdrElemPr";
	static const char _LN_default_cell_style_name[] = "default-cell-style-name";
	static const char _LN_chr[] = "chr";
	static const char _LN_tscellpaddr[] = "tscellpaddr";
	static const char _LN_NoViewCalculatedMembers[] = "NoViewCalculatedMembers";
	static const char _LN_Missing[] = "Missing";
	static const char _LN_texture_generation_mode_x[] = "texture-generation-mode-x";
	static const char _LN_numbered_paragraph[] = "numbered-paragraph";
	static const char _LN_offset[] = "offset";
	static const char _LN_ccFmla[] = "ccFmla";
	static const char _LN_Workbook[] = "Workbook";
	static const char _LN_levelpicture[] = "levelpicture";
	static const char _LN_LeafOnly[] = "LeafOnly";
	static const char _LN_accPr[] = "accPr";
	static const char _LN_spTree[] = "spTree";
	static const char _LN_bgPict[] = "bgPict";
	static const char _LN_PrintPageOrientation[] = "PrintPageOrientation";
	static const char _LN_attached_axis[] = "attached-axis";
	static const char _LN_SeqNum[] = "SeqNum";
	static const char _LN_min_height[] = "min-height";
	static const char _LN_count_empty_lines[] = "count-empty-lines";
	static const char _LN_trFmla[] = "trFmla";
	static const char _LN_tscellpaddt[] = "tscellpaddt";
	static const char _LN_e[] = "e";
	static const char _LN_lineWrapLikeWord6[] = "lineWrapLikeWord6";
	static const char _LN_AllLevelsRequired[] = "AllLevelsRequired";
	static const char _LN_tagLst[] = "tagLst";
	static const char _LN_level[] = "level";
	static const char _LN_levelprev[] = "levelprev";
	static const char _LN_visibility[] = "visibility";
	static const char _LN_Expanded[] = "Expanded";
	static const char _LN_VisualTotals[] = "VisualTotals";
	static const char _LN_texture_generation_mode_y[] = "texture-generation-mode-y";
	static const char _LN_xmlCPr[] = "xmlCPr";
	static const char _LN_barPr[] = "barPr";
	static const char _LN_min_width[] = "min-width";
	static const char _LN_domain[] = "domain";
	static const char _LN_Names[] = "Names";
	static const char _LN_tscellpct[] = "tscellpct";
	static const char _LN_section[] = "section";
	static const char _LN_sldBase[] = "sldBase";
	static const char _LN_PaperKind[] = "PaperKind";
	static const char _LN_count_in_text_boxes[] = "count-in-text-boxes";
	static const char _LN_levelprevspace[] = "levelprevspace";
	static const char _LN_RowLast[] = "RowLast";
	static const char _LN_table_cell[] = "table-cell";
	static const char _LN_begChr[] = "begChr";
	static const char _LN_printBodyTextBeforeHeader[] = "printBodyTextBeforeHeader";
	static const char _LN_Worksheet[] = "Worksheet";
	static const char _LN_categories[] = "categories";
	static const char _LN_Location[] = "Location";
	static const char _LN_OnlyTableValuesAllowed[] = "OnlyTableValuesAllowed";
	static const char _LN_max_height[] = "max-height";
	static const char _LN_tscellwidth[] = "tscellwidth";
	static const char _LN_cmLst[] = "cmLst";
	static const char _LN_PaperSource[] = "PaperSource";
	static const char _LN_Pane[] = "Pane";
	static const char _LN_sepChr[] = "sepChr";
	static const char _LN_protected[] = "protected";
	static const char _LN_covered_table_cell[] = "covered-table-cell";
	static const char _LN_texture_kind[] = "texture-kind";
	static const char _LN_NamedRange[] = "NamedRange";
	static const char _LN_PLPivotField[] = "PLPivotField";
	static const char _LN_levelspace[] = "levelspace";
	static const char _LN_data_point[] = "data-point";
	static const char _LN_restart_on_page[] = "restart-on-page";
	static const char _LN_max_width[] = "max-width";
	static const char _LN_transition[] = "transition";
	static const char _LN_ActiveRow[] = "ActiveRow";
	static const char _LN_tscellwidthfts[] = "tscellwidthfts";
	static const char _LN_endChr[] = "endChr";
	static const char _LN_Font[] = "Font";
	static const char _LN_Masks[] = "Masks";
	static const char _LN_protection_key[] = "protection-key";
	static const char _LN_printColBlack[] = "printColBlack";
	static const char _LN_texture_filter[] = "texture-filter";
	static const char _LN_levelstartat[] = "levelstartat";
	static const char _LN_repeated[] = "repeated";
	static const char _LN_PLTotal[] = "PLTotal";
	static const char _LN_sheets[] = "sheets";
	static const char _LN_timingInfo[] = "timingInfo";
	static const char _LN_grow[] = "grow";
	static const char _LN_ActiveCol[] = "ActiveCol";
	static const char _LN_Color[] = "Color";
	static const char _LN_number_columns_repeated[] = "number-columns-repeated";
	static const char _LN_linenumbering_separator[] = "linenumbering-separator";
	static const char _LN_Borders[] = "Borders";
	static const char _LN_Mask[] = "Mask";
	static const char _LN_binary_data[] = "binary-data";
	static const char _LN_tscfirstcol[] = "tscfirstcol";
	static const char _LN_wbPr[] = "wbPr";
	static const char _LN_volTypes[] = "volTypes";
	static const char _LN_DataMember[] = "DataMember";
	static const char _LN_shp[] = "shp";
	static const char _LN_mean_value[] = "mean-value";
	static const char _LN_wpSpaceWidth[] = "wpSpaceWidth";
	static const char _LN_display[] = "display";
	static const char _LN_Style[] = "Style";
	static const char _LN_leveltemplateid[] = "leveltemplateid";
	static const char _LN_hf[] = "hf";
	static const char _LN_Level[] = "Level";
	static const char _LN_texture_mode[] = "texture-mode";
	static const char _LN_RangeSelection[] = "RangeSelection";
	static const char _LN_dPr[] = "dPr";
	static const char _LN_webPub[] = "webPub";
	static const char _LN_colOff[] = "colOff";
	static const char _LN_tscfirstrow[] = "tscfirstrow";
	static const char _LN_Interior[] = "Interior";
	static const char _LN_sldLayout[] = "sldLayout";
	static const char _LN_Case[] = "Case";
	static const char _LN_volType[] = "volType";
	static const char _LN_condition[] = "condition";
	static const char _LN_DataAxisEmpty[] = "DataAxisEmpty";
	static const char _LN_Length[] = "Length";
	static const char _LN_error_indicator[] = "error-indicator";
	static const char _LN_number_columns_spanned[] = "number-columns-spanned";
	static const char _LN_object[] = "object";
	static const char _LN_notes_configuration[] = "notes-configuration";
	static const char _LN_showBreaksInFrames[] = "showBreaksInFrames";
	static const char _LN_baseJc[] = "baseJc";
	static const char _LN_emissive_color[] = "emissive-color";
	static const char _LN_leveltext[] = "leveltext";
	static const char _LN_Selection[] = "Selection";
	static const char _LN_Pos[] = "Pos";
	static const char _LN_main[] = "main";
	static const char _LN_defLink[] = "defLink";
	static const char _LN_tsclastcol[] = "tsclastcol";
	static const char _LN_Separator[] = "Separator";
	static const char _LN_section_source[] = "section-source";
	static const char _LN_object_ole[] = "object-ole";
	static const char _LN_smartTag[] = "smartTag";
	static const char _LN_rowOff[] = "rowOff";
	static const char _LN_maxDist[] = "maxDist";
	static const char _LN_DisplayFieldList[] = "DisplayFieldList";
	static const char _LN_FontScale[] = "FontScale";
	static const char _LN_regression_curve[] = "regression-curve";
	static const char _LN_topic[] = "topic";
	static const char _LN_li[] = "li";
	static const char _LN_Border[] = "Border";
	static const char _LN_ExcelType[] = "ExcelType";
	static const char _LN_subFontBySize[] = "subFontBySize";
	static const char _LN_citation_style_name[] = "citation-style-name";
	static const char _LN_specular_color[] = "specular-color";
	static const char _LN_outline1Link[] = "outline1Link";
	static const char _LN_number_rows_spanned[] = "number-rows-spanned";
	static const char _LN_WBSMasks[] = "WBSMasks";
	static const char _LN_smartTagTypes[] = "smartTagTypes";
	static const char _LN_tsclastrow[] = "tsclastrow";
	static const char _LN_ns[] = "ns";
	static const char _LN_notify_on_update_of_ranges[] = "notify-on-update-of-ranges";
	static const char _LN_objDist[] = "objDist";
	static const char _LN_section_name[] = "section-name";
	static const char _LN_Locale[] = "Locale";
	static const char _LN_linebetcol[] = "linebetcol";
	static const char _LN_ParameterType[] = "ParameterType";
	static const char _LN_NoAutoFit[] = "NoAutoFit";
	static const char _LN_clientData[] = "clientData";
	static const char _LN_subtopic[] = "subtopic";
	static const char _LN_stock_gain_marker[] = "stock-gain-marker";
	static const char _LN_psover[] = "psover";
	static const char _LN_VerifyUniqueCodes[] = "VerifyUniqueCodes";
	static const char _LN_outline2Link[] = "outline2Link";
	static const char _LN_suppressBottomSpacing[] = "suppressBottomSpacing";
	static const char _LN_eqArrPr[] = "eqArrPr";
	static const char _LN_tscnecell[] = "tscnecell";
	static const char _LN_shininess[] = "shininess";
	static const char _LN_Size[] = "Size";
	static const char _LN_fileRecovery[] = "fileRecovery";
	static const char _LN_citation_body_style_name[] = "citation-body-style-name";
	static const char _LN_filter_name[] = "filter-name";
	static const char _LN_content_validation_name[] = "content-validation-name";
	static const char _LN_RDF[] = "RDF";
	static const char _LN_rdf[] = "rdf";
	static const char _LN_psz[] = "psz";
	static const char _LN_linecont[] = "linecont";
	static const char _LN_extents[] = "extents";
	static const char _LN_DblUnderline[] = "DblUnderline";
	static const char _LN_class_id[] = "class-id";
	static const char _LN_tscnwcell[] = "tscnwcell";
	static const char _LN_lvl1Link[] = "lvl1Link";
	static const char _LN_except[] = "except";
	static const char _LN_stock_loss_marker[] = "stock-loss-marker";
	static const char _LN_GenerateCodes[] = "GenerateCodes";
	static const char _LN_numJc[] = "numJc";
	static const char _LN_calcPr[] = "calcPr";
	static const char _LN_pubauto[] = "pubauto";
	static const char _LN_horizontal_pos[] = "horizontal-pos";
	static const char _LN_tracked_changes[] = "tracked-changes";
	static const char _LN_suppressTopSpacing[] = "suppressTopSpacing";
	static const char _LN_linemod[] = "linemod";
	static const char _LN_NamedCell[] = "NamedCell";
	static const char _LN_applet[] = "applet";
	static const char _LN_nsName[] = "nsName";
	static const char _LN_tscsecell[] = "tscsecell";
	static const char _LN_Overline[] = "Overline";
	static const char _LN_lvl2Link[] = "lvl2Link";
	static const char _LN_denJc[] = "denJc";
	static const char _LN_twoCellAnchor[] = "twoCellAnchor";
	static const char _LN_default_style_name[] = "default-style-name";
	static const char _LN_extRefs[] = "extRefs";
	static const char _LN_pvmrg[] = "pvmrg";
	static const char _LN_MemberExpand[] = "MemberExpand";
	static const char _LN_lineppage[] = "lineppage";
	static const char _LN_Prefix[] = "Prefix";
	static const char _LN_number_matrix_columns_spanned[] = "number-matrix-columns-spanned";
	static const char _LN_Table[] = "Table";
	static const char _LN_stock_range_line[] = "stock-range-line";
	static const char _LN_code[] = "code";
	static const char _LN_horizontal_rel[] = "horizontal-rel";
	static const char _LN_track_changes[] = "track-changes";
	static const char _LN_fPr[] = "fPr";
	static const char _LN_Strikethru[] = "Strikethru";
	static const char _LN_lvl3Link[] = "lvl3Link";
	static const char _LN_tscswcell[] = "tscswcell";
	static const char _LN_pvpara[] = "pvpara";
	static const char _LN_namedRanges[] = "namedRanges";
	static const char _LN_suppressTopSpacingMac5[] = "suppressTopSpacingMac5";
	static const char _LN_nodeID[] = "nodeID";
	static const char _LN_oneCellAnchor[] = "oneCellAnchor";
	static const char _LN_WBSMask[] = "WBSMask";
	static const char _LN_archive[] = "archive";
	static const char _LN_linerestart[] = "linerestart";
	static const char _LN_num[] = "num";
	static const char _LN_start_numbering_at[] = "start-numbering-at";
	static const char _LN_pvpg[] = "pvpg";
	static const char _LN_lvl4Link[] = "lvl4Link";
	static const char _LN_tsd[] = "tsd";
	static const char _LN_Highlight[] = "Highlight";
	static const char _LN_vertical_pos[] = "vertical-pos";
	static const char _LN_about[] = "about";
	static const char _LN_CacheDetails[] = "CacheDetails";
	static const char _LN_changed_region[] = "changed-region";
	static const char _LN_forms[] = "forms";
	static const char _LN_pivotCaches[] = "pivotCaches";
	static const char _LN_den[] = "den";
	static const char _LN_ExtendedAttributes[] = "ExtendedAttributes";
	static const char _LN_may_script[] = "may-script";
	static const char _LN_book[] = "book";
	static const char _LN_absoluteAnchor[] = "absoluteAnchor";
	static const char _LN_pwd[] = "pwd";
	static const char _LN_linestart[] = "linestart";
	static const char _LN_tsnowrap[] = "tsnowrap";
	static const char _LN_number_matrix_rows_spanned[] = "number-matrix-rows-spanned";
	static const char _LN_suppressTopSpacingWP[] = "suppressTopSpacingWP";
	static const char _LN_resource[] = "resource";
	static const char _LN_fName[] = "fName";
	static const char _LN_automatic_focus[] = "automatic-focus";
	static const char _LN_lvl5Link[] = "lvl5Link";
	static const char _LN_Perpendicular[] = "Perpendicular";
	static const char _LN_fileVersion[] = "fileVersion";
	static const char _LN_footnotes_position[] = "footnotes-position";
	static const char _LN_vertical_rel[] = "vertical-rel";
	static const char _LN_pxe[] = "pxe";
	static const char _LN_FieldListTop[] = "FieldListTop";
	static const char _LN_docRoute[] = "docRoute";
	static const char _LN_id[] = "id";
	static const char _LN_plugin[] = "plugin";
	static const char _LN_tsrowd[] = "tsrowd";
	static const char _LN_linestarts[] = "linestarts";
	static const char _LN_groupChrPr[] = "groupChrPr";
	static const char _LN_ExtendedAttribute[] = "ExtendedAttribute";
	static const char _LN_lvl6Link[] = "lvl6Link";
	static const char _LN_qc[] = "qc";
	static const char _LN_insertion[] = "insertion";
	static const char _LN_apply_design_mode[] = "apply-design-mode";
	static const char _LN_oleSize[] = "oleSize";
	static const char _LN_fileSharing[] = "fileSharing";
	static const char _LN_parseType[] = "parseType";
	static const char _LN_MainFile[] = "MainFile";
	static const char _LN_DoubleStrikethrough[] = "DoubleStrikethrough";
	static const char _LN_mime_type[] = "mime-type";
	static const char _LN_editable[] = "editable";
	static const char _LN_FieldListBottom[] = "FieldListBottom";
	static const char _LN_suppressSpBfAfterPgBrk[] = "suppressSpBfAfterPgBrk";
	static const char _LN_tsvertalb[] = "tsvertalb";
	static const char _LN_qd[] = "qd";
	static const char _LN_spLock[] = "spLock";
	static const char _LN_note_continuation_notice_forward[] = "note-continuation-notice-forward";
	static const char _LN_linex[] = "linex";
	static const char _LN_protect[] = "protect";
	static const char _LN_lim[] = "lim";
	static const char _LN_lvl7Link[] = "lvl7Link";
	static const char _LN_deletion[] = "deletion";
	static const char _LN_bookProtection[] = "bookProtection";
	static const char _LN_qj[] = "qj";
	static const char _LN_RollupType[] = "RollupType";
	static const char _LN_parentRef[] = "parentRef";
	static const char _LN_wrap[] = "wrap";
	static const char _LN_bookViews[] = "bookViews";
	static const char _LN_tsvertalc[] = "tsvertalc";
	static const char _LN_linkself[] = "linkself";
	static const char _LN_cxnSpLock[] = "cxnSpLock";
	static const char _LN_table_column[] = "table-column";
	static const char _LN_form[] = "form";
	static const char _LN_lvl8Link[] = "lvl8Link";
	static const char _LN_qk[] = "qk";
	static const char _LN_FieldListRight[] = "FieldListRight";
	static const char _LN_RTLText[] = "RTLText";
	static const char _LN_mcJc[] = "mcJc";
	static const char _LN_format_change[] = "format-change";
	static const char _LN_wrap_dynamic_treshold[] = "wrap-dynamic-treshold";
	static const char _LN_floating_frame[] = "floating-frame";
	static const char _LN_swapBordersFacingPages[] = "swapBordersFacingPages";
	static const char _LN_notAllowed[] = "notAllowed";
	static const char _LN_CalculationType[] = "CalculationType";
	static const char _LN_userBookViews[] = "userBookViews";
	static const char _LN_ql[] = "ql";
	static const char _LN_target_frame[] = "target-frame";
	static const char _LN_linkstyles[] = "linkstyles";
	static const char _LN_tsvertalt[] = "tsvertalt";
	static const char _LN_stCxn[] = "stCxn";
	static const char _LN_LastAuthor[] = "LastAuthor";
	static const char _LN_mcPr[] = "mcPr";
	static const char _LN_lvl9Link[] = "lvl9Link";
	static const char _LN_UseVertical[] = "UseVertical";
	static const char _LN_table_header_columns[] = "table-header-columns";
	static const char _LN_note_continuation_notice_backward[] = "note-continuation-notice-backward";
	static const char _LN_qmspace[] = "qmspace";
	static const char _LN_FieldListLeft[] = "FieldListLeft";
	static const char _LN_change[] = "change";
	static const char _LN_endCxn[] = "endCxn";
	static const char _LN_mc[] = "mc";
	static const char _LN_bookView[] = "bookView";
	static const char _LN_externalRef[] = "externalRef";
	static const char _LN_frame_name[] = "frame-name";
	static const char _LN_twoonone[] = "twoonone";
	static const char _LN_linkval[] = "linkval";
	static const char _LN_method[] = "method";
	static const char _LN_Formula[] = "Formula";
	static const char _LN_number_wrapped_paragraphs[] = "number-wrapped-paragraphs";
	static const char _LN_Letterspace[] = "Letterspace";
	static const char _LN_qr[] = "qr";
	static const char _LN_plcHide[] = "plcHide";
	static const char _LN_convMailMergeEsc[] = "convMailMergeEsc";
	static const char _LN_extConns[] = "extConns";
	static const char _LN_change_start[] = "change-start";
	static const char _LN_AppName[] = "AppName";
	static const char _LN_themeLinkLst[] = "themeLinkLst";
	static const char _LN_gspLock[] = "gspLock";
	static const char _LN_userBookView[] = "userBookView";
	static const char _LN_PivotAxis[] = "PivotAxis";
	static const char _LN_table_columns[] = "table-columns";
	static const char _LN_enctype[] = "enctype";
	static const char _LN_qt[] = "qt";
	static const char _LN_RestrictValues[] = "RestrictValues";
	static const char _LN_contour_polygon[] = "contour-polygon";
	static const char _LN_mcs[] = "mcs";
	static const char _LN_lisa[] = "lisa";
	static const char _LN_txe[] = "txe";
	static const char _LN_extConn[] = "extConn";
	static const char _LN_TotalTime[] = "TotalTime";
	static const char _LN_ColorTrans[] = "ColorTrans";
	static const char _LN_graphicFrameLock[] = "graphicFrameLock";
	static const char _LN_rawclbgbdiag[] = "rawclbgbdiag";
	static const char _LN_change_end[] = "change-end";
	static const char _LN_titleStyle[] = "titleStyle";
	static const char _LN_VersionUpdateableMin[] = "VersionUpdateableMin";
	static const char _LN_allow_deletes[] = "allow-deletes";
	static const char _LN_bibliography_configuration[] = "bibliography-configuration";
	static const char _LN_truncateFontHeight[] = "truncateFontHeight";
	static const char _LN_mrJc[] = "mrJc";
	static const char _LN_uc[] = "uc";
	static const char _LN_lisb[] = "lisb";
	static const char _LN_table_column_group[] = "table-column-group";
	static const char _LN_wrap_contour[] = "wrap-contour";
	static const char _LN_ValuelistSortOrder[] = "ValuelistSortOrder";
	static const char _LN_dbProps[] = "dbProps";
	static const char _LN_contour_path[] = "contour-path";
	static const char _LN_smartTagType[] = "smartTagType";
	static const char _LN_AsianFont[] = "AsianFont";
	static const char _LN_ud[] = "ud";
	static const char _LN_mrPr[] = "mrPr";
	static const char _LN_change_id[] = "change-id";
	static const char _LN_rawclbgcross[] = "rawclbgcross";
	static const char _LN_listhybrid[] = "listhybrid";
	static const char _LN_bodyStyle[] = "bodyStyle";
	static const char _LN_Created[] = "Created";
	static const char _LN_allow_inserts[] = "allow-inserts";
	static const char _LN_olapProps[] = "olapProps";
	static const char _LN_ul[] = "ul";
	static const char _LN_wrap_contour_mode[] = "wrap-contour-mode";
	static const char _LN_mwSmallCaps[] = "mwSmallCaps";
	static const char _LN_VersionLastUpdate[] = "VersionLastUpdate";
	static const char _LN_mPr[] = "mPr";
	static const char _LN_ComplexScriptFont[] = "ComplexScriptFont";
	static const char _LN_recreate_on_edit[] = "recreate-on-edit";
	static const char _LN_variable_decls[] = "variable-decls";
	static const char _LN_prefix[] = "prefix";
	static const char _LN_AppendNewValues[] = "AppendNewValues";
	static const char _LN_notesStyle[] = "notesStyle";
	static const char _LN_listid[] = "listid";
	static const char _LN_ulc[] = "ulc";
	static const char _LN_rawclbgdcross[] = "rawclbgdcross";
	static const char _LN_table_header_rows[] = "table-header-rows";
	static const char _LN_mr[] = "mr";
	static const char _LN_ab[] = "ab";
	static const char _LN_webqProps[] = "webqProps";
	static const char _LN_allow_updates[] = "allow-updates";
	static const char _LN_usePrinterMetrics[] = "usePrinterMetrics";
	static const char _LN_uld[] = "uld";
	static const char _LN_suffix[] = "suffix";
	static const char _LN_listname[] = "listname";
	static const char _LN_Words[] = "Words";
	static const char _LN_limLoc[] = "limLoc";
	static const char _LN_Toggle[] = "Toggle";
	static const char _LN_run_through[] = "run-through";
	static const char _LN_VersionRefreshableMin[] = "VersionRefreshableMin";
	static const char _LN_otherStyle[] = "otherStyle";
	static const char _LN_desc[] = "desc";
	static const char _LN_LocalizeFont[] = "LocalizeFont";
	static const char _LN_sequence_decls[] = "sequence-decls";
	static const char _LN_rawbgdkbdiag[] = "rawbgdkbdiag";
	static const char _LN_txtWiz[] = "txtWiz";
	static const char _LN_uldash[] = "uldash";
	static const char _LN_customMenu[] = "customMenu";
	static const char _LN_numbered_entries[] = "numbered-entries";
	static const char _LN_Characters[] = "Characters";
	static const char _LN_apply_filter[] = "apply-filter";
	static const char _LN_table_rows[] = "table-rows";
	static const char _LN_subHide[] = "subHide";
	static const char _LN_absh[] = "absh";
	static const char _LN_ValueList[] = "ValueList";
	static const char _LN_listoverridecount[] = "listoverridecount";
	static const char _LN_flow_with_text[] = "flow-with-text";
	static const char _LN_ww6BorderRules[] = "ww6BorderRules";
	static const char _LN_sldLayoutId[] = "sldLayoutId";
	static const char _LN_qParams[] = "qParams";
	static const char _LN_uldashd[] = "uldashd";
	static const char _LN_server_map[] = "server-map";
	static const char _LN_ComplexScriptSize[] = "ComplexScriptSize";
	static const char _LN_bounds[] = "bounds";
	static const char _LN_user_field_decls[] = "user-field-decls";
	static const char _LN_rawclbgdkcross[] = "rawclbgdkcross";
	static const char _LN_supHide[] = "supHide";
	static const char _LN_DefaultVersion[] = "DefaultVersion";
	static const char _LN_ID[] = "ID";
	static const char _LN_table_row_group[] = "table-row-group";
	static const char _LN_command_type[] = "command-type";
	static const char _LN_abslock[] = "abslock";
	static const char _LN_sort_by_position[] = "sort-by-position";
	static const char _LN_help[] = "help";
	static const char _LN_fsb[] = "fsb";
	static const char _LN_uldashdd[] = "uldashdd";
	static const char _LN_PresentationFormat[] = "PresentationFormat";
	static const char _LN_sldLayoutIdLst[] = "sldLayoutIdLst";
	static const char _LN_listoverrideformat[] = "listoverrideformat";
	static const char _LN_image_map[] = "image-map";
	static const char _LN_overflow_behavior[] = "overflow-behavior";
	static const char _LN_wrapTrailSpaces[] = "wrapTrailSpaces";
	static const char _LN_Calendars[] = "Calendars";
	static const char _LN_naryPr[] = "naryPr";
	static const char _LN_statusBar[] = "statusBar";
	static const char _LN_IndFirst[] = "IndFirst";
	static const char _LN_tables[] = "tables";
	static const char _LN_rawclbgdkdcross[] = "rawclbgdkdcross";
	static const char _LN_Flag[] = "Flag";
	static const char _LN_dde_connection_decls[] = "dde-connection-decls";
	static const char _LN_PivotField[] = "PivotField";
	static const char _LN_command[] = "command";
	static const char _LN_uldb[] = "uldb";
	static const char _LN_is_sub_table[] = "is-sub-table";
	static const char _LN_area_rectangle[] = "area-rectangle";
	static const char _LN_sub[] = "sub";
	static const char _LN_sort_key[] = "sort-key";
	static const char _LN_txStyles[] = "txStyles";
	static const char _LN_absnoovrlp[] = "absnoovrlp";
	static const char _LN_qParam[] = "qParam";
	static const char _LN_footnoteLayoutLikeWW8[] = "footnoteLayoutLikeWW8";
	static const char _LN_IndLeft[] = "IndLeft";
	static const char _LN_supBookSource[] = "supBookSource";
	static const char _LN_listoverridestart[] = "listoverridestart";
	static const char _LN_mirror[] = "mirror";
	static const char _LN_ulhair[] = "ulhair";
	static const char _LN_IsBaseCalendar[] = "IsBaseCalendar";
	static const char _LN_datasource[] = "datasource";
	static const char _LN_sup[] = "sup";
	static const char _LN_CubeField[] = "CubeField";
	static const char _LN_rawclbgdkfdiag[] = "rawclbgdkfdiag";
	static const char _LN_data[] = "data";
	static const char _LN_key[] = "key";
	static const char _LN_sldMaster[] = "sldMaster";
	static const char _LN_s[] = "s";
	static const char _LN_Guid[] = "Guid";
	static const char _LN_absw[] = "absw";
	static const char _LN_IndRight[] = "IndRight";
	static const char _LN_area_circle[] = "area-circle";
	static const char _LN_clip[] = "clip";
	static const char _LN_ulhwave[] = "ulhwave";
	static const char _LN_master_fields[] = "master-fields";
	static const char _LN_imageRelation[] = "imageRelation";
	static const char _LN_sort_ascending[] = "sort-ascending";
	static const char _LN_CalculatedMember[] = "CalculatedMember";
	static const char _LN_c[] = "c";
	static const char _LN_BaseCalendarUID[] = "BaseCalendarUID";
	static const char _LN_asc[] = "asc";
	static const char _LN_listpicture[] = "listpicture";
	static const char _LN_acaps[] = "acaps";
	static const char _LN_wrap_influence_on_position[] = "wrap-influence-on-position";
	static const char _LN_shapeLayoutLikeWW8[] = "shapeLayoutLikeWW8";
	static const char _LN_rawclbgdkhor[] = "rawclbgdkhor";
	static const char _LN_param[] = "param";
	static const char _LN_handoutMaster[] = "handoutMaster";
	static const char _LN_ulldash[] = "ulldash";
	static const char _LN_SpLine[] = "SpLine";
	static const char _LN_tab[] = "tab";
	static const char _LN_graphicData[] = "graphicData";
	static const char _LN_Bytes[] = "Bytes";
	static const char _LN_area_polygon[] = "area-polygon";
	static const char _LN_fldInfos[] = "fldInfos";
	static const char _LN_acccomma[] = "acccomma";
	static const char _LN_detail_fields[] = "detail-fields";
	static const char _LN_pivotCache[] = "pivotCache";
	static const char _LN_table_source[] = "table-source";
	static const char _LN_phantPr[] = "phantPr";
	static const char _LN_tab_ref[] = "tab-ref";
	static const char _LN_list_style[] = "list-style";
	static const char _LN_listrestarthdn[] = "listrestarthdn";
	static const char _LN_SpBefore[] = "SpBefore";
	static const char _LN_ulnone[] = "ulnone";
	static const char _LN_MemberName[] = "MemberName";
	static const char _LN_WeekDays[] = "WeekDays";
	static const char _LN_Lines[] = "Lines";
	static const char _LN_rawclbgdkvert[] = "rawclbgdkvert";
	static const char _LN_notesMaster[] = "notesMaster";
	static const char _LN_graphic[] = "graphic";
	static const char _LN_fldInfo[] = "fldInfo";
	static const char _LN_alignTablesRowByRow[] = "alignTablesRowByRow";
	static const char _LN_nohref[] = "nohref";
	static const char _LN_accdot[] = "accdot";
	static const char _LN_degHide[] = "degHide";
	static const char _LN_line_break[] = "line-break";
	static const char _LN_docRouteInfo[] = "docRouteInfo";
	static const char _LN_ulth[] = "ulth";
	static const char _LN_Paragraphs[] = "Paragraphs";
	static const char _LN_frame_display_scrollbar[] = "frame-display-scrollbar";
	static const char _LN_SpAfter[] = "SpAfter";
	static const char _LN_consecutive_numbering[] = "consecutive-numbering";
	static const char _LN_WeekDay[] = "WeekDay";
	static const char _LN_mode[] = "mode";
	static const char _LN_escape_processing[] = "escape-processing";
	static const char _LN_ParentUniqueName[] = "ParentUniqueName";
	static const char _LN_listsimple[] = "listsimple";
	static const char _LN_scene[] = "scene";
	static const char _LN_rawclbgfdiag[] = "rawclbgfdiag";
	static const char _LN_accnone[] = "accnone";
	static const char _LN_ulthd[] = "ulthd";
	static const char _LN_radPr[] = "radPr";
	static const char _LN_HorzAlign[] = "HorzAlign";
	static const char _LN_span[] = "span";
	static const char _LN_DayType[] = "DayType";
	static const char _LN_font_size_asian[] = "font-size-asian";
	static const char _LN_CharactersWithSpaces[] = "CharactersWithSpaces";
	static const char _LN_docRouteOriginator[] = "docRouteOriginator";
	static const char _LN_forgetLastTabAlignment[] = "forgetLastTabAlignment";
	static const char _LN_vrp[] = "vrp";
	static const char _LN_liststyleid[] = "liststyleid";
	static const char _LN_ulthdash[] = "ulthdash";
	static const char _LN_acf[] = "acf";
	static const char _LN_deg[] = "deg";
	static const char _LN_SolveOrder[] = "SolveOrder";
	static const char _LN_a[] = "a";
	static const char _LN_rawclbghoriz[] = "rawclbghoriz";
	static const char _LN_filter_options[] = "filter-options";
	static const char _LN_list_level_style_number[] = "list-level-style-number";
	static const char _LN_frame_display_border[] = "frame-display-border";
	static const char _LN_DayWorking[] = "DayWorking";
	static const char _LN_vpn[] = "vpn";
	static const char _LN_Bullet[] = "Bullet";
	static const char _LN_ignore_result[] = "ignore-result";
	static const char _LN_font_size_complex[] = "font-size-complex";
	static const char _LN_ulthdashd[] = "ulthdashd";
	static const char _LN_liststylename[] = "liststylename";
	static const char _LN_vup[] = "vup";
	static const char _LN_adjustright[] = "adjustright";
	static const char _LN_docRouteRecipient[] = "docRouteRecipient";
	static const char _LN_Invalid[] = "Invalid";
	static const char _LN_visited_style_name[] = "visited-style-name";
	static const char _LN_BulletStr[] = "BulletStr";
	static const char _LN_adjustLineHeightInTable[] = "adjustLineHeightInTable";
	static const char _LN_rawclbgvert[] = "rawclbgvert";
	static const char _LN_TimePeriod[] = "TimePeriod";
	static const char _LN_refresh_delay[] = "refresh-delay";
	static const char _LN_projection[] = "projection";
	static const char _LN_navigation_mode[] = "navigation-mode";
	static const char _LN_frame_margin_horizontal[] = "frame-margin-horizontal";
	static const char _LN_ulthdashdd[] = "ulthdashdd";
	static const char _LN_display_levels[] = "display-levels";
	static const char _LN_SmartTags[] = "SmartTags";
	static const char _LN_Set[] = "Set";
	static const char _LN_Metadata[] = "Metadata";
	static const char _LN_igx[] = "igx";
	static const char _LN_listtemplateid[] = "listtemplateid";
	static const char _LN_adn[] = "adn";
	static const char _LN_BulletFont[] = "BulletFont";
	static const char _LN_font_size_rel[] = "font-size-rel";
	static const char _LN_rdblquote[] = "rdblquote";
	static const char _LN_FromDate[] = "FromDate";
	static const char _LN_bookmark[] = "bookmark";
	static const char _LN_PTLineItems[] = "PTLineItems";
	static const char _LN_scenario[] = "scenario";
	static const char _LN_distance[] = "distance";
	static const char _LN_aenddoc[] = "aenddoc";
	static const char _LN_ulthldash[] = "ulthldash";
	static const char _LN_metadataTypes[] = "metadataTypes";
	static const char _LN_CustomDocumentProperties[] = "CustomDocumentProperties";
	static const char _LN_autoSpaceLikeWord95[] = "autoSpaceLikeWord95";
	static const char _LN_list_level_style_bullet[] = "list-level-style-bullet";
	static const char _LN_LocalizeBulletFont[] = "LocalizeBulletFont";
	static const char _LN_ToDate[] = "ToDate";
	static const char _LN_listtext[] = "listtext";
	static const char _LN_tab_cycle[] = "tab-cycle";
	static const char _LN_font_size_rel_asian[] = "font-size-rel-asian";
	static const char _LN_bookmark_start[] = "bookmark-start";
	static const char _LN_scenario_ranges[] = "scenario-ranges";
	static const char _LN_frame_margin_vertical[] = "frame-margin-vertical";
	static const char _LN_focal_length[] = "focal-length";
	static const char _LN_aendnotes[] = "aendnotes";
	static const char _LN_PTSource[] = "PTSource";
	static const char _LN_rempersonalinfo[] = "rempersonalinfo";
	static const char _LN_ululdbwave[] = "ululdbwave";
	static const char _LN_spcPct[] = "spcPct";
	static const char _LN_cellMetadata[] = "cellMetadata";
	static const char _LN_WorkingTimes[] = "WorkingTimes";
	static const char _LN_lnbrkrule[] = "lnbrkrule";
	static const char _LN_connection_resource[] = "connection-resource";
	static const char _LN_OfficeDataConnection[] = "OfficeDataConnection";
	static const char _LN_noSpaceRaiseLower[] = "noSpaceRaiseLower";
	static const char _LN_aexpnd[] = "aexpnd";
	static const char _LN_PTFormula[] = "PTFormula";
	static const char _LN_BulletFontSize[] = "BulletFontSize";
	static const char _LN_spcPts[] = "spcPts";
	static const char _LN_OfficeDocumentSettings[] = "OfficeDocumentSettings";
	static const char _LN_bookmark_end[] = "bookmark-end";
	static const char _LN_shadow_slant[] = "shadow-slant";
	static const char _LN_is_active[] = "is-active";
	static const char _LN_ulw[] = "ulw";
	static const char _LN_bullet_char[] = "bullet-char";
	static const char _LN_font_size_rel_complex[] = "font-size-rel-complex";
	static const char _LN_bldIgx[] = "bldIgx";
	static const char _LN_result[] = "result";
	static const char _LN_visible_area_left[] = "visible-area-left";
	static const char _LN_valueMetadata[] = "valueMetadata";
	static const char _LN_lndscpsxn[] = "lndscpsxn";
	static const char _LN_WorkingTime[] = "WorkingTime";
	static const char _LN_af[] = "af";
	static const char _LN_ulwave[] = "ulwave";
	static const char _LN_PTFormat[] = "PTFormat";
	static const char _LN_restoredLeft[] = "restoredLeft";
	static const char _LN_display_border[] = "display-border";
	static const char _LN_bldChart[] = "bldChart";
	static const char _LN_reference_mark[] = "reference-mark";
	static const char _LN_shade_mode[] = "shade-mode";
	static const char _LN_model[] = "model";
	static const char _LN_revauth[] = "revauth";
	static const char _LN_TextPosAfterBullet[] = "TextPosAfterBullet";
	static const char _LN_lineSpc[] = "lineSpc";
	static const char _LN_doNotUseHTMLParagraphAutoSpacing[] = "doNotUseHTMLParagraphAutoSpacing";
	static const char _LN_bullet_relative_size[] = "bullet-relative-size";
	static const char _LN_affixed[] = "affixed";
	static const char _LN_Connection[] = "Connection";
	static const char _LN_lnongrid[] = "lnongrid";
	static const char _LN_FromTime[] = "FromTime";
	static const char _LN_DownloadComponents[] = "DownloadComponents";
	static const char _LN_mdxMetadata[] = "mdxMetadata";
	static const char _LN_CacheFile[] = "CacheFile";
	static const char _LN_script_type[] = "script-type";
	static const char _LN_visible_area_top[] = "visible-area-top";
	static const char _LN_revauthdel[] = "revauthdel";
	static const char _LN_spcBef[] = "spcBef";
	static const char _LN_restoredTop[] = "restoredTop";
	static const char _LN_afs[] = "afs";
	static const char _LN_ambient_color[] = "ambient-color";
	static const char _LN_border_color[] = "border-color";
	static const char _LN_textarea[] = "textarea";
	static const char _LN_ConnectionString[] = "ConnectionString";
	static const char _LN_reference_mark_start[] = "reference-mark-start";
	static const char _LN_loch[] = "loch";
	static const char _LN_upr[] = "upr";
	static const char _LN_ToTime[] = "ToTime";
	static const char _LN_Flags[] = "Flags";
	static const char _LN_QuerySource[] = "QuerySource";
	static const char _LN_strMetadata[] = "strMetadata";
	static const char _LN_aftnbj[] = "aftnbj";
	static const char _LN_list_level_style_image[] = "list-level-style-image";
	static const char _LN_spcAft[] = "spcAft";
	static const char _LN_letter_spacing[] = "letter-spacing";
	static const char _LN_revbar[] = "revbar";
	static const char _LN_urtf[] = "urtf";
	static const char _LN_lquote[] = "lquote";
	static const char _LN_LocationOfComponents[] = "LocationOfComponents";
	static const char _LN_visible_area_width[] = "visible-area-width";
	static const char _LN_scale[] = "scale";
	static const char _LN_Tasks[] = "Tasks";
	static const char _LN_lighting_mode[] = "lighting-mode";
	static const char _LN_Tab[] = "Tab";
	static const char _LN_layoutRawTableWidth[] = "layoutRawTableWidth";
	static const char _LN_copy_back[] = "copy-back";
	static const char _LN_aftncn[] = "aftncn";
	static const char _LN_tabLst[] = "tabLst";
	static const char _LN_echo_char[] = "echo-char";
	static const char _LN_CommandType[] = "CommandType";
	static const char _LN_useltbaln[] = "useltbaln";
	static const char _LN_DefaultItem[] = "DefaultItem";
	static const char _LN_futureMetadata[] = "futureMetadata";
	static const char _LN_revdttm[] = "revdttm";
	static const char _LN_ls[] = "ls";
	static const char _LN_reference_mark_end[] = "reference-mark-end";
	static const char _LN_Position[] = "Position";
	static const char _LN_Task[] = "Task";
	static const char _LN_language_asian[] = "language-asian";
	static const char _LN_viewPrBase[] = "viewPrBase";
	static const char _LN_Destination[] = "Destination";
	static const char _LN_copy_styles[] = "copy-styles";
	static const char _LN_defRPr[] = "defRPr";
	static const char _LN_outline_style[] = "outline-style";
	static const char _LN_ltrch[] = "ltrch";
	static const char _LN_file[] = "file";
	static const char _LN_visible_area_height[] = "visible-area-height";
	static const char _LN_shapedefaults[] = "shapedefaults";
	static const char _LN_revdttmdel[] = "revdttmdel";
	static const char _LN_CommandText[] = "CommandText";
	static const char _LN_userprops[] = "userprops";
	static const char _LN_IsNull[] = "IsNull";
	static const char _LN_AllItemName[] = "AllItemName";
	static const char _LN_layoutTableRowsApart[] = "layoutTableRowsApart";
	static const char _LN_Alignment[] = "Alignment";
	static const char _LN_light[] = "light";
	static const char _LN_ltrdoc[] = "ltrdoc";
	static const char _LN_br[] = "br";
	static const char _LN_formatted_text[] = "formatted-text";
	static const char _LN_language_complex[] = "language-complex";
	static const char _LN_note[] = "note";
	static const char _LN_guide[] = "guide";
	static const char _LN_aftnnalc[] = "aftnnalc";
	static const char _LN_copy_formulas[] = "copy-formulas";
	static const char _LN_metadataType[] = "metadataType";
	static const char _LN_CreateDate[] = "CreateDate";
	static const char _LN_outline_level_style[] = "outline-level-style";
	static const char _LN_diffuse_color[] = "diffuse-color";
	static const char _LN_revised[] = "revised";
	static const char _LN_Parameter[] = "Parameter";
	static const char _LN_colormru[] = "colormru";
	static const char _LN_Leader[] = "Leader";
	static const char _LN_Dimension[] = "Dimension";
	static const char _LN_ltrmark[] = "ltrmark";
	static const char _LN_note_citation[] = "note-citation";
	static const char _LN_ole_draw_aspect[] = "ole-draw-aspect";
	static const char _LN_guideLst[] = "guideLst";
	static const char _LN_vern[] = "vern";
	static const char _LN_aftnnar[] = "aftnnar";
	static const char _LN_useWord97LineBreakingRules[] = "useWord97LineBreakingRules";
	static const char _LN_max_value[] = "max-value";
	static const char _LN_Contact[] = "Contact";
	static const char _LN_comment[] = "comment";
	static const char _LN_country_asian[] = "country-asian";
	static const char _LN_X[] = "X";
	static const char _LN_colormenu[] = "colormenu";
	static const char _LN_SourceFile[] = "SourceFile";
	static const char _LN_ltrpar[] = "ltrpar";
	static const char _LN_DontShowInFieldList[] = "DontShowInFieldList";
	static const char _LN_revprop[] = "revprop";
	static const char _LN_sldViewPrBase[] = "sldViewPrBase";
	static const char _LN_graphic_properties[] = "graphic-properties";
	static const char _LN_aftnnauc[] = "aftnnauc";
	static const char _LN_Y[] = "Y";
	static const char _LN_label[] = "label";
	static const char _LN_vertalb[] = "vertalb";
	static const char _LN_chart_properties[] = "chart-properties";
	static const char _LN_WBS[] = "WBS";
	static const char _LN_shapes[] = "shapes";
	static const char _LN_min_value[] = "min-value";
	static const char _LN_ltrrow[] = "ltrrow";
	static const char _LN_colors[] = "colors";
	static const char _LN_specular[] = "specular";
	static const char _LN_A[] = "A";
	static const char _LN_HeaderFile[] = "HeaderFile";
	static const char _LN_country_complex[] = "country-complex";
	static const char _LN_revprot[] = "revprot";
	static const char _LN_note_body[] = "note-body";
	static const char _LN_WBSLevel[] = "WBSLevel";
	static const char _LN_vertalc[] = "vertalc";
	static const char _LN_aftnnchi[] = "aftnnchi";
	static const char _LN_normalViewPr[] = "normalViewPr";
	static const char _LN_B[] = "B";
	static const char _LN_cell_range_source[] = "cell-range-source";
	static const char _LN_breakWrappedTables[] = "breakWrappedTables";
	static const char _LN_Measure[] = "Measure";
	static const char _LN_txBody[] = "txBody";
	static const char _LN_validation[] = "validation";
	static const char _LN_ltrsect[] = "ltrsect";
	static const char _LN_shapelayout[] = "shapelayout";
	static const char _LN_drawing_page_properties[] = "drawing-page-properties";
	static const char _LN_cube[] = "cube";
	static const char _LN_revtbl[] = "revtbl";
	static const char _LN_scale_text[] = "scale-text";
	static const char _LN_k[] = "k";
	static const char _LN_MailMerge[] = "MailMerge";
	static const char _LN_C[] = "C";
	static const char _LN_note_class[] = "note-class";
	static const char _LN_vertalj[] = "vertalj";
	static const char _LN_OutlineNumber[] = "OutlineNumber";
	static const char _LN_aftnnchosung[] = "aftnnchosung";
	static const char _LN_xfrm[] = "xfrm";
	static const char _LN_EnableMultiplePageItems[] = "EnableMultiplePageItems";
	static const char _LN_min_edge[] = "min-edge";
	static const char _LN_slideViewPr[] = "slideViewPr";
	static const char _LN_lytcalctblwd[] = "lytcalctblwd";
	static const char _LN_D[] = "D";
	static const char _LN_revtim[] = "revtim";
	static const char _LN_font_style_asian[] = "font-style-asian";
	static const char _LN_idmap[] = "idmap";
	static const char _LN_last_column_spanned[] = "last-column-spanned";
	static const char _LN_ib[] = "ib";
	static const char _LN_three_dimensional[] = "three-dimensional";
	static const char _LN_snapToGridInCell[] = "snapToGridInCell";
	static const char _LN_vertalt[] = "vertalt";
	static const char _LN_DirX[] = "DirX";
	static const char _LN_useSpRect[] = "useSpRect";
	static const char _LN_t[] = "t";
	static const char _LN_ruby[] = "ruby";
	static const char _LN_aftnncnum[] = "aftnncnum";
	static const char _LN_OutlineLevel[] = "OutlineLevel";
	static const char _LN_AlwaysUseConnectionFile[] = "AlwaysUseConnectionFile";
	static const char _LN_max_edge[] = "max-edge";
	static const char _LN_gradient[] = "gradient";
	static const char _LN_ri[] = "ri";
	static const char _LN_regrouptable[] = "regrouptable";
	static const char _LN_outlineViewPr[] = "outlineViewPr";
	static const char _LN_lytexcttp[] = "lytexcttp";
	static const char _LN_DirY[] = "DirY";
	static const char _LN_ruby_base[] = "ruby-base";
	static const char _LN_vertdoc[] = "vertdoc";
	static const char _LN_anvPr[] = "anvPr";
	static const char _LN_font_style_complex[] = "font-style-complex";
	static const char _LN_rin[] = "rin";
	static const char _LN_GroupLevel[] = "GroupLevel";
	static const char _LN_dontAllowFieldEndSelect[] = "dontAllowFieldEndSelect";
	static const char _LN_aftnndbar[] = "aftnndbar";
	static const char _LN_sphere[] = "sphere";
	static const char _LN_deep[] = "deep";
	static const char _LN_last_row_spanned[] = "last-row-spanned";
	static const char _LN_Priority[] = "Priority";
	static const char _LN_Type[] = "Type";
	static const char _LN_lytprtmet[] = "lytprtmet";
	static const char _LN_rules[] = "rules";
	static const char _LN_vertsect[] = "vertsect";
	static const char _LN_notesTextViewPr[] = "notesTextViewPr";
	static const char _LN_ruby_text[] = "ruby-text";
	static const char _LN_anvSpPr[] = "anvSpPr";
	static const char _LN_fixed_text[] = "fixed-text";
	static const char _LN_symbol_type[] = "symbol-type";
	static const char _LN_CredentialsMethod[] = "CredentialsMethod";
	static const char _LN_center[] = "center";
	static const char _LN_rquote[] = "rquote";
	static const char _LN_IsGroupLevel[] = "IsGroupLevel";
	static const char _LN_AutoGen[] = "AutoGen";
	static const char _LN_aftnndbnum[] = "aftnndbnum";
	static const char _LN_entry[] = "entry";
	static const char _LN_Duration[] = "Duration";
	static const char _LN_font_relief[] = "font-relief";
	static const char _LN_lyttblrtgr[] = "lyttblrtgr";
	static const char _LN_viewkind[] = "viewkind";
	static const char _LN_nvSpPr[] = "nvSpPr";
	static const char _LN_detective[] = "detective";
	static const char _LN_multi_line[] = "multi-line";
	static const char _LN_size[] = "size";
	static const char _LN_applyBreakingRules[] = "applyBreakingRules";
	static const char _LN_Prompt[] = "Prompt";
	static const char _LN_sorterViewPr[] = "sorterViewPr";
	static const char _LN_start_color[] = "start-color";
	static const char _LN_symbol_name[] = "symbol-name";
	static const char _LN_date_value[] = "date-value";
	static const char _LN_aftnndbnumd[] = "aftnndbnumd";
	static const char _LN_GroupDefinition[] = "GroupDefinition";
	static const char _LN_Work[] = "Work";
	static const char _LN_rsidroot[] = "rsidroot";
	static const char _LN_spPr[] = "spPr";
	static const char _LN_SSOApplicationID[] = "SSOApplicationID";
	static const char _LN_viewnobound[] = "viewnobound";
	static const char _LN_proxy[] = "proxy";
	static const char _LN_text_shadow[] = "text-shadow";
	static const char _LN_pivotCacheDefinition[] = "pivotCacheDefinition";
	static const char _LN_mac[] = "mac";
	static const char _LN_extrude[] = "extrude";
	static const char _LN_operation[] = "operation";
	static const char _LN_Value[] = "Value";
	static const char _LN_combobox[] = "combobox";
	static const char _LN_Stop[] = "Stop";
	static const char _LN_txsp[] = "txsp";
	static const char _LN_end_color[] = "end-color";
	static const char _LN_symbol_image[] = "symbol-image";
	static const char _LN_notesViewPr[] = "notesViewPr";
	static const char _LN_date_adjust[] = "date-adjust";
	static const char _LN_macpict[] = "macpict";
	static const char _LN_rsidtbl[] = "rsidtbl";
	static const char _LN_skew[] = "skew";
	static const char _LN_aftnndbnumk[] = "aftnndbnumk";
	static const char _LN_wrapTextWithPunct[] = "wrapTextWithPunct";
	static const char _LN_rotate[] = "rotate";
	static const char _LN_LengthLevelUniqueName[] = "LengthLevelUniqueName";
	static const char _LN_EditMode[] = "EditMode";
	static const char _LN_viewscale[] = "viewscale";
	static const char _LN_index[] = "index";
	static const char _LN_text_underline_type[] = "text-underline-type";
	static const char _LN_Resume[] = "Resume";
	static const char _LN_auto_complete[] = "auto-complete";
	static const char _LN_Name[] = "Name";
	static const char _LN_makebackup[] = "makebackup";
	static const char _LN_start_intensity[] = "start-intensity";
	static const char _LN_pivotCacheRecords[] = "pivotCacheRecords";
	static const char _LN_rsltbmp[] = "rsltbmp";
	static const char _LN_gridSpacing[] = "gridSpacing";
	static const char _LN_time[] = "time";
	static const char _LN_custom_shape[] = "custom-shape";
	static const char _LN_anvCxnSpPr[] = "anvCxnSpPr";
	static const char _LN_symbol_width[] = "symbol-width";
	static const char _LN_highlighted_range[] = "highlighted-range";
	static const char _LN_aftnndbnumt[] = "aftnndbnumt";
	static const char _LN_callout[] = "callout";
	static const char _LN_ResumeValid[] = "ResumeValid";
	static const char _LN_Format[] = "Format";
	static const char _LN_DataType[] = "DataType";
	static const char _LN_viewzk[] = "viewzk";
	static const char _LN_useAsianBreakRules[] = "useAsianBreakRules";
	static const char _NS_anium[] = "urn:oasis:names:tc:opendocument:xmlns:animation:1.0";
	static const char _NS_math[] = "http://www.w3.org/1998/Math/MathML";
	static const char _NS_w10[] = "urn:schemas-microsoft-com:office:word";
	static const char _NS_chart[] = "urn:oasis:names:tc:opendocument:xmlns:chart:1.0";
	static const char _NS_sc12[] = "http://schemas.microsoft.com/office/excel/2005/8/singlecells";
	static const char _NS_odc12[] = "urn:schemas-microsoft-com:office:odc";
	static const char _NS_c11[] = "urn:schemas-microsoft-com:office:component:spreadsheet";
	static const char _NS_rng[] = "http://relaxng.org/ns/structure/1.0";
	static const char _NS_meta[] = "urn:oasis:names:tc:opendocument:xmlns:meta:1.0";
	static const char _NS_udcs[] = "http://schemas.microsoft.com/data/udc/soap";
	static const char _NS_xlink[] = "http://www.w3.org/1999/xlink";
	static const char _NS_style[] = "urn:oasis:names:tc:opendocument:xmlns:style:1.0";
	static const char _NS_fo[] = "urn:oasis:names:tc:opendocument:xmlns:xsl-fo-compatible:1.0";
	static const char _NS_xforms[] = "http://www.w3.org/2002/xforms";
	static const char _NS_draw[] = "urn:oasis:names:tc:opendocument:xmlns:drawing:1.0";
	static const char _NS_prj12[] = "http://schemas.microsoft.com/project";
	static const char _NS_mdc12[] = "http://schemas.microsoft.com/package/2005/06/metadata/core-properties";
	static const char _NS_table[] = "urn:oasis:names:tc:opendocument:xmlns:table:1.0";
	static const char _NS_rs11[] = "urn:schemas-microsoft-com:rowset";
	static const char _NS_sxr12[] = "http://schemas.microsoft.com/office/excel/2005/8/sxref";
	static const char _NS_xldr12[] = "http://schemas.microsoft.com/office/officeart/2005/8/xldr";
	static const char _NS_m12[] = "http://schemas.microsoft.com/office/omml/2004/12/core";
	static const char _NS_number[] = "urn:oasis:names:tc:opendocument:xmlns:datastyle:1.0";
	static const char _NS_p12[] = "http://schemas.microsoft.com/package/2005/06/content-types";
	static const char _NS_voldeps12[] = "http://schemas.microsoft.com/office/excel/2005/8/voldeps";
	static const char _NS_pt12[] = "http://schemas.microsoft.com/office/excel/2005/8/pivot";
	static const char _NS_xml[] = "http://www.w3.org/XML/1998/namespace";
	static const char _NS_w11[] = "http://schemas.microsoft.com/office/word/2003/wordml";
	static const char _NS_sl12[] = "http://schemas.microsoft.com/schemaLibrary/2003/core";
	static const char _NS_s12[] = "http://schemas.microsoft.com/office/excel/2005/8/styles";
	static const char _NS_udc[] = "http://schemas.microsoft.com/data/udc";
	static const char _NS_dr3d[] = "urn:oasis:names:tc:opendocument:xmlns:dr3d:1.0";
	static const char _NS_rdf[] = "http://www.w3.org/1999/02/22-rdf-syntax-ns#";
	static const char _NS_sb12[] = "http://schemas.microsoft.com/office/excel/2005/8/supbook";
	static const char _NS_wb12[] = "http://schemas.microsoft.com/office/excel/2005/8/workbook";
	static const char _NS_RowsetSchema[] = "#RowsetSchema";
	static const char _NS_ec12[] = "http://schemas.microsoft.com/office/excel/2005/8/externalconnection";
	static const char _NS_qt12[] = "http://schemas.microsoft.com/office/excel/2005/8/querytable";
	static const char _NS_o12[] = "urn:schemas-microsoft-com:office:office";
	static const char _NS_s11[] = "uuid:BDC6E3F0-6DA3-11d1-A2A3-00AA00C14882";
	static const char _NS_x11[] = "urn:schemas-microsoft-com:office:excel";
	static const char _NS_ss11[] = "urn:schemas-microsoft-com:office:spreadsheet";
	static const char _NS_rtf[] = "urn:rtf:1.7";
	static const char _NS_smil[] = "urn:oasis:names:tc:opendocument:xmlns:smil-compatible:1.0";
	static const char _NS_w12[] = "http://schemas.microsoft.com/office/word/2005/8/25/wordml";
	static const char _NS_rtfs[] = "urn:rtf:schema";
	static const char _NS_x2[] = "http://schemas.microsoft.com/office/excel/2003/xml";
	static const char _NS_ip12[] = "http://schemas.microsoft.com/office/infopath/2003/solutionDefinition";
	static const char _NS_rel12[] = "http://schemas.microsoft.com/package/2005/06/relationships";
	static const char _NS_config[] = "urn:oasis:names:tc:opendocument:xmlns:config:1.0";
	static const char _NS_text[] = "urn:oasis:names:tc:opendocument:xmlns:text:1.0";
	static const char _NS_tbl12[] = "http://schemas.microsoft.com/office/excel/2005/8/table";
	static const char _NS_pr[] = "urn:oasis:names:tc:opendocument:xmlns:presentation:1.0";
	static const char _NS_udcxf11[] = "http://schemas.microsoft.com/data/udc/xmlfile";
	static const char _NS_office[] = "urn:oasis:names:tc:opendocument:xmlns:office:1.0";
	static const char _NS_md12[] = "http://schemas.microsoft.com/office/excel/2005/8/metadata";
	static const char _NS_pp12[] = "http://schemas.microsoft.com/office/powerpoint/2005/8/pml";
	static const char _NS_v11[] = "http://schemas.microsoft.com/visio/2003/core";
	static const char _NS_sst12[] = "http://schemas.microsoft.com/office/excel/2005/8/sst";
	static const char _NS_swb12[] = "http://schemas.microsoft.com/office/excel/2005/8/sharedworkbook";
	static const char _NS_pic12[] = "http://schemas.microsoft.com/office/officeart/2005/8/picture";
	static const char _NS_vml[] = "urn:schemas-microsoft-com:vml";
	static const char _NS_ax12[] = "http://schemas.microsoft.com/office/activeX/2005/ax";
	static const char _NS_ws12[] = "http://schemas.microsoft.com/office/excel/2005/8/worksheet";
	static const char _NS_wddr12[] = "http://schemas.microsoft.com/office/officeart/2005/8/wddr";
	static const char _NS_a12[] = "http://schemas.microsoft.com/office/officeart/2005/8/oartml";
	static const char _NS_svg[] = "urn:oasis:names:tc:opendocument:xmlns:svg-compatible:1.0";
	static const char _NS_on12[] = "http://schemas.microsoft.com/office/onenote/2004/import";
	static const char _NS_form[] = "urn:oasis:names:tc:opendocument:xmlns:form:1.0";
	static const char _NS_script[] = "urn:oasis:names:tc:opendocument:xmlns:script:1.0";
	static const char _NS_aml12[] = "http://schemas.microsoft.com/aml/2001/core";
	static const char _NS_dc[] = "http://purl.org/dc/elements/1.1/";
	static const char _NS_wa11[] = "http://schemas.microsoft.com/office/word/2003/auxHint";
	static const char _NS_swbu12[] = "http://schemas.microsoft.com/office/excel/2005/8/sharedworkbookusers";
	static const char _NS_c12[] = "http://schemas.microsoft.com/office/2004/7/core";


	static const char _LN_anium[] = "anium";
	static const char _LN_w10[] = "w10";
	static const char _LN_sc12[] = "sc12";
	static const char _LN_odc12[] = "odc12";
	static const char _LN_c11[] = "c11";
	static const char _LN_rng[] = "rng";
	static const char _LN_udcs[] = "udcs";
	static const char _LN_xlink[] = "xlink";
	static const char _LN_fo[] = "fo";
	static const char _LN_xforms[] = "xforms";
	static const char _LN_draw[] = "draw";
	static const char _LN_prj12[] = "prj12";
	static const char _LN_mdc12[] = "mdc12";
	static const char _LN_rs11[] = "rs11";
	static const char _LN_sxr12[] = "sxr12";
	static const char _LN_xldr12[] = "xldr12";
	static const char _LN_m12[] = "m12";
	static const char _LN_p12[] = "p12";
	static const char _LN_voldeps12[] = "voldeps12";
	static const char _LN_pt12[] = "pt12";
	static const char _LN_xml[] = "xml";
	static const char _LN_w11[] = "w11";
	static const char _LN_sl12[] = "sl12";
	static const char _LN_s12[] = "s12";
	static const char _LN_udc[] = "udc";
	static const char _LN_dr3d[] = "dr3d";
	static const char _LN_sb12[] = "sb12";
	static const char _LN_wb12[] = "wb12";
	static const char _LN_RowsetSchema[] = "RowsetSchema";
	static const char _LN_ec12[] = "ec12";
	static const char _LN_qt12[] = "qt12";
	static const char _LN_o12[] = "o12";
	static const char _LN_s11[] = "s11";
	static const char _LN_x11[] = "x11";
	static const char _LN_ss11[] = "ss11";
	static const char _LN_smil[] = "smil";
	static const char _LN_w12[] = "w12";
	static const char _LN_rtfs[] = "rtfs";
	static const char _LN_ip12[] = "ip12";
	static const char _LN_rel12[] = "rel12";
	static const char _LN_config[] = "config";
	static const char _LN_tbl12[] = "tbl12";
	static const char _LN_pr[] = "pr";
	static const char _LN_udcxf11[] = "udcxf11";
	static const char _LN_office[] = "office";
	static const char _LN_md12[] = "md12";
	static const char _LN_pp12[] = "pp12";
	static const char _LN_v11[] = "v11";
	static const char _LN_sst12[] = "sst12";
	static const char _LN_swb12[] = "swb12";
	static const char _LN_pic12[] = "pic12";
	static const char _LN_vml[] = "vml";
	static const char _LN_ax12[] = "ax12";
	static const char _LN_ws12[] = "ws12";
	static const char _LN_wddr12[] = "wddr12";
	static const char _LN_a12[] = "a12";
	static const char _LN_svg[] = "svg";
	static const char _LN_on12[] = "on12";
	static const char _LN_aml12[] = "aml12";
	static const char _LN_dc[] = "dc";
	static const char _LN_wa11[] = "wa11";
	static const char _LN_swbu12[] = "swbu12";
	static const char _LN_c12[] = "c12";


	struct QNameTuple{ const char* namespaceUri; const char* localName; };
#define QNAMETABLESIZE 20000
#define QNAME_SIZE_STATIC 6873
	static QNameTuple qNameTable[QNAMETABLESIZE] = {
		{_NS_ec12, _LN_webqProps}, 
		{_NS_x11, _LN_CacheDetails}, 
		{_NS_rtf, _LN_tsvertalb}, 
		{_NS_form, _LN_detail_fields}, 
		{_NS_dr3d, _LN_distance}, 
		{_NS_fo, _LN_clip}, 
		{_NS_ec12, _LN_txtWiz}, 
		{_NS_wb12, _LN_supBookSource}, 
		{_NS_w12, _LN_truncateFontHeight}, 
		{_NS_rtf, _LN_tsvertalc}, 
		{_NS_x11, _LN_FieldListTop}, 
		{_NS_dr3d, _LN_focal_length}, 
		{_NS_ec12, _LN_qParams}, 
		{_NS_form, _LN_escape_processing}, 
		{_NS_rtf, _LN_tsvertalt}, 
		{_NS_w12, _LN_mwSmallCaps}, 
		{_NS_wb12, _LN_imageRelation}, 
		{_NS_a12, _LN_spcPct}, 
		{_NS_form, _LN_filter}, 
		{_NS_ec12, _LN_fsb}, 
		{_NS_x11, _LN_FieldListBottom}, 
		{_NS_draw, _LN_wrap_influence_on_position}, 
		{_NS_dr3d, _LN_shadow_slant}, 
		{_NS_rtf, _LN_twoonone}, 
		{_NS_a12, _LN_spcPts}, 
		{_NS_wb12, _LN_pivotCache}, 
		{_NS_ec12, _LN_tables}, 
		{_NS_rtf, _LN_tx}, 
		{_NS_w12, _LN_usePrinterMetrics}, 
		{_NS_form, _LN_ignore_result}, 
		{_NS_a12, _LN_tab}, 
		{_NS_x11, _LN_FieldListRight}, 
		{_NS_dr3d, _LN_shade_mode}, 
		{_NS_ec12, _LN_qParam}, 
		{_NS_rtf, _LN_txe}, 
		{_NS_draw, _LN_frame_display_scrollbar}, 
		{_NS_wb12, _LN_docRouteInfo}, 
		{_NS_ec12, _LN_m}, 
		{_NS_a12, _LN_lineSpc}, 
		{_NS_rtf, _LN_uc}, 
		{_NS_w12, _LN_ww6BorderRules}, 
		{_NS_form, _LN_navigation_mode}, 
		{_NS_ec12, _LN_s}, 
		{_NS_x11, _LN_FieldListLeft}, 
		{_NS_dr3d, _LN_ambient_color}, 
		{_NS_rtf, _LN_ud}, 
		{_NS_a12, _LN_spcBef}, 
		{_NS_form, _LN_order}, 
		{_NS_ec12, _LN_x}, 
		{_NS_wb12, _LN_docRouteOriginator}, 
		{_NS_rtf, _LN_ul}, 
		{_NS_draw, _LN_frame_display_border}, 
		{_NS_x11, _LN_PivotAxis}, 
		{_NS_w12, _LN_wrapTrailSpaces}, 
		{_NS_a12, _LN_spcAft}, 
		{_NS_ec12, _LN_fldInfos}, 
		{_NS_dr3d, _LN_lighting_mode}, 
		{_NS_form, _LN_tab_cycle}, 
		{_NS_rtf, _LN_ulc}, 
		{_NS_dr3d, _LN_transform}, 
		{_NS_a12, _LN_tabLst}, 
		{_NS_ec12, _LN_fldInfo}, 
		{_NS_rtf, _LN_uld}, 
		{_NS_wb12, _LN_docRouteRecipient}, 
		{_NS_dr3d, _LN_light}, 
		{_NS_x11, _LN_VersionUpdateableMin}, 
		{_NS_draw, _LN_frame_margin_horizontal}, 
		{_NS_w12, _LN_footnoteLayoutLikeWW8}, 
		{_NS_a12, _LN_defRPr}, 
		{_NS_form, _LN_connection_resource}, 
		{_NS_rtf, _LN_uldash}, 
		{_NS_a12, _LN_br}, 
		{_NS_xforms, _LN_model}, 
		{_NS_dr3d, _LN_diffuse_color}, 
		{_NS_a12, _LN_field}, 
		{_NS_rtf, _LN_uldashd}, 
		{_NS_x11, _LN_VersionLastUpdate}, 
		{_NS_form, _LN_text}, 
		{_NS_dr3d, _LN_direction}, 
		{_NS_w12, _LN_shapeLayoutLikeWW8}, 
		{_NS_a12, _LN_math}, 
		{_NS_draw, _LN_frame_margin_vertical}, 
		{_NS_dr3d, _LN_enabled}, 
		{_NS_rtf, _LN_uldashdd}, 
		{_NS_form, _LN_textarea}, 
		{_NS_form, _LN_password}, 
		{_NS_rtf, _LN_uldb}, 
		{_NS_dr3d, _LN_specular}, 
		{_NS_x11, _LN_VersionRefreshableMin}, 
		{_NS_draw, _LN_visible_area_left}, 
		{_NS_w12, _LN_alignTablesRowByRow}, 
		{_NS_odc12, _LN_OfficeDataConnection}, 
		{_NS_dr3d, _LN_cube}, 
		{_NS_md12, _LN_Metadata}, 
		{_NS_rtf, _LN_ulhair}, 
		{_NS_form, _LN_echo_char}, 
		{_NS_x11, _LN_DefaultVersion}, 
		{_NS_odc12, _LN_Connection}, 
		{_NS_draw, _LN_visible_area_top}, 
		{_NS_form, _LN_file}, 
		{_NS_dr3d, _LN_min_edge}, 
		{_NS_rtf, _LN_ulhwave}, 
		{_NS_md12, _LN_metadataTypes}, 
		{_NS_w12, _LN_forgetLastTabAlignment}, 
		{_NS_x11, _LN_PivotField}, 
		{_NS_rtf, _LN_ulldash}, 
		{_NS_dr3d, _LN_max_edge}, 
		{_NS_a12, _LN_t}, 
		{_NS_odc12, _LN_ConnectionString}, 
		{_NS_form, _LN_formatted_text}, 
		{_NS_draw, _LN_visible_area_width}, 
		{_NS_md12, _LN_cellMetadata}, 
		{_NS_x11, _LN_CubeField}, 
		{_NS_rtf, _LN_ulnone}, 
		{_NS_dr3d, _LN_sphere}, 
		{_NS_w12, _LN_adjustLineHeightInTable}, 
		{_NS_form, _LN_max_value}, 
		{_NS_odc12, _LN_CommandType}, 
		{_NS_rtf, _LN_ulth}, 
		{_NS_dr3d, _LN_center}, 
		{_NS_md12, _LN_valueMetadata}, 
		{_NS_draw, _LN_visible_area_height}, 
		{_NS_x11, _LN_CalculatedMember}, 
		{_NS_form, _LN_min_value}, 
		{_NS_rtf, _LN_ulthd}, 
		{_NS_dr3d, _LN_size}, 
		{_NS_odc12, _LN_CommandText}, 
		{_NS_w12, _LN_autoSpaceLikeWord95}, 
		{_NS_md12, _LN_mdxMetadata}, 
		{_NS_x11, _LN_MemberName}, 
		{_NS_rtf, _LN_ulthdash}, 
		{_NS_dr3d, _LN_extrude}, 
		{_NS_form, _LN_validation}, 
		{_NS_draw, _LN_ole_draw_aspect}, 
		{_NS_odc12, _LN_Parameter}, 
		{_NS_form, _LN_number}, 
		{_NS_md12, _LN_strMetadata}, 
		{_NS_dr3d, _LN_rotate}, 
		{_NS_w12, _LN_noSpaceRaiseLower}, 
		{_NS_form, _LN_value}, 
		{_NS_rtf, _LN_ulthdashd}, 
		{_NS_odc12, _LN_SourceFile}, 
		{_NS_x11, _LN_ParentUniqueName}, 
		{_NS_form, _LN_current_value}, 
		{_NS_style, _LN_chart_properties}, 
		{_NS_form, _LN_date}, 
		{_NS_md12, _LN_futureMetadata}, 
		{_NS_draw, _LN_custom_shape}, 
		{_NS_rtf, _LN_ulthdashdd}, 
		{_NS_odc12, _LN_HeaderFile}, 
		{_NS_form, _LN_time}, 
		{_NS_md12, _LN_fsb}, 
		{_NS_x11, _LN_SolveOrder}, 
		{_NS_chart, _LN_scale_text}, 
		{_NS_draw, _LN_engine}, 
		{_NS_rtf, _LN_ulthldash}, 
		{_NS_w12, _LN_doNotUseHTMLParagraphAutoSpacing}, 
		{_NS_odc12, _LN_MailMerge}, 
		{_NS_x11, _LN_Invalid}, 
		{_NS_draw, _LN_data}, 
		{_NS_form, _LN_fixed_text}, 
		{_NS_md12, _LN_metadataType}, 
		{_NS_odc12, _LN_Version}, 
		{_NS_chart, _LN_three_dimensional}, 
		{_NS_x11, _LN_Set}, 
		{_NS_md12, _LN_b}, 
		{_NS_rtf, _LN_ululdbwave}, 
		{_NS_form, _LN_multi_line}, 
		{_NS_md12, _LN_r}, 
		{_NS_chart, _LN_deep}, 
		{_NS_w12, _LN_layoutRawTableWidth}, 
		{_NS_draw, _LN_enhanced_geometry}, 
		{_NS_rtf, _LN_ulw}, 
		{_NS_md12, _LN_t}, 
		{_NS_x11, _LN_PTLineItems}, 
		{_NS_form, _LN_combobox}, 
		{_NS_md12, _LN_s}, 
		{_NS_odc12, _LN_AlwaysUseConnectionFile}, 
		{_NS_rtf, _LN_ulwave}, 
		{_NS_chart, _LN_symbol_type}, 
		{_NS_md12, _LN_p}, 
		{_NS_x11, _LN_PTSource}, 
		{_NS_rtf, _LN_u}, 
		{_NS_draw, _LN_mirror_vertical}, 
		{_NS_c12, _LN_SmartTagType}, 
		{_NS_w12, _LN_layoutTableRowsApart}, 
		{_NS_md12, _LN_k}, 
		{_NS_rtf, _LN_up}, 
		{_NS_form, _LN_auto_complete}, 
		{_NS_md12, _LN_i}, 
		{_NS_chart, _LN_symbol_name}, 
		{_NS_odc12, _LN_CredentialsMethod}, 
		{_NS_x11, _LN_PTFormula}, 
		{_NS_rtf, _LN_upr}, 
		{_NS_c12, _LN_schemaRef}, 
		{_NS_form, _LN_item}, 
		{_NS_md12, _LN_ib}, 
		{_NS_draw, _LN_mirror_horizontal}, 
		{_NS_rtf, _LN_urtf}, 
		{_NS_x11, _LN_PTFormat}, 
		{_NS_chart, _LN_symbol_image}, 
		{_NS_form, _LN_listbox}, 
		{_NS_w12, _LN_useWord97LineBreakingRules}, 
		{_NS_odc12, _LN_SSOApplicationID}, 
		{_NS_rtf, _LN_useltbaln}, 
		{_NS_x11, _LN_CacheFile}, 
		{_NS_draw, _LN_text_rotate_angle}, 
		{_NS_odc12, _LN_Name}, 
		{_NS_form, _LN_multiple}, 
		{_NS_chart, _LN_symbol_width}, 
		{_NS_w12, _LN_breakWrappedTables}, 
		{_NS_rtf, _LN_userprops}, 
		{_NS_odc12, _LN_DataType}, 
		{_NS_x11, _LN_QuerySource}, 
		{_NS_rtf, _LN_v}, 
		{_NS_draw, _LN_extrusion_allowed}, 
		{_NS_chart, _LN_symbol_height}, 
		{_NS_form, _LN_xforms_list_source}, 
		{_NS_m12, _LN_nor}, 
		{_NS_rtf, _LN_vern}, 
		{_NS_w12, _LN_snapToGridInCell}, 
		{_NS_x11, _LN_DefaultItem}, 
		{_NS_chart, _LN_vertical}, 
		{_NS_form, _LN_option}, 
		{_NS_rtf, _LN_version}, 
		{_NS_m12, _LN_lit}, 
		{_NS_draw, _LN_text_path_allowed}, 
		{_NS_pt12, _LN_pivotCacheDefinition}, 
		{_NS_x11, _LN_AllItemName}, 
		{_NS_form, _LN_button}, 
		{_NS_m12, _LN_scr}, 
		{_NS_rtf, _LN_vertalb}, 
		{_NS_chart, _LN_connect_bars}, 
		{_NS_w12, _LN_dontAllowFieldEndSelect}, 
		{_NS_m12, _LN_sty}, 
		{_NS_rtf, _LN_vertalc}, 
		{_NS_x11, _LN_Dimension}, 
		{_NS_form, _LN_default_button}, 
		{_NS_chart, _LN_gap_width}, 
		{_NS_m12, _LN_brk}, 
		{_NS_o12, _LN_DocumentProperties}, 
		{_NS_pt12, _LN_pivotCacheRecords}, 
		{_NS_draw, _LN_concentric_gradient_fill_allowed}, 
		{_NS_o12, _LN_Name}, 
		{_NS_rtf, _LN_vertalj}, 
		{_NS_m12, _LN_aln}, 
		{_NS_w12, _LN_applyBreakingRules}, 
		{_NS_chart, _LN_overlap}, 
		{_NS_o12, _LN_Description}, 
		{_NS_m12, _LN_rPr}, 
		{_NS_x11, _LN_DontShowInFieldList}, 
		{_NS_o12, _LN_Keywords}, 
		{_NS_rtf, _LN_vertalt}, 
		{_NS_draw, _LN_extrusion}, 
		{_NS_m12, _LN_t}, 
		{_NS_pt12, _LN_pivotTableDefinition}, 
		{_NS_form, _LN_toggle}, 
		{_NS_m12, _LN_chr}, 
		{_NS_x11, _LN_Measure}, 
		{_NS_w12, _LN_wrapTextWithPunct}, 
		{_NS_rtf, _LN_vertdoc}, 
		{_NS_m12, _LN_accPr}, 
		{_NS_chart, _LN_japanese_candle_stick}, 
		{_NS_pt12, _LN_cacheSource}, 
		{_NS_m12, _LN_e}, 
		{_NS_draw, _LN_extrusion_brightness}, 
		{_NS_rtf, _LN_vertsect}, 
		{_NS_form, _LN_focus_on_click}, 
		{_NS_m12, _LN_pos}, 
		{_NS_pt12, _LN_fields}, 
		{_NS_chart, _LN_interpolation}, 
		{_NS_w12, _LN_useAsianBreakRules}, 
		{_NS_m12, _LN_barPr}, 
		{_NS_x11, _LN_EnableMultiplePageItems}, 
		{_NS_rtf, _LN_viewkind}, 
		{_NS_draw, _LN_extrusion_depth}, 
		{_NS_m12, _LN_opEmu}, 
		{_NS_form, _LN_xforms_submission}, 
		{_NS_pt12, _LN_hierarchies}, 
		{_NS_chart, _LN_spline_order}, 
		{_NS_x11, _LN_GroupLevel}, 
		{_NS_form, _LN_image}, 
		{_NS_rtf, _LN_viewnobound}, 
		{_NS_pt12, _LN_kpis}, 
		{_NS_m12, _LN_noBreak}, 
		{_NS_sl12, _LN_schemaLibrary}, 
		{_NS_w12, _LN_useWord2002TableStyleRules}, 
		{_NS_draw, _LN_extrusion_diffusion}, 
		{_NS_sl12, _LN_schema}, 
		{_NS_x11, _LN_IsGroupLevel}, 
		{_NS_form, _LN_checkbox}, 
		{_NS_m12, _LN_boxPr}, 
		{_NS_rtf, _LN_viewscale}, 
		{_NS_chart, _LN_spline_resolution}, 
		{_NS_pt12, _LN_sdTupleCache}, 
		{_NS_w12, _LN_dontGrowAutofit}, 
		{_NS_rtf, _LN_viewzk}, 
		{_NS_m12, _LN_hideTop}, 
		{_NS_form, _LN_current_state}, 
		{_NS_chart, _LN_pie_offset}, 
		{_NS_x11, _LN_GroupDefinition}, 
		{_NS_pt12, _LN_ciFmlas}, 
		{_NS_chart, _LN_lines}, 
		{_NS_rtf, _LN_wbitmap}, 
		{_NS_m12, _LN_hideBot}, 
		{_NS_w12, _LN_useFELayout}, 
		{_NS_draw, _LN_extrusion_number_of_line_segments}, 
		{_NS_form, _LN_is_tristate}, 
		{_NS_pt12, _LN_calcMbrs}, 
		{_NS_chart, _LN_solid_type}, 
		{_NS_pt12, _LN_fsb}, 
		{_NS_m12, _LN_hideLeft}, 
		{_NS_x11, _LN_LengthLevelUniqueName}, 
		{_NS_rtf, _LN_wbmbitspixel}, 
		{_NS_form, _LN_state}, 
		{_NS_pt12, _LN_field}, 
		{_NS_v11, _LN_VisioDocument}, 
		{_NS_chart, _LN_stacked}, 
		{_NS_w12, _LN_useNormalStyleForList}, 
		{_NS_draw, _LN_extrusion_light_face}, 
		{_NS_m12, _LN_hideRight}, 
		{_NS_form, _LN_radio}, 
		{_NS_x11, _LN_DisplayIn}, 
		{_NS_rtf, _LN_wbmplanes}, 
		{_NS_pt12, _LN_shItems}, 
		{_NS_form, _LN_frame}, 
		{_NS_pt12, _LN_group}, 
		{_NS_chart, _LN_percentage}, 
		{_NS_v11, _LN_SolutionXML}, 
		{_NS_m12, _LN_strikeH}, 
		{_NS_x11, _LN_GroupNumber}, 
		{_NS_rtf, _LN_wbmwidthbytes}, 
		{_NS_v11, _LN_cp}, 
		{_NS_pt12, _LN_wksht}, 
		{_NS_form, _LN_image_frame}, 
		{_NS_m12, _LN_strikeV}, 
		{_NS_draw, _LN_extrusion_first_light_harsh}, 
		{_NS_w12, _LN_dontUseIndentAsNumberingTabStop}, 
		{_NS_v11, _LN_pp}, 
		{_NS_pt12, _LN_consol}, 
		{_NS_x11, _LN_ParentName}, 
		{_NS_form, _LN_hidden}, 
		{_NS_rtf, _LN_webhidden}, 
		{_NS_v11, _LN_tp}, 
		{_NS_chart, _LN_link_data_style_to_source}, 
		{_NS_pt12, _LN_pages}, 
		{_NS_m12, _LN_strikeBLTR}, 
		{_NS_form, _LN_grid}, 
		{_NS_v11, _LN_fld}, 
		{_NS_form, _LN_column}, 
		{_NS_rtf, _LN_widctlpar}, 
		{_NS_x11, _LN_ParentIsOther}, 
		{_NS_chart, _LN_visible}, 
		{_NS_pt12, _LN_rngSets}, 
		{_NS_form, _LN_text_style_name}, 
		{_NS_w12, _LN_dontUseWord11FELineBreak}, 
		{_NS_m12, _LN_strikeTLBR}, 
		{_NS_v11, _LN_XProp}, 
		{_NS_draw, _LN_extrusion_second_light_harsh}, 
		{_NS_pt12, _LN_page}, 
		{_NS_rtf, _LN_widowctrl}, 
		{_NS_pt12, _LN_item}, 
		{_NS_v11, _LN_Text}, 
		{_NS_x11, _LN_GroupMember}, 
		{_NS_chart, _LN_logarithmic}, 
		{_NS_form, _LN_value_range}, 
		{_NS_m12, _LN_boxFormPr}, 
		{_NS_m12, _LN_begChr}, 
		{_NS_pt12, _LN_rngSet}, 
		{_NS_v11, _LN_XForm}, 
		{_NS_chart, _LN_maximum}, 
		{_NS_rtf, _LN_windowcaption}, 
		{_NS_m12, _LN_sepChr}, 
		{_NS_pt12, _LN_m}, 
		{_NS_x11, _LN_IsNotFiltered}, 
		{_NS_w12, _LN_allowSpaceOfSameStyleInTable}, 
		{_NS_draw, _LN_extrusion_first_light_level}, 
		{_NS_v11, _LN_Line}, 
		{_NS_m12, _LN_endChr}, 
		{_NS_pt12, _LN_n}, 
		{_NS_chart, _LN_minimum}, 
		{_NS_rtf, _LN_wmetafile}, 
		{_NS_m12, _LN_grow}, 
		{_NS_pt12, _LN_b}, 
		{_NS_v11, _LN_Fill}, 
		{_NS_form, _LN_step_size}, 
		{_NS_chart, _LN_origin}, 
		{_NS_m12, _LN_shp}, 
		{_NS_x11, _LN_TotalAllMembers}, 
		{_NS_pt12, _LN_e}, 
		{_NS_w12, _LN_ww11IndentRules}, 
		{_NS_rtf, _LN_wpeqn}, 
		{_NS_m12, _LN_dPr}, 
		{_NS_pt12, _LN_s}, 
		{_NS_v11, _LN_XForm1D}, 
		{_NS_m12, _LN_baseJc}, 
		{_NS_draw, _LN_extrusion_second_light_level}, 
		{_NS_pt12, _LN_d}, 
		{_NS_rtf, _LN_wpjst}, 
		{_NS_form, _LN_page_step_size}, 
		{_NS_chart, _LN_interval_major}, 
		{_NS_m12, _LN_maxDist}, 
		{_NS_v11, _LN_Event}, 
		{_NS_x11, _LN_DetailMaxHeight}, 
		{_NS_pt12, _LN_mbrs}, 
		{_NS_m12, _LN_objDist}, 
		{_NS_rtf, _LN_wpsp}, 
		{_NS_w12, _LN_dontAutofitConstrainedTables}, 
		{_NS_v11, _LN_LayerMem}, 
		{_NS_chart, _LN_interval_minor}, 
		{_NS_pt12, _LN_mpm}, 
		{_NS_form, _LN_delay_for_repeat}, 
		{_NS_m12, _LN_rSpRule}, 
		{_NS_x11, _LN_DetailMaxWidth}, 
		{_NS_rtf, _LN_wraptrsp}, 
		{_NS_form, _LN_orientation}, 
		{_NS_pt12, _LN_mpn}, 
		{_NS_draw, _LN_extrusion_first_light_direction}, 
		{_NS_m12, _LN_rSp}, 
		{_NS_v11, _LN_StyleProp}, 
		{_NS_w12, _LN_autofitLikeWW11}, 
		{_NS_rtf, _LN_wrppunct}, 
		{_NS_pt12, _LN_mpb}, 
		{_NS_m12, _LN_eqArrPr}, 
		{_NS_x11, _LN_NoDetailAutoFit}, 
		{_NS_chart, _LN_tick_marks_major_inner}, 
		{_NS_m12, _LN_type}, 
		{_NS_v11, _LN_Foreign}, 
		{_NS_form, _LN_generic_control}, 
		{_NS_rtf, _LN_xe}, 
		{_NS_pt12, _LN_mps}, 
		{_NS_m12, _LN_numJc}, 
		{_NS_form, _LN_name}, 
		{_NS_rtf, _LN_xef}, 
		{_NS_pt12, _LN_mpd}, 
		{_NS_m12, _LN_denJc}, 
		{_NS_w12, _LN_underlineTabInNumList}, 
		{_NS_v11, _LN_PageProps}, 
		{_NS_x11, _LN_DetailRowHeight}, 
		{_NS_draw, _LN_extrusion_second_light_direction}, 
		{_NS_m12, _LN_fPr}, 
		{_NS_rtf, _LN_yr}, 
		{_NS_chart, _LN_tick_marks_major_outer}, 
		{_NS_m12, _LN_num}, 
		{_NS_w12, _LN_docVar}, 
		{_NS_pt12, _LN_rangeDesc}, 
		{_NS_v11, _LN_TextBlock}, 
		{_NS_rtf, _LN_yts}, 
		{_NS_m12, _LN_den}, 
		{_NS_form, _LN_control_implementation}, 
		{_NS_x11, _LN_TotalFormat}, 
		{_NS_draw, _LN_extrusion_metal}, 
		{_NS_m12, _LN_fName}, 
		{_NS_rtf, _LN_yxe}, 
		{_NS_w12, _LN_rsidRoot}, 
		{_NS_xforms, _LN_bind}, 
		{_NS_pt12, _LN_discMapping}, 
		{_NS_v11, _LN_TextXForm}, 
		{_NS_m12, _LN_groupChrPr}, 
		{_NS_chart, _LN_tick_marks_minor_inner}, 
		{_NS_x11, _LN_MemberFormat}, 
		{_NS_rtf, _LN_zwbo}, 
		{_NS_w12, _LN_rsid}, 
		{_NS_m12, _LN_lim}, 
		{_NS_v11, _LN_Align}, 
		{_NS_pt12, _LN_grItems}, 
		{_NS_m12, _LN_count}, 
		{_NS_form, _LN_button_type}, 
		{_NS_rtf, _LN_zwj}, 
		{_NS_pt12, _LN_x}, 
		{_NS_draw, _LN_extrusion_rotation_angle}, 
		{_NS_m12, _LN_mcJc}, 
		{_NS_form, _LN_id}, 
		{_NS_w12, _LN_rPrDefault}, 
		{_NS_pt12, _LN_r}, 
		{_NS_x11, _LN_FieldLabelFormat}, 
		{_NS_v11, _LN_Protection}, 
		{_NS_rtf, _LN_zwnbo}, 
		{_NS_m12, _LN_mcPr}, 
		{_NS_chart, _LN_tick_marks_minor_outer}, 
		{_NS_m12, _LN_mc}, 
		{_NS_pt12, _LN_kpi}, 
		{_NS_v11, _LN_Help}, 
		{_NS_w12, _LN_pPrDefault}, 
		{_NS_rtf, _LN_zwnj}, 
		{_NS_m12, _LN_plcHide}, 
		{_NS_form, _LN_current_selected}, 
		{_NS_x11, _LN_DetailSortOrder}, 
		{_NS_draw, _LN_extrusion_rotation_center}, 
		{_NS_chart, _LN_display_label}, 
		{_NS_v11, _LN_Misc}, 
		{_NS_w12, _LN_view}, 
		{_NS_pt12, _LN_hierarchy}, 
		{_NS_m12, _LN_cGpRule}, 
		{_NS_w12, _LN_zoom}, 
		{_NS_form, _LN_disabled}, 
		{_NS_pt12, _LN_flds}, 
		{_NS_v11, _LN_RulerGrid}, 
		{_NS_m12, _LN_cSp}, 
		{_NS_x11, _LN_CellsExpanded}, 
		{_NS_chart, _LN_text_overlap}, 
		{_NS_rtf, _LN_dobypara}, 
		{_NS_form, _LN_dropdown}, 
		{_NS_draw, _LN_extrusion_shininess}, 
		{_NS_pt12, _LN_grLvls}, 
		{_NS_m12, _LN_cGp}, 
		{_NS_v11, _LN_DocProps}, 
		{_NS_m12, _LN_mcs}, 
		{_NS_form, _LN_for}, 
		{_NS_pt12, _LN_fld}, 
		{_NS_w12, _LN_removePersonalInformation}, 
		{_NS_rtf, _LN_doccomm}, 
		{_NS_x11, _LN_CellsNotExpanded}, 
		{_NS_m12, _LN_mPr}, 
		{_NS_chart, _LN_label_arrangement}, 
		{_NS_v11, _LN_Image}, 
		{_NS_draw, _LN_extrusion_skew}, 
		{_NS_m12, _LN_mr}, 
		{_NS_pt12, _LN_grLvl}, 
		{_NS_form, _LN_image_data}, 
		{_NS_rtf, _LN_doctemp}, 
		{_NS_m12, _LN_limLoc}, 
		{_NS_v11, _LN_Group}, 
		{_NS_form, _LN_label}, 
		{_NS_w12, _LN_removeDateAndTime}, 
		{_NS_pt12, _LN_grps}, 
		{_NS_m12, _LN_subHide}, 
		{_NS_x11, _LN_CellsExpandedSeqNum}, 
		{_NS_chart, _LN_data_label_number}, 
		{_NS_rtf, _LN_doctype}, 
		{_NS_v11, _LN_Layout}, 
		{_NS_m12, _LN_supHide}, 
		{_NS_draw, _LN_extrusion_specularity}, 
		{_NS_pt12, _LN_grp}, 
		{_NS_form, _LN_max_length}, 
		{_NS_m12, _LN_naryPr}, 
		{_NS_x11, _LN_Operator}, 
		{_NS_rtf, _LN_docvar}, 
		{_NS_m12, _LN_sub}, 
		{_NS_pt12, _LN_grMbrs}, 
		{_NS_chart, _LN_data_label_text}, 
		{_NS_v11, _LN_PageLayout}, 
		{_NS_x11, _LN_Alignment}, 
		{_NS_w12, _LN_dontDisplayPageBoundaries}, 
		{_NS_m12, _LN_sup}, 
		{_NS_form, _LN_printable}, 
		{_NS_rtf, _LN_dodhgt}, 
		{_NS_draw, _LN_extrusion_viewpoint}, 
		{_NS_pt12, _LN_grMbr}, 
		{_NS_m12, _LN_show}, 
		{_NS_v11, _LN_PrintProps}, 
		{_NS_rtf, _LN_dolock}, 
		{_NS_form, _LN_readonly}, 
		{_NS_x11, _LN_CaptionAlignment}, 
		{_NS_chart, _LN_data_label_symbol}, 
		{_NS_pt12, _LN_entries}, 
		{_NS_m12, _LN_zeroWid}, 
		{_NS_v11, _LN_Char}, 
		{_NS_w12, _LN_displayBackgroundShape}, 
		{_NS_draw, _LN_extrusion_origin}, 
		{_NS_form, _LN_selected}, 
		{_NS_udc, _LN_DataSource}, 
		{_NS_pt12, _LN_sets}, 
		{_NS_x11, _LN_FitWidth}, 
		{_NS_v11, _LN_Para}, 
		{_NS_m12, _LN_zeroAsc}, 
		{_NS_form, _LN_size}, 
		{_NS_rtf, _LN_donotshowcomments}, 
		{_NS_chart, _LN_error_category}, 
		{_NS_udc, _LN_Type}, 
		{_NS_v11, _LN_Tabs}, 
		{_NS_pt12, _LN_qryCache}, 
		{_NS_x11, _LN_FitHeight}, 
		{_NS_draw, _LN_extrusion_color}, 
		{_NS_m12, _LN_zeroDesc}, 
		{_NS_udc, _LN_SubType}, 
		{_NS_form, _LN_tab_index}, 
		{_NS_w12, _LN_printPostScriptOverText}, 
		{_NS_pt12, _LN_mbr}, 
		{_NS_rtf, _LN_donotshowinsdel}, 
		{_NS_udc, _LN_Name}, 
		{_NS_chart, _LN_error_percentage}, 
		{_NS_v11, _LN_Scratch}, 
		{_NS_m12, _LN_transp}, 
		{_NS_pt12, _LN_set}, 
		{_NS_x11, _LN_LeftToRight}, 
		{_NS_v11, _LN_Connection}, 
		{_NS_draw, _LN_enhanced_path}, 
		{_NS_form, _LN_tab_stop}, 
		{_NS_m12, _LN_phantPr}, 
		{_NS_form, _LN_title}, 
		{_NS_m12, _LN_degHide}, 
		{_NS_pt12, _LN_tuples}, 
		{_NS_chart, _LN_error_margin}, 
		{_NS_rtf, _LN_donotshowmarkup}, 
		{_NS_udc, _LN_ConnectionInfo}, 
		{_NS_m12, _LN_radPr}, 
		{_NS_x11, _LN_BlackAndWhite}, 
		{_NS_udc, _LN_Description}, 
		{_NS_v11, _LN_ConnectionABCD}, 
		{_NS_w12, _LN_printFractionalCharacterWidth}, 
		{_NS_m12, _LN_deg}, 
		{_NS_draw, _LN_path_stretchpoint_x}, 
		{_NS_udc, _LN_Keywords}, 
		{_NS_form, _LN_visual_effect}, 
		{_NS_pt12, _LN_sortByTuple}, 
		{_NS_m12, _LN_acc}, 
		{_NS_v11, _LN_Field}, 
		{_NS_rtf, _LN_donotshowprops}, 
		{_NS_chart, _LN_error_lower_limit}, 
		{_NS_x11, _LN_NoPrinterInfo}, 
		{_NS_m12, _LN_bar}, 
		{_NS_pt12, _LN_qry}, 
		{_NS_w12, _LN_printFormsData}, 
		{_NS_udc, _LN_TooltipInfo}, 
		{_NS_m12, _LN_box}, 
		{_NS_v11, _LN_Control}, 
		{_NS_form, _LN_image_position}, 
		{_NS_rtf, _LN_dpaendhol}, 
		{_NS_draw, _LN_path_stretchpoint_y}, 
		{_NS_pt12, _LN_ciFmla}, 
		{_NS_x11, _LN_DraftQuality}, 
		{_NS_chart, _LN_error_upper_limit}, 
		{_NS_v11, _LN_Geom}, 
		{_NS_m12, _LN_boxForm}, 
		{_NS_rtf, _LN_dpaendl}, 
		{_NS_pt12, _LN_rule}, 
		{_NS_form, _LN_image_align}, 
		{_NS_w12, _LN_embedTrueTypeFonts}, 
		{_NS_draw, _LN_text_areas}, 
		{_NS_m12, _LN_d}, 
		{_NS_v11, _LN_Act}, 
		{_NS_udc, _LN_ClientParameterBindings}, 
		{_NS_x11, _LN_CommentsLayout}, 
		{_NS_pt12, _LN_calcMbr}, 
		{_NS_rtf, _LN_dpaendsol}, 
		{_NS_m12, _LN_eqArr}, 
		{_NS_v11, _LN_Layer}, 
		{_NS_form, _LN_bound_column}, 
		{_NS_chart, _LN_error_upper_indicator}, 
		{_NS_draw, _LN_glue_points}, 
		{_NS_m12, _LN_f}, 
		{_NS_x11, _LN_Scale}, 
		{_NS_pt12, _LN_filters}, 
		{_NS_w12, _LN_doNotEmbedSystemFonts}, 
		{_NS_v11, _LN_User}, 
		{_NS_rtf, _LN_dpaendw}, 
		{_NS_m12, _LN_func}, 
		{_NS_pt12, _LN_filter}, 
		{_NS_udc, _LN_ClientParameterBinding}, 
		{_NS_m12, _LN_groupChr}, 
		{_NS_v11, _LN_Prop}, 
		{_NS_rtf, _LN_dparc}, 
		{_NS_draw, _LN_glue_point_type}, 
		{_NS_x11, _LN_PrintErrors}, 
		{_NS_form, _LN_convert_empty_to_null}, 
		{_NS_chart, _LN_error_lower_indicator}, 
		{_NS_w12, _LN_saveSubsetFonts}, 
		{_NS_m12, _LN_limLow}, 
		{_NS_pt12, _LN_location}, 
		{_NS_form, _LN_data_field}, 
		{_NS_v11, _LN_Hyperlink}, 
		{_NS_rtf, _LN_dparcflipx}, 
		{_NS_udc, _LN_CredentialBinding}, 
		{_NS_m12, _LN_limUpp}, 
		{_NS_pt12, _LN_rwFlds}, 
		{_NS_x11, _LN_ValidPrinterInfo}, 
		{_NS_chart, _LN_series_source}, 
		{_NS_w12, _LN_saveFormsData}, 
		{_NS_m12, _LN_m}, 
		{_NS_form, _LN_list_source}, 
		{_NS_v11, _LN_Reviewer}, 
		{_NS_rtf, _LN_dparcflipy}, 
		{_NS_draw, _LN_glue_point_leaving_directions}, 
		{_NS_udc, _LN_Credential}, 
		{_NS_pt12, _LN_rwItms}, 
		{_NS_m12, _LN_nary}, 
		{_NS_m12, _LN_phant}, 
		{_NS_x11, _LN_PaperSizeIndex}, 
		{_NS_chart, _LN_regression_type}, 
		{_NS_v11, _LN_Annotation}, 
		{_NS_w12, _LN_mirrorMargins}, 
		{_NS_draw, _LN_text_path}, 
		{_NS_m12, _LN_rad}, 
		{_NS_pt12, _LN_rwDrills}, 
		{_NS_udc, _LN_Attribute}, 
		{_NS_rtf, _LN_dpastarthol}, 
		{_NS_form, _LN_list_source_type}, 
		{_NS_m12, _LN_sPre}, 
		{_NS_udc, _LN_If}, 
		{_NS_m12, _LN_sSub}, 
		{_NS_v11, _LN_SmartTagDef}, 
		{_NS_pt12, _LN_colFlds}, 
		{_NS_pr, _LN_transition_type}, 
		{_NS_rtf, _LN_dpastartl}, 
		{_NS_form, _LN_properties}, 
		{_NS_x11, _LN_HorizontalResolution}, 
		{_NS_draw, _LN_text_path_mode}, 
		{_NS_m12, _LN_sSubSup}, 
		{_NS_w12, _LN_alignBordersAndEdges}, 
		{_NS_v11, _LN_Data1}, 
		{_NS_m12, _LN_sSup}, 
		{_NS_pt12, _LN_colItms}, 
		{_NS_form, _LN_property}, 
		{_NS_rtf, _LN_dpastartsol}, 
		{_NS_m12, _LN_r}, 
		{_NS_udc, _LN_ClientParameterValue}, 
		{_NS_pr, _LN_transition_style}, 
		{_NS_v11, _LN_Data2}, 
		{_NS_draw, _LN_text_path_scale}, 
		{_NS_x11, _LN_VerticalResolution}, 
		{_NS_pt12, _LN_colDrills}, 
		{_NS_m12, _LN_argSz}, 
		{_NS_rtf, _LN_dpastartw}, 
		{_NS_v11, _LN_Data3}, 
		{_NS_w12, _LN_bordersDontSurroundHeader}, 
		{_NS_form, _LN_property_name}, 
		{_NS_udc, _LN_CredentialValue}, 
		{_NS_x11, _LN_Gridlines}, 
		{_NS_pt12, _LN_pgFields}, 
		{_NS_m12, _LN_argPr}, 
		{_NS_pr, _LN_transition_speed}, 
		{_NS_rtf, _LN_dpcallout}, 
		{_NS_pr, _LN_duration}, 
		{_NS_v11, _LN_ForeignData}, 
		{_NS_m12, _LN_jc}, 
		{_NS_form, _LN_list_property}, 
		{_NS_draw, _LN_text_path_same_letter_heights}, 
		{_NS_pr, _LN_visibility}, 
		{_NS_pt12, _LN_datFields}, 
		{_NS_x11, _LN_NumberOfCopies}, 
		{_NS_rtf, _LN_dpcoa}, 
		{_NS_v11, _LN_PinX}, 
		{_NS_w12, _LN_bordersDontSurroundFooter}, 
		{_NS_m12, _LN_mathFont}, 
		{_NS_pt12, _LN_fmts}, 
		{_NS_form, _LN_list_value}, 
		{_NS_draw, _LN_modifiers}, 
		{_NS_v11, _LN_PinY}, 
		{_NS_rtf, _LN_dpcoaccent}, 
		{_NS_draw, _LN_background_size}, 
		{_NS_m12, _LN_defSz}, 
		{_NS_x11, _LN_RowColHeadings}, 
		{_NS_w12, _LN_gutterAtTop}, 
		{_NS_pt12, _LN_condfmts}, 
		{_NS_udcs, _LN_Location}, 
		{_NS_v11, _LN_Width}, 
		{_NS_draw, _LN_equation}, 
		{_NS_office, _LN_annotation}, 
		{_NS_m12, _LN_brkBin}, 
		{_NS_x11, _LN_PTRule}, 
		{_NS_draw, _LN_formula}, 
		{_NS_office, _LN_display}, 
		{_NS_rtf, _LN_dpcobestfit}, 
		{_NS_udcs, _LN_Proxy}, 
		{_NS_pt12, _LN_crtfmts}, 
		{_NS_v11, _LN_Height}, 
		{_NS_w12, _LN_hideSpellingErrors}, 
		{_NS_draw, _LN_handle}, 
		{_NS_m12, _LN_brkBinSub}, 
		{_NS_pr, _LN_background_objects_visible}, 
		{_NS_x11, _LN_FormatType}, 
		{_NS_rtf, _LN_dpcoborder}, 
		{_NS_meta, _LN_date_string}, 
		{_NS_v11, _LN_LocPinX}, 
		{_NS_pt12, _LN_drillSaved}, 
		{_NS_udcs, _LN_Authentication}, 
		{_NS_udcs, _LN_Windows}, 
		{_NS_m12, _LN_smallFrac}, 
		{_NS_rtf, _LN_dpcodabs}, 
		{_NS_v11, _LN_LocPinY}, 
		{_NS_style, _LN_num_prefix}, 
		{_NS_pt12, _LN_strSelects}, 
		{_NS_w12, _LN_hideGrammaticalErrors}, 
		{_NS_pr, _LN_background_visible}, 
		{_NS_draw, _LN_handle_mirror_vertical}, 
		{_NS_udcs, _LN_Basic}, 
		{_NS_x11, _LN_IndividualCellBorders}, 
		{_NS_m12, _LN_dispDef}, 
		{_NS_v11, _LN_Angle}, 
		{_NS_pt12, _LN_dims}, 
		{_NS_rtf, _LN_dpcodbottom}, 
		{_NS_style, _LN_num_suffix}, 
		{_NS_m12, _LN_lMargin}, 
		{_NS_v11, _LN_FlipX}, 
		{_NS_x11, _LN_ItemType}, 
		{_NS_pr, _LN_display_header}, 
		{_NS_pt12, _LN_mgs}, 
		{_NS_udcs, _LN_useExplicit}, 
		{_NS_w12, _LN_activeWritingStyle}, 
		{_NS_pt12, _LN_tableStyle}, 
		{_NS_rtf, _LN_dpcodcenter}, 
		{_NS_style, _LN_num_format}, 
		{_NS_draw, _LN_handle_mirror_horizontal}, 
		{_NS_v11, _LN_FlipY}, 
		{_NS_m12, _LN_rMargin}, 
		{_NS_udcs, _LN_userId}, 
		{_NS_x11, _LN_BlockTotal}, 
		{_NS_w12, _LN_proofState}, 
		{_NS_pr, _LN_display_footer}, 
		{_NS_udcs, _LN_password}, 
		{_NS_pt12, _LN_sxfilters}, 
		{_NS_m12, _LN_defJc}, 
		{_NS_rtf, _LN_dpcodescent}, 
		{_NS_v11, _LN_ResizeMode}, 
		{_NS_style, _LN_num_letter_sync}, 
		{_NS_draw, _LN_handle_switched}, 
		{_NS_pt12, _LN_items}, 
		{_NS_w12, _LN_formsDesign}, 
		{_NS_udcs, _LN_SOAPAction}, 
		{_NS_m12, _LN_preSp}, 
		{_NS_x11, _LN_CountOfSameItems}, 
		{_NS_rtf, _LN_dpcodtop}, 
		{_NS_pr, _LN_display_page_number}, 
		{_NS_v11, _LN_LineWeight}, 
		{_NS_pt12, _LN_cdls}, 
		{_NS_office, _LN_change_info}, 
		{_NS_m12, _LN_postSp}, 
		{_NS_draw, _LN_handle_position}, 
		{_NS_x11, _LN_PTLineItem}, 
		{_NS_rtf, _LN_dpcolength}, 
		{_NS_w12, _LN_attachedTemplate}, 
		{_NS_v11, _LN_LineColor}, 
		{_NS_m12, _LN_intraSp}, 
		{_NS_pt12, _LN_autoSortScope}, 
		{_NS_pr, _LN_display_date_time}, 
		{_NS_office, _LN_event_listeners}, 
		{_NS_x11, _LN_RuleType}, 
		{_NS_rtf, _LN_dpcominusx}, 
		{_NS_w12, _LN_linkStyles}, 
		{_NS_script, _LN_event_listener}, 
		{_NS_pt12, _LN_cdl}, 
		{_NS_v11, _LN_LinePattern}, 
		{_NS_m12, _LN_interSp}, 
		{_NS_draw, _LN_handle_range_x_minimum}, 
		{_NS_x11, _LN_Rule}, 
		{_NS_udcxf11, _LN_File}, 
		{_NS_pt12, _LN_pgField}, 
		{_NS_rtf, _LN_dpcominusy}, 
		{_NS_script, _LN_macro_name}, 
		{_NS_v11, _LN_Rounding}, 
		{_NS_m12, _LN_wrapRight}, 
		{_NS_x11, _LN_RuleV10}, 
		{_NS_w12, _LN_stylePaneFormatFilter}, 
		{_NS_math, _LN_math}, 
		{_NS_pt12, _LN_datField}, 
		{_NS_draw, _LN_handle_range_x_maximum}, 
		{_NS_rtf, _LN_dpcooffset}, 
		{_NS_m12, _LN_wrapIndent}, 
		{_NS_v11, _LN_EndArrowSize}, 
		{_NS_pt12, _LN_i}, 
		{_NS_x11, _LN_ParseRuleAsV10}, 
		{_NS_rng, _LN_anyName}, 
		{_NS_pt12, _LN_fmt}, 
		{_NS_vml, _LN_shape}, 
		{_NS_rtf, _LN_dpcosmarta}, 
		{_NS_m12, _LN_naryGrow}, 
		{_NS_w12, _LN_stylePaneSortMethod}, 
		{_NS_x11, _LN_Area}, 
		{_NS_v11, _LN_BeginArrow}, 
		{_NS_vml, _LN_group}, 
		{_NS_x11, _LN_Outline}, 
		{_NS_pt12, _LN_cfmt}, 
		{_NS_draw, _LN_handle_range_y_minimum}, 
		{_NS_m12, _LN_intLim}, 
		{_NS_rtf, _LN_dpcotdouble}, 
		{_NS_text, _LN_dde_connection_decl}, 
		{_NS_v11, _LN_EndArrow}, 
		{_NS_w12, _LN_documentType}, 
		{_NS_vml, _LN_shapetype}, 
		{_NS_pt12, _LN_crtfmt}, 
		{_NS_m12, _LN_naryLim}, 
		{_NS_x11, _LN_MissingItemsLimit}, 
		{_NS_v11, _LN_LineCap}, 
		{_NS_rtf, _LN_dpcotright}, 
		{_NS_pt12, _LN_drill}, 
		{_NS_w12, _LN_mailMerge}, 
		{_NS_vml, _LN_VMLFrame}, 
		{_NS_draw, _LN_handle_range_y_maximum}, 
		{_NS_office, _LN_dde_application}, 
		{_NS_m12, _LN_mathPr}, 
		{_NS_vml, _LN_background}, 
		{_NS_vml, _LN_path}, 
		{_NS_pt12, _LN_strSelect}, 
		{_NS_rtf, _LN_dpcotsingle}, 
		{_NS_v11, _LN_BeginArrowSize}, 
		{_NS_w12, _LN_revisionView}, 
		{_NS_office, _LN_dde_topic}, 
		{_NS_x11, _LN_VersionLastRefresh}, 
		{_NS_draw, _LN_handle_polar}, 
		{_NS_m12, _LN_oMathParaPr}, 
		{_NS_vml, _LN_formulas}, 
		{_NS_pt12, _LN_mbrProps}, 
		{_NS_rtf, _LN_dpcottriple}, 
		{_NS_office, _LN_dde_item}, 
		{_NS_m12, _LN_oMath}, 
		{_NS_vml, _LN_f}, 
		{_NS_v11, _LN_LineColorTrans}, 
		{_NS_x11, _LN_RefreshName}, 
		{_NS_pt12, _LN_itms}, 
		{_NS_rtf, _LN_dpcount}, 
		{_NS_vml, _LN_handles}, 
		{_NS_draw, _LN_handle_radius_range_minimum}, 
		{_NS_m12, _LN_oMathPara}, 
		{_NS_v11, _LN_FillForegnd}, 
		{_NS_x11, _LN_RefreshDate}, 
		{_NS_pt12, _LN_selctPg}, 
		{_NS_office, _LN_automatic_update}, 
		{_NS_vml, _LN_h}, 
		{_NS_rtf, _LN_dpellipse}, 
		{_NS_vml, _LN_fill}, 
		{_NS_pt12, _LN_mp}, 
		{_NS_v11, _LN_FillBkgnd}, 
		{_NS_table, _LN_dde_link}, 
		{_NS_vml, _LN_stroke}, 
		{_NS_pt12, _LN_dim}, 
		{_NS_x11, _LN_RefreshDateCopy}, 
		{_NS_vml, _LN_shadow}, 
		{_NS_ip12, _LN_partFragment}, 
		{_NS_rtf, _LN_dpendgroup}, 
		{_NS_aml12, _LN_annotation}, 
		{_NS_pt12, _LN_mg}, 
		{_NS_draw, _LN_handle_radius_range_maximum}, 
		{_NS_office, _LN_dde_source}, 
		{_NS_v11, _LN_FillPattern}, 
		{_NS_pt12, _LN_map}, 
		{_NS_vml, _LN_textbox}, 
		{_NS_pr, _LN_class}, 
		{_NS_rtf, _LN_dpfillbgcb}, 
		{_NS_ip12, _LN_xmlFileAdapter}, 
		{_NS_pr, _LN_placeholder}, 
		{_NS_x11, _LN_ConsolidationReference}, 
		{_NS_pt12, _LN_sxfilter}, 
		{_NS_v11, _LN_ShdwForegnd}, 
		{_NS_vml, _LN_textpath}, 
		{_NS_office, _LN_conversion_mode}, 
		{_NS_rtf, _LN_dpfillbgcg}, 
		{_NS_p12, _LN_Types}, 
		{_NS_x11, _LN_NoSaveData}, 
		{_NS_pt12, _LN_autofilter}, 
		{_NS_v11, _LN_ShdwBkgnd}, 
		{_NS_anium, _LN_animate}, 
		{_NS_vml, _LN_imagedata}, 
		{_NS_pr, _LN_user_transformed}, 
		{_NS_ip12, _LN_sharepointListAdapter}, 
		{_NS_rtf, _LN_dpfillbgcr}, 
		{_NS_vml, _LN_line}, 
		{_NS_ip12, _LN_field}, 
		{_NS_p12, _LN_Default}, 
		{_NS_anium, _LN_set}, 
		{_NS_v11, _LN_ShdwPattern}, 
		{_NS_pr, _LN_animations}, 
		{_NS_x11, _LN_SourceConsolidation}, 
		{_NS_vml, _LN_Behavior}, 
		{_NS_p12, _LN_Override}, 
		{_NS_rtf, _LN_dpfillbggray}, 
		{_NS_ip12, _LN_davAdapter}, 
		{_NS_anium, _LN_animateMotion}, 
		{_NS_pr, _LN_sound}, 
		{_NS_svg, _LN_path}, 
		{_NS_vml, _LN_Sequence}, 
		{_NS_v11, _LN_FillForegndTrans}, 
		{_NS_ip12, _LN_folderURL}, 
		{_NS_x11, _LN_BackgroundQuery}, 
		{_NS_rtf, _LN_dpfillbgpal}, 
		{_NS_vml, _LN_Action}, 
		{_NS_pr, _LN_play_full}, 
		{_NS_svg, _LN_origin}, 
		{_NS_vml, _LN_arc}, 
		{_NS_ip12, _LN_fileName}, 
		{_NS_v11, _LN_FillBkgndTrans}, 
		{_NS_x11, _LN_CubeSource}, 
		{_NS_rtf, _LN_dpfillfgcb}, 
		{_NS_pr, _LN_show_shape}, 
		{_NS_vml, _LN_curve}, 
		{_NS_smil, _LN_calcMode}, 
		{_NS_qt12, _LN_querytable}, 
		{_NS_vml, _LN_polyline}, 
		{_NS_ip12, _LN_emailAdapter}, 
		{_NS_rtf, _LN_dpfillfgcg}, 
		{_NS_vml, _LN_rect}, 
		{_NS_x11, _LN_DisableRefresh}, 
		{_NS_v11, _LN_ShdwForegndTrans}, 
		{_NS_qt12, _LN_qtRefr}, 
		{_NS_ip12, _LN_to}, 
		{_NS_anium, _LN_animateColor}, 
		{_NS_qt12, _LN_fsb}, 
		{_NS_ip12, _LN_cc}, 
		{_NS_rtf, _LN_dpfillfgcr}, 
		{_NS_vml, _LN_roundrect}, 
		{_NS_x11, _LN_HasNoRecords}, 
		{_NS_v11, _LN_ShdwBkgndTrans}, 
		{_NS_qt12, _LN_delNms}, 
		{_NS_ip12, _LN_bcc}, 
		{_NS_vml, _LN_oval}, 
		{_NS_anium, _LN_color_interpolation}, 
		{_NS_ip12, _LN_subject}, 
		{_NS_rtf, _LN_dpfillfggray}, 
		{_NS_vml, _LN_image}, 
		{_NS_qt12, _LN_qtflds}, 
		{_NS_x11, _LN_OptimizeCache}, 
		{_NS_v11, _LN_ShapeShdwType}, 
		{_NS_ip12, _LN_intro}, 
		{_NS_qt12, _LN_delNm}, 
		{_NS_rtf, _LN_dpfillfgpal}, 
		{_NS_qt12, _LN_qtfld}, 
		{_NS_x11, _LN_RefreshOnFileOpen}, 
		{_NS_anium, _LN_color_interpolation_direction}, 
		{_NS_v11, _LN_ShapeShdwOffsetX}, 
		{_NS_mdc12, _LN_CoreProperties}, 
		{_NS_rtf, _LN_dpfillpat}, 
		{_NS_ip12, _LN_attachmentFileName}, 
		{_NS_x11, _LN_PublishObject}, 
		{_NS_rtf, _LN_dpgroup}, 
		{_NS_anium, _LN_animateTransform}, 
		{_NS_v11, _LN_ShapeShdwOffsetY}, 
		{_NS_ip12, _LN_dataAdapters}, 
		{_NS_mdc12, _LN_Creator}, 
		{_NS_svg, _LN_type}, 
		{_NS_x11, _LN_DivID}, 
		{_NS_rtf, _LN_dpline}, 
		{_NS_mdc12, _LN_Title}, 
		{_NS_ip12, _LN_documentSchemas}, 
		{_NS_x11, _LN_SourceType}, 
		{_NS_rtf, _LN_dplinecob}, 
		{_NS_v11, _LN_ShapeShdwObliqueAngle}, 
		{_NS_anium, _LN_transitionFilter}, 
		{_NS_mdc12, _LN_Subject}, 
		{_NS_smil, _LN_type}, 
		{_NS_x11, _LN_HtmlType}, 
		{_NS_rtf, _LN_dplinecog}, 
		{_NS_ws12, _LN_workSheet}, 
		{_NS_ip12, _LN_documentSchema}, 
		{_NS_mdc12, _LN_Category}, 
		{_NS_aml12, _LN_arc}, 
		{_NS_smil, _LN_subtype}, 
		{_NS_v11, _LN_ShapeShdwScaleFactor}, 
		{_NS_rtf, _LN_dplinecor}, 
		{_NS_smil, _LN_direction}, 
		{_NS_ws12, _LN_macroSheet}, 
		{_NS_x11, _LN_AutoRepublish}, 
		{_NS_mdc12, _LN_Keywords}, 
		{_NS_ip12, _LN_customValidation}, 
		{_NS_v11, _LN_BeginX}, 
		{_NS_smil, _LN_fadeColor}, 
		{_NS_rtf, _LN_dplinedado}, 
		{_NS_x11, _LN_QueryType}, 
		{_NS_ws12, _LN_dialogSheet}, 
		{_NS_mdc12, _LN_Description}, 
		{_NS_v11, _LN_BeginY}, 
		{_NS_smil, _LN_mode}, 
		{_NS_x11, _LN_CommandText}, 
		{_NS_ip12, _LN_errorCondition}, 
		{_NS_anium, _LN_id}, 
		{_NS_x11, _LN_CommandType}, 
		{_NS_v11, _LN_EndX}, 
		{_NS_rtf, _LN_dplinedadodo}, 
		{_NS_ws12, _LN_chartSheet}, 
		{_NS_mdc12, _LN_ContentType}, 
		{_NS_w10, _LN_bordertop}, 
		{_NS_v11, _LN_EndY}, 
		{_NS_ip12, _LN_errorMessage}, 
		{_NS_x11, _LN_Maintain}, 
		{_NS_ws12, _LN_wsPr}, 
		{_NS_smil, _LN_targetElement}, 
		{_NS_rtf, _LN_dplinedash}, 
		{_NS_x11, _LN_Connection}, 
		{_NS_ws12, _LN_dimension}, 
		{_NS_w10, _LN_borderleft}, 
		{_NS_mdc12, _LN_ContentStatus}, 
		{_NS_v11, _LN_TheData}, 
		{_NS_x11, _LN_Parameter}, 
		{_NS_aml12, _LN_content}, 
		{_NS_rtf, _LN_dplinedot}, 
		{_NS_ip12, _LN_domEventHandlers}, 
		{_NS_smil, _LN_attributeName}, 
		{_NS_ws12, _LN_wsViews}, 
		{_NS_mdc12, _LN_Version}, 
		{_NS_v11, _LN_TheText}, 
		{_NS_w10, _LN_borderright}, 
		{_NS_aml12, _LN_context}, 
		{_NS_anium, _LN_sub_item}, 
		{_NS_rtf, _LN_dplinegray}, 
		{_NS_x11, _LN_CommandTextOrignal}, 
		{_NS_mdc12, _LN_Revision}, 
		{_NS_ws12, _LN_userShViews}, 
		{_NS_aml12, _LN_property}, 
		{_NS_ip12, _LN_domEventHandler}, 
		{_NS_v11, _LN_EventDblClick}, 
		{_NS_w10, _LN_borderbottom}, 
		{_NS_smil, _LN_values}, 
		{_NS_w10, _LN_wrap}, 
		{_NS_ws12, _LN_wsFmtInfo}, 
		{_NS_rtf, _LN_dplinehollow}, 
		{_NS_mdc12, _LN_DateCreated}, 
		{_NS_v11, _LN_EventXFMod}, 
		{_NS_x11, _LN_SourceConnectionFile}, 
		{_NS_smil, _LN_keyTimes}, 
		{_NS_w10, _LN_anchorlock}, 
		{_NS_ws12, _LN_colInfos}, 
		{_NS_rtf, _LN_dplinepal}, 
		{_NS_mdc12, _LN_DateModified}, 
		{_NS_v11, _LN_EventDrop}, 
		{_NS_smil, _LN_keySplines}, 
		{_NS_x11, _LN_SourceDataFile}, 
		{_NS_ws12, _LN_sheetData}, 
		{_NS_rtf, _LN_dplinesolid}, 
		{_NS_ip12, _LN_importParameters}, 
		{_NS_v11, _LN_LayerMember}, 
		{_NS_mdc12, _LN_LastModifiedBy}, 
		{_NS_smil, _LN_accumulate}, 
		{_NS_ws12, _LN_calcChains}, 
		{_NS_x11, _LN_RobustConnect}, 
		{_NS_rtf, _LN_dplinew}, 
		{_NS_ip12, _LN_importSource}, 
		{_NS_ws12, _LN_autofilter}, 
		{_NS_mdc12, _LN_LastPrinted}, 
		{_NS_smil, _LN_additive}, 
		{_NS_v11, _LN_EnableLineProps}, 
		{_NS_anium, _LN_formula}, 
		{_NS_rtf, _LN_dppolycount}, 
		{_NS_w11, _LN_ilvl}, 
		{_NS_x11, _LN_RefreshTimeSpan}, 
		{_NS_ws12, _LN_sortState}, 
		{_NS_mdc12, _LN_Language}, 
		{_NS_ip12, _LN_listProperties}, 
		{_NS_smil, _LN_to}, 
		{_NS_w11, _LN_ilfo}, 
		{_NS_ip12, _LN_fields}, 
		{_NS_rtf, _LN_dppolygon}, 
		{_NS_v11, _LN_EnableFillProps}, 
		{_NS_w11, _LN_top}, 
		{_NS_ws12, _LN_rwBrks}, 
		{_NS_smil, _LN_from}, 
		{_NS_w11, _LN_left}, 
		{_NS_mdc12, _LN_Identifier}, 
		{_NS_x11, _LN_LocalConnection}, 
		{_NS_ip12, _LN_submit}, 
		{_NS_smil, _LN_by}, 
		{_NS_w11, _LN_bottom}, 
		{_NS_ws12, _LN_colBrks}, 
		{_NS_rtf, _LN_dppolyline}, 
		{_NS_v11, _LN_EnableTextProps}, 
		{_NS_w11, _LN_right}, 
		{_NS_smil, _LN_begin}, 
		{_NS_x11, _LN_NoRefreshCache}, 
		{_NS_rtf, _LN_dpptx}, 
		{_NS_w11, _LN_between}, 
		{_NS_ip12, _LN_submitAction}, 
		{_NS_smil, _LN_end}, 
		{_NS_ws12, _LN_pageMargins}, 
		{_NS_w11, _LN_bar}, 
		{_NS_v11, _LN_HideForApply}, 
		{_NS_rtf, _LN_dppty}, 
		{_NS_w11, _LN_tab}, 
		{_NS_smil, _LN_dur}, 
		{_NS_w11, _LN_pStyle}, 
		{_NS_ip12, _LN_successMessage}, 
		{_NS_ws12, _LN_printOptions}, 
		{_NS_x11, _LN_UseLocalConnection}, 
		{_NS_rtf, _LN_dprect}, 
		{_NS_v11, _LN_ImgOffsetX}, 
		{_NS_w11, _LN_keepNext}, 
		{_NS_smil, _LN_endsync}, 
		{_NS_w11, _LN_keepLines}, 
		{_NS_ws12, _LN_pageSetup}, 
		{_NS_rtf, _LN_dproundr}, 
		{_NS_ip12, _LN_useHttpHandler}, 
		{_NS_w11, _LN_pageBreakBefore}, 
		{_NS_v11, _LN_ImgOffsetY}, 
		{_NS_x11, _LN_VersionLastEdit}, 
		{_NS_smil, _LN_repeatDur}, 
		{_NS_w11, _LN_framePr}, 
		{_NS_rel12, _LN_Relationships}, 
		{_NS_rtf, _LN_dpshadow}, 
		{_NS_w11, _LN_widowControl}, 
		{_NS_ws12, _LN_headerFooter}, 
		{_NS_v11, _LN_ImgWidth}, 
		{_NS_smil, _LN_repeatCount}, 
		{_NS_w11, _LN_listPr}, 
		{_NS_ip12, _LN_useScriptHandler}, 
		{_NS_ws12, _LN_drawing}, 
		{_NS_x11, _LN_DoNotJoinDelimiters}, 
		{_NS_rtf, _LN_dpshadx}, 
		{_NS_w11, _LN_supressLineNumbers}, 
		{_NS_rel12, _LN_Relationship}, 
		{_NS_ws12, _LN_fsb}, 
		{_NS_smil, _LN_fill}, 
		{_NS_v11, _LN_ImgHeight}, 
		{_NS_w11, _LN_pBdr}, 
		{_NS_rtf, _LN_dpshady}, 
		{_NS_w11, _LN_shd}, 
		{_NS_ip12, _LN_useQueryAdapter}, 
		{_NS_ws12, _LN_phonetic}, 
		{_NS_x11, _LN_NoTextToColumns}, 
		{_NS_v11, _LN_PageWidth}, 
		{_NS_w11, _LN_tabs}, 
		{_NS_smil, _LN_fillDefault}, 
		{_NS_ip12, _LN_onLoad}, 
		{_NS_rtf, _LN_dptxbtlr}, 
		{_NS_w11, _LN_suppressAutoHyphens}, 
		{_NS_w11, _LN_kinsoku}, 
		{_NS_smil, _LN_restart}, 
		{_NS_v11, _LN_PageHeight}, 
		{_NS_ws12, _LN_dataConsolidation}, 
		{_NS_ip12, _LN_roles}, 
		{_NS_rtf, _LN_dptxbx}, 
		{_NS_w11, _LN_wordWrap}, 
		{_NS_x11, _LN_DisableDateRecognition}, 
		{_NS_w11, _LN_overflowPunct}, 
		{_NS_ip12, _LN_role}, 
		{_NS_v11, _LN_ShdwOffsetX}, 
		{_NS_smil, _LN_restartDefault}, 
		{_NS_ws12, _LN_mergeCells}, 
		{_NS_w11, _LN_topLinePunct}, 
		{_NS_rtf, _LN_dptxbxmar}, 
		{_NS_w11, _LN_autoSpaceDE}, 
		{_NS_pp12, _LN_snd}, 
		{_NS_ax12, _LN_ocx}, 
		{_NS_ws12, _LN_dVals}, 
		{_NS_ip12, _LN_membership}, 
		{_NS_w11, _LN_autoSpaceDN}, 
		{_NS_x11, _LN_EnableRedirections}, 
		{_NS_smil, _LN_accelerate}, 
		{_NS_v11, _LN_ShdwOffsetY}, 
		{_NS_rtf, _LN_dptxbxtext}, 
		{_NS_pp12, _LN_stSnd}, 
		{_NS_w11, _LN_bidi}, 
		{_NS_ax12, _LN_ocxPr}, 
		{_NS_ws12, _LN_smartTags}, 
		{_NS_w11, _LN_adjustRightInd}, 
		{_NS_v11, _LN_PageScale}, 
		{_NS_x11, _LN_EditWebPage}, 
		{_NS_rtf, _LN_dptxlrtb}, 
		{_NS_smil, _LN_decelerate}, 
		{_NS_pp12, _LN_endSnd}, 
		{_NS_w11, _LN_snapToGrid}, 
		{_NS_ip12, _LN_getUserNameFromData}, 
		{_NS_ax12, _LN_ocxMBed}, 
		{_NS_w11, _LN_spacing}, 
		{_NS_ws12, _LN_hyperlinks}, 
		{_NS_pp12, _LN_blinds}, 
		{_NS_rtf, _LN_dptxlrtbv}, 
		{_NS_w11, _LN_ind}, 
		{_NS_smil, _LN_autoReverse}, 
		{_NS_v11, _LN_DrawingScale}, 
		{_NS_ip12, _LN_userName}, 
		{_NS_x11, _LN_RefreshedInXl9}, 
		{_NS_w11, _LN_contextualSpacing}, 
		{_NS_ip12, _LN_group}, 
		{_NS_pp12, _LN_checker}, 
		{_NS_anium, _LN_par}, 
		{_NS_rtf, _LN_dptxtbrl}, 
		{_NS_w11, _LN_suppressOverlap}, 
		{_NS_pp12, _LN_circle}, 
		{_NS_x11, _LN_EntirePage}, 
		{_NS_w11, _LN_jc}, 
		{_NS_ws12, _LN_conditionalFormatting}, 
		{_NS_anium, _LN_seq}, 
		{_NS_v11, _LN_DrawingSizeType}, 
		{_NS_ip12, _LN_hwsWorkflow}, 
		{_NS_w11, _LN_textDirection}, 
		{_NS_rtf, _LN_dptxtbrlv}, 
		{_NS_pp12, _LN_dissolve}, 
		{_NS_ip12, _LN_location}, 
		{_NS_w11, _LN_textAlignment}, 
		{_NS_anium, _LN_iterate}, 
		{_NS_x11, _LN_HTMLFormat}, 
		{_NS_rtf, _LN_dpx}, 
		{_NS_ws12, _LN_legacyDrawing}, 
		{_NS_w11, _LN_outlineLvl}, 
		{_NS_v11, _LN_DrawingScaleType}, 
		{_NS_pp12, _LN_comb}, 
		{_NS_w11, _LN_divId}, 
		{_NS_x11, _LN_URLString}, 
		{_NS_ip12, _LN_allowedActions}, 
		{_NS_rtf, _LN_dpxsize}, 
		{_NS_anium, _LN_iterate_type}, 
		{_NS_w11, _LN_cnfStyle}, 
		{_NS_ws12, _LN_scenarios}, 
		{_NS_pp12, _LN_cover}, 
		{_NS_ip12, _LN_action}, 
		{_NS_v11, _LN_InhibitSnap}, 
		{_NS_w11, _LN_rPr}, 
		{_NS_rtf, _LN_dpy}, 
		{_NS_pp12, _LN_cut}, 
		{_NS_w11, _LN_sectPr}, 
		{_NS_x11, _LN_WebPostString}, 
		{_NS_ws12, _LN_cellWatches}, 
		{_NS_anium, _LN_iterate_interval}, 
		{_NS_w11, _LN_binData}, 
		{_NS_rtf, _LN_dpysize}, 
		{_NS_ip12, _LN_allowedTasks}, 
		{_NS_v11, _LN_UIVisibility}, 
		{_NS_pp12, _LN_diamond}, 
		{_NS_w11, _LN_movie}, 
		{_NS_a12, _LN_st}, 
		{_NS_anium, _LN_audio}, 
		{_NS_x11, _LN_RefreshInfo}, 
		{_NS_ip12, _LN_task}, 
		{_NS_w11, _LN_background}, 
		{_NS_pp12, _LN_fade}, 
		{_NS_a12, _LN_end}, 
		{_NS_rtf, _LN_dropcapli}, 
		{_NS_v11, _LN_ShdwType}, 
		{_NS_ws12, _LN_sheetProtection}, 
		{_NS_w11, _LN_applet}, 
		{_NS_ip12, _LN_fileNew}, 
		{_NS_w11, _LN_msAccessHTML}, 
		{_NS_anium, _LN_audio_level}, 
		{_NS_pp12, _LN_newsflash}, 
		{_NS_rtf, _LN_dropcapt}, 
		{_NS_x11, _LN_TextWizardSettings}, 
		{_NS_anium, _LN_command}, 
		{_NS_a12, _LN_audioCd}, 
		{_NS_ws12, _LN_protectedRanges}, 
		{_NS_rtf, _LN_ds}, 
		{_NS_v11, _LN_ShdwObliqueAngle}, 
		{_NS_pp12, _LN_plus}, 
		{_NS_anium, _LN_param}, 
		{_NS_ws12, _LN_picture}, 
		{_NS_w11, _LN_scriptAnchor}, 
		{_NS_anium, _LN_name}, 
		{_NS_ip12, _LN_initialXmlDocument}, 
		{_NS_x11, _LN_HTMLTables}, 
		{_NS_w11, _LN_ocx}, 
		{_NS_a12, _LN_audioFile}, 
		{_NS_rtf, _LN_dxfrtext}, 
		{_NS_pp12, _LN_pull}, 
		{_NS_anium, _LN_value}, 
		{_NS_w11, _LN_args}, 
		{_NS_v11, _LN_ShdwScaleFactor}, 
		{_NS_ws12, _LN_webPubItems}, 
		{_NS_rtf, _LN_dy}, 
		{_NS_pp12, _LN_push}, 
		{_NS_w11, _LN_appletText}, 
		{_NS_style, _LN_style}, 
		{_NS_a12, _LN_videoFile}, 
		{_NS_ws12, _LN_row}, 
		{_NS_ip12, _LN_customCategory}, 
		{_NS_x11, _LN_UseSameSettings}, 
		{_NS_style, _LN_name}, 
		{_NS_rtf, _LN_edmins}, 
		{_NS_v11, _LN_LeftMargin}, 
		{_NS_pp12, _LN_random}, 
		{_NS_w11, _LN_scriptId}, 
		{_NS_ws12, _LN_chain}, 
		{_NS_ip12, _LN_package}, 
		{_NS_x11, _LN_Query97}, 
		{_NS_rtf, _LN_embo}, 
		{_NS_w11, _LN_language}, 
		{_NS_ws12, _LN_c}, 
		{_NS_pp12, _LN_randomBar}, 
		{_NS_style, _LN_display_name}, 
		{_NS_v11, _LN_RightMargin}, 
		{_NS_ip12, _LN_files}, 
		{_NS_rtf, _LN_emdash}, 
		{_NS_ws12, _LN_colInfo}, 
		{_NS_ip12, _LN_file}, 
		{_NS_w11, _LN_scriptText}, 
		{_NS_pp12, _LN_split}, 
		{_NS_v11, _LN_TopMargin}, 
		{_NS_ws12, _LN_f}, 
		{_NS_x11, _LN_DoNotPromptForFile}, 
		{_NS_w11, _LN_fldData}, 
		{_NS_rtf, _LN_emfblip}, 
		{_NS_style, _LN_parent_style_name}, 
		{_NS_ws12, _LN_v}, 
		{_NS_w11, _LN_r}, 
		{_NS_pp12, _LN_strips}, 
		{_NS_ip12, _LN_fileProperties}, 
		{_NS_w11, _LN_fldSimple}, 
		{_NS_v11, _LN_BottomMargin}, 
		{_NS_rtf, _LN_emspace}, 
		{_NS_ws12, _LN_rst}, 
		{_NS_ip12, _LN_property}, 
		{_NS_x11, _LN_AutoFormatWidth}, 
		{_NS_pp12, _LN_wedge}, 
		{_NS_w11, _LN_hlink}, 
		{_NS_a12, _LN_dark1}, 
		{_NS_style, _LN_next_style_name}, 
		{_NS_w11, _LN_col}, 
		{_NS_rtf, _LN_endash}, 
		{_NS_pp12, _LN_wheel}, 
		{_NS_ws12, _LN_tabColor}, 
		{_NS_a12, _LN_light1}, 
		{_NS_w11, _LN_hdr}, 
		{_NS_v11, _LN_VerticalAlign}, 
		{_NS_ip12, _LN_permissions}, 
		{_NS_x11, _LN_DisableEdit}, 
		{_NS_w11, _LN_ftr}, 
		{_NS_rtf, _LN_enddoc}, 
		{_NS_pp12, _LN_wipe}, 
		{_NS_ws12, _LN_wsView}, 
		{_NS_style, _LN_list_style_name}, 
		{_NS_a12, _LN_dark2}, 
		{_NS_w11, _LN_footnotePr}, 
		{_NS_x11, _LN_Filled}, 
		{_NS_v11, _LN_TextBkgnd}, 
		{_NS_style, _LN_master_page_name}, 
		{_NS_pp12, _LN_zoom}, 
		{_NS_w11, _LN_endnotePr}, 
		{_NS_ws12, _LN_pane}, 
		{_NS_ip12, _LN_allowedControl}, 
		{_NS_rtf, _LN_endnhere}, 
		{_NS_a12, _LN_light2}, 
		{_NS_w11, _LN_type}, 
		{_NS_rtf, _LN_endnotes}, 
		{_NS_pp12, _LN_sndAc}, 
		{_NS_w11, _LN_pgSz}, 
		{_NS_style, _LN_auto_update}, 
		{_NS_v11, _LN_DefaultTabStop}, 
		{_NS_x11, _LN_InsertEntireRows}, 
		{_NS_ws12, _LN_selection}, 
		{_NS_a12, _LN_accent1}, 
		{_NS_ip12, _LN_externalViews}, 
		{_NS_w11, _LN_pgMar}, 
		{_NS_style, _LN_class}, 
		{_NS_rtf, _LN_enspace}, 
		{_NS_ws12, _LN_brk}, 
		{_NS_pp12, _LN_sldRef}, 
		{_NS_w11, _LN_paperSrc}, 
		{_NS_x11, _LN_NewAsync}, 
		{_NS_a12, _LN_accent2}, 
		{_NS_v11, _LN_TextDirection}, 
		{_NS_ws12, _LN_afdd}, 
		{_NS_rtf, _LN_expnd}, 
		{_NS_w11, _LN_pgBorders}, 
		{_NS_ip12, _LN_externalView}, 
		{_NS_pp12, _LN_ssJumpAc}, 
		{_NS_ws12, _LN_filter}, 
		{_NS_w11, _LN_lnNumType}, 
		{_NS_a12, _LN_accent3}, 
		{_NS_x11, _LN_NoAutofit}, 
		{_NS_style, _LN_default_outline_level}, 
		{_NS_rtf, _LN_expndtw}, 
		{_NS_w11, _LN_pgNumType}, 
		{_NS_pp12, _LN_sldJumpAc}, 
		{_NS_v11, _LN_TextBkgndTrans}, 
		{_NS_w11, _LN_cols}, 
		{_NS_ip12, _LN_attributeData}, 
		{_NS_style, _LN_map}, 
		{_NS_a12, _LN_accent4}, 
		{_NS_ws12, _LN_DateGroupItem}, 
		{_NS_x11, _LN_NoTitles}, 
		{_NS_w11, _LN_formProt}, 
		{_NS_rtf, _LN_expshrtn}, 
		{_NS_ip12, _LN_button}, 
		{_NS_style, _LN_condition}, 
		{_NS_pp12, _LN_urlAc}, 
		{_NS_v11, _LN_TxtPinX}, 
		{_NS_w11, _LN_vAlign}, 
		{_NS_rtf, _LN_f}, 
		{_NS_a12, _LN_accent5}, 
		{_NS_ws12, _LN_criteria}, 
		{_NS_w11, _LN_noEndnote}, 
		{_NS_pp12, _LN_fileAc}, 
		{_NS_x11, _LN_OverwriteCells}, 
		{_NS_v11, _LN_TxtPinY}, 
		{_NS_rtf, _LN_faauto}, 
		{_NS_w11, _LN_titlePg}, 
		{_NS_ip12, _LN_chooseFragment}, 
		{_NS_style, _LN_apply_style_name}, 
		{_NS_a12, _LN_accent6}, 
		{_NS_w11, _LN_textFlow}, 
		{_NS_pp12, _LN_presAc}, 
		{_NS_style, _LN_base_cell_address}, 
		{_NS_ws12, _LN_criteriaColor}, 
		{_NS_v11, _LN_TxtWidth}, 
		{_NS_w11, _LN_rtlGutter}, 
		{_NS_x11, _LN_Synchronous}, 
		{_NS_ip12, _LN_editWith}, 
		{_NS_a12, _LN_hlink}, 
		{_NS_rtf, _LN_facenter7_0}, 
		{_NS_w11, _LN_docGrid}, 
		{_NS_pp12, _LN_tooltip}, 
		{_NS_x11, _LN_NextId}, 
		{_NS_w11, _LN_br}, 
		{_NS_v11, _LN_TxtHeight}, 
		{_NS_style, _LN_default_style}, 
		{_NS_a12, _LN_folHlink}, 
		{_NS_rtf, _LN_facingp}, 
		{_NS_ws12, _LN_criteriaCellIcon}, 
		{_NS_w11, _LN_t}, 
		{_NS_ip12, _LN_unboundControls}, 
		{_NS_pp12, _LN_custShow}, 
		{_NS_w11, _LN_delText}, 
		{_NS_x11, _LN_ColumnInfo}, 
		{_NS_rtf, _LN_fahang}, 
		{_NS_v11, _LN_TxtLocPinX}, 
		{_NS_style, _LN_page_layout}, 
		{_NS_ws12, _LN_dataRefs}, 
		{_NS_w11, _LN_instrText}, 
		{_NS_ip12, _LN_editing}, 
		{_NS_a12, _LN_custClr}, 
		{_NS_pp12, _LN_programAc}, 
		{_NS_rtf, _LN_falt}, 
		{_NS_w11, _LN_delInstrText}, 
		{_NS_ws12, _LN_dataRef}, 
		{_NS_v11, _LN_TxtLocPinY}, 
		{_NS_x11, _LN_DoNotPersist}, 
		{_NS_w11, _LN_noBreakHyphen}, 
		{_NS_style, _LN_page_usage}, 
		{_NS_a12, _LN_latin}, 
		{_NS_pp12, _LN_macroAc}, 
		{_NS_ip12, _LN_masterDetail}, 
		{_NS_rtf, _LN_faroman}, 
		{_NS_w11, _LN_softHyphen}, 
		{_NS_w11, _LN_annotationRef}, 
		{_NS_v11, _LN_TxtAngle}, 
		{_NS_ws12, _LN_mergeCell}, 
		{_NS_a12, _LN_highAnsi}, 
		{_NS_pp12, _LN_hlinkAc}, 
		{_NS_rtf, _LN_favar}, 
		{_NS_style, _LN_header_style}, 
		{_NS_w11, _LN_footnoteRef}, 
		{_NS_x11, _LN_DoNotPersistSort}, 
		{_NS_ip12, _LN_fragmentToInsert}, 
		{_NS_w11, _LN_endnoteRef}, 
		{_NS_v11, _LN_AlignLeft}, 
		{_NS_rtf, _LN_fbias}, 
		{_NS_ws12, _LN_cellSmartTags}, 
		{_NS_pp12, _LN_oleVerbAc}, 
		{_NS_w11, _LN_separator}, 
		{_NS_a12, _LN_complexScript}, 
		{_NS_style, _LN_footer_style}, 
		{_NS_x11, _LN_DoNotPersstAF}, 
		{_NS_w11, _LN_continuationSeparator}, 
		{_NS_ip12, _LN_mainpane}, 
		{_NS_rtf, _LN_fbidi}, 
		{_NS_v11, _LN_AlignCenter}, 
		{_NS_pp12, _LN_mediaAc}, 
		{_NS_w11, _LN_footnote}, 
		{_NS_a12, _LN_eastAsian}, 
		{_NS_ws12, _LN_cellSmartTag}, 
		{_NS_x11, _LN_Sort}, 
		{_NS_style, _LN_master_page}, 
		{_NS_w11, _LN_endnote}, 
		{_NS_rtf, _LN_fchars}, 
		{_NS_ip12, _LN_printSettings}, 
		{_NS_style, _LN_header}, 
		{_NS_w11, _LN_sym}, 
		{_NS_v11, _LN_AlignRight}, 
		{_NS_pp12, _LN_custShowAc}, 
		{_NS_a12, _LN_majorFont}, 
		{_NS_ip12, _LN_header}, 
		{_NS_style, _LN_footer}, 
		{_NS_w11, _LN_pgNum}, 
		{_NS_x11, _LN_CaseSensitive}, 
		{_NS_rtf, _LN_fcharset}, 
		{_NS_ws12, _LN_cellSmartTagPr}, 
		{_NS_ip12, _LN_footer}, 
		{_NS_w11, _LN_cr}, 
		{_NS_pp12, _LN_onClick}, 
		{_NS_v11, _LN_AlignTop}, 
		{_NS_a12, _LN_minorFont}, 
		{_NS_w11, _LN_pict}, 
		{_NS_rtf, _LN_fdecor}, 
		{_NS_style, _LN_header_left}, 
		{_NS_ws12, _LN_userShView}, 
		{_NS_ip12, _LN_toolbar}, 
		{_NS_x11, _LN_FuturePersist}, 
		{_NS_w11, _LN_fldChar}, 
		{_NS_rtf, _LN_fet}, 
		{_NS_pp12, _LN_onMouseOver}, 
		{_NS_v11, _LN_AlignMiddle}, 
		{_NS_w11, _LN_ruby}, 
		{_NS_a12, _LN_fillStyle}, 
		{_NS_ws12, _LN_dVal}, 
		{_NS_ip12, _LN_menu}, 
		{_NS_pp12, _LN_seconds}, 
		{_NS_style, _LN_footer_left}, 
		{_NS_w11, _LN_rStyle}, 
		{_NS_a12, _LN_line}, 
		{_NS_x11, _LN_HeaderRow}, 
		{_NS_rtf, _LN_fetch}, 
		{_NS_style, _LN_display}, 
		{_NS_w11, _LN_rFonts}, 
		{_NS_v11, _LN_AlignBottom}, 
		{_NS_ws12, _LN_formula1}, 
		{_NS_ip12, _LN_menuArea}, 
		{_NS_pp12, _LN_percent}, 
		{_NS_w11, _LN_b}, 
		{_NS_a12, _LN_lineStyle}, 
		{_NS_x11, _LN_IdWrapped}, 
		{_NS_rtf, _LN_ffdefres}, 
		{_NS_pp12, _LN_chRg}, 
		{_NS_style, _LN_region_left}, 
		{_NS_v11, _LN_LockWidth}, 
		{_NS_ws12, _LN_formula2}, 
		{_NS_w11, _LN_b_cs}, 
		{_NS_ip12, _LN_taskpane}, 
		{_NS_a12, _LN_scene3d}, 
		{_NS_w11, _LN_i}, 
		{_NS_rtf, _LN_ffdeftext}, 
		{_NS_pp12, _LN_pRg}, 
		{_NS_x11, _LN_AlertVersion}, 
		{_NS_ws12, _LN_cfRule}, 
		{_NS_ip12, _LN_views}, 
		{_NS_a12, _LN_sp3d}, 
		{_NS_v11, _LN_LockHeight}, 
		{_NS_style, _LN_region_center}, 
		{_NS_w11, _LN_i_cs}, 
		{_NS_pp12, _LN_bg}, 
		{_NS_ws12, _LN_formula}, 
		{_NS_ip12, _LN_view}, 
		{_NS_rtf, _LN_ffentrymcr}, 
		{_NS_w11, _LN_caps}, 
		{_NS_x11, _LN_SortOrder}, 
		{_NS_pp12, _LN_subSp}, 
		{_NS_v11, _LN_LockMoveX}, 
		{_NS_w11, _LN_smallCaps}, 
		{_NS_a12, _LN_effectStyle}, 
		{_NS_rng, _LN_grammar}, 
		{_NS_ip12, _LN_xmlToEdit}, 
		{_NS_style, _LN_region_right}, 
		{_NS_ws12, _LN_colorScale}, 
		{_NS_w11, _LN_strike}, 
		{_NS_rtf, _LN_ffexitmcr}, 
		{_NS_w11, _LN_dstrike}, 
		{_NS_pp12, _LN_oleChartEl}, 
		{_NS_v11, _LN_LockMoveY}, 
		{_NS_x11, _LN_ExtraLeftColumns}, 
		{_NS_pr, _LN_notes}, 
		{_NS_ws12, _LN_databar}, 
		{_NS_a12, _LN_bgFillStyle}, 
		{_NS_w11, _LN_outline}, 
		{_NS_rtf, _LN_ffformat}, 
		{QName::EMPTY_STRING, _LN_datatypeLibrary}, 
		{_NS_pp12, _LN_tx}, 
		{_NS_ip12, _LN_documentSignatures}, 
		{_NS_w11, _LN_shadow}, 
		{_NS_v11, _LN_LockAspect}, 
		{_NS_ws12, _LN_iconset}, 
		{_NS_w11, _LN_emboss}, 
		{_NS_pp12, _LN_e2o}, 
		{_NS_table, _LN_table_template}, 
		{_NS_rng, _LN_define}, 
		{_NS_a12, _LN_fillStyleLst}, 
		{_NS_x11, _LN_ExtraRightColumns}, 
		{_NS_ws12, _LN_hyperlink}, 
		{_NS_rtf, _LN_ffhaslistbox}, 
		{_NS_w11, _LN_imprint}, 
		{_NS_ip12, _LN_signedDataBlock}, 
		{QName::EMPTY_STRING, _LN_name}, 
		{_NS_pp12, _LN_sldTgt}, 
		{_NS_v11, _LN_LockDelete}, 
		{_NS_w11, _LN_noProof}, 
		{_NS_ws12, _LN_cfvo}, 
		{_NS_a12, _LN_lineStyleLst}, 
		{_NS_w11, _LN_vanish}, 
		{_NS_x11, _LN_DeletedTitle}, 
		{_NS_ws12, _LN_color}, 
		{_NS_rtf, _LN_ffhelptext}, 
		{_NS_pp12, _LN_sndTgt}, 
		{_NS_ip12, _LN_message}, 
		{_NS_w11, _LN_webHidden}, 
		{_NS_rng, _LN_optional}, 
		{_NS_text, _LN_first_row_start_column}, 
		{_NS_v11, _LN_LockBegin}, 
		{_NS_rtf, _LN_ffhps}, 
		{_NS_w11, _LN_color}, 
		{_NS_ws12, _LN_oddHeader}, 
		{_NS_pp12, _LN_spTgt}, 
		{_NS_a12, _LN_effectStyleLst}, 
		{_NS_w11, _LN_w}, 
		{_NS_v11, _LN_LockEnd}, 
		{_NS_rng, _LN_attribute}, 
		{_NS_x11, _LN_NoToggleDataHeader}, 
		{_NS_rtf, _LN_ffl}, 
		{_NS_w11, _LN_kern}, 
		{_NS_pp12, _LN_tgtEl}, 
		{_NS_text, _LN_first_row_end_column}, 
		{_NS_ws12, _LN_oddFooter}, 
		{_NS_ip12, _LN_documentVersionUpgrade}, 
		{_NS_w11, _LN_position}, 
		{_NS_v11, _LN_LockRotate}, 
		{_NS_rtf, _LN_ffmaxlen}, 
		{_NS_pp12, _LN_tn}, 
		{_NS_a12, _LN_bgFillStyleLst}, 
		{_NS_w11, _LN_sz}, 
		{_NS_x11, _LN_AlternateMethod}, 
		{_NS_ws12, _LN_evenHeader}, 
		{_NS_pp12, _LN_rtn}, 
		{_NS_ip12, _LN_useTransform}, 
		{_NS_rtf, _LN_ffname}, 
		{_NS_v11, _LN_LockCrop}, 
		{_NS_w11, _LN_sz_cs}, 
		{_NS_a12, _LN_clrScheme}, 
		{_NS_text, _LN_last_row_start_column}, 
		{_NS_w11, _LN_highlight}, 
		{_NS_pp12, _LN_cond}, 
		{_NS_x11, _LN_NoAutoPage}, 
		{_NS_office, _LN_process_content}, 
		{_NS_ws12, _LN_evenFooter}, 
		{_NS_ip12, _LN_extensions}, 
		{_NS_rtf, _LN_ffownhelp}, 
		{_NS_w11, _LN_u}, 
		{_NS_pp12, _LN_par}, 
		{_NS_v11, _LN_LockVtxEdit}, 
		{_NS_a12, _LN_custClrLst}, 
		{_NS_rng, _LN_ref}, 
		{_NS_ip12, _LN_extension}, 
		{_NS_w11, _LN_effect}, 
		{_NS_pp12, _LN_seq}, 
		{_NS_x11, _LN_SheetName}, 
		{_NS_w11, _LN_bdr}, 
		{_NS_text, _LN_last_row_end_column}, 
		{_NS_ws12, _LN_firstHeader}, 
		{_NS_rtf, _LN_ffownstat}, 
		{_NS_rng, _LN_start}, 
		{_NS_pp12, _LN_excl}, 
		{_NS_a12, _LN_fontScheme}, 
		{_NS_w11, _LN_fitText}, 
		{_NS_v11, _LN_LockTextEdit}, 
		{_NS_pp12, _LN_animate}, 
		{_NS_ip12, _LN_ruleSetAction}, 
		{_NS_x11, _LN_ExternName}, 
		{_NS_w11, _LN_vertAlign}, 
		{_NS_rtf, _LN_ffprot}, 
		{_NS_rng, _LN_choice}, 
		{_NS_table, _LN_first_row}, 
		{_NS_pp12, _LN_animateColor}, 
		{_NS_ws12, _LN_firstFooter}, 
		{_NS_ip12, _LN_rule}, 
		{_NS_w11, _LN_rtl}, 
		{_NS_x11, _LN_Xct}, 
		{_NS_v11, _LN_LockFormat}, 
		{_NS_a12, _LN_effectScheme}, 
		{_NS_w11, _LN_cs}, 
		{_NS_rtf, _LN_ffrecalc}, 
		{_NS_rng, _LN_element}, 
		{_NS_table, _LN_last_row}, 
		{_NS_w11, _LN_em}, 
		{_NS_ip12, _LN_exitRuleSet}, 
		{_NS_pp12, _LN_animateEffect}, 
		{_NS_ws12, _LN_sortCondition}, 
		{_NS_v11, _LN_LockGroup}, 
		{_NS_rtf, _LN_ffres}, 
		{_NS_w11, _LN_hyphen}, 
		{_NS_x11, _LN_FormatSettings}, 
		{_NS_office, _LN_document}, 
		{_NS_pp12, _LN_animateMotion}, 
		{_NS_ws12, _LN_sct}, 
		{_NS_w11, _LN_lang}, 
		{_NS_table, _LN_first_column}, 
		{_NS_rtf, _LN_ffsize}, 
		{_NS_w11, _LN_asianLayout}, 
		{_NS_v11, _LN_LockCalcWH}, 
		{_NS_x11, _LN_Delimiters}, 
		{_NS_w11, _LN_specVanish}, 
		{_NS_ip12, _LN_dialogBoxMessageAction}, 
		{_NS_pp12, _LN_animateRotation}, 
		{_NS_office, _LN_document_content}, 
		{_NS_table, _LN_last_column}, 
		{_NS_ws12, _LN_rangeProtection}, 
		{_NS_w11, _LN_lists}, 
		{_NS_x11, _LN_Comma}, 
		{_NS_rtf, _LN_ffstattext}, 
		{_NS_v11, _LN_LockSelect}, 
		{_NS_table, _LN_body}, 
		{_NS_w11, _LN_styles}, 
		{_NS_ws12, _LN_slc}, 
		{_NS_w11, _LN_fonts}, 
		{_NS_pp12, _LN_animateScale}, 
		{_NS_rtf, _LN_fftype}, 
		{_NS_office, _LN_document_styles}, 
		{_NS_x11, _LN_Consecutive}, 
		{_NS_table, _LN_even_rows}, 
		{_NS_w11, _LN_rubyAlign}, 
		{_NS_pp12, _LN_command}, 
		{_NS_v11, _LN_LockCustProp}, 
		{_NS_ip12, _LN_dialogBoxExpressionAction}, 
		{_NS_ws12, _LN_cellWatch}, 
		{_NS_w11, _LN_hps}, 
		{_NS_pp12, _LN_set}, 
		{_NS_rtf, _LN_fftypetxt}, 
		{_NS_x11, _LN_Custom}, 
		{_NS_ws12, _LN_t}, 
		{_NS_w11, _LN_hpsRaise}, 
		{_NS_pp12, _LN_audio}, 
		{_NS_table, _LN_odd_rows}, 
		{_NS_office, _LN_document_meta}, 
		{_NS_v11, _LN_HelpTopic}, 
		{_NS_ws12, _LN_r}, 
		{_NS_rtf, _LN_fi}, 
		{_NS_w11, _LN_hpsBaseText}, 
		{_NS_ip12, _LN_switchViewAction}, 
		{_NS_ws12, _LN_rt}, 
		{_NS_pp12, _LN_video}, 
		{_NS_x11, _LN_SemiColon}, 
		{_NS_w11, _LN_lid}, 
		{_NS_rtf, _LN_fid}, 
		{_NS_v11, _LN_Copyright}, 
		{_NS_table, _LN_even_columns}, 
		{_NS_ws12, _LN_rubyBase}, 
		{_NS_w11, _LN_rubyPr}, 
		{_NS_rtf, _LN_field}, 
		{_NS_office, _LN_document_settings}, 
		{_NS_x11, _LN_Space}, 
		{_NS_ws12, _LN_rPr}, 
		{_NS_w11, _LN_rt}, 
		{_NS_rtf, _LN_file}, 
		{_NS_pp12, _LN_beginCondLst}, 
		{_NS_ip12, _LN_assignmentAction}, 
		{_NS_x11, _LN_Tab}, 
		{_NS_ws12, _LN_font}, 
		{_NS_w11, _LN_rubyBase}, 
		{_NS_table, _LN_odd_columns}, 
		{_NS_v11, _LN_NoObjHandles}, 
		{QName::EMPTY_STRING, _LN_combine}, 
		{_NS_ws12, _LN_charset}, 
		{_NS_w11, _LN_pPr}, 
		{_NS_rtf, _LN_filetbl}, 
		{_NS_ws12, _LN_family}, 
		{_NS_pp12, _LN_endCondLst}, 
		{_NS_w11, _LN_subDoc}, 
		{_NS_ip12, _LN_queryAction}, 
		{_NS_x11, _LN_TextQualifier}, 
		{_NS_style, _LN_font_face}, 
		{_NS_office, _LN_version}, 
		{_NS_v11, _LN_NonPrinting}, 
		{_NS_ws12, _LN_b}, 
		{_NS_a12, _LN_sx}, 
		{_NS_w11, _LN_gridCol}, 
		{_NS_rtf, _LN_fittext}, 
		{_NS_ws12, _LN_i}, 
		{_NS_pp12, _LN_endSync}, 
		{_NS_w11, _LN_insideH}, 
		{_NS_a12, _LN_sy}, 
		{_NS_x11, _LN_StartRow}, 
		{_NS_office, _LN_mimetype}, 
		{_NS_ws12, _LN_strike}, 
		{_NS_pp12, _LN_iterate}, 
		{_NS_svg, _LN_font_family}, 
		{_NS_w11, _LN_insideV}, 
		{_NS_rtf, _LN_fldalt}, 
		{_NS_v11, _LN_NoCtlHandles}, 
		{_NS_ws12, _LN_outline}, 
		{_NS_a12, _LN_off}, 
		{_NS_ip12, _LN_openNewDocumentAction}, 
		{_NS_w11, _LN_tl2br}, 
		{_NS_office, _LN_meta}, 
		{_NS_x11, _LN_Decimal}, 
		{_NS_ws12, _LN_shadow}, 
		{_NS_pp12, _LN_childLst}, 
		{_NS_w11, _LN_tr2bl}, 
		{_NS_rtf, _LN_flddirty}, 
		{_NS_svg, _LN_font_style}, 
		{_NS_a12, _LN_ext}, 
		{_NS_v11, _LN_NoAlignBox}, 
		{_NS_w11, _LN_tcW}, 
		{_NS_rng, _LN_zeroOrMore}, 
		{_NS_pp12, _LN_subLst}, 
		{_NS_ws12, _LN_condense}, 
		{_NS_w11, _LN_gridSpan}, 
		{_NS_a12, _LN_chOff}, 
		{_NS_rtf, _LN_fldedit}, 
		{_NS_ip12, _LN_closeDocumentAction}, 
		{_NS_x11, _LN_ThousandSeparator}, 
		{_NS_svg, _LN_font_variant}, 
		{_NS_w11, _LN_hmerge}, 
		{_NS_office, _LN_body}, 
		{_NS_v11, _LN_UpdateAlignBox}, 
		{_NS_ws12, _LN_extend}, 
		{_NS_a12, _LN_chExt}, 
		{_NS_w11, _LN_vmerge}, 
		{_NS_rtf, _LN_fldinst}, 
		{_NS_ip12, _LN_ruleSet}, 
		{_NS_pp12, _LN_timeNodeBase}, 
		{_NS_ws12, _LN_sz}, 
		{_NS_office, _LN_text}, 
		{_NS_x11, _LN_FitToPage}, 
		{_NS_w11, _LN_tcBorders}, 
		{_NS_svg, _LN_font_weight}, 
		{_NS_v11, _LN_HideText}, 
		{_NS_ws12, _LN_u}, 
		{_NS_w11, _LN_noWrap}, 
		{_NS_rtf, _LN_fldlock}, 
		{_NS_ip12, _LN_ruleSets}, 
		{_NS_rng, _LN_group}, 
		{_NS_ws12, _LN_vertAlign}, 
		{_NS_pp12, _LN_prevCondLst}, 
		{_NS_w11, _LN_tcMar}, 
		{_NS_a12, _LN_tint}, 
		{_NS_ws12, _LN_ruby}, 
		{_NS_w11, _LN_tcFitText}, 
		{_NS_svg, _LN_font_stretch}, 
		{_NS_v11, _LN_DynFeedback}, 
		{_NS_rtf, _LN_fldpriv}, 
		{_NS_text, _LN_global}, 
		{_NS_x11, _LN_DoNotDisplayColHeaders}, 
		{_NS_w11, _LN_tcPr}, 
		{_NS_ip12, _LN_calculations}, 
		{_NS_a12, _LN_shade}, 
		{_NS_pp12, _LN_nextCondLst}, 
		{_NS_ws12, _LN_sstItem}, 
		{_NS_w11, _LN_gridBefore}, 
		{_NS_rtf, _LN_fldrslt}, 
		{_NS_svg, _LN_font_size}, 
		{_NS_v11, _LN_GlueType}, 
		{_NS_office, _LN_drawing}, 
		{_NS_a12, _LN_comp}, 
		{_NS_w11, _LN_gridAfter}, 
		{_NS_w11, _LN_wBefore}, 
		{_NS_ws12, _LN_webPubItem}, 
		{_NS_ip12, _LN_calculatedField}, 
		{_NS_rtf, _LN_fldtype}, 
		{_NS_rng, _LN_empty}, 
		{_NS_a12, _LN_inv}, 
		{_NS_x11, _LN_DoNotDisplayRowHeaders}, 
		{_NS_w11, _LN_wAfter}, 
		{_NS_svg, _LN_unicode_range}, 
		{_NS_v11, _LN_WalkPreference}, 
		{_NS_a12, _LN_gray}, 
		{_NS_w11, _LN_cantSplit}, 
		{_NS_ws12, _LN_csViews}, 
		{_NS_rtf, _LN_fmodern}, 
		{_NS_office, _LN_presentation}, 
		{_NS_w11, _LN_trHeight}, 
		{_NS_x11, _LN_ViewableRange}, 
		{_NS_v11, _LN_BegTrigger}, 
		{_NS_rtf, _LN_fn}, 
		{_NS_svg, _LN_units_per_em}, 
		{_NS_a12, _LN_alpha}, 
		{_NS_ws12, _LN_csView}, 
		{_NS_w11, _LN_tblHeader}, 
		{_NS_w11, _LN_tblCellSpacing}, 
		{_NS_rtf, _LN_fname}, 
		{_NS_office, _LN_spreadsheet}, 
		{_NS_svg, _LN_panose_1}, 
		{_NS_a12, _LN_alphaOff}, 
		{_NS_v11, _LN_EndTrigger}, 
		{_NS_w11, _LN_tblPrEx}, 
		{_NS_x11, _LN_GridlineColor}, 
		{_NS_w11, _LN_trPr}, 
		{_NS_office, _LN_chart}, 
		{_NS_rtf, _LN_fnetwork}, 
		{_NS_svg, _LN_stemv}, 
		{_NS_a12, _LN_alphaMod}, 
		{_NS_v11, _LN_ObjType}, 
		{_NS_w11, _LN_tc}, 
		{_NS_office, _LN_image}, 
		{_NS_on12, _LN_Import}, 
		{_NS_w11, _LN_tblStyle}, 
		{_NS_rtf, _LN_fnil}, 
		{_NS_svg, _LN_stemh}, 
		{_NS_a12, _LN_hue}, 
		{_NS_x11, _LN_ExcelWorksheetType}, 
		{_NS_v11, _LN_Comment}, 
		{_NS_w11, _LN_tblpPr}, 
		{_NS_w11, _LN_tblOverlap}, 
		{_NS_office, _LN_settings}, 
		{_NS_svg, _LN_slope}, 
		{_NS_a12, _LN_hueOff}, 
		{_NS_rtf, _LN_fnonfilesys}, 
		{_NS_x11, _LN_IntlMacro}, 
		{_NS_w11, _LN_tblRtl}, 
		{_NS_on12, _LN_EnsurePage}, 
		{_NS_v11, _LN_IsDropSource}, 
		{_NS_w11, _LN_bidiVisual}, 
		{_NS_a12, _LN_hueMod}, 
		{_NS_rng, _LN_oneOrMore}, 
		{_NS_svg, _LN_cap_height}, 
		{_NS_rtf, _LN_fontemb}, 
		{_NS_x11, _LN_Unsynced}, 
		{_NS_w11, _LN_tblStyleRowBandSize}, 
		{_NS_a12, _LN_sat}, 
		{_NS_on12, _LN_PlaceObjects}, 
		{_NS_w11, _LN_tblStyleColBandSize}, 
		{_NS_swb12, _LN_revisions}, 
		{_NS_v11, _LN_NoLiveDynamics}, 
		{_NS_svg, _LN_x_height}, 
		{_NS_rtf, _LN_fontfile}, 
		{_NS_w11, _LN_tblW}, 
		{_NS_x11, _LN_Selected}, 
		{_NS_config, _LN_config_item_set}, 
		{_NS_a12, _LN_satOff}, 
		{_NS_swb12, _LN_info}, 
		{_NS_on12, _LN_Object}, 
		{_NS_w11, _LN_tblInd}, 
		{_NS_config, _LN_name}, 
		{_NS_rtf, _LN_fonttbl}, 
		{_NS_swb12, _LN_header}, 
		{_NS_w11, _LN_tblBorders}, 
		{_NS_v11, _LN_LocalizeMerge}, 
		{_NS_x11, _LN_CodeName}, 
		{_NS_svg, _LN_accent_height}, 
		{_NS_a12, _LN_satMod}, 
		{_NS_on12, _LN_Delete}, 
		{_NS_rtf, _LN_footer}, 
		{_NS_w11, _LN_tblLayout}, 
		{_NS_pp12, _LN_attrName}, 
		{_NS_swb12, _LN_rwCol}, 
		{_NS_on12, _LN_Position}, 
		{_NS_config, _LN_config_item}, 
		{_NS_a12, _LN_lum}, 
		{_NS_w11, _LN_tblCellMar}, 
		{_NS_svg, _LN_ascent}, 
		{_NS_v11, _LN_Calendar}, 
		{_NS_x11, _LN_PageSetup}, 
		{_NS_on12, _LN_Image}, 
		{_NS_rtf, _LN_footerf}, 
		{_NS_rng, _LN_text}, 
		{_NS_w11, _LN_tblLook}, 
		{_NS_swb12, _LN_move}, 
		{_NS_a12, _LN_lumOff}, 
		{_NS_pp12, _LN_attrNameLst}, 
		{_NS_w11, _LN_tblPr}, 
		{_NS_svg, _LN_descent}, 
		{_NS_v11, _LN_LangID}, 
		{_NS_on12, _LN_Ink}, 
		{_NS_config, _LN_type}, 
		{_NS_rtf, _LN_footerl}, 
		{_NS_w11, _LN_tblGrid}, 
		{_NS_swb12, _LN_userView}, 
		{_NS_x11, _LN_DisplayPageBreak}, 
		{_NS_a12, _LN_lumMod}, 
		{_NS_pp12, _LN_boolVal}, 
		{_NS_w11, _LN_tr}, 
		{_NS_svg, _LN_widths}, 
		{_NS_rng, _LN_value}, 
		{_NS_on12, _LN_Outline}, 
		{_NS_a12, _LN_red}, 
		{_NS_rtf, _LN_footery}, 
		{_NS_v11, _LN_ShapeKeywords}, 
		{_NS_w11, _LN_pos}, 
		{_NS_swb12, _LN_sheetName}, 
		{_NS_svg, _LN_bbox}, 
		{_NS_rtf, _LN_footnote}, 
		{_NS_on12, _LN_Html}, 
		{_NS_pp12, _LN_intVal}, 
		{_NS_w11, _LN_numFmt}, 
		{_NS_a12, _LN_redOff}, 
		{_NS_w11, _LN_numStart}, 
		{_NS_swb12, _LN_sheet}, 
		{_NS_on12, _LN_File}, 
		{_NS_pp12, _LN_fltVal}, 
		{_NS_rtf, _LN_formdisp}, 
		{_NS_v11, _LN_DropOnPageScale}, 
		{_NS_w11, _LN_numRestart}, 
		{_NS_config, _LN_config_item_map_indexed}, 
		{_NS_svg, _LN_ideographic}, 
		{_NS_a12, _LN_redMod}, 
		{_NS_x11, _LN_TransitionExpressionEvaluation}, 
		{_NS_swb12, _LN_cell}, 
		{_NS_w11, _LN_active}, 
		{_NS_on12, _LN_Data}, 
		{_NS_a12, _LN_green}, 
		{_NS_pp12, _LN_strVal}, 
		{_NS_swb12, _LN_formatting}, 
		{_NS_w11, _LN_hash}, 
		{_NS_rtf, _LN_formfield}, 
		{_NS_svg, _LN_alphabetic}, 
		{_NS_v11, _LN_XRulerDensity}, 
		{_NS_w11, _LN_column}, 
		{_NS_pp12, _LN_val}, 
		{_NS_a12, _LN_greenOff}, 
		{_NS_config, _LN_config_item_map_entry}, 
		{_NS_swb12, _LN_autofmt}, 
		{_NS_w11, _LN_uniqueTag}, 
		{_NS_rtf, _LN_formprot}, 
		{_NS_x11, _LN_TransitionFormulaEntry}, 
		{_NS_pp12, _LN_tav}, 
		{_NS_swb12, _LN_name}, 
		{_NS_w11, _LN_name}, 
		{_NS_svg, _LN_mathematical}, 
		{_NS_x11, _LN_Print}, 
		{_NS_v11, _LN_YRulerDensity}, 
		{_NS_a12, _LN_greenMod}, 
		{_NS_swb12, _LN_comment}, 
		{_NS_w11, _LN_mappedName}, 
		{_NS_rtf, _LN_formshade}, 
		{_NS_a12, _LN_blue}, 
		{_NS_prj12, _LN_Type}, 
		{_NS_x11, _LN_Zoom}, 
		{_NS_w11, _LN_dynamicAddress}, 
		{_NS_svg, _LN_hanging}, 
		{_NS_pp12, _LN_behaviorBase}, 
		{_NS_config, _LN_config_item_map_named}, 
		{_NS_swb12, _LN_qtField}, 
		{_NS_w11, _LN_udl}, 
		{_NS_v11, _LN_XRulerOrigin}, 
		{_NS_prj12, _LN_UID}, 
		{_NS_rtf, _LN_fosnum}, 
		{_NS_a12, _LN_blueOff}, 
		{_NS_w11, _LN_table}, 
		{_NS_pp12, _LN_tavLst}, 
		{_NS_office, _LN_scripts}, 
		{_NS_x11, _LN_PageBreakZoom}, 
		{_NS_svg, _LN_v_ideographic}, 
		{_NS_prj12, _LN_Start}, 
		{_NS_rtf, _LN_fprq}, 
		{_NS_w11, _LN_src}, 
		{_NS_swb12, _LN_conflict}, 
		{_NS_pp12, _LN_rgb}, 
		{_NS_a12, _LN_blueMod}, 
		{_NS_v11, _LN_YRulerOrigin}, 
		{_NS_w11, _LN_filter}, 
		{_NS_office, _LN_script}, 
		{_NS_pp12, _LN_hsl}, 
		{_NS_prj12, _LN_Finish}, 
		{_NS_w11, _LN_sort}, 
		{_NS_swb12, _LN_revdata}, 
		{_NS_rtf, _LN_fracwidth}, 
		{_NS_svg, _LN_v_alphabetic}, 
		{_NS_pp12, _LN_by}, 
		{_NS_x11, _LN_ShowPageBreakZoom}, 
		{_NS_w11, _LN_fieldMapData}, 
		{_NS_rng, _LN_mixed}, 
		{_NS_v11, _LN_XGridDensity}, 
		{_NS_prj12, _LN_Unit}, 
		{_NS_pp12, _LN_from}, 
		{_NS_a12, _LN_scrgbClr}, 
		{_NS_w11, _LN_colDelim}, 
		{_NS_swb12, _LN_tabidMap}, 
		{_NS_prj12, _LN_Value}, 
		{_NS_rtf, _LN_frelative}, 
		{_NS_pp12, _LN_to}, 
		{_NS_w11, _LN_jdsoType}, 
		{_NS_script, _LN_language}, 
		{_NS_svg, _LN_v_mathematical}, 
		{_NS_a12, _LN_srgbClr}, 
		{_NS_v11, _LN_YGridDensity}, 
		{_NS_swb12, _LN_tabId}, 
		{_NS_w11, _LN_fHdr}, 
		{_NS_x11, _LN_DefaultRowHeight}, 
		{_NS_prj12, _LN_Project}, 
		{_NS_pp12, _LN_progress}, 
		{_NS_rtf, _LN_frmtxbtlr}, 
		{_NS_w11, _LN_recipientData}, 
		{_NS_text, _LN_object_index_source}, 
		{_NS_prj12, _LN_Name}, 
		{_NS_svg, _LN_v_hanging}, 
		{_NS_a12, _LN_hslClr}, 
		{_NS_w11, _LN_mainDocumentType}, 
		{_NS_swb12, _LN_undoinfo}, 
		{_NS_office, _LN_font_face_decls}, 
		{_NS_prj12, _LN_Title}, 
		{_NS_v11, _LN_XGridSpacing}, 
		{_NS_pp12, _LN_rCenter}, 
		{_NS_rtf, _LN_frmtxlrtb}, 
		{_NS_w11, _LN_linkToQuery}, 
		{_NS_prj12, _LN_Subject}, 
		{_NS_x11, _LN_DefaultColumnWidth}, 
		{_NS_a12, _LN_sysClr}, 
		{_NS_w11, _LN_dataType}, 
		{_NS_swb12, _LN_cellOld}, 
		{_NS_office, _LN_styles}, 
		{_NS_prj12, _LN_Category}, 
		{_NS_w11, _LN_defaultSQL}, 
		{_NS_svg, _LN_underline_position}, 
		{_NS_v11, _LN_YGridSpacing}, 
		{_NS_text, _LN_use_spreadsheet_objects}, 
		{_NS_rtf, _LN_frmtxlrtbv}, 
		{_NS_prj12, _LN_Company}, 
		{_NS_pp12, _LN_mediaNodeBase}, 
		{_NS_a12, _LN_schemeClr}, 
		{_NS_w11, _LN_connectString}, 
		{_NS_swb12, _LN_cellNew}, 
		{_NS_x11, _LN_StandardWidth}, 
		{_NS_prj12, _LN_Manager}, 
		{_NS_rng, _LN_interleave}, 
		{_NS_w11, _LN_query}, 
		{_NS_x11, _LN_Visible}, 
		{_NS_rtf, _LN_frmtxtbrl}, 
		{_NS_v11, _LN_XGridOrigin}, 
		{_NS_a12, _LN_prstClr}, 
		{_NS_swb12, _LN_dxfOld}, 
		{_NS_pp12, _LN_timeNodeLst}, 
		{_NS_w11, _LN_dataSource}, 
		{_NS_prj12, _LN_Author}, 
		{_NS_text, _LN_use_math_objects}, 
		{_NS_svg, _LN_underline_thickness}, 
		{_NS_w11, _LN_headerSource}, 
		{_NS_pp12, _LN_tmpl}, 
		{_NS_office, _LN_automatic_styles}, 
		{_NS_swb12, _LN_dxfNew}, 
		{_NS_rtf, _LN_frmtxtbrlv}, 
		{_NS_w11, _LN_doNotSuppressBlankLines}, 
		{_NS_v11, _LN_YGridOrigin}, 
		{_NS_x11, _LN_LeftColumnVisible}, 
		{_NS_prj12, _LN_CreationDate}, 
		{_NS_w11, _LN_destination}, 
		{_NS_text, _LN_use_draw_objects}, 
		{_NS_pp12, _LN_tmplLst}, 
		{_NS_swb12, _LN_dxfFmt}, 
		{_NS_rtf, _LN_froman}, 
		{_NS_prj12, _LN_Revision}, 
		{_NS_w11, _LN_addressFieldName}, 
		{_NS_svg, _LN_strikethrough_position}, 
		{_NS_office, _LN_master_styles}, 
		{_NS_swb12, _LN_formula}, 
		{_NS_v11, _LN_OutputFormat}, 
		{_NS_w11, _LN_mailSubject}, 
		{_NS_pp12, _LN_bldAsOne}, 
		{_NS_x11, _LN_DisplayRightToLeft}, 
		{_NS_rtf, _LN_fromhtml}, 
		{_NS_w11, _LN_mailAsAttachment}, 
		{_NS_prj12, _LN_LastSaved}, 
		{_NS_meta, _LN_generator}, 
		{_NS_text, _LN_use_chart_objects}, 
		{_NS_w11, _LN_viewMergedData}, 
		{_NS_swb12, _LN_formulaOld}, 
		{_NS_v11, _LN_LockPreview}, 
		{_NS_pp12, _LN_bldSub}, 
		{_NS_w11, _LN_activeRecord}, 
		{_NS_rtf, _LN_fromtext}, 
		{_NS_svg, _LN_strikethrough_thickness}, 
		{_NS_dc, _LN_title}, 
		{_NS_w11, _LN_checkErrors}, 
		{_NS_pp12, _LN_bldP}, 
		{_NS_x11, _LN_GridlineColorIndex}, 
		{_NS_prj12, _LN_ScheduleFromStart}, 
		{_NS_rtf, _LN_fs}, 
		{_NS_v11, _LN_AddMarkup}, 
		{_NS_w11, _LN_odso}, 
		{_NS_text, _LN_use_other_objects}, 
		{_NS_w11, _LN_origWordTableRules}, 
		{_NS_dc, _LN_description}, 
		{_NS_pp12, _LN_bldDgm}, 
		{_NS_rtf, _LN_fscript}, 
		{_NS_svg, _LN_overline_position}, 
		{_NS_prj12, _LN_StartDate}, 
		{_NS_v11, _LN_ViewMarkup}, 
		{_NS_w11, _LN_wpJustification}, 
		{_NS_x11, _LN_DisplayFormulas}, 
		{_NS_w11, _LN_noTabHangInd}, 
		{_NS_dc, _LN_subject}, 
		{_NS_rtf, _LN_fswiss}, 
		{_NS_pp12, _LN_bldOleChart}, 
		{_NS_w11, _LN_noLeading}, 
		{_NS_prj12, _LN_FinishDate}, 
		{_NS_text, _LN_object_index_entry_template}, 
		{_NS_v11, _LN_PreviewQuality}, 
		{_NS_w11, _LN_spaceForUL}, 
		{_NS_meta, _LN_keyword}, 
		{_NS_svg, _LN_overline_thickness}, 
		{_NS_rtf, _LN_ftnalt}, 
		{_NS_pp12, _LN_bldE2o}, 
		{_NS_x11, _LN_DoNotDisplayGridlines}, 
		{_NS_w11, _LN_noColumnBalance}, 
		{_NS_prj12, _LN_FYStartDate}, 
		{_NS_text, _LN_user_index}, 
		{_NS_rtf, _LN_ftnbj}, 
		{_NS_w11, _LN_balanceSingleByteDoubleByteWidth}, 
		{_NS_v11, _LN_PreviewScope}, 
		{_NS_swbu12, _LN_users}, 
		{_NS_pp12, _LN_buildLst}, 
		{_NS_svg, _LN_font_face_src}, 
		{_NS_w11, _LN_transparentMetafiles}, 
		{_NS_meta, _LN_initial_creator}, 
		{_NS_rtf, _LN_ftncn}, 
		{_NS_w11, _LN_noExtraLineSpacing}, 
		{_NS_swbu12, _LN_usrinfo}, 
		{_NS_x11, _LN_DoNotDisplayHeadings}, 
		{_NS_v11, _LN_DocLangID}, 
		{_NS_prj12, _LN_CriticalSlackLimit}, 
		{_NS_w11, _LN_doNotLeaveBackslashAlone}, 
		{_NS_dc, _LN_creator}, 
		{_NS_text, _LN_user_index_source}, 
		{_NS_rtf, _LN_ftnil}, 
		{_NS_svg, _LN_font_face_uri}, 
		{_NS_w11, _LN_ulTrailSpace}, 
		{_NS_v11, _LN_Gamma}, 
		{_NS_w11, _LN_doNotExpandShiftReturn}, 
		{_NS_meta, _LN_printed_by}, 
		{_NS_text, _LN_use_graphics}, 
		{_NS_prj12, _LN_CurrencyDigits}, 
		{_NS_rtf, _LN_ftnlytwnine}, 
		{_NS_w11, _LN_spacingInWholePoints}, 
		{_NS_x11, _LN_DoNotDisplayOutline}, 
		{_NS_v11, _LN_Contrast}, 
		{_NS_svg, _LN_font_face_format}, 
		{_NS_w11, _LN_lineWrapLikeWord6}, 
		{_NS_rtf, _LN_ftnnalc}, 
		{_NS_w11, _LN_printBodyTextBeforeHeader}, 
		{_NS_meta, _LN_creation_date}, 
		{_NS_text, _LN_use_tables}, 
		{_NS_svg, _LN_string}, 
		{_NS_prj12, _LN_CurrencySymbol}, 
		{_NS_v11, _LN_Brightness}, 
		{_NS_w11, _LN_printColBlack}, 
		{_NS_pp12, _LN_sld}, 
		{_NS_dc, _LN_date}, 
		{_NS_rtf, _LN_ftnnar}, 
		{_NS_w11, _LN_wpSpaceWidth}, 
		{_NS_pp12, _LN_all}, 
		{_NS_x11, _LN_ApplyAutomaticOutlineStyles}, 
		{_NS_v11, _LN_Sharpen}, 
		{_NS_w11, _LN_showBreaksInFrames}, 
		{_NS_svg, _LN_font_face_name}, 
		{_NS_text, _LN_use_floating_frames}, 
		{_NS_rtf, _LN_ftnnauc}, 
		{_NS_w11, _LN_subFontBySize}, 
		{_NS_pp12, _LN_range}, 
		{_NS_meta, _LN_print_date}, 
		{_NS_svg, _LN_name}, 
		{_NS_v11, _LN_Blur}, 
		{_NS_prj12, _LN_CurrencySymbolPosition}, 
		{_NS_sc12, _LN_singleCells}, 
		{_NS_w11, _LN_suppressBottomSpacing}, 
		{_NS_rtf, _LN_ftnnchi}, 
		{_NS_sc12, _LN_table}, 
		{_NS_w11, _LN_suppressTopSpacing}, 
		{_NS_text, _LN_use_objects}, 
		{_NS_pp12, _LN_custData}, 
		{_NS_meta, _LN_template}, 
		{_NS_v11, _LN_Denoise}, 
		{_NS_x11, _LN_NoSummaryRowsBelowDetail}, 
		{_NS_w11, _LN_suppressTopSpacingMac5}, 
		{_NS_prj12, _LN_CalendarUID}, 
		{_NS_svg, _LN_definition_src}, 
		{_NS_xlink, _LN_href}, 
		{_NS_w11, _LN_suppressTopSpacingWP}, 
		{_NS_rtf, _LN_ftnnchosung}, 
		{_NS_xlink, _LN_type}, 
		{_NS_w11, _LN_suppressSpBfAfterPgBrk}, 
		{_NS_v11, _LN_Transparency}, 
		{_NS_text, _LN_copy_outline_levels}, 
		{_NS_w11, _LN_swapBordersFacingPages}, 
		{_NS_prj12, _LN_DefaultStartTime}, 
		{_NS_style, _LN_font_adornments}, 
		{_NS_rtf, _LN_ftnncnum}, 
		{_NS_xlink, _LN_actuate}, 
		{_NS_w11, _LN_convMailMergeEsc}, 
		{_NS_x11, _LN_NoSummaryColumnsRightDetail}, 
		{_NS_v11, _LN_SelectMode}, 
		{_NS_xlink, _LN_title}, 
		{_NS_w11, _LN_truncateFontHeight}, 
		{_NS_rtf, _LN_ftnndbar}, 
		{_NS_meta, _LN_date}, 
		{_NS_w11, _LN_mwSmallCaps}, 
		{_NS_prj12, _LN_DefaultFinishTime}, 
		{_NS_style, _LN_font_family_generic}, 
		{_NS_text, _LN_user_index_entry_template}, 
		{_NS_w11, _LN_usePrinterMetrics}, 
		{_NS_v11, _LN_DisplayMode}, 
		{_NS_sst12, _LN_Sst}, 
		{_NS_x11, _LN_DoNotDisplayZeros}, 
		{_NS_rtf, _LN_ftnndbnum}, 
		{_NS_w11, _LN_ww6BorderRules}, 
		{_NS_meta, _LN_auto_reload}, 
		{_NS_w11, _LN_wrapTrailSpaces}, 
		{_NS_style, _LN_font_pitch}, 
		{_NS_prj12, _LN_MinutesPerDay}, 
		{_NS_pp12, _LN_commentAuthor}, 
		{_NS_v11, _LN_IsDropTarget}, 
		{_NS_w11, _LN_footnoteLayoutLikeWW8}, 
		{_NS_xlink, _LN_show}, 
		{_NS_x11, _LN_ActiveColumn}, 
		{_NS_pp12, _LN_pos}, 
		{_NS_rtf, _LN_ftnndbnumd}, 
		{_NS_text, _LN_alphabetical_index}, 
		{_NS_w11, _LN_shapeLayoutLikeWW8}, 
		{_NS_pp12, _LN_text}, 
		{_NS_meta, _LN_delay}, 
		{_NS_style, _LN_font_charset}, 
		{_NS_w11, _LN_alignTablesRowByRow}, 
		{_NS_prj12, _LN_MinutesPerWeek}, 
		{_NS_x11, _LN_FilterOn}, 
		{_NS_v11, _LN_IsSnapTarget}, 
		{_NS_pp12, _LN_cm}, 
		{_NS_rtf, _LN_ftnndbnumk}, 
		{_NS_w11, _LN_forgetLastTabAlignment}, 
		{_NS_w11, _LN_adjustLineHeightInTable}, 
		{_NS_number, _LN_number_style}, 
		{_NS_text, _LN_alphabetical_index_source}, 
		{_NS_prj12, _LN_DaysPerMonth}, 
		{_NS_x11, _LN_TopRowVisible}, 
		{_NS_rtf, _LN_ftnndbnumt}, 
		{_NS_w11, _LN_autoSpaceLikeWord95}, 
		{_NS_meta, _LN_hyperlink_behaviour}, 
		{_NS_number, _LN_number}, 
		{_NS_v11, _LN_IsTextEditTarget}, 
		{_NS_w11, _LN_noSpaceRaiseLower}, 
		{_NS_s12, _LN_Styles}, 
		{_NS_w11, _LN_doNotUseHTMLParagraphAutoSpacing}, 
		{_NS_text, _LN_ignore_case}, 
		{_NS_s12, _LN_fmts}, 
		{_NS_rtf, _LN_ftnnganada}, 
		{_NS_prj12, _LN_DefaultTaskType}, 
		{_NS_w11, _LN_layoutRawTableWidth}, 
		{_NS_x11, _LN_TopRowBottomPane}, 
		{_NS_s12, _LN_fonts}, 
		{_NS_office, _LN_target_frame_name}, 
		{_NS_v11, _LN_DontMoveChildren}, 
		{_NS_number, _LN_decimal_replacement}, 
		{_NS_w11, _LN_layoutTableRowsApart}, 
		{_NS_dc, _LN_language}, 
		{_NS_rtf, _LN_ftnngbnum}, 
		{_NS_s12, _LN_fills}, 
		{_NS_w11, _LN_useWord97LineBreakingRules}, 
		{_NS_text, _LN_main_entry_style_name}, 
		{_NS_s12, _LN_borders}, 
		{_NS_w11, _LN_breakWrappedTables}, 
		{_NS_pic12, _LN_spPr}, 
		{_NS_v11, _LN_ShapePermeableX}, 
		{_NS_x11, _LN_LeftColumnRightPane}, 
		{_NS_prj12, _LN_DefaultFixedCostAccrual}, 
		{_NS_number, _LN_display_factor}, 
		{_NS_rtf, _LN_ftnngbnumd}, 
		{_NS_w11, _LN_snapToGridInCell}, 
		{_NS_meta, _LN_editing_cycles}, 
		{_NS_s12, _LN_xfs}, 
		{_NS_w11, _LN_dontAllowFieldEndSelect}, 
		{_NS_s12, _LN_styles}, 
		{_NS_x11, _LN_ActivePane}, 
		{_NS_w11, _LN_applyBreakingRules}, 
		{_NS_rtf, _LN_ftnngbnumk}, 
		{_NS_number, _LN_embedded_text}, 
		{_NS_v11, _LN_ShapePermeableY}, 
		{_NS_text, _LN_alphabetical_separators}, 
		{_NS_w11, _LN_wrapTextWithPunct}, 
		{_NS_s12, _LN_dxfs}, 
		{_NS_prj12, _LN_DefaultStandardRate}, 
		{_NS_meta, _LN_editing_duration}, 
		{_NS_number, _LN_position}, 
		{_NS_w11, _LN_useAsianBreakRules}, 
		{_NS_rtf, _LN_ftnngbnuml}, 
		{_NS_x11, _LN_SplitHorizontal}, 
		{_NS_w11, _LN_useWord2002TableStyleRules}, 
		{_NS_s12, _LN_tableStyles}, 
		{_NS_text, _LN_combine_entries}, 
		{_NS_w11, _LN_dontGrowAutofit}, 
		{_NS_v11, _LN_ShapePermeablePlace}, 
		{_NS_prj12, _LN_DefaultOvertimeRate}, 
		{_NS_rtf, _LN_ftnnrlc}, 
		{_NS_number, _LN_scientific_number}, 
		{_NS_meta, _LN_document_statistic}, 
		{_NS_w11, _LN_useFELayout}, 
		{_NS_x11, _LN_SplitVertical}, 
		{_NS_w11, _LN_docVar}, 
		{_NS_s12, _LN_colorpalette}, 
		{_NS_pp12, _LN_embed}, 
		{_NS_rtf, _LN_ftnnruc}, 
		{_NS_v11, _LN_ShapeFixedCode}, 
		{_NS_w11, _LN_view}, 
		{_NS_s12, _LN_fsb}, 
		{_NS_pp12, _LN_link}, 
		{_NS_meta, _LN_page_count}, 
		{_NS_prj12, _LN_DurationFormat}, 
		{_NS_w11, _LN_zoom}, 
		{_NS_text, _LN_combine_entries_with_dash}, 
		{_NS_s12, _LN_color}, 
		{_NS_x11, _LN_FreezePanes}, 
		{_NS_number, _LN_min_exponent_digits}, 
		{_NS_w11, _LN_removePersonalInformation}, 
		{_NS_rtf, _LN_ftnnzodiac}, 
		{_NS_s12, _LN_left}, 
		{_NS_v11, _LN_ShapePlowCode}, 
		{_NS_meta, _LN_table_count}, 
		{_NS_prj12, _LN_WorkFormat}, 
		{_NS_w11, _LN_dontDisplayPageBoundaries}, 
		{_NS_s12, _LN_right}, 
		{_NS_number, _LN_fraction}, 
		{_NS_x11, _LN_FrozenNoSplit}, 
		{_NS_w11, _LN_displayBackgroundShape}, 
		{_NS_s12, _LN_top}, 
		{_NS_rtf, _LN_ftnnzodiacd}, 
		{_NS_w11, _LN_printPostScriptOverText}, 
		{_NS_text, _LN_combine_entries_with_pp}, 
		{_NS_meta, _LN_draw_count}, 
		{_NS_s12, _LN_bottom}, 
		{_NS_v11, _LN_ShapeRouteStyle}, 
		{_NS_w11, _LN_printFractionalCharacterWidth}, 
		{_NS_prj12, _LN_EditableActualCosts}, 
		{_NS_x11, _LN_TabColorIndex}, 
		{_NS_s12, _LN_diag}, 
		{_NS_w11, _LN_printFormsData}, 
		{_NS_number, _LN_min_numerator_digits}, 
		{_NS_rtf, _LN_ftnnzodiacl}, 
		{_NS_meta, _LN_image_count}, 
		{_NS_s12, _LN_vertical}, 
		{_NS_w11, _LN_embedTrueTypeFonts}, 
		{_NS_v11, _LN_ConFixedCode}, 
		{_NS_x11, _LN_Panes}, 
		{_NS_text, _LN_use_keys_as_entries}, 
		{_NS_w11, _LN_doNotEmbedSystemFonts}, 
		{_NS_pp12, _LN_sldId}, 
		{_NS_rtf, _LN_ftnrestart}, 
		{_NS_prj12, _LN_HonorConstraints}, 
		{_NS_w11, _LN_saveSubsetFonts}, 
		{_NS_s12, _LN_horizontal}, 
		{_NS_meta, _LN_ole_object_count}, 
		{_NS_number, _LN_min_denominator_digits}, 
		{_NS_w11, _LN_saveFormsData}, 
		{_NS_v11, _LN_ConLineJumpCode}, 
		{_NS_x11, _LN_ProtectContents}, 
		{_NS_pp12, _LN_sldMasterId}, 
		{_NS_rtf, _LN_ftnrstcont}, 
		{_NS_text, _LN_capitalize_entries}, 
		{_NS_w11, _LN_mirrorMargins}, 
		{_NS_s12, _LN_fontPr}, 
		{_NS_prj12, _LN_EarnedValueMethod}, 
		{_NS_w11, _LN_alignBordersAndEdges}, 
		{_NS_s12, _LN_fill}, 
		{_NS_meta, _LN_paragraph_count}, 
		{_NS_rtf, _LN_ftnrstpg}, 
		{_NS_w11, _LN_bordersDontSurroundHeader}, 
		{_NS_s12, _LN_pattern}, 
		{_NS_number, _LN_denominator_value}, 
		{_NS_x11, _LN_ProtectObjects}, 
		{_NS_v11, _LN_ConLineJumpStyle}, 
		{_NS_pp12, _LN_notesMasterId}, 
		{_NS_text, _LN_comma_separated}, 
		{_NS_w11, _LN_bordersDontSurroundFooter}, 
		{_NS_s12, _LN_gradient}, 
		{_NS_rtf, _LN_ftnsep}, 
		{_NS_fo, _LN_language}, 
		{_NS_w11, _LN_gutterAtTop}, 
		{_NS_s12, _LN_image}, 
		{_NS_meta, _LN_word_count}, 
		{_NS_w11, _LN_hideSpellingErrors}, 
		{_NS_number, _LN_currency_style}, 
		{_NS_prj12, _LN_InsertedProjectsLikeSummary}, 
		{_NS_v11, _LN_ConLineJumpDirX}, 
		{_NS_pp12, _LN_handoutMasterId}, 
		{_NS_x11, _LN_ProtectScenarios}, 
		{_NS_rtf, _LN_ftnsepc}, 
		{_NS_fo, _LN_country}, 
		{_NS_w11, _LN_hideGrammaticalErrors}, 
		{_NS_s12, _LN_fgColor}, 
		{_NS_w11, _LN_activeWritingStyle}, 
		{_NS_meta, _LN_character_count}, 
		{_NS_pp12, _LN_sldLst}, 
		{_NS_rtf, _LN_ftnstart}, 
		{_NS_w11, _LN_proofState}, 
		{_NS_number, _LN_currency_symbol}, 
		{_NS_s12, _LN_bgColor}, 
		{_NS_v11, _LN_ConLineJumpDirY}, 
		{_NS_x11, _LN_EnableSelection}, 
		{_NS_text, _LN_sort_algorithm}, 
		{_NS_w11, _LN_formsDesign}, 
		{_NS_number, _LN_language}, 
		{_NS_s12, _LN_stop}, 
		{_NS_prj12, _LN_MultipleCriticalPaths}, 
		{_NS_meta, _LN_frame_count}, 
		{_NS_rtf, _LN_ftntj}, 
		{_NS_w11, _LN_attachedTemplate}, 
		{_NS_number, _LN_country}, 
		{_NS_s12, _LN_fmt}, 
		{_NS_pp12, _LN_sldMasterIdLst}, 
		{_NS_w11, _LN_linkStyles}, 
		{_NS_v11, _LN_ShapePlaceFlip}, 
		{_NS_s12, _LN_xf}, 
		{_NS_x11, _LN_AllowFormatCells}, 
		{_NS_w11, _LN_stylePaneFormatFilter}, 
		{_NS_rtf, _LN_fttruetype}, 
		{_NS_pp12, _LN_sldIdLst}, 
		{_NS_meta, _LN_sentence_count}, 
		{_NS_prj12, _LN_NewTasksEffortDriven}, 
		{_NS_w11, _LN_documentType}, 
		{_NS_number, _LN_percentage_style}, 
		{_NS_s12, _LN_alignment}, 
		{_NS_w11, _LN_mailMerge}, 
		{_NS_text, _LN_alphabetical_index_auto_mark_file}, 
		{_NS_v11, _LN_ConLineRouteExt}, 
		{_NS_x11, _LN_AllowSizeCols}, 
		{_NS_rtf, _LN_fvaliddos}, 
		{_NS_w11, _LN_revisionView}, 
		{_NS_meta, _LN_syllable_count}, 
		{_NS_pp12, _LN_notesMasterIdLst}, 
		{_NS_number, _LN_date_style}, 
		{_NS_prj12, _LN_NewTasksEstimated}, 
		{_NS_w11, _LN_trackRevisions}, 
		{_NS_s12, _LN_protection}, 
		{_NS_v11, _LN_ShapeSplit}, 
		{_NS_rtf, _LN_fvalidhpfs}, 
		{_NS_w11, _LN_documentProtection}, 
		{_NS_number, _LN_day}, 
		{_NS_x11, _LN_AllowSizeRows}, 
		{_NS_w11, _LN_autoFormatOverride}, 
		{_NS_number, _LN_style}, 
		{_NS_s12, _LN_styleInfo}, 
		{_NS_pp12, _LN_handoutMasterIdLst}, 
		{_NS_rtf, _LN_fvalidmac}, 
		{_NS_w11, _LN_defaultTabStop}, 
		{_NS_text, _LN_alphabetical_index_entry_template}, 
		{_NS_s12, _LN_style}, 
		{_NS_v11, _LN_ShapeSplittable}, 
		{_NS_prj12, _LN_SplitsInProgressTasks}, 
		{_NS_number, _LN_month}, 
		{_NS_w11, _LN_autoHyphenation}, 
		{_NS_x11, _LN_AllowInsertCols}, 
		{_NS_meta, _LN_non_whitespace_character_count}, 
		{_NS_s12, _LN_dxf}, 
		{_NS_w11, _LN_consecutiveHyphenLimit}, 
		{_NS_pp12, _LN_slideSize}, 
		{_NS_rtf, _LN_fvalidntfs}, 
		{_NS_number, _LN_textual}, 
		{_NS_s12, _LN_font}, 
		{_NS_text, _LN_bibliography}, 
		{_NS_w11, _LN_hyphenationZone}, 
		{_NS_v11, _LN_ResizePage}, 
		{_NS_rtf, _LN_g}, 
		{_NS_meta, _LN_row_count}, 
		{_NS_prj12, _LN_SpreadActualCost}, 
		{_NS_w11, _LN_doNotHyphenateCaps}, 
		{_NS_x11, _LN_AllowInsertRows}, 
		{_NS_pp12, _LN_notesSize}, 
		{_NS_rtf, _LN_gcw}, 
		{_NS_w11, _LN_showEnvelope}, 
		{_NS_v11, _LN_EnableGrid}, 
		{_NS_s12, _LN_indexedColors}, 
		{_NS_number, _LN_possessive_form}, 
		{_NS_rtf, _LN_generator}, 
		{_NS_meta, _LN_cell_count}, 
		{_NS_w11, _LN_summaryLength}, 
		{_NS_text, _LN_bibliography_source}, 
		{_NS_pp12, _LN_custShowLst}, 
		{_NS_rtf, _LN_green}, 
		{_NS_w11, _LN_clickAndTypeStyle}, 
		{_NS_number, _LN_year}, 
		{_NS_prj12, _LN_SpreadPercentComplete}, 
		{_NS_x11, _LN_AllowInsertHyperlinks}, 
		{_NS_v11, _LN_DynamicsOff}, 
		{_NS_w11, _LN_defaultTableStyle}, 
		{_NS_s12, _LN_themeColors}, 
		{_NS_meta, _LN_object_count}, 
		{_NS_rtf, _LN_gridtbl}, 
		{_NS_number, _LN_era}, 
		{_NS_w11, _LN_evenAndOddHeaders}, 
		{_NS_pp12, _LN_photoAlbum}, 
		{_NS_w11, _LN_bookFoldRevPrinting}, 
		{_NS_s12, _LN_mruColors}, 
		{_NS_v11, _LN_CtrlAsInput}, 
		{_NS_rtf, _LN_gutter}, 
		{_NS_x11, _LN_AllowDeleteCols}, 
		{_NS_text, _LN_bibliography_entry_template}, 
		{_NS_w11, _LN_bookFoldPrinting}, 
		{_NS_prj12, _LN_TaskUpdatesResource}, 
		{_NS_meta, _LN_user_defined}, 
		{_NS_number, _LN_day_of_week}, 
		{_NS_w11, _LN_bookFoldPrintingSheets}, 
		{_NS_meta, _LN_name}, 
		{_NS_pp12, _LN_commentAuthorLst}, 
		{_NS_s12, _LN_rgbcolor}, 
		{_NS_v11, _LN_PlaceStyle}, 
		{_NS_rtf, _LN_gutterprl}, 
		{_NS_w11, _LN_drawingGridHorizontalSpacing}, 
		{_NS_s12, _LN_tableStyle}, 
		{_NS_x11, _LN_AllowDeleteRows}, 
		{_NS_w11, _LN_drawingGridVerticalSpacing}, 
		{_NS_number, _LN_week_of_year}, 
		{_NS_prj12, _LN_FiscalYearStart}, 
		{_NS_text, _LN_index_source_styles}, 
		{_NS_meta, _LN_value_type}, 
		{_NS_pp12, _LN_custDataLst}, 
		{_NS_w11, _LN_displayHorizontalDrawingGridEvery}, 
		{_NS_v11, _LN_RouteStyle}, 
		{_NS_rtf, _LN_guttersxn}, 
		{_NS_text, _LN_h}, 
		{_NS_w11, _LN_displayVerticalDrawingGridEvery}, 
		{_NS_number, _LN_quarter}, 
		{_NS_x11, _LN_AllowSort}, 
		{_NS_rtf, _LN_header}, 
		{_NS_s12, _LN_tableStyleElement}, 
		{_NS_prj12, _LN_WeekStartDay}, 
		{_NS_w11, _LN_useMarginsForDrawingGridOrigin}, 
		{_NS_v11, _LN_PlaceDepth}, 
		{_NS_s12, _LN_charset}, 
		{_NS_text, _LN_index_source_style}, 
		{_NS_w11, _LN_drawingGridHorizontalOrigin}, 
		{_NS_pp12, _LN_creatingAppVersion}, 
		{_NS_rtf, _LN_headerf}, 
		{_NS_number, _LN_time_style}, 
		{_NS_text, _LN_outline_level}, 
		{_NS_x11, _LN_AllowFilter}, 
		{_NS_s12, _LN_family}, 
		{_NS_w11, _LN_drawingGridVerticalOrigin}, 
		{_NS_v11, _LN_PlowCode}, 
		{_NS_s12, _LN_b}, 
		{_NS_w11, _LN_doNotShadeFormData}, 
		{_NS_rtf, _LN_headerl}, 
		{_NS_prj12, _LN_MoveCompletedEndsBack}, 
		{_NS_s12, _LN_i}, 
		{_NS_w11, _LN_punctuationKerning}, 
		{_NS_text, _LN_index_title_template}, 
		{_NS_pp12, _LN_editingAppVersion}, 
		{_NS_s12, _LN_strike}, 
		{_NS_text, _LN_restart_numbering}, 
		{_NS_w11, _LN_characterSpacingControl}, 
		{_NS_number, _LN_truncate_on_overflow}, 
		{_NS_x11, _LN_AllowUsePivotTables}, 
		{_NS_rtf, _LN_headery}, 
		{_NS_v11, _LN_LineJumpCode}, 
		{_NS_pp12, _LN_presentation}, 
		{_NS_s12, _LN_outline}, 
		{_NS_w11, _LN_printTwoOnOne}, 
		{_NS_s12, _LN_shadow}, 
		{_NS_number, _LN_hours}, 
		{_NS_rtf, _LN_hich}, 
		{_NS_w11, _LN_strictFirstAndLastChars}, 
		{_NS_text, _LN_start_value}, 
		{_NS_prj12, _LN_MoveRemainingStartsBack}, 
		{_NS_x11, _LN_SpaceAbove}, 
		{_NS_s12, _LN_condense}, 
		{_NS_rtf, _LN_highlight}, 
		{_NS_text, _LN_index_entry_chapter}, 
		{_NS_w11, _LN_noLineBreaksAfter}, 
		{_NS_v11, _LN_LineJumpStyle}, 
		{_NS_s12, _LN_extend}, 
		{_NS_number, _LN_minutes}, 
		{_NS_w11, _LN_noLineBreaksBefore}, 
		{_NS_rtf, _LN_hlfr}, 
		{_NS_s12, _LN_sz}, 
		{_NS_x11, _LN_SpaceBelow}, 
		{_NS_w11, _LN_webPageEncoding}, 
		{_NS_text, _LN_is_list_header}, 
		{_NS_s12, _LN_u}, 
		{_NS_number, _LN_seconds}, 
		{_NS_w11, _LN_optimizeForBrowser}, 
		{_NS_text, _LN_index_entry_text}, 
		{_NS_v11, _LN_PageLineJumpDirX}, 
		{_NS_s12, _LN_vertAlign}, 
		{_NS_rtf, _LN_hlinkbase}, 
		{_NS_prj12, _LN_MoveRemainingStartsForward}, 
		{_NS_w11, _LN_relyOnVML}, 
		{_NS_text, _LN_number}, 
		{_NS_x11, _LN_ZeroHeight}, 
		{_NS_s12, _LN_fontScheme}, 
		{_NS_w11, _LN_allowPNG}, 
		{_NS_x11, _LN_Layout}, 
		{_NS_text, _LN_p}, 
		{_NS_rtf, _LN_hlloc}, 
		{_NS_number, _LN_decimal_places}, 
		{_NS_w11, _LN_doNotRelyOnCSS}, 
		{_NS_v11, _LN_PageLineJumpDirY}, 
		{_NS_pp12, _LN_pubWhat}, 
		{_NS_w11, _LN_doNotSaveWebPagesAsSingleFile}, 
		{_NS_text, _LN_index_entry_page_number}, 
		{_NS_x11, _LN_Header}, 
		{_NS_rtf, _LN_hlsrc}, 
		{_NS_number, _LN_am_pm}, 
		{_NS_text, _LN_style_name}, 
		{_NS_prj12, _LN_MoveCompletedEndsForward}, 
		{_NS_w11, _LN_doNotOrganizeInFolder}, 
		{_NS_pp12, _LN_pubAs}, 
		{_NS_v11, _LN_LineToNodeX}, 
		{_NS_x11, _LN_Footer}, 
		{_NS_w11, _LN_doNotUseLongFileNames}, 
		{_NS_rtf, _LN_horzdoc}, 
		{_NS_w11, _LN_pixelsPerInch}, 
		{_NS_text, _LN_class_names}, 
		{_NS_text, _LN_index_entry_span}, 
		{_NS_number, _LN_boolean_style}, 
		{_NS_pp12, _LN_present}, 
		{_NS_w11, _LN_targetScreenSz}, 
		{_NS_v11, _LN_LineToNodeY}, 
		{_NS_rtf, _LN_horzsect}, 
		{_NS_x11, _LN_PageMargins}, 
		{_NS_prj12, _LN_BaselineForEarnedValue}, 
		{_NS_w11, _LN_savePreviewPicture}, 
		{_NS_number, _LN_boolean}, 
		{_NS_pp12, _LN_browse}, 
		{_NS_rtf, _LN_hr}, 
		{_NS_w11, _LN_validateAgainstSchema}, 
		{_NS_text, _LN_cond_style_name}, 
		{_NS_x11, _LN_Count}, 
		{_NS_v11, _LN_BlockSizeX}, 
		{_NS_sb12, _LN_supBook}, 
		{_NS_w11, _LN_saveInvalidXML}, 
		{_NS_pp12, _LN_kiosk}, 
		{_NS_text, _LN_index_entry_bibliography}, 
		{_NS_number, _LN_text_style}, 
		{_NS_rtf, _LN_htmautsp}, 
		{_NS_w11, _LN_ignoreMixedContent}, 
		{_NS_number, _LN_text}, 
		{_NS_x11, _LN_SheetIndex}, 
		{_NS_text, _LN_page_sequence}, 
		{_NS_w11, _LN_alwaysShowPlaceholderText}, 
		{_NS_v11, _LN_BlockSizeY}, 
		{_NS_sb12, _LN_sheetNames}, 
		{_NS_pp12, _LN_showType}, 
		{_NS_prj12, _LN_AutoAddNewResourcesAndTasks}, 
		{_NS_rtf, _LN_htmlbase}, 
		{_NS_w11, _LN_doNotUnderlineInvalidXML}, 
		{_NS_x11, _LN_Crn}, 
		{_NS_text, _LN_page}, 
		{_NS_w11, _LN_removeWordSchemaOnSave}, 
		{_NS_number, _LN_text_content}, 
		{_NS_text, _LN_bibliography_data_field}, 
		{_NS_pp12, _LN_showWhat}, 
		{_NS_v11, _LN_AvenueSizeX}, 
		{_NS_sb12, _LN_namedLinks}, 
		{_NS_prj12, _LN_StatusDate}, 
		{_NS_x11, _LN_Moper}, 
		{_NS_rtf, _LN_htmlrtf}, 
		{_NS_w11, _LN_useXSLTWhenSaving}, 
		{_NS_number, _LN_title}, 
		{_NS_w11, _LN_saveThroughXSLT}, 
		{_NS_pp12, _LN_penClr}, 
		{_NS_text, _LN_master_page_name}, 
		{_NS_sb12, _LN_supSheet}, 
		{_NS_rtf, _LN_htmltag}, 
		{_NS_w11, _LN_showXMLTags}, 
		{_NS_v11, _LN_AvenueSizeY}, 
		{_NS_x11, _LN_ExcelName}, 
		{_NS_prj12, _LN_CurrentDate}, 
		{_NS_style, _LN_volatile}, 
		{_NS_sb12, _LN_sheetName}, 
		{_NS_text, _LN_index_entry_tab_stop}, 
		{_NS_w11, _LN_alwaysMergeEmptyNamespace}, 
		{_NS_text, _LN_list}, 
		{_NS_pp12, _LN_htmlPubPr}, 
		{_NS_w11, _LN_hdrShapeDefaults}, 
		{_NS_rtf, _LN_hyphauto}, 
		{_NS_sb12, _LN_xrow}, 
		{_NS_v11, _LN_LineToLineX}, 
		{_NS_x11, _LN_FunctionGroup}, 
		{_NS_w11, _LN_compat}, 
		{_NS_pp12, _LN_webPr}, 
		{_NS_number, _LN_automatic_order}, 
		{_NS_style, _LN_leader_char}, 
		{_NS_sb12, _LN_xc}, 
		{_NS_w11, _LN_docVars}, 
		{_NS_rtf, _LN_hyphcaps}, 
		{_NS_style, _LN_type}, 
		{_NS_prj12, _LN_MicrosoftProjectServerURL}, 
		{_NS_text, _LN_continue_numbering}, 
		{_NS_pp12, _LN_prnPr}, 
		{_NS_w11, _LN_sourceFileName}, 
		{_NS_v11, _LN_LineToLineY}, 
		{_NS_sb12, _LN_xv}, 
		{_NS_w11, _LN_marW}, 
		{_NS_number, _LN_format_source}, 
		{_NS_x11, _LN_FunctionGroupIndex}, 
		{_NS_style, _LN_position}, 
		{_NS_prj12, _LN_Autolink}, 
		{_NS_rtf, _LN_hyphconsec}, 
		{_NS_pp12, _LN_showPr}, 
		{_NS_text, _LN_list_item}, 
		{_NS_w11, _LN_marH}, 
		{_NS_sb12, _LN_namedRange}, 
		{_NS_w11, _LN_scrollbar}, 
		{_NS_v11, _LN_LineJumpFactorX}, 
		{_NS_rtf, _LN_hyphhotz}, 
		{_NS_w11, _LN_noResizeAllowed}, 
		{_NS_x11, _LN_KeyboardShortcut}, 
		{_NS_text, _LN_list_header}, 
		{_NS_sb12, _LN_DDELink}, 
		{_NS_prj12, _LN_NewTaskStartDate}, 
		{_NS_pp12, _LN_presentationPr}, 
		{_NS_number, _LN_transliteration_format}, 
		{_NS_w11, _LN_linkedToFile}, 
		{_NS_text, _LN_index_entry_link_start}, 
		{_NS_rtf, _LN_hyphpar}, 
		{_NS_w11, _LN_noBorder}, 
		{_NS_v11, _LN_LineJumpFactorY}, 
		{_NS_x11, _LN_Macro}, 
		{_NS_sb12, _LN_OLELink}, 
		{_NS_rtf, _LN_i}, 
		{_NS_w11, _LN_flatBorders}, 
		{_NS_sb12, _LN_values}, 
		{_NS_rtf, _LN_id}, 
		{_NS_text, _LN_numbered_paragraph}, 
		{_NS_w11, _LN_framesetSplitbar}, 
		{_NS_prj12, _LN_DefaultTaskEVMethod}, 
		{_NS_sb12, _LN_value}, 
		{_NS_x11, _LN_phonetictext}, 
		{_NS_rtf, _LN_ilvl}, 
		{_NS_v11, _LN_LineAdjustFrom}, 
		{_NS_w11, _LN_frameLayout}, 
		{_NS_text, _LN_index_entry_link_end}, 
		{_NS_number, _LN_transliteration_language}, 
		{_NS_sb12, _LN_val}, 
		{_NS_text, _LN_level}, 
		{_NS_w11, _LN_frameset}, 
		{_NS_rtf, _LN_impr}, 
		{_NS_table, _LN_table}, 
		{_NS_w11, _LN_frame}, 
		{_NS_rtf, _LN_info}, 
		{_NS_text, _LN_section}, 
		{_NS_v11, _LN_LineAdjustTo}, 
		{_NS_table, _LN_name}, 
		{_NS_x11, _LN_AutoFilterColumn}, 
		{_NS_w11, _LN_start}, 
		{_NS_prj12, _LN_ProjectExternallyEdited}, 
		{_NS_text, _LN_name}, 
		{_NS_table, _LN_style_name}, 
		{_NS_w11, _LN_nfc}, 
		{_NS_number, _LN_transliteration_country}, 
		{_NS_rtf, _LN_insrsid}, 
		{_NS_pp12, _LN_str}, 
		{_NS_table, _LN_protected}, 
		{_NS_v11, _LN_PlaceFlip}, 
		{_NS_w11, _LN_lvlRestart}, 
		{_NS_text, _LN_protected}, 
		{_NS_table, _LN_protection_key}, 
		{_NS_x11, _LN_WorkbookOptions}, 
		{_NS_pp12, _LN_bin}, 
		{_NS_w11, _LN_isLgl}, 
		{_NS_rtf, _LN_intbl}, 
		{_NS_prj12, _LN_ExtendedCreationDate}, 
		{_NS_w11, _LN_suff}, 
		{_NS_v11, _LN_LineRouteExt}, 
		{_NS_pp12, _LN_tag}, 
		{_NS_table, _LN_print}, 
		{_NS_rtf, _LN_ipgp}, 
		{_NS_number, _LN_transliteration_style}, 
		{_NS_w11, _LN_lvlText}, 
		{_NS_text, _LN_protection_key}, 
		{_NS_x11, _LN_MoveAfterReturn}, 
		{_NS_w11, _LN_lvlPicBulletId}, 
		{_NS_rtf, _LN_irow}, 
		{_NS_sxr12, _LN_pivotRefs}, 
		{_NS_prj12, _LN_ActualsInSync}, 
		{_NS_w11, _LN_legacy}, 
		{_NS_table, _LN_print_ranges}, 
		{_NS_v11, _LN_PageShapeSplit}, 
		{_NS_text, _LN_display}, 
		{_NS_w11, _LN_lvlJc}, 
		{_NS_number, _LN_min_integer_digits}, 
		{_NS_rtf, _LN_irowband}, 
		{_NS_sxr12, _LN_pivotRef}, 
		{_NS_w11, _LN_lsid}, 
		{_NS_table, _LN_table_row}, 
		{_NS_sxr12, _LN_rule}, 
		{_NS_text, _LN_condition}, 
		{_NS_w11, _LN_plt}, 
		{_NS_rtf, _LN_itap}, 
		{_NS_v11, _LN_PageLeftMargin}, 
		{_NS_prj12, _LN_RemoveFileProperties}, 
		{_NS_number, _LN_grouping}, 
		{_NS_w11, _LN_tmpl}, 
		{_NS_rtf, _LN_ixe}, 
		{_NS_w11, _LN_styleLink}, 
		{_NS_number, _LN_calendar}, 
		{_NS_text, _LN_section_source}, 
		{_NS_sxr12, _LN_pivotRefInternal}, 
		{_NS_prj12, _LN_AdminProject}, 
		{_NS_w11, _LN_listStyleLink}, 
		{_NS_table, _LN_number_rows_repeated}, 
		{_NS_pp12, _LN_ph}, 
		{_NS_v11, _LN_PageRightMargin}, 
		{_NS_rtf, _LN_jcompress}, 
		{_NS_w11, _LN_lvl}, 
		{_NS_style, _LN_family}, 
		{_NS_x2, _LN_MapInfo}, 
		{_NS_sxr12, _LN_flags}, 
		{_NS_pp12, _LN_media}, 
		{_NS_w11, _LN_startOverride}, 
		{_NS_text, _LN_section_name}, 
		{_NS_prj12, _LN_OutlineCodes}, 
		{_NS_rtf, _LN_jexpand}, 
		{_NS_v11, _LN_PageTopMargin}, 
		{_NS_w11, _LN_ilst}, 
		{_NS_x2, _LN_Binding}, 
		{_NS_sxr12, _LN_slicers}, 
		{_NS_pp12, _LN_actions}, 
		{_NS_w11, _LN_lvlOverride}, 
		{_NS_table, _LN_default_cell_style_name}, 
		{_NS_pp12, _LN_anvPr}, 
		{_NS_text, _LN_filter_name}, 
		{_NS_prj12, _LN_OutlineCode}, 
		{_NS_rtf, _LN_jpegblip}, 
		{_NS_w11, _LN_listPicBullet}, 
		{_NS_x2, _LN_MapID}, 
		{_NS_sxr12, _LN_slicer}, 
		{_NS_text, _LN_linenumbering_configuration}, 
		{_NS_pp12, _LN_anvSpPr}, 
		{_NS_w11, _LN_listDef}, 
		{_NS_v11, _LN_PageBottomMargin}, 
		{_NS_table, _LN_visibility}, 
		{_NS_rtf, _LN_jsksu}, 
		{_NS_prj12, _LN_FieldID}, 
		{_NS_w11, _LN_list}, 
		{_NS_x2, _LN_Schema}, 
		{_NS_sxr12, _LN_elements}, 
		{_NS_pp12, _LN_pnvPr}, 
		{_NS_text, _LN_tracked_changes}, 
		{_NS_v11, _LN_ScaleX}, 
		{_NS_w11, _LN_ilfoMacAtCleanup}, 
		{_NS_rtf, _LN_keep}, 
		{_NS_text, _LN_number_lines}, 
		{_NS_sxr12, _LN_element}, 
		{_NS_pp12, _LN_nvSpPr}, 
		{_NS_x2, _LN_Map}, 
		{_NS_table, _LN_table_cell}, 
		{_NS_w11, _LN_aliases}, 
		{_NS_prj12, _LN_FieldName}, 
		{_NS_sxr12, _LN_n}, 
		{_NS_pp12, _LN_spPr}, 
		{_NS_v11, _LN_ScaleY}, 
		{_NS_w11, _LN_sti}, 
		{_NS_rtf, _LN_keepn}, 
		{_NS_sxr12, _LN_d}, 
		{_NS_pp12, _LN_style}, 
		{_NS_text, _LN_track_changes}, 
		{_NS_text, _LN_increment}, 
		{_NS_prj12, _LN_Alias}, 
		{_NS_w11, _LN_basedOn}, 
		{_NS_sxr12, _LN_x}, 
		{_NS_pp12, _LN_txBody}, 
		{_NS_v11, _LN_PagesX}, 
		{_NS_w11, _LN_next}, 
		{_NS_rtf, _LN_kerning}, 
		{_NS_table, _LN_covered_table_cell}, 
		{_NS_sxr12, _LN_s}, 
		{_NS_pp12, _LN_anvCxnSpPr}, 
		{_NS_w11, _LN_link}, 
		{_NS_text, _LN_changed_region}, 
		{_NS_v11, _LN_PagesY}, 
		{_NS_sxr12, _LN_b}, 
		{_NS_pp12, _LN_nvCxnSpPr}, 
		{_NS_text, _LN_number_position}, 
		{_NS_prj12, _LN_PhoneticAlias}, 
		{_NS_x2, _LN_ShowImportExportValidationErrors}, 
		{_NS_w11, _LN_autoRedefine}, 
		{_NS_rtf, _LN_keycode}, 
		{_NS_sxr12, _LN_e}, 
		{_NS_pp12, _LN_anvGraphicFramePr}, 
		{_NS_text, _LN_id}, 
		{_NS_w11, _LN_hidden}, 
		{_NS_rtf, _LN_keywords}, 
		{_NS_text, _LN_offset}, 
		{_NS_v11, _LN_CenterX}, 
		{_NS_prj12, _LN_Values}, 
		{_NS_pp12, _LN_nvGraphicFramePr}, 
		{_NS_x2, _LN_Append}, 
		{_NS_w11, _LN_semiHidden}, 
		{_NS_table, _LN_number_columns_repeated}, 
		{_NS_pp12, _LN_xfrm}, 
		{_NS_w11, _LN_locked}, 
		{_NS_rtf, _LN_ksulang}, 
		{_NS_text, _LN_insertion}, 
		{_NS_x2, _LN_Entry}, 
		{_NS_v11, _LN_CenterY}, 
		{_NS_prj12, _LN_ValueID}, 
		{_NS_pp12, _LN_graphic}, 
		{_NS_w11, _LN_personal}, 
		{_NS_pp12, _LN_anvGspPr}, 
		{_NS_text, _LN_count_empty_lines}, 
		{_NS_w11, _LN_personalCompose}, 
		{_NS_text, _LN_deletion}, 
		{_NS_x2, _LN_Mapdata}, 
		{_NS_rtf, _LN_landscape}, 
		{_NS_v11, _LN_OnPage}, 
		{_NS_pp12, _LN_nvGspPr}, 
		{_NS_w11, _LN_personalReply}, 
		{_NS_table, _LN_number_columns_spanned}, 
		{_NS_prj12, _LN_ParentValueID}, 
		{_NS_x2, _LN_Range}, 
		{_NS_rtf, _LN_lang}, 
		{_NS_pp12, _LN_gspPr}, 
		{_NS_w11, _LN_rsid}, 
		{_NS_prj12, _LN_Description}, 
		{_NS_pp12, _LN_sp}, 
		{_NS_v11, _LN_PrintGrid}, 
		{_NS_w11, _LN_tblStylePr}, 
		{_NS_text, _LN_format_change}, 
		{_NS_rtf, _LN_langfe}, 
		{_NS_text, _LN_count_in_text_boxes}, 
		{_NS_tbl12, _LN_table}, 
		{_NS_pp12, _LN_gsp}, 
		{_NS_x2, _LN_HeaderRange}, 
		{_NS_w11, _LN_lsdException}, 
		{_NS_tbl12, _LN_autofilter}, 
		{_NS_table, _LN_number_rows_spanned}, 
		{_NS_pp12, _LN_graphicFrame}, 
		{_NS_prj12, _LN_Enterprise}, 
		{_NS_text, _LN_change}, 
		{_NS_w11, _LN_versionOfBuiltInStylenames}, 
		{_NS_rtf, _LN_langfenp}, 
		{_NS_tbl12, _LN_sortState}, 
		{_NS_pp12, _LN_cxnsp}, 
		{_NS_x2, _LN_XPath}, 
		{_NS_w11, _LN_latentStyles}, 
		{_NS_text, _LN_restart_on_page}, 
		{_NS_v11, _LN_PrintPageOrientation}, 
		{_NS_tbl12, _LN_cols}, 
		{_NS_pp12, _LN_clrMap}, 
		{_NS_x2, _LN_Field}, 
		{_NS_w11, _LN_style}, 
		{_NS_rtf, _LN_langnp}, 
		{_NS_tbl12, _LN_tableStyle}, 
		{_NS_text, _LN_change_start}, 
		{_NS_w11, _LN_altName}, 
		{_NS_tbl12, _LN_fsb}, 
		{_NS_table, _LN_content_validation_name}, 
		{_NS_v11, _LN_PaperKind}, 
		{_NS_pp12, _LN_clrMapOvr}, 
		{_NS_x2, _LN_ActiveRows}, 
		{_NS_w11, _LN_panose_1}, 
		{_NS_prj12, _LN_EnterpriseOutlineCodeAlias}, 
		{_NS_rtf, _LN_lastrow}, 
		{_NS_tbl12, _LN_col}, 
		{_NS_table, _LN_formula}, 
		{_NS_pp12, _LN_fill}, 
		{_NS_text, _LN_change_end}, 
		{_NS_w11, _LN_charset}, 
		{_NS_text, _LN_linenumbering_separator}, 
		{_NS_rtf, _LN_lbr}, 
		{_NS_w11, _LN_family}, 
		{_NS_v11, _LN_PaperSource}, 
		{_NS_x2, _LN_NoInserts}, 
		{_NS_tbl12, _LN_ccFmla}, 
		{_NS_pp12, _LN_bgPr}, 
		{_NS_w11, _LN_notTrueType}, 
		{_NS_text, _LN_change_id}, 
		{_NS_x2, _LN_ReadOnly}, 
		{_NS_rtf, _LN_lchars}, 
		{_NS_v11, _LN_Font}, 
		{_NS_w11, _LN_pitch}, 
		{_NS_tbl12, _LN_trFmla}, 
		{_NS_pp12, _LN_bgLink}, 
		{_NS_prj12, _LN_ResourceSubstitutionEnabled}, 
		{_NS_text, _LN_notes_configuration}, 
		{_NS_w11, _LN_sig}, 
		{_NS_table, _LN_number_matrix_columns_spanned}, 
		{_NS_v11, _LN_Color}, 
		{_NS_rtf, _LN_ldblquote}, 
		{_NS_x2, _LN_InstanceShape}, 
		{_NS_w11, _LN_defaultFonts}, 
		{_NS_text, _LN_variable_decls}, 
		{_NS_tbl12, _LN_xmlCPr}, 
		{_NS_pp12, _LN_bgMaster}, 
		{_NS_prj12, _LN_LeafOnly}, 
		{_NS_rtf, _LN_level}, 
		{_NS_x2, _LN_ActiveRow}, 
		{_NS_w11, _LN_font}, 
		{_NS_v11, _LN_Style}, 
		{_NS_w11, _LN_blockQuote}, 
		{_NS_text, _LN_citation_style_name}, 
		{_NS_pp12, _LN_spTree}, 
		{_NS_v11, _LN_Case}, 
		{_NS_text, _LN_sequence_decls}, 
		{_NS_x2, _LN_XSDType}, 
		{_NS_w11, _LN_bodyDiv}, 
		{_NS_rtf, _LN_levelfollow}, 
		{_NS_table, _LN_number_matrix_rows_spanned}, 
		{_NS_prj12, _LN_AllLevelsRequired}, 
		{_NS_w11, _LN_marLeft}, 
		{_NS_pp12, _LN_tagLst}, 
		{_NS_v11, _LN_Pos}, 
		{_NS_w11, _LN_marRight}, 
		{_NS_x2, _LN_Aggregate}, 
		{_NS_table, _LN_protect}, 
		{_NS_rtf, _LN_levelindent}, 
		{_NS_w11, _LN_marTop}, 
		{_NS_text, _LN_user_field_decls}, 
		{_NS_pp12, _LN_sldBase}, 
		{_NS_text, _LN_citation_body_style_name}, 
		{_NS_v11, _LN_FontScale}, 
		{_NS_w11, _LN_marBottom}, 
		{_NS_a12, _LN_spLock}, 
		{_NS_prj12, _LN_OnlyTableValuesAllowed}, 
		{_NS_rtf, _LN_leveljc}, 
		{_NS_pp12, _LN_cmLst}, 
		{_NS_w11, _LN_divBdr}, 
		{_NS_table, _LN_table_column}, 
		{_NS_v11, _LN_Locale}, 
		{_NS_w11, _LN_divsChild}, 
		{_NS_a12, _LN_cxnSpLock}, 
		{_NS_voldeps12, _LN_volTypes}, 
		{_NS_prj12, _LN_Masks}, 
		{_NS_text, _LN_dde_connection_decls}, 
		{_NS_text, _LN_default_style_name}, 
		{_NS_rtf, _LN_leveljcn}, 
		{_NS_w11, _LN_div}, 
		{_NS_v11, _LN_Size}, 
		{_NS_pp12, _LN_transition}, 
		{_NS_text, _LN_s}, 
		{_NS_w11, _LN_txbxContent}, 
		{_NS_a12, _LN_stCxn}, 
		{_NS_prj12, _LN_Mask}, 
		{_NS_voldeps12, _LN_volType}, 
		{_NS_table, _LN_table_header_columns}, 
		{_NS_w11, _LN_p}, 
		{_NS_text, _LN_c}, 
		{_NS_rtf, _LN_levellegal}, 
		{_NS_v11, _LN_DblUnderline}, 
		{_NS_pp12, _LN_timingInfo}, 
		{_NS_a12, _LN_endCxn}, 
		{_NS_voldeps12, _LN_main}, 
		{_NS_prj12, _LN_Level}, 
		{_NS_w11, _LN_tbl}, 
		{_NS_text, _LN_start_numbering_at}, 
		{_NS_text, _LN_tab}, 
		{_NS_w11, _LN_cfChunk}, 
		{_NS_pp12, _LN_hf}, 
		{_NS_rtf, _LN_levelnfc}, 
		{_NS_table, _LN_table_columns}, 
		{_NS_voldeps12, _LN_topic}, 
		{_NS_v11, _LN_Overline}, 
		{_NS_prj12, _LN_Length}, 
		{_NS_a12, _LN_gspLock}, 
		{_NS_w11, _LN_proofErr}, 
		{_NS_text, _LN_tab_ref}, 
		{_NS_voldeps12, _LN_value}, 
		{_NS_w11, _LN_permStart}, 
		{_NS_pp12, _LN_sldLayout}, 
		{_NS_text, _LN_footnotes_position}, 
		{_NS_rtf, _LN_levelnfcn}, 
		{_NS_w11, _LN_permEnd}, 
		{_NS_prj12, _LN_Separator}, 
		{_NS_v11, _LN_Strikethru}, 
		{_NS_voldeps12, _LN_subtopic}, 
		{_NS_ss11, _LN_Workbook}, 
		{_NS_w11, _LN_ignoreSubtree}, 
		{_NS_text, _LN_line_break}, 
		{_NS_table, _LN_table_column_group}, 
		{_NS_a12, _LN_graphicFrameLock}, 
		{_NS_pp12, _LN_defLink}, 
		{_NS_voldeps12, _LN_ref}, 
		{_NS_ss11, _LN_Styles}, 
		{_NS_w11, _LN_ignoreElements}, 
		{_NS_table, _LN_display}, 
		{_NS_prj12, _LN_WBSMasks}, 
		{_NS_v11, _LN_Highlight}, 
		{_NS_rtf, _LN_levelnorestart}, 
		{_NS_text, _LN_span}, 
		{_NS_ss11, _LN_Style}, 
		{_NS_w11, _LN_divs}, 
		{_NS_text, _LN_a}, 
		{_NS_pp12, _LN_outline1Link}, 
		{_NS_ss11, _LN_Names}, 
		{_NS_text, _LN_note_continuation_notice_forward}, 
		{_NS_office, _LN_name}, 
		{_NS_v11, _LN_Perpendicular}, 
		{_NS_rtf, _LN_levelnumbers}, 
		{_NS_table, _LN_table_header_rows}, 
		{_NS_prj12, _LN_VerifyUniqueCodes}, 
		{_NS_w11, _LN_docOleData}, 
		{_NS_pp12, _LN_outline2Link}, 
		{_NS_ss11, _LN_Worksheet}, 
		{_NS_rtf, _LN_levelold}, 
		{_NS_table, _LN_table_rows}, 
		{_NS_prj12, _LN_GenerateCodes}, 
		{_NS_text, _LN_visited_style_name}, 
		{_NS_w11, _LN_docSuppData}, 
		{_NS_v11, _LN_DoubleStrikethrough}, 
		{_NS_pp12, _LN_lvl1Link}, 
		{_NS_ss11, _LN_NamedRange}, 
		{_NS_text, _LN_note_continuation_notice_backward}, 
		{_NS_rtf, _LN_levelpicture}, 
		{_NS_prj12, _LN_Prefix}, 
		{_NS_ss11, _LN_Alignment}, 
		{_NS_text, _LN_bookmark}, 
		{_NS_a12, _LN_bounds}, 
		{_NS_table, _LN_table_row_group}, 
		{_NS_v11, _LN_RTLText}, 
		{_NS_wb12, _LN_book}, 
		{_NS_pp12, _LN_lvl2Link}, 
		{_NS_w11, _LN_shapeDefaults}, 
		{_NS_prj12, _LN_WBSMask}, 
		{_NS_ss11, _LN_Borders}, 
		{_NS_rtf, _LN_levelprev}, 
		{_NS_pp12, _LN_lvl3Link}, 
		{_NS_w11, _LN_bgPict}, 
		{_NS_v11, _LN_UseVertical}, 
		{_NS_wb12, _LN_fileVersion}, 
		{_NS_ss11, _LN_Font}, 
		{_NS_text, _LN_bookmark_start}, 
		{_NS_table, _LN_is_sub_table}, 
		{_NS_w11, _LN_docPr}, 
		{_NS_text, _LN_bibliography_configuration}, 
		{_NS_rng, _LN_data}, 
		{_NS_pp12, _LN_lvl4Link}, 
		{_NS_rtf, _LN_levelprevspace}, 
		{_NS_w11, _LN_body}, 
		{_NS_ss11, _LN_Interior}, 
		{_NS_rtf, _LN_psover}, 
		{_NS_prj12, _LN_ExtendedAttributes}, 
		{_NS_v11, _LN_Letterspace}, 
		{_NS_wb12, _LN_fileSharing}, 
		{_NS_text, _LN_bookmark_end}, 
		{QName::EMPTY_STRING, _LN_type}, 
		{_NS_w11, _LN_wordDocument}, 
		{_NS_text, _LN_prefix}, 
		{_NS_ss11, _LN_NumberFormat}, 
		{_NS_rtf, _LN_psz}, 
		{_NS_pp12, _LN_lvl5Link}, 
		{_NS_ss11, _LN_Protection}, 
		{_NS_rtf, _LN_levelspace}, 
		{_NS_rng, _LN_param}, 
		{_NS_wb12, _LN_bookViews}, 
		{_NS_text, _LN_suffix}, 
		{_NS_v11, _LN_ColorTrans}, 
		{_NS_prj12, _LN_ExtendedAttribute}, 
		{_NS_text, _LN_reference_mark}, 
		{_NS_rtf, _LN_pubauto}, 
		{_NS_ss11, _LN_Border}, 
		{_NS_pp12, _LN_lvl6Link}, 
		{_NS_rtf, _LN_levelstartat}, 
		{_NS_ss11, _LN_Column}, 
		{_NS_table, _LN_table_source}, 
		{_NS_v11, _LN_AsianFont}, 
		{_NS_a12, _LN_graphicData}, 
		{_NS_rtf, _LN_pvmrg}, 
		{_NS_prj12, _LN_RollupType}, 
		{_NS_text, _LN_numbered_entries}, 
		{_NS_ss11, _LN_Row}, 
		{_NS_pp12, _LN_lvl7Link}, 
		{_NS_table, _LN_mode}, 
		{_NS_ss11, _LN_Cell}, 
		{_NS_text, _LN_reference_mark_start}, 
		{_NS_a12, _LN_graphic}, 
		{_NS_rtf, _LN_pvpara}, 
		{_NS_table, _LN_table_name}, 
		{_NS_rtf, _LN_leveltemplateid}, 
		{_NS_ss11, _LN_Data}, 
		{_NS_v11, _LN_ComplexScriptFont}, 
		{_NS_pp12, _LN_lvl8Link}, 
		{_NS_table, _LN_filter_name}, 
		{_NS_prj12, _LN_CalculationType}, 
		{_NS_rtf, _LN_pvpg}, 
		{_NS_ss11, _LN_Comment}, 
		{_NS_text, _LN_sort_by_position}, 
		{_NS_rtf, _LN_leveltext}, 
		{_NS_rtf, _LN_pwd}, 
		{_NS_pp12, _LN_lvl9Link}, 
		{_NS_text, _LN_reference_mark_end}, 
		{_NS_prj12, _LN_Formula}, 
		{_NS_v11, _LN_LocalizeFont}, 
		{_NS_text, _LN_sort_key}, 
		{_NS_ss11, _LN_NamedCell}, 
		{_NS_pp12, _LN_txLstStyle}, 
		{_NS_rtf, _LN_li}, 
		{_NS_table, _LN_filter_options}, 
		{_NS_rtf, _LN_pxe}, 
		{_NS_text, _LN_note}, 
		{_NS_rtf, _LN_line}, 
		{_NS_text, _LN_key}, 
		{_NS_ss11, _LN_Table}, 
		{_NS_rtf, _LN_qc}, 
		{_NS_prj12, _LN_RestrictValues}, 
		{_NS_pp12, _LN_themeLinkLst}, 
		{_NS_v11, _LN_ComplexScriptSize}, 
		{_NS_table, _LN_refresh_delay}, 
		{_NS_rtf, _LN_qd}, 
		{_NS_rtf, _LN_linebetcol}, 
		{_NS_text, _LN_note_citation}, 
		{_NS_text, _LN_sort_ascending}, 
		{QName::EMPTY_STRING, _LN_ns}, 
		{_NS_rtf, _LN_qj}, 
		{_NS_a12, _LN_igx}, 
		{_NS_v11, _LN_IndFirst}, 
		{_NS_table, _LN_scenario}, 
		{_NS_pp12, _LN_titleStyle}, 
		{_NS_xml, _LN_lang}, 
		{_NS_text, _LN_label}, 
		{_NS_rtf, _LN_linecont}, 
		{_NS_prj12, _LN_ValuelistSortOrder}, 
		{_NS_a12, _LN_chart}, 
		{_NS_rtf, _LN_qk}, 
		{_NS_xml, _LN_base}, 
		{_NS_text, _LN_list_style}, 
		{_NS_v11, _LN_IndLeft}, 
		{_NS_pp12, _LN_bodyStyle}, 
		{_NS_rtf, _LN_ql}, 
		{_NS_text, _LN_note_body}, 
		{_NS_rtf, _LN_linemod}, 
		{_NS_rdf, _LN_RDF}, 
		{_NS_o12, _LN_File}, 
		{_NS_table, _LN_scenario_ranges}, 
		{_NS_prj12, _LN_AppendNewValues}, 
		{_NS_v11, _LN_IndRight}, 
		{_NS_rtf, _LN_qmspace}, 
		{_NS_rng, _LN_except}, 
		{_NS_prj12, _LN_Default}, 
		{_NS_pp12, _LN_notesStyle}, 
		{_NS_rtf, _LN_lineppage}, 
		{_NS_text, _LN_note_class}, 
		{_NS_o12, _LN_MainFile}, 
		{_NS_table, _LN_is_active}, 
		{_NS_text, _LN_consecutive_numbering}, 
		{_NS_rtf, _LN_qr}, 
		{_NS_v11, _LN_SpLine}, 
		{_NS_o12, _LN_Title}, 
		{_NS_rng, _LN_nsName}, 
		{_NS_text, _LN_ruby}, 
		{_NS_a12, _LN_bldIgx}, 
		{_NS_fo, _LN_font_size}, 
		{_NS_prj12, _LN_ValueList}, 
		{_NS_o12, _LN_Subject}, 
		{_NS_pp12, _LN_otherStyle}, 
		{_NS_rtf, _LN_qt}, 
		{_NS_rtf, _LN_linerestart}, 
		{_NS_rdf, _LN_ID}, 
		{_NS_o12, _LN_Author}, 
		{_NS_v11, _LN_SpBefore}, 
		{_NS_table, _LN_display_border}, 
		{_NS_prj12, _LN_ID}, 
		{_NS_text, _LN_ruby_base}, 
		{_NS_a12, _LN_bldChart}, 
		{_NS_text, _LN_list_level_style_number}, 
		{_NS_rdf, _LN_nodeID}, 
		{_NS_pp12, _LN_sldLayoutId}, 
		{_NS_rtf, _LN_linestart}, 
		{_NS_rtf, _LN_rawclbgbdiag}, 
		{_NS_style, _LN_font_size_asian}, 
		{_NS_v11, _LN_SpAfter}, 
		{_NS_o12, _LN_LastAuthor}, 
		{_NS_prj12, _LN_Calendars}, 
		{_NS_table, _LN_border_color}, 
		{_NS_text, _LN_ruby_text}, 
		{_NS_rdf, _LN_about}, 
		{_NS_o12, _LN_Revision}, 
		{_NS_prj12, _LN_Calendar}, 
		{_NS_rtf, _LN_linestarts}, 
		{_NS_text, _LN_date}, 
		{_NS_text, _LN_display_levels}, 
		{_NS_v11, _LN_HorzAlign}, 
		{_NS_rtf, _LN_rawclbgcross}, 
		{_NS_pp12, _LN_sldLayoutIdLst}, 
		{_NS_table, _LN_copy_back}, 
		{_NS_style, _LN_font_size_complex}, 
		{_NS_rdf, _LN_resource}, 
		{_NS_o12, _LN_AppName}, 
		{_NS_rtf, _LN_linex}, 
		{_NS_v11, _LN_Bullet}, 
		{_NS_rdf, _LN_datatype}, 
		{_NS_text, _LN_date_value}, 
		{_NS_prj12, _LN_IsBaseCalendar}, 
		{_NS_pp12, _LN_txStyles}, 
		{_NS_rtf, _LN_rawclbgdcross}, 
		{_NS_table, _LN_copy_styles}, 
		{_NS_o12, _LN_TotalTime}, 
		{_NS_rtf, _LN_linkself}, 
		{_NS_style, _LN_font_size_rel}, 
		{_NS_text, _LN_list_level_style_bullet}, 
		{_NS_v11, _LN_BulletStr}, 
		{_NS_rdf, _LN_parseType}, 
		{_NS_o12, _LN_LastPrinted}, 
		{_NS_pp12, _LN_sldMaster}, 
		{_NS_text, _LN_date_adjust}, 
		{_NS_prj12, _LN_BaseCalendarUID}, 
		{_NS_rtf, _LN_rawbgdkbdiag}, 
		{_NS_rtf, _LN_linkstyles}, 
		{_NS_table, _LN_copy_formulas}, 
		{_NS_text, _LN_time}, 
		{_NS_o12, _LN_Created}, 
		{_NS_text, _LN_bullet_char}, 
		{_NS_rng, _LN_parentRef}, 
		{_NS_v11, _LN_BulletFont}, 
		{_NS_style, _LN_font_size_rel_asian}, 
		{_NS_prj12, _LN_WeekDays}, 
		{_NS_pp12, _LN_handoutMaster}, 
		{_NS_o12, _LN_LastSaved}, 
		{_NS_rtf, _LN_linkval}, 
		{_NS_table, _LN_comment}, 
		{_NS_o12, _LN_Pages}, 
		{_NS_rtf, _LN_rawclbgdkcross}, 
		{_NS_text, _LN_time_value}, 
		{_NS_rng, _LN_notAllowed}, 
		{_NS_rtf, _LN_lin}, 
		{_NS_prj12, _LN_WeekDay}, 
		{_NS_pp12, _LN_notesMaster}, 
		{_NS_table, _LN_shapes}, 
		{_NS_v11, _LN_LocalizeBulletFont}, 
		{_NS_text, _LN_bullet_relative_size}, 
		{_NS_o12, _LN_Words}, 
		{_NS_rtf, _LN_lisa}, 
		{_NS_pp12, _LN_notes}, 
		{_NS_style, _LN_font_size_rel_complex}, 
		{_NS_text, _LN_time_adjust}, 
		{_NS_prj12, _LN_DayType}, 
		{_NS_rng, _LN_externalRef}, 
		{_NS_rtf, _LN_rawclbgdkdcross}, 
		{_NS_rtf, _LN_lisb}, 
		{QName::EMPTY_STRING, _LN_href}, 
		{_NS_o12, _LN_Characters}, 
		{_NS_v11, _LN_BulletFontSize}, 
		{_NS_table, _LN_cell_range_source}, 
		{_NS_rng, _LN_div}, 
		{_NS_style, _LN_script_type}, 
		{_NS_o12, _LN_Category}, 
		{_NS_a12, _LN_txBody}, 
		{_NS_prj12, _LN_DayWorking}, 
		{_NS_text, _LN_page_number}, 
		{_NS_text, _LN_list_level_style_image}, 
		{_NS_rtf, _LN_rawclbgdkfdiag}, 
		{_NS_rng, _LN_include}, 
		{_NS_rtf, _LN_listhybrid}, 
		{_NS_a12, _LN_xfrm}, 
		{_NS_prj12, _LN_TimePeriod}, 
		{_NS_v11, _LN_TextPosAfterBullet}, 
		{_NS_text, _LN_page_adjust}, 
		{_NS_fo, _LN_letter_spacing}, 
		{_NS_rtf, _LN_listid}, 
		{_NS_table, _LN_last_column_spanned}, 
		{_NS_text, _LN_outline_style}, 
		{_NS_o12, _LN_PresentationFormat}, 
		{_NS_rtf, _LN_rawclbgdkhor}, 
		{_NS_a12, _LN_useSpRect}, 
		{_NS_o12, _LN_Manager}, 
		{_NS_v11, _LN_Flags}, 
		{_NS_prj12, _LN_FromDate}, 
		{_NS_rtf, _LN_listname}, 
		{_NS_text, _LN_select_page}, 
		{_NS_o12, _LN_Company}, 
		{_NS_style, _LN_language_asian}, 
		{_NS_a12, _LN_anvPr}, 
		{_NS_v11, _LN_Tab}, 
		{_NS_rtf, _LN_rawclbgdkvert}, 
		{_NS_table, _LN_last_row_spanned}, 
		{_NS_rtf, _LN_ab}, 
		{_NS_prj12, _LN_ToDate}, 
		{_NS_text, _LN_outline_level_style}, 
		{_NS_o12, _LN_Guid}, 
		{_NS_a12, _LN_anvSpPr}, 
		{_NS_o12, _LN_HyperlinkBase}, 
		{_NS_v11, _LN_Position}, 
		{_NS_rtfs, _LN_Toggle}, 
		{_NS_rtf, _LN_listoverridecount}, 
		{_NS_text, _LN_page_continuation}, 
		{_NS_table, _LN_detective}, 
		{_NS_rtf, _LN_rawclbgfdiag}, 
		{_NS_style, _LN_language_complex}, 
		{_NS_prj12, _LN_WorkingTimes}, 
		{_NS_a12, _LN_nvSpPr}, 
		{_NS_o12, _LN_Bytes}, 
		{_NS_rtf, _LN_absh}, 
		{_NS_style, _LN_graphic_properties}, 
		{_NS_v11, _LN_Alignment}, 
		{_NS_table, _LN_operation}, 
		{_NS_rtfs, _LN_Value}, 
		{_NS_text, _LN_string_value}, 
		{_NS_a12, _LN_spPr}, 
		{_NS_rtf, _LN_rawclbghoriz}, 
		{_NS_o12, _LN_Lines}, 
		{_NS_style, _LN_country_asian}, 
		{_NS_prj12, _LN_WorkingTime}, 
		{_NS_rtf, _LN_listoverrideformat}, 
		{_NS_v11, _LN_Leader}, 
		{_NS_table, _LN_index}, 
		{_NS_rtf, _LN_abslock}, 
		{_NS_a12, _LN_txsp}, 
		{_NS_v11, _LN_X}, 
		{_NS_prj12, _LN_FromTime}, 
		{_NS_o12, _LN_Paragraphs}, 
		{_NS_rtf, _LN_rawclbgvert}, 
		{_NS_a12, _LN_style}, 
		{_NS_style, _LN_drawing_page_properties}, 
		{_NS_text, _LN_sender_firstname}, 
		{_NS_pp12, _LN_restoredLeft}, 
		{_NS_rtfs, _LN_Flag}, 
		{_NS_style, _LN_country_complex}, 
		{_NS_v11, _LN_Y}, 
		{_NS_rtf, _LN_listoverridestart}, 
		{_NS_fo, _LN_font_style}, 
		{_NS_prj12, _LN_ToTime}, 
		{_NS_table, _LN_highlighted_range}, 
		{_NS_v11, _LN_A}, 
		{_NS_rtf, _LN_rdblquote}, 
		{_NS_draw, _LN_gradient}, 
		{_NS_a12, _LN_anvCxnSpPr}, 
		{_NS_rtf, _LN_absnoovrlp}, 
		{_NS_pp12, _LN_restoredTop}, 
		{_NS_rtf, _LN_red}, 
		{_NS_draw, _LN_display_name}, 
		{_NS_v11, _LN_B}, 
		{_NS_text, _LN_sender_lastname}, 
		{_NS_prj12, _LN_Tasks}, 
		{_NS_o12, _LN_CharactersWithSpaces}, 
		{_NS_rtf, _LN_listpicture}, 
		{_NS_draw, _LN_style}, 
		{_NS_v11, _LN_C}, 
		{_NS_o12, _LN_Version}, 
		{_NS_rtf, _LN_absw}, 
		{_NS_a12, _LN_nvCxnSpPr}, 
		{_NS_style, _LN_font_style_asian}, 
		{_NS_pp12, _LN_scale}, 
		{_NS_prj12, _LN_Task}, 
		{_NS_draw, _LN_cx}, 
		{_NS_table, _LN_cell_range_address}, 
		{_NS_o12, _LN_SmartTagType}, 
		{_NS_v11, _LN_D}, 
		{_NS_pp12, _LN_origin}, 
		{_NS_rtf, _LN_rempersonalinfo}, 
		{_NS_draw, _LN_cy}, 
		{_NS_rtf, _LN_acaps}, 
		{_NS_text, _LN_sender_initials}, 
		{_NS_rtf, _LN_listrestarthdn}, 
		{_NS_prj12, _LN_IsNull}, 
		{_NS_v11, _LN_DirX}, 
		{_NS_table, _LN_direction}, 
		{_NS_o12, _LN_SmartTags}, 
		{_NS_a12, _LN_anvGraphicFramePr}, 
		{_NS_rtf, _LN_result}, 
		{_NS_style, _LN_font_style_complex}, 
		{_NS_pp12, _LN_viewPrBase}, 
		{_NS_rtf, _LN_acccomma}, 
		{_NS_draw, _LN_start_color}, 
		{_NS_rtf, _LN_listsimple}, 
		{_NS_text, _LN_sender_title}, 
		{_NS_v11, _LN_DirY}, 
		{_NS_prj12, _LN_CreateDate}, 
		{_NS_pp12, _LN_guide}, 
		{_NS_rtf, _LN_revauth}, 
		{_NS_rtf, _LN_accdot}, 
		{_NS_table, _LN_contains_error}, 
		{_NS_style, _LN_font_relief}, 
		{_NS_v11, _LN_Type}, 
		{_NS_draw, _LN_end_color}, 
		{_NS_a12, _LN_nvGraphicFramePr}, 
		{_NS_rtf, _LN_liststyleid}, 
		{_NS_prj12, _LN_Contact}, 
		{_NS_o12, _LN_CustomDocumentProperties}, 
		{_NS_text, _LN_sender_position}, 
		{_NS_pp12, _LN_guideLst}, 
		{_NS_rtf, _LN_accnone}, 
		{_NS_rtf, _LN_revauthdel}, 
		{_NS_v11, _LN_AutoGen}, 
		{_NS_prj12, _LN_WBS}, 
		{_NS_fo, _LN_text_shadow}, 
		{_NS_a12, _LN_anvGspPr}, 
		{_NS_table, _LN_marked_invalid}, 
		{_NS_rtf, _LN_acf}, 
		{_NS_rtf, _LN_liststylename}, 
		{_NS_draw, _LN_start_intensity}, 
		{_NS_rtf, _LN_revbar}, 
		{_NS_v11, _LN_Prompt}, 
		{_NS_text, _LN_sender_email}, 
		{_NS_pp12, _LN_sldViewPrBase}, 
		{_NS_rtf, _LN_additive}, 
		{_NS_prj12, _LN_WBSLevel}, 
		{_NS_a12, _LN_nvGspPr}, 
		{_NS_o12, _LN_OfficeDocumentSettings}, 
		{_NS_v11, _LN_Value}, 
		{_NS_rtf, _LN_revdttm}, 
		{_NS_style, _LN_text_underline_type}, 
		{_NS_table, _LN_structure_protected}, 
		{_NS_draw, _LN_end_intensity}, 
		{_NS_rtf, _LN_listtemplateid}, 
		{_NS_a12, _LN_gspPr}, 
		{_NS_rtf, _LN_adjustright}, 
		{_NS_pp12, _LN_normalViewPr}, 
		{_NS_prj12, _LN_OutlineNumber}, 
		{_NS_text, _LN_sender_phone_private}, 
		{_NS_a12, _LN_sp}, 
		{_NS_v11, _LN_EditMode}, 
		{_NS_draw, _LN_angle}, 
		{_NS_rtf, _LN_revdttmdel}, 
		{_NS_rtf, _LN_adn}, 
		{_NS_rtf, _LN_listtext}, 
		{_NS_o12, _LN_DownloadComponents}, 
		{_NS_pp12, _LN_slideViewPr}, 
		{_NS_a12, _LN_cxnsp}, 
		{_NS_v11, _LN_Format}, 
		{_NS_style, _LN_text_underline_style}, 
		{_NS_prj12, _LN_OutlineLevel}, 
		{_NS_table, _LN_calculation_settings}, 
		{_NS_draw, _LN_border}, 
		{_NS_text, _LN_sender_fax}, 
		{_NS_rtf, _LN_revised}, 
		{_NS_rtf, _LN_aenddoc}, 
		{_NS_rtf, _LN_lnbrkrule}, 
		{_NS_rtf, _LN_revisions}, 
		{_NS_v11, _LN_UICat}, 
		{_NS_prj12, _LN_Priority}, 
		{_NS_pp12, _LN_outlineViewPr}, 
		{_NS_a12, _LN_graphicFrame}, 
		{_NS_o12, _LN_LocationOfComponents}, 
		{_NS_rtf, _LN_aendnotes}, 
		{_NS_table, _LN_case_sensitive}, 
		{_NS_svg, _LN_linearGradient}, 
		{_NS_text, _LN_sender_company}, 
		{_NS_rtf, _LN_lndscpsxn}, 
		{_NS_rtf, _LN_revprop}, 
		{_NS_v11, _LN_UICod}, 
		{_NS_style, _LN_text_underline_width}, 
		{_NS_a12, _LN_gsp}, 
		{_NS_prj12, _LN_Duration}, 
		{_NS_rtf, _LN_aexpnd}, 
		{_NS_o12, _LN_shapedefaults}, 
		{_NS_v11, _LN_UIFmt}, 
		{_NS_pp12, _LN_notesTextViewPr}, 
		{_NS_rtf, _LN_lnongrid}, 
		{_NS_rtf, _LN_revprot}, 
		{_NS_svg, _LN_radialGradient}, 
		{_NS_prj12, _LN_Work}, 
		{_NS_rtf, _LN_af}, 
		{_NS_text, _LN_sender_phone_work}, 
		{_NS_table, _LN_precision_as_shown}, 
		{_NS_rtf, _LN_loch}, 
		{_NS_svg, _LN_fx}, 
		{_NS_style, _LN_text_underline_color}, 
		{_NS_rtf, _LN_revtbl}, 
		{_NS_o12, _LN_colormru}, 
		{_NS_prj12, _LN_Stop}, 
		{_NS_v11, _LN_ObjectKind}, 
		{_NS_pp12, _LN_sorterViewPr}, 
		{_NS_rtf, _LN_affixed}, 
		{_NS_fo, _LN_font_weight}, 
		{_NS_svg, _LN_fy}, 
		{_NS_rtf, _LN_lquote}, 
		{_NS_text, _LN_sender_street}, 
		{_NS_rtf, _LN_revtim}, 
		{_NS_v11, _LN_XDyn}, 
		{_NS_prj12, _LN_Resume}, 
		{_NS_rtf, _LN_afs}, 
		{_NS_o12, _LN_colormenu}, 
		{_NS_rtf, _LN_ls}, 
		{_NS_svg, _LN_stop}, 
		{_NS_pp12, _LN_notesViewPr}, 
		{_NS_rtf, _LN_ri}, 
		{_NS_v11, _LN_YDyn}, 
		{_NS_svg, _LN_offset}, 
		{_NS_rtf, _LN_aftnbj}, 
		{_NS_o12, _LN_colors}, 
		{_NS_style, _LN_font_weight_asian}, 
		{_NS_text, _LN_sender_city}, 
		{_NS_rtf, _LN_ltrch}, 
		{_NS_prj12, _LN_ResumeValid}, 
		{_NS_rtf, _LN_rin}, 
		{_NS_table, _LN_search_criteria_must_apply_to_whole_cell}, 
		{_NS_v11, _LN_XCon}, 
		{_NS_pp12, _LN_gridSpacing}, 
		{_NS_a12, _LN_spid}, 
		{_NS_rtf, _LN_row}, 
		{_NS_rtf, _LN_aftncn}, 
		{_NS_svg, _LN_stop_color}, 
		{_NS_rtf, _LN_ltrdoc}, 
		{_NS_o12, _LN_shapelayout}, 
		{_NS_a12, _LN_name}, 
		{_NS_v11, _LN_YCon}, 
		{_NS_prj12, _LN_EffortDriven}, 
		{_NS_pp12, _LN_viewPr}, 
		{_NS_rtf, _LN_rquote}, 
		{_NS_text, _LN_sender_postal_code}, 
		{_NS_style, _LN_font_weight_complex}, 
		{_NS_o12, _LN_idmap}, 
		{_NS_rtfs, _LN_Destination}, 
		{_NS_a12, _LN_descr}, 
		{_NS_rtf, _LN_ltrmark}, 
		{_NS_rtf, _LN_rsid}, 
		{_NS_v11, _LN_CanGlue}, 
		{_NS_table, _LN_automatic_find_labels}, 
		{_NS_svg, _LN_stop_opacity}, 
		{_NS_prj12, _LN_Recurring}, 
		{_NS_rtf, _LN_ltrpar}, 
		{_NS_rtf, _LN_aftnnalc}, 
		{_NS_text, _LN_sender_country}, 
		{_NS_v11, _LN_NoFill}, 
		{_NS_rtf, _LN_rsidroot}, 
		{_NS_o12, _LN_regrouptable}, 
		{_NS_style, _LN_text_underline_mode}, 
		{_NS_svg, _LN_gradientUnits}, 
		{_NS_rtf, _LN_ltrrow}, 
		{_NS_prj12, _LN_OverAllocated}, 
		{_NS_rtf, _LN_aftnnar}, 
		{_NS_v11, _LN_NoLine}, 
		{_NS_rtf, _LN_rsidtbl}, 
		{_NS_o12, _LN_rules}, 
		{_NS_table, _LN_use_regular_expressions}, 
		{_NS_rtf, _LN_ltrsect}, 
		{_NS_v11, _LN_NoShow}, 
		{_NS_o12, _LN_entry}, 
		{_NS_prj12, _LN_Estimated}, 
		{_NS_rtf, _LN_aftnnauc}, 
		{_NS_text, _LN_sender_state_or_province}, 
		{_NS_rtf, _LN_rsltbmp}, 
		{_NS_svg, _LN_gradientTransform}, 
		{_NS_style, _LN_text_line_through_mode}, 
		{_NS_table, _LN_null_year}, 
		{_NS_o12, _LN_r}, 
		{_NS_v11, _LN_NoSnap}, 
		{_NS_wddr12, _LN_extent}, 
		{_NS_rtf, _LN_aftnnchi}, 
		{_NS_prj12, _LN_Milestone}, 
		{_NS_rtf, _LN_rslthtml}, 
		{_NS_rtf, _LN_lytcalctblwd}, 
		{_NS_text, _LN_author_name}, 
		{_NS_o12, _LN_proxy}, 
		{_NS_table, _LN_null_date}, 
		{_NS_a12, _LN_rot}, 
		{_NS_svg, _LN_spreadMethod}, 
		{_NS_style, _LN_letter_kerning}, 
		{_NS_v11, _LN_MoveTo}, 
		{_NS_table, _LN_value_type}, 
		{_NS_prj12, _LN_Summary}, 
		{_NS_o12, _LN_skew}, 
		{_NS_rtf, _LN_rsltmerge}, 
		{_NS_rtf, _LN_lytexcttp}, 
		{_NS_draw, _LN_hatch}, 
		{_NS_wddr12, _LN_effectExtent}, 
		{_NS_rtf, _LN_aftnnchosung}, 
		{_NS_o12, _LN_extrusion}, 
		{_NS_v11, _LN_LineTo}, 
		{_NS_text, _LN_author_initials}, 
		{_NS_draw, _LN_distance}, 
		{_NS_style, _LN_text_blinking}, 
		{_NS_prj12, _LN_Critical}, 
		{_NS_wddr12, _LN_docPr}, 
		{_NS_rtf, _LN_rsltpict}, 
		{_NS_rtf, _LN_lytprtmet}, 
		{_NS_table, _LN_date_value_type}, 
		{_NS_rtf, _LN_aftnncnum}, 
		{_NS_v11, _LN_ArcTo}, 
		{_NS_o12, _LN_callout}, 
		{_NS_wddr12, _LN_graphic}, 
		{_NS_text, _LN_chapter}, 
		{_NS_draw, _LN_rotation}, 
		{_NS_o12, _LN_lock}, 
		{_NS_wddr12, _LN_start}, 
		{_NS_rtf, _LN_rsltrtf}, 
		{_NS_style, _LN_text_combine}, 
		{_NS_prj12, _LN_IsSubproject}, 
		{_NS_table, _LN_iteration}, 
		{_NS_rtf, _LN_lyttblrtgr}, 
		{_NS_rtf, _LN_aftnndbar}, 
		{_NS_wddr12, _LN_lineTo}, 
		{_NS_text, _LN_file_name}, 
		{_NS_v11, _LN_InfiniteLine}, 
		{_NS_draw, _LN_fill_image}, 
		{_NS_o12, _LN_OLEObject}, 
		{_NS_rtf, _LN_rslttxt}, 
		{_NS_rtf, _LN_mac}, 
		{_NS_table, _LN_status}, 
		{_NS_rtf, _LN_aftnndbnum}, 
		{_NS_v11, _LN_Ellipse}, 
		{_NS_wddr12, _LN_wrapPolygon}, 
		{_NS_rtf, _LN_rtf}, 
		{_NS_draw, _LN_opacity}, 
		{_NS_text, _LN_template_name}, 
		{_NS_prj12, _LN_IsSubprojectReadOnly}, 
		{_NS_o12, _LN_complex}, 
		{_NS_rtf, _LN_macpict}, 
		{_NS_table, _LN_steps}, 
		{_NS_style, _LN_text_combine_start_char}, 
		{_NS_draw, _LN_start}, 
		{_NS_rtf, _LN_rtlch}, 
		{_NS_wddr12, _LN_wrapNone}, 
		{_NS_rtf, _LN_aftnndbnumd}, 
		{_NS_draw, _LN_end}, 
		{_NS_v11, _LN_EllipticalArcTo}, 
		{_NS_text, _LN_sheet_name}, 
		{_NS_rtf, _LN_makebackup}, 
		{_NS_RowsetSchema, _LN_row}, 
		{_NS_prj12, _LN_SubprojectName}, 
		{_NS_rtf, _LN_rtldoc}, 
		{_NS_table, _LN_maximum_difference}, 
		{_NS_draw, _LN_marker}, 
		{_NS_style, _LN_text_combine_end_char}, 
		{_NS_wddr12, _LN_wrapSquare}, 
		{_NS_rtf, _LN_aftnndbnumk}, 
		{_NS_rtf, _LN_manager}, 
		{_NS_v11, _LN_SplineStart}, 
		{_NS_text, _LN_variable_decl}, 
		{_NS_rtf, _LN_rtlgutter}, 
		{_NS_prj12, _LN_ExternalTask}, 
		{_NS_draw, _LN_stroke_dash}, 
		{_NS_wddr12, _LN_wrapTight}, 
		{_NS_rtf, _LN_margb}, 
		{_NS_rtf, _LN_aftnndbnumt}, 
		{_NS_style, _LN_text_emphasize}, 
		{_NS_table, _LN_content_validations}, 
		{_NS_a12, _LN_camera}, 
		{_NS_v11, _LN_SplineKnot}, 
		{_NS_rtf, _LN_rtlmark}, 
		{_NS_text, _LN_variable_set}, 
		{_NS_draw, _LN_dots1}, 
		{_NS_rs11, _LN_data}, 
		{_NS_rtf, _LN_margbsxn}, 
		{_NS_wddr12, _LN_wrapThrough}, 
		{_NS_prj12, _LN_ExternalTaskProject}, 
		{_NS_style, _LN_text_scale}, 
		{_NS_rtf, _LN_aftnnganada}, 
		{_NS_a12, _LN_lightRig}, 
		{_NS_rtf, _LN_rtlpar}, 
		{_NS_v11, _LN_PolylineTo}, 
		{_NS_rtf, _LN_margl}, 
		{_NS_text, _LN_variable_get}, 
		{_NS_table, _LN_content_validation}, 
		{_NS_draw, _LN_dots1_length}, 
		{_NS_rtf, _LN_rtlrow}, 
		{_NS_table, _LN_condition}, 
		{_NS_a12, _LN_backdrop}, 
		{_NS_prj12, _LN_EarlyStart}, 
		{_NS_v11, _LN_NURBSTo}, 
		{_NS_rtf, _LN_aftnngbnum}, 
		{_NS_wddr12, _LN_wrapTopAndBottom}, 
		{_NS_draw, _LN_dots2}, 
		{_NS_rtf, _LN_marglsxn}, 
		{_NS_style, _LN_text_rotation_angle}, 
		{_NS_v11, _LN_E}, 
		{_NS_wddr12, _LN_align}, 
		{_NS_rtf, _LN_rtlsect}, 
		{_NS_text, _LN_variable_input}, 
		{_NS_wddr12, _LN_offset}, 
		{_NS_prj12, _LN_EarlyFinish}, 
		{_NS_rtf, _LN_aftnngbnumd}, 
		{_NS_v11, _LN_Menu}, 
		{_NS_rtf, _LN_margmirror}, 
		{_NS_table, _LN_base_cell_address}, 
		{_NS_rtf, _LN_rxe}, 
		{_NS_draw, _LN_dots2_length}, 
		{_NS_s11, _LN_Schema}, 
		{_NS_rtf, _LN_s}, 
		{_NS_wddr12, _LN_simplePos}, 
		{_NS_style, _LN_text_rotation_scale}, 
		{_NS_prj12, _LN_LateStart}, 
		{_NS_text, _LN_user_field_decl}, 
		{_NS_v11, _LN_Action}, 
		{_NS_rtf, _LN_margr}, 
		{_NS_rtf, _LN_aftnngbnumk}, 
		{_NS_rtf, _LN_sa}, 
		{_NS_s11, _LN_ElementType}, 
		{_NS_table, _LN_allow_empty_cell}, 
		{_NS_v11, _LN_Checked}, 
		{_NS_wddr12, _LN_positionH}, 
		{_NS_s11, _LN_attribute}, 
		{_NS_fo, _LN_hyphenate}, 
		{_NS_rtf, _LN_margrsxn}, 
		{_NS_prj12, _LN_LateFinish}, 
		{_NS_style, _LN_presentation_page_layout}, 
		{_NS_rtf, _LN_saauto}, 
		{_NS_rtf, _LN_aftnngbnuml}, 
		{_NS_text, _LN_user_field_get}, 
		{_NS_pr, _LN_object}, 
		{_NS_rtf, _LN_margt}, 
		{_NS_s11, _LN_extends}, 
		{_NS_table, _LN_display_list}, 
		{_NS_v11, _LN_Disabled}, 
		{_NS_wddr12, _LN_positionV}, 
		{_NS_a12, _LN_anchor}, 
		{_NS_rtf, _LN_aftnnrlc}, 
		{_NS_rtf, _LN_saftnnalc}, 
		{_NS_prj12, _LN_StartVariance}, 
		{_NS_text, _LN_user_field_input}, 
		{_NS_rtf, _LN_margtsxn}, 
		{_NS_v11, _LN_ReadOnly}, 
		{_NS_a12, _LN_normal}, 
		{_NS_table, _LN_help_message}, 
		{_NS_s11, _LN_AttributeType}, 
		{_NS_fo, _LN_hyphenation_remain_char_count}, 
		{_NS_rtf, _LN_aftnnruc}, 
		{_NS_rtf, _LN_saftnnar}, 
		{_NS_style, _LN_page_layout_properties}, 
		{_NS_table, _LN_title}, 
		{_NS_a12, _LN_up}, 
		{_NS_prj12, _LN_FinishVariance}, 
		{_NS_rtf, _LN_mhtmltag}, 
		{_NS_v11, _LN_Invisible}, 
		{_NS_text, _LN_sequence_decl}, 
		{_NS_s11, _LN_datatype}, 
		{_NS_rtf, _LN_saftnnauc}, 
		{_NS_rtf, _LN_aftnnzodiac}, 
		{_NS_rtf, _LN_min}, 
		{_NS_fo, _LN_page_width}, 
		{_NS_table, _LN_error_message}, 
		{_NS_prj12, _LN_WorkVariance}, 
		{_NS_v11, _LN_BeginGroup}, 
		{_NS_rtf, _LN_mo}, 
		{_NS_fo, _LN_hyphenation_push_char_count}, 
		{_NS_rtf, _LN_saftnnchi}, 
		{_NS_rtf, _LN_aftnnzodiacd}, 
		{_NS_fo, _LN_page_height}, 
		{_NS_text, _LN_display_outline_level}, 
		{_NS_prj12, _LN_FreeSlack}, 
		{_NS_v11, _LN_TagName}, 
		{_NS_rtf, _LN_msmcap}, 
		{_NS_table, _LN_message_type}, 
		{_NS_rtf, _LN_saftnnchosung}, 
		{_NS_rtf, _LN_aftnnzodiacl}, 
		{_NS_style, _LN_paragraph_properties}, 
		{_NS_rtf, _LN_nestcell}, 
		{_NS_prj12, _LN_TotalSlack}, 
		{_NS_v11, _LN_ButtonFace}, 
		{_NS_style, _LN_paper_tray_name}, 
		{_NS_table, _LN_error_macro}, 
		{_NS_text, _LN_separation_character}, 
		{_NS_rtf, _LN_saftnncnum}, 
		{_NS_rtf, _LN_nestrow}, 
		{_NS_a12, _LN_bevelT}, 
		{_NS_v11, _LN_SortKey}, 
		{_NS_rtf, _LN_aftnrestart}, 
		{_NS_prj12, _LN_FixedCost}, 
		{_NS_fo, _LN_line_height}, 
		{_NS_table, _LN_execute}, 
		{_NS_w12, _LN_ilvl}, 
		{_NS_text, _LN_sequence}, 
		{_NS_v11, _LN_Name}, 
		{_NS_style, _LN_print_orientation}, 
		{_NS_a12, _LN_bevelB}, 
		{_NS_rtf, _LN_saftnndbar}, 
		{_NS_w12, _LN_ilfo}, 
		{_NS_rtf, _LN_aftnrstcont}, 
		{_NS_rtf, _LN_nesttableprops}, 
		{_NS_v11, _LN_Status}, 
		{_NS_table, _LN_label_ranges}, 
		{_NS_w12, _LN_top}, 
		{_NS_text, _LN_ref_name}, 
		{_NS_prj12, _LN_FixedCostAccrual}, 
		{_NS_style, _LN_line_height_at_least}, 
		{_NS_w12, _LN_left}, 
		{_NS_rtf, _LN_saftnndbnum}, 
		{_NS_a12, _LN_extrusionClr}, 
		{_NS_rtf, _LN_aftnsep}, 
		{_NS_rtf, _LN_nextfile}, 
		{_NS_v11, _LN_Visible}, 
		{_NS_w12, _LN_bottom}, 
		{_NS_text, _LN_expression}, 
		{_NS_table, _LN_label_range}, 
		{_NS_w12, _LN_right}, 
		{_NS_style, _LN_register_truth_ref_style_name}, 
		{_NS_prj12, _LN_PercentComplete}, 
		{_NS_style, _LN_line_spacing}, 
		{_NS_v11, _LN_Print}, 
		{_NS_rtf, _LN_aftnsepc}, 
		{_NS_a12, _LN_contourClr}, 
		{_NS_rtf, _LN_saftnndbnumd}, 
		{_NS_style, _LN_print}, 
		{_NS_rtf, _LN_nobrkwrptbl}, 
		{_NS_text, _LN_text_input}, 
		{_NS_w12, _LN_between}, 
		{_NS_v11, _LN_Active}, 
		{_NS_a12, _LN_flatTx}, 
		{_NS_text, _LN_initial_creator}, 
		{_NS_rtf, _LN_aftnstart}, 
		{_NS_w12, _LN_bar}, 
		{_NS_rtf, _LN_nocolbal}, 
		{_NS_rtf, _LN_saftnndbnumk}, 
		{_NS_table, _LN_label_cell_range_address}, 
		{_NS_prj12, _LN_PercentWorkComplete}, 
		{_NS_text, _LN_creation_date}, 
		{_NS_w12, _LN_tab}, 
		{_NS_v11, _LN_Lock}, 
		{_NS_style, _LN_print_page_order}, 
		{_NS_rtf, _LN_aftntj}, 
		{_NS_style, _LN_font_independent_line_spacing}, 
		{_NS_prj12, _LN_Cost}, 
		{_NS_v11, _LN_Snap}, 
		{_NS_w12, _LN_pStyle}, 
		{_NS_rtf, _LN_ai}, 
		{_NS_rtf, _LN_saftnndbnumt}, 
		{_NS_rtf, _LN_nocompatoptions}, 
		{_NS_text, _LN_creation_time}, 
		{_NS_v11, _LN_Glue}, 
		{_NS_style, _LN_first_page_number}, 
		{_NS_text, _LN_description}, 
		{_NS_table, _LN_data_cell_range_address}, 
		{_NS_fo, _LN_text_align}, 
		{_NS_rtf, _LN_alang}, 
		{_NS_prj12, _LN_OvertimeCost}, 
		{_NS_w12, _LN_keepNext}, 
		{_NS_rtf, _LN_nocwrap}, 
		{_NS_text, _LN_user_defined}, 
		{_NS_rtf, _LN_saftnnganada}, 
		{_NS_v11, _LN_NameUniv}, 
		{_NS_office, _LN_value}, 
		{_NS_style, _LN_scale_to}, 
		{_NS_table, _LN_orientation}, 
		{_NS_w12, _LN_keepLines}, 
		{_NS_office, _LN_date_value}, 
		{_NS_fo, _LN_text_align_last}, 
		{_NS_prj12, _LN_OvertimeWork}, 
		{_NS_rtf, _LN_noextrasprl}, 
		{_NS_rtf, _LN_allowfieldendsel}, 
		{_NS_v11, _LN_Label}, 
		{_NS_rtf, _LN_saftnngbnum}, 
		{_NS_office, _LN_time_value}, 
		{_NS_a12, _LN_clrFrom}, 
		{_NS_style, _LN_scale_to_pages}, 
		{_NS_rtf, _LN_allprot}, 
		{_NS_rtf, _LN_nofchars}, 
		{_NS_v11, _LN_Verify}, 
		{_NS_prj12, _LN_ActualStart}, 
		{_NS_w12, _LN_pageBreakBefore}, 
		{_NS_table, _LN_named_expressions}, 
		{_NS_a12, _LN_clrTo}, 
		{_NS_rtf, _LN_saftnngbnumd}, 
		{_NS_style, _LN_justify_single_word}, 
		{_NS_v11, _LN_Description}, 
		{_NS_office, _LN_boolean_value}, 
		{_NS_w12, _LN_framePr}, 
		{_NS_rtf, _LN_alntblind}, 
		{_NS_office, _LN_string_value}, 
		{_NS_rtf, _LN_nofcharsws}, 
		{_NS_style, _LN_table_centering}, 
		{_NS_prj12, _LN_ActualFinish}, 
		{_NS_table, _LN_named_range}, 
		{_NS_v11, _LN_Address}, 
		{_NS_a12, _LN_fillToRect}, 
		{_NS_rtf, _LN_saftnngbnumk}, 
		{_NS_rtf, _LN_alt}, 
		{_NS_fo, _LN_keep_together}, 
		{_NS_rtf, _LN_nofpages}, 
		{_NS_a12, _LN_gs}, 
		{_NS_text, _LN_print_time}, 
		{_NS_w12, _LN_widowControl}, 
		{_NS_v11, _LN_SubAddress}, 
		{_NS_prj12, _LN_ActualDuration}, 
		{_NS_text, _LN_print_date}, 
		{_NS_fo, _LN_widows}, 
		{_NS_table, _LN_range_usable_as}, 
		{_NS_rtf, _LN_animtext}, 
		{_NS_style, _LN_footnote_max_height}, 
		{_NS_rtf, _LN_saftnngbnuml}, 
		{_NS_a12, _LN_gsList}, 
		{_NS_rtf, _LN_nofwords}, 
		{_NS_text, _LN_printed_by}, 
		{_NS_w12, _LN_listPr}, 
		{_NS_rng, _LN_list}, 
		{_NS_rtf, _LN_annotation}, 
		{_NS_text, _LN_title}, 
		{_NS_fo, _LN_orphans}, 
		{_NS_v11, _LN_ExtraInfo}, 
		{_NS_a12, _LN_lin}, 
		{_NS_prj12, _LN_ActualCost}, 
		{_NS_rtf, _LN_saftnnrlc}, 
		{_NS_rtf, _LN_nolead}, 
		{_NS_style, _LN_footnote_sep}, 
		{_NS_text, _LN_subject}, 
		{_NS_a12, _LN_path}, 
		{_NS_rtf, _LN_annotprot}, 
		{_NS_v11, _LN_Frame}, 
		{_NS_style, _LN_width}, 
		{_NS_table, _LN_named_expression}, 
		{_NS_style, _LN_tab_stops}, 
		{_NS_w12, _LN_supressLineNumbers}, 
		{_NS_rtf, _LN_noline}, 
		{_NS_style, _LN_color}, 
		{_NS_rtf, _LN_saftnnruc}, 
		{_NS_rtf, _LN_ansi}, 
		{_NS_text, _LN_keywords}, 
		{_NS_table, _LN_expression}, 
		{_NS_style, _LN_tab_stop}, 
		{_NS_a12, _LN_tileRect}, 
		{_NS_prj12, _LN_ActualOvertimeCost}, 
		{_NS_v11, _LN_NewWindow}, 
		{_NS_w12, _LN_pBdr}, 
		{_NS_text, _LN_editing_cycles}, 
		{_NS_style, _LN_char}, 
		{_NS_v11, _LN_Default}, 
		{_NS_rtf, _LN_ansicpg}, 
		{_NS_text, _LN_editing_duration}, 
		{_NS_style, _LN_line_style}, 
		{_NS_rtf, _LN_nolnhtadjtbl}, 
		{_NS_a12, _LN_fillRect}, 
		{_NS_w12, _LN_shd}, 
		{_NS_rtf, _LN_saftnnzodiac}, 
		{_NS_prj12, _LN_ActualWork}, 
		{_NS_table, _LN_database_ranges}, 
		{_NS_rtf, _LN_aoutl}, 
		{_NS_v11, _LN_Initials}, 
		{_NS_w12, _LN_tabs}, 
		{_NS_text, _LN_duration}, 
		{_NS_style, _LN_leader_type}, 
		{_NS_style, _LN_adjustment}, 
		{_NS_rtf, _LN_nonesttables}, 
		{_NS_a12, _LN_alphaBiLevel}, 
		{_NS_rtf, _LN_saftnnzodiacd}, 
		{_NS_table, _LN_database_range}, 
		{_NS_prj12, _LN_ActualOvertimeWork}, 
		{_NS_v11, _LN_ReviewerID}, 
		{_NS_rtf, _LN_ApplyBrkRules}, 
		{_NS_style, _LN_leader_style}, 
		{_NS_rtf, _LN_nonshppict}, 
		{_NS_text, _LN_modification_time}, 
		{_NS_w12, _LN_suppressAutoHyphens}, 
		{_NS_a12, _LN_alphaCeiling}, 
		{_NS_rtf, _LN_saftnnzodiacl}, 
		{_NS_style, _LN_distance_before_sep}, 
		{_NS_rtf, _LN_ascaps}, 
		{_NS_table, _LN_is_selection}, 
		{_NS_prj12, _LN_RegularWork}, 
		{_NS_v11, _LN_CurrentIndex}, 
		{_NS_style, _LN_leader_width}, 
		{_NS_w12, _LN_kinsoku}, 
		{_NS_rtf, _LN_nooverflow}, 
		{_NS_a12, _LN_alphaFloor}, 
		{_NS_rtf, _LN_ashad}, 
		{_NS_rtf, _LN_saftnrestart}, 
		{_NS_text, _LN_modification_date}, 
		{_NS_style, _LN_distance_after_sep}, 
		{_NS_v11, _LN_MarkerIndex}, 
		{_NS_rtf, _LN_noproof}, 
		{_NS_text, _LN_creator}, 
		{_NS_w12, _LN_wordWrap}, 
		{_NS_prj12, _LN_RemainingDuration}, 
		{_NS_a12, _LN_alphaInv}, 
		{_NS_style, _LN_leader_color}, 
		{_NS_table, _LN_on_update_keep_styles}, 
		{_NS_rng, _LN_name}, 
		{_NS_rtf, _LN_asianbrkrule}, 
		{_NS_v11, _LN_Date}, 
		{_NS_rtf, _LN_saftnrstcont}, 
		{_NS_rtf, _LN_nosectexpand}, 
		{_NS_w12, _LN_overflowPunct}, 
		{_NS_style, _LN_layout_grid_mode}, 
		{_NS_a12, _LN_alphaModFix}, 
		{_NS_style, _LN_leader_text}, 
		{_NS_prj12, _LN_RemainingCost}, 
		{_NS_rtf, _LN_aspalpha}, 
		{_NS_v11, _LN_XJustify}, 
		{_NS_text, _LN_database_name}, 
		{_NS_rtf, _LN_saftnstart}, 
		{_NS_table, _LN_on_update_keep_size}, 
		{_NS_rtf, _LN_aspnum}, 
		{_NS_a12, _LN_alphaRepl}, 
		{_NS_w12, _LN_topLinePunct}, 
		{_NS_rtf, _LN_nosnaplinegrid}, 
		{_NS_v11, _LN_YJustify}, 
		{_NS_prj12, _LN_RemainingWork}, 
		{_NS_rtf, _LN_sautoupd}, 
		{_NS_text, _LN_table_name}, 
		{_NS_style, _LN_leader_text_style}, 
		{_NS_v11, _LN_Title}, 
		{_NS_style, _LN_layout_grid_base_height}, 
		{_NS_rtf, _LN_astrike}, 
		{_NS_a12, _LN_biLevel}, 
		{_NS_rtf, _LN_sb}, 
		{_NS_table, _LN_has_persistent_data}, 
		{_NS_v11, _LN_Subject}, 
		{_NS_w12, _LN_autoSpaceDE}, 
		{_NS_rtf, _LN_nospaceforul}, 
		{_NS_text, _LN_table_type}, 
		{_NS_v11, _LN_Creator}, 
		{_NS_a12, _LN_blur}, 
		{_NS_rtf, _LN_atnauthor}, 
		{_NS_style, _LN_tab_stop_distance}, 
		{_NS_rtf, _LN_sbasedon}, 
		{_NS_prj12, _LN_RemainingOvertimeCost}, 
		{_NS_w12, _LN_autoSpaceDN}, 
		{_NS_rtf, _LN_nosupersub}, 
		{_NS_table, _LN_contains_header}, 
		{_NS_a12, _LN_clrBal}, 
		{_NS_v11, _LN_Manager}, 
		{_NS_style, _LN_layout_grid_ruby_height}, 
		{_NS_rtf, _LN_atndate}, 
		{_NS_rtf, _LN_sbauto}, 
		{_NS_text, _LN_database_display}, 
		{_NS_w12, _LN_bidi}, 
		{_NS_rtf, _LN_notabind}, 
		{_NS_fo, _LN_hyphenation_keep}, 
		{_NS_v11, _LN_Company}, 
		{_NS_a12, _LN_clrChange}, 
		{_NS_rtf, _LN_atnicn}, 
		{_NS_rtf, _LN_sbkcol}, 
		{_NS_prj12, _LN_RemainingOvertimeWork}, 
		{_NS_v11, _LN_Category}, 
		{_NS_text, _LN_column_name}, 
		{_NS_style, _LN_layout_grid_lines}, 
		{_NS_table, _LN_display_filter_buttons}, 
		{_NS_v11, _LN_Keywords}, 
		{_NS_w12, _LN_adjustRightInd}, 
		{_NS_rtf, _LN_atnid}, 
		{_NS_rtf, _LN_noultrlspc}, 
		{_NS_a12, _LN_clrRepl}, 
		{_NS_prj12, _LN_ACWP}, 
		{_NS_rtf, _LN_sbkeven}, 
		{_NS_v11, _LN_Desc}, 
		{_NS_prj12, _LN_CV}, 
		{_NS_fo, _LN_hyphenation_ladder_count}, 
		{_NS_text, _LN_database_next}, 
		{_NS_a12, _LN_duotone}, 
		{_NS_rtf, _LN_atnparent}, 
		{_NS_w12, _LN_snapToGrid}, 
		{_NS_rtf, _LN_sbknone}, 
		{_NS_style, _LN_layout_grid_color}, 
		{_NS_rtf, _LN_nowidctlpar}, 
		{_NS_table, _LN_target_range_address}, 
		{_NS_style, _LN_drop_cap}, 
		{_NS_rtf, _LN_atnref}, 
		{_NS_v11, _LN_HyperlinkBase}, 
		{_NS_rtf, _LN_sbkodd}, 
		{_NS_prj12, _LN_ConstraintType}, 
		{_NS_w12, _LN_spacing}, 
		{_NS_rtf, _LN_nowrap}, 
		{_NS_a12, _LN_fillOverlay}, 
		{_NS_text, _LN_database_row_select}, 
		{_NS_style, _LN_length}, 
		{_NS_w12, _LN_ind}, 
		{_NS_rtf, _LN_atntime}, 
		{_NS_rtf, _LN_sbkpage}, 
		{_NS_style, _LN_layout_grid_ruby_below}, 
		{_NS_rtf, _LN_nowwrap}, 
		{_NS_a12, _LN_grayscl}, 
		{_NS_table, _LN_database_source_sql}, 
		{_NS_v11, _LN_AlternateNames}, 
		{_NS_prj12, _LN_ConstraintDate}, 
		{_NS_style, _LN_lines}, 
		{_NS_text, _LN_row_number}, 
		{_NS_table, _LN_database_name}, 
		{_NS_rtf, _LN_sbys}, 
		{_NS_a12, _LN_hsl}, 
		{_NS_rtf, _LN_atrfend}, 
		{_NS_style, _LN_distance}, 
		{_NS_rtf, _LN_noxlattoyen}, 
		{_NS_a12, _LN_luminance}, 
		{_NS_w12, _LN_contextualSpacing}, 
		{_NS_v11, _LN_Template}, 
		{_NS_prj12, _LN_Deadline}, 
		{_NS_style, _LN_layout_grid_print}, 
		{_NS_style, _LN_style_name}, 
		{_NS_rtf, _LN_scaps}, 
		{_NS_rtf, _LN_atrfstart}, 
		{_NS_table, _LN_sql_statement}, 
		{_NS_rtf, _LN_objalias}, 
		{_NS_a12, _LN_sharpen}, 
		{_NS_text, _LN_database_row_number}, 
		{_NS_rtf, _LN_scompose}, 
		{_NS_w12, _LN_mirrorIndents}, 
		{_NS_rtf, _LN_aul}, 
		{_NS_text, _LN_value}, 
		{_NS_style, _LN_register_true}, 
		{_NS_prj12, _LN_LevelAssignments}, 
		{_NS_v11, _LN_BuildNumberCreated}, 
		{_NS_a12, _LN_blip}, 
		{_NS_style, _LN_layout_grid_display}, 
		{_NS_rtf, _LN_objalign}, 
		{_NS_rtf, _LN_sec}, 
		{_NS_rtf, _LN_auld}, 
		{_NS_table, _LN_parse_sql_statement}, 
		{_NS_rtf, _LN_sect}, 
		{_NS_a12, _LN_srcRect}, 
		{_NS_fo, _LN_margin_left}, 
		{_NS_w12, _LN_textboxTightWrap}, 
		{_NS_rtf, _LN_auldb}, 
		{_NS_rtf, _LN_objattph}, 
		{_NS_text, _LN_page_variable_set}, 
		{_NS_prj12, _LN_LevelingCanSplit}, 
		{_NS_v11, _LN_BuildNumberEdited}, 
		{_NS_rtf, _LN_sectd}, 
		{_NS_a12, _LN_tile}, 
		{_NS_style, _LN_header_footer_properties}, 
		{_NS_rtf, _LN_aulnone}, 
		{_NS_text, _LN_active}, 
		{_NS_fo, _LN_margin_right}, 
		{_NS_rtf, _LN_objautlink}, 
		{_NS_table, _LN_database_source_table}, 
		{_NS_w12, _LN_suppressOverlap}, 
		{_NS_a12, _LN_stretch}, 
		{_NS_prj12, _LN_LevelingDelay}, 
		{_NS_v11, _LN_PreviewPicture}, 
		{_NS_rtf, _LN_sectdefaultcl}, 
		{_NS_rtf, _LN_aulw}, 
		{_NS_w12, _LN_jc}, 
		{_NS_rtf, _LN_objclass}, 
		{_NS_fo, _LN_text_indent}, 
		{_NS_a12, _LN_fgClr}, 
		{_NS_style, _LN_dynamic_spacing}, 
		{_NS_rtf, _LN_aup}, 
		{_NS_text, _LN_page_variable_get}, 
		{_NS_rtf, _LN_sectexpand}, 
		{_NS_v11, _LN_CustomProps}, 
		{_NS_table, _LN_database_table_name}, 
		{_NS_w12, _LN_textDirection}, 
		{_NS_a12, _LN_bgClr}, 
		{_NS_prj12, _LN_LevelingDelayFormat}, 
		{_NS_rtf, _LN_objcropb}, 
		{_NS_rtf, _LN_author}, 
		{_NS_style, _LN_auto_text_indent}, 
		{_NS_text, _LN_placeholder}, 
		{_NS_style, _LN_text_properties}, 
		{_NS_rtf, _LN_b}, 
		{_NS_a12, _LN_noFill}, 
		{_NS_v11, _LN_TimeCreated}, 
		{_NS_rtf, _LN_sectlinegrid}, 
		{_NS_rtf, _LN_objcropl}, 
		{_NS_fo, _LN_font_variant}, 
		{_NS_rtf, _LN_background}, 
		{_NS_w12, _LN_textAlignment}, 
		{_NS_prj12, _LN_PreLeveledStart}, 
		{_NS_table, _LN_database_source_query}, 
		{_NS_fo, _LN_margin_top}, 
		{_NS_rtf, _LN_sectnum}, 
		{_NS_a12, _LN_solidFill}, 
		{_NS_v11, _LN_TimeSaved}, 
		{_NS_rtf, _LN_objcropr}, 
		{_NS_rtf, _LN_bdbfhdr}, 
		{_NS_text, _LN_placeholder_type}, 
		{_NS_w12, _LN_outlineLvl}, 
		{_NS_fo, _LN_text_transform}, 
		{_NS_table, _LN_query_name}, 
		{_NS_rtf, _LN_sectrsid}, 
		{_NS_prj12, _LN_PreLeveledFinish}, 
		{_NS_a12, _LN_gradFill}, 
		{_NS_fo, _LN_color}, 
		{_NS_fo, _LN_margin_bottom}, 
		{_NS_rtf, _LN_objcropt}, 
		{_NS_v11, _LN_TimeEdited}, 
		{_NS_w12, _LN_divId}, 
		{_NS_rtf, _LN_bdrrlswsix}, 
		{_NS_prj12, _LN_Hyperlink}, 
		{_NS_table, _LN_sort}, 
		{_NS_text, _LN_conditional_text}, 
		{_NS_fo, _LN_margin}, 
		{_NS_a12, _LN_blipFill}, 
		{_NS_rtf, _LN_objdata}, 
		{_NS_rtf, _LN_bgbdiag}, 
		{_NS_rtf, _LN_sectspecifycl}, 
		{_NS_w12, _LN_cnfStyle}, 
		{_NS_v11, _LN_TimePrinted}, 
		{_NS_rtf, _LN_object}, 
		{_NS_w12, _LN_rPr}, 
		{_NS_style, _LN_use_window_font_color}, 
		{_NS_prj12, _LN_HyperlinkAddress}, 
		{_NS_a12, _LN_pattFill}, 
		{_NS_rtf, _LN_bgcross}, 
		{_NS_fo, _LN_break_before}, 
		{_NS_table, _LN_bind_styles_to_content}, 
		{_NS_text, _LN_string_value_if_true}, 
		{_NS_rtf, _LN_objemb}, 
		{_NS_v11, _LN_CustomProp}, 
		{_NS_rtf, _LN_sectspecifygen}, 
		{_NS_w12, _LN_sectPr}, 
		{_NS_table, _LN_language}, 
		{_NS_a12, _LN_grpFill}, 
		{_NS_style, _LN_text_outline}, 
		{_NS_rtf, _LN_bgdcross}, 
		{_NS_rtf, _LN_objh}, 
		{_NS_table, _LN_country}, 
		{_NS_fo, _LN_break_after}, 
		{_NS_prj12, _LN_HyperlinkSubAddress}, 
		{_NS_v11, _LN_GlueSettings}, 
		{_NS_a12, _LN_cont}, 
		{_NS_rtf, _LN_sectspecifyl}, 
		{_NS_rtf, _LN_objhtml}, 
		{_NS_text, _LN_string_value_if_false}, 
		{_NS_a12, _LN_effect}, 
		{_NS_rtf, _LN_bgdkbdiag}, 
		{_NS_table, _LN_algorithm}, 
		{_NS_style, _LN_text_line_through_type}, 
		{_NS_fo, _LN_background_color}, 
		{_NS_v11, _LN_SnapSettings}, 
		{_NS_w12, _LN_movie}, 
		{_NS_rtf, _LN_sectunlocked}, 
		{_NS_rtf, _LN_objicemb}, 
		{_NS_table, _LN_sort_by}, 
		{_NS_rtf, _LN_bgdkcross}, 
		{_NS_a12, _LN_alphaOutset}, 
		{_NS_prj12, _LN_IgnoreResourceCalendar}, 
		{_NS_text, _LN_current_value}, 
		{_NS_rtf, _LN_sftnbj}, 
		{_NS_w12, _LN_background}, 
		{_NS_rtf, _LN_objlink}, 
		{_NS_a12, _LN_blend}, 
		{_NS_prj12, _LN_Notes}, 
		{_NS_v11, _LN_SnapExtensions}, 
		{_NS_style, _LN_background_image}, 
		{_NS_rtf, _LN_bgdkdcross}, 
		{_NS_table, _LN_field_number}, 
		{_NS_w12, _LN_applet}, 
		{_NS_text, _LN_hidden_text}, 
		{_NS_a12, _LN_fill}, 
		{_NS_style, _LN_text_line_through_style}, 
		{_NS_rtf, _LN_sftnnalc}, 
		{_NS_rtf, _LN_objlock}, 
		{_NS_prj12, _LN_HideBar}, 
		{_NS_style, _LN_repeat}, 
		{_NS_v11, _LN_SnapAngles}, 
		{_NS_a12, _LN_glow}, 
		{_NS_rtf, _LN_bgdkfdiag}, 
		{_NS_table, _LN_data_type}, 
		{_NS_text, _LN_is_hidden}, 
		{_NS_style, _LN_filter_name}, 
		{_NS_w12, _LN_msAccessHTML}, 
		{_NS_rtf, _LN_sftnnar}, 
		{_NS_rtf, _LN_objname}, 
		{_NS_prj12, _LN_Rollup}, 
		{_NS_fo, _LN_border}, 
		{_NS_w12, _LN_anchor}, 
		{_NS_table, _LN_order}, 
		{_NS_a12, _LN_innerShdw}, 
		{_NS_rtf, _LN_bgdkhoriz}, 
		{_NS_style, _LN_text_line_through_width}, 
		{_NS_text, _LN_note_ref}, 
		{_NS_prj12, _LN_BCWS}, 
		{_NS_rtf, _LN_objocx}, 
		{_NS_rtf, _LN_sftnnauc}, 
		{_NS_v11, _LN_DynamicGridEnabled}, 
		{_NS_w12, _LN_inline}, 
		{_NS_fo, _LN_border_top}, 
		{_NS_rtf, _LN_bgdkvert}, 
		{_NS_a12, _LN_outerShdw}, 
		{_NS_prj12, _LN_BCWP}, 
		{_NS_rtf, _LN_objpub}, 
		{_NS_table, _LN_subtotal_rules}, 
		{_NS_w12, _LN_args}, 
		{_NS_rtf, _LN_sftnnchi}, 
		{_NS_text, _LN_sequence_ref}, 
		{_NS_v11, _LN_ProtectStyles}, 
		{_NS_rtf, _LN_bgfdiag}, 
		{_NS_style, _LN_text_line_through_color}, 
		{_NS_a12, _LN_prstShdw}, 
		{_NS_fo, _LN_border_bottom}, 
		{_NS_rtf, _LN_objscalex}, 
		{_NS_w12, _LN_appletText}, 
		{_NS_rtf, _LN_sftnnchosung}, 
		{_NS_text, _LN_reference_format}, 
		{_NS_rtf, _LN_bghoriz}, 
		{_NS_prj12, _LN_PhysicalPercentComplete}, 
		{_NS_v11, _LN_ProtectShapes}, 
		{_NS_a12, _LN_reflection}, 
		{_NS_fo, _LN_border_left}, 
		{_NS_rtf, _LN_objscaley}, 
		{_NS_text, _LN_script}, 
		{_NS_table, _LN_page_breaks_on_group_change}, 
		{_NS_w12, _LN_fldData}, 
		{_NS_rtf, _LN_sftnncnum}, 
		{_NS_rtf, _LN_bgvert}, 
		{_NS_style, _LN_text_line_through_text}, 
		{_NS_w12, _LN_r}, 
		{_NS_a12, _LN_relOff}, 
		{_NS_rtf, _LN_bin}, 
		{_NS_rtf, _LN_objsect}, 
		{_NS_prj12, _LN_PredecessorLink}, 
		{_NS_v11, _LN_ProtectMasters}, 
		{_NS_fo, _LN_border_right}, 
		{_NS_table, _LN_sort_groups}, 
		{_NS_text, _LN_execute_macro}, 
		{_NS_rtf, _LN_sftnndbar}, 
		{_NS_a12, _LN_sftEdg}, 
		{_NS_w12, _LN_fldSimple}, 
		{_NS_rtf, _LN_binfsxn}, 
		{_NS_rtf, _LN_objsetsize}, 
		{_NS_v11, _LN_ProtectBkgnds}, 
		{_NS_prj12, _LN_PredecessorUID}, 
		{_NS_table, _LN_subtotal_rule}, 
		{_NS_rtf, _LN_sftnndbnum}, 
		{_NS_a12, _LN_effectLst}, 
		{_NS_style, _LN_text_line_through_text_style}, 
		{_NS_style, _LN_border_line_width}, 
		{_NS_w12, _LN_hyperlink}, 
		{_NS_rtf, _LN_binsxn}, 
		{_NS_text, _LN_hidden_paragraph}, 
		{_NS_rtf, _LN_objsub}, 
		{_NS_w12, _LN_ffData}, 
		{_NS_prj12, _LN_CrossProject}, 
		{_NS_rtf, _LN_sftnndbnumd}, 
		{_NS_a12, _LN_effectDag}, 
		{_NS_rtf, _LN_bkmkcolf}, 
		{_NS_v11, _LN_CustomMenusFile}, 
		{_NS_rtf, _LN_objtime}, 
		{_NS_style, _LN_text_position}, 
		{_NS_w12, _LN_name}, 
		{_NS_text, _LN_dde_connection}, 
		{_NS_table, _LN_group_by_field_number}, 
		{_NS_style, _LN_border_line_width_top}, 
		{_NS_w12, _LN_trackRevisions}, 
		{_NS_w12, _LN_enabled}, 
		{_NS_rtf, _LN_bkmkcoll}, 
		{_NS_rtf, _LN_sftnndbnumk}, 
		{_NS_rtf, _LN_objtransy}, 
		{_NS_style, _LN_font_name}, 
		{_NS_prj12, _LN_CrossProjectName}, 
		{_NS_v11, _LN_CustomToolbarsFile}, 
		{_NS_text, _LN_connection_name}, 
		{_NS_table, _LN_subtotal_field}, 
		{_NS_w12, _LN_trackMoves}, 
		{_NS_w12, _LN_calcOnExit}, 
		{_NS_rtf, _LN_bkmkend}, 
		{_NS_rtf, _LN_objupdate}, 
		{_NS_prj12, _LN_LinkLag}, 
		{_NS_rtf, _LN_sftnndbnumt}, 
		{_NS_style, _LN_border_line_width_bottom}, 
		{_NS_style, _LN_font_name_asian}, 
		{_NS_text, _LN_measure}, 
		{_NS_table, _LN_function}, 
		{_NS_rtf, _LN_bkmkpub}, 
		{_NS_w12, _LN_entryMacro}, 
		{_NS_rtf, _LN_objw}, 
		{_NS_v11, _LN_AttachedToolbars}, 
		{_NS_w12, _LN_trackFormatting}, 
		{_NS_prj12, _LN_LagFormat}, 
		{_NS_rtf, _LN_sftnnganada}, 
		{_NS_text, _LN_kind}, 
		{_NS_table, _LN_filter}, 
		{_NS_rtf, _LN_oldas}, 
		{_NS_rtf, _LN_bkmkstart}, 
		{_NS_w12, _LN_exitMacro}, 
		{_NS_v11, _LN_SnapAngle}, 
		{_NS_style, _LN_font_name_complex}, 
		{_NS_style, _LN_border_line_width_left}, 
		{_NS_rtf, _LN_sftnngbnum}, 
		{_NS_fo, _LN_font_family}, 
		{_NS_text, _LN_table_formula}, 
		{_NS_w12, _LN_documentProtection}, 
		{_NS_rtf, _LN_bliptag}, 
		{_NS_rtf, _LN_oldcprops}, 
		{_NS_prj12, _LN_ActualWorkProtected}, 
		{_NS_w12, _LN_helpText}, 
		{_NS_table, _LN_condition_source}, 
		{_NS_office, _LN_value_type}, 
		{_NS_v11, _LN_ColorEntry}, 
		{_NS_rtf, _LN_sftnngbnumd}, 
		{_NS_rtf, _LN_blipuid}, 
		{_NS_rtf, _LN_oldpprops}, 
		{_NS_w12, _LN_statusText}, 
		{_NS_style, _LN_font_family_asian}, 
		{_NS_office, _LN_currency}, 
		{_NS_style, _LN_border_line_width_right}, 
		{_NS_v11, _LN_PaperSize}, 
		{_NS_w12, _LN_autoFormatOverride}, 
		{_NS_rtf, _LN_blipupi}, 
		{_NS_rtf, _LN_sftnngbnumk}, 
		{_NS_rtf, _LN_oldsprops}, 
		{_NS_text, _LN_fixed}, 
		{_NS_prj12, _LN_ActualOvertimeWorkProtected}, 
		{_NS_w12, _LN_checkBox}, 
		{_NS_rtf, _LN_blue}, 
		{_NS_fo, _LN_padding}, 
		{_NS_v11, _LN_PaperWidth}, 
		{_NS_table, _LN_condition_source_range_address}, 
		{_NS_a12, _LN_gd}, 
		{_NS_w12, _LN_styleLockTheme}, 
		{_NS_style, _LN_font_family_complex}, 
		{_NS_text, _LN_formula}, 
		{_NS_w12, _LN_ddList}, 
		{_NS_rtf, _LN_oldtprops}, 
		{_NS_prj12, _LN_Baseline}, 
		{_NS_rtf, _LN_sftnngbnuml}, 
		{_NS_rtf, _LN_bookfold}, 
		{_NS_a12, _LN_pos}, 
		{_NS_fo, _LN_padding_top}, 
		{_NS_v11, _LN_PaperHeight}, 
		{_NS_table, _LN_display_duplicates}, 
		{_NS_w12, _LN_textInput}, 
		{_NS_rtf, _LN_sftnnrlc}, 
		{_NS_a12, _LN_ahXY}, 
		{_NS_w12, _LN_styleLockQFSet}, 
		{_NS_rtf, _LN_oldlinewrap}, 
		{_NS_rtf, _LN_bookfoldrev}, 
		{_NS_style, _LN_data_style_name}, 
		{_NS_prj12, _LN_TimephasedData}, 
		{_NS_w12, _LN_size}, 
		{_NS_rtf, _LN_operator}, 
		{_NS_fo, _LN_padding_bottom}, 
		{_NS_style, _LN_font_family_generic_asian}, 
		{_NS_v11, _LN_PrintCenteredH}, 
		{_NS_a12, _LN_ahPolar}, 
		{_NS_rtf, _LN_sftnnruc}, 
		{_NS_table, _LN_filter_and}, 
		{_NS_prj12, _LN_Number}, 
		{_NS_w12, _LN_defaultTabStop}, 
		{_NS_w12, _LN_sizeAuto}, 
		{_NS_rtf, _LN_otblrul}, 
		{_NS_rtf, _LN_bookfoldsheets}, 
		{_NS_text, _LN_toc_mark_start}, 
		{_NS_a12, _LN_cxn}, 
		{_NS_fo, _LN_padding_left}, 
		{_NS_w12, _LN_default}, 
		{_NS_rtf, _LN_box}, 
		{_NS_table, _LN_filter_or}, 
		{_NS_a12, _LN_rect}, 
		{_NS_prj12, _LN_Interim}, 
		{_NS_rtf, _LN_sftnnzodiac}, 
		{_NS_v11, _LN_PrintCenteredV}, 
		{_NS_rtf, _LN_outl}, 
		{_NS_a12, _LN_pt}, 
		{_NS_w12, _LN_autoHyphenation}, 
		{_NS_style, _LN_font_family_generic_complex}, 
		{_NS_text, _LN_toc_mark_end}, 
		{_NS_w12, _LN_listEntry}, 
		{_NS_rtf, _LN_brdrart}, 
		{_NS_fo, _LN_padding_right}, 
		{_NS_rtf, _LN_sftnnzodiacd}, 
		{_NS_w12, _LN_type}, 
		{_NS_rtf, _LN_outlinelevel}, 
		{_NS_table, _LN_filter_condition}, 
		{_NS_a12, _LN_close}, 
		{_NS_v11, _LN_PrintFitOnPages}, 
		{_NS_prj12, _LN_EstimatedDuration}, 
		{_NS_rtf, _LN_brdrb}, 
		{_NS_text, _LN_toc_mark}, 
		{_NS_table, _LN_value}, 
		{_NS_style, _LN_shadow}, 
		{_NS_style, _LN_font_style_name}, 
		{_NS_w12, _LN_maxLengh}, 
		{_NS_rtf, _LN_overlay}, 
		{_NS_a12, _LN_moveTo}, 
		{_NS_w12, _LN_consecutiveHyphenLimit}, 
		{_NS_rtf, _LN_sftnnzodiacl}, 
		{_NS_prj12, _LN_Resources}, 
		{_NS_rtf, _LN_brdrbar}, 
		{_NS_rtf, _LN_page}, 
		{_NS_v11, _LN_PrintLandscape}, 
		{_NS_table, _LN_operator}, 
		{_NS_w12, _LN_format}, 
		{_NS_a12, _LN_lineTo}, 
		{_NS_fo, _LN_keep_with_next}, 
		{_NS_text, _LN_user_index_mark_start}, 
		{_NS_prj12, _LN_Resource}, 
		{_NS_rtf, _LN_brdrbtw}, 
		{_NS_rtf, _LN_sftnrestart}, 
		{_NS_rtf, _LN_pagebb}, 
		{_NS_style, _LN_font_style_name_asian}, 
		{_NS_w12, _LN_hyphenationZone}, 
		{_NS_w12, _LN_col}, 
		{_NS_a12, _LN_arcTo}, 
		{_NS_prj12, _LN_Initials}, 
		{_NS_v11, _LN_PrintPagesAcross}, 
		{_NS_table, _LN_data_pilot_tables}, 
		{_NS_text, _LN_line_number}, 
		{_NS_rtf, _LN_brdrcf}, 
		{_NS_rtf, _LN_panose}, 
		{_NS_w12, _LN_hdr}, 
		{_NS_rtf, _LN_sftnrstcont}, 
		{_NS_a12, _LN_quadBezTo}, 
		{_NS_prj12, _LN_Phonetics}, 
		{_NS_text, _LN_user_index_mark_end}, 
		{_NS_w12, _LN_ftr}, 
		{_NS_rtf, _LN_paperh}, 
		{_NS_w12, _LN_doNotHyphenateCaps}, 
		{_NS_rtf, _LN_brdrdash}, 
		{_NS_v11, _LN_PrintPagesDown}, 
		{_NS_style, _LN_font_style_name_complex}, 
		{_NS_style, _LN_text_autospace}, 
		{_NS_rtf, _LN_sftnrstpg}, 
		{_NS_table, _LN_data_pilot_table}, 
		{_NS_prj12, _LN_NTAccount}, 
		{_NS_a12, _LN_cubicBezTo}, 
		{_NS_rtf, _LN_paperw}, 
		{_NS_rtf, _LN_brdrdashd}, 
		{_NS_text, _LN_user_index_mark}, 
		{_NS_w12, _LN_showEnvelope}, 
		{_NS_w12, _LN_headerReference}, 
		{_NS_v11, _LN_PrintScale}, 
		{_NS_rtf, _LN_par}, 
		{_NS_rtf, _LN_sftnstart}, 
		{_NS_a12, _LN_avLst}, 
		{_NS_style, _LN_font_pitch_asian}, 
		{_NS_style, _LN_punctuation_wrap}, 
		{_NS_table, _LN_application_data}, 
		{_NS_prj12, _LN_MaterialLabel}, 
		{_NS_rtf, _LN_brdrdashdd}, 
		{_NS_text, _LN_index_name}, 
		{_NS_style, _LN_line_break}, 
		{_NS_rtf, _LN_pararsid}, 
		{_NS_v11, _LN_FontEntry}, 
		{_NS_rtf, _LN_sftntj}, 
		{_NS_a12, _LN_gdLst}, 
		{_NS_w12, _LN_summaryLength}, 
		{_NS_w12, _LN_footerReference}, 
		{_NS_prj12, _LN_Code}, 
		{_NS_rtf, _LN_pard}, 
		{_NS_table, _LN_grand_total}, 
		{_NS_rtf, _LN_shad}, 
		{_NS_a12, _LN_ahLst}, 
		{_NS_prj12, _LN_Group}, 
		{_NS_style, _LN_font_pitch_complex}, 
		{_NS_v11, _LN_FaceName}, 
		{_NS_rtf, _LN_brdrdashdotstr}, 
		{_NS_style, _LN_vertical_align}, 
		{_NS_rtf, _LN_pc}, 
		{_NS_w12, _LN_footnotePr}, 
		{_NS_w12, _LN_clickAndTypeStyle}, 
		{_NS_rtf, _LN_shading}, 
		{_NS_a12, _LN_cxnLst}, 
		{_NS_prj12, _LN_WorkGroup}, 
		{_NS_rtf, _LN_pca}, 
		{_NS_text, _LN_alphabetical_index_mark_start}, 
		{_NS_v11, _LN_StyleSheet}, 
		{_NS_table, _LN_ignore_empty_rows}, 
		{_NS_rtf, _LN_brdrdashsm}, 
		{_NS_w12, _LN_endnotePr}, 
		{_NS_style, _LN_writing_mode}, 
		{_NS_rtf, _LN_shidden}, 
		{_NS_a12, _LN_rectLst}, 
		{_NS_v11, _LN_Master}, 
		{_NS_rtf, _LN_pgbrdrb}, 
		{_NS_prj12, _LN_EmailAddress}, 
		{_NS_rtf, _LN_brdrdb}, 
		{_NS_w12, _LN_defaultTableStyle}, 
		{_NS_w12, _LN_pgSz}, 
		{_NS_rtf, _LN_shift}, 
		{_NS_a12, _LN_pathLst}, 
		{_NS_table, _LN_identify_categories}, 
		{_NS_w12, _LN_pgMar}, 
		{_NS_rtf, _LN_brdrdot}, 
		{_NS_rtf, _LN_pgbrdrfoot}, 
		{_NS_prj12, _LN_MaxUnits}, 
		{_NS_text, _LN_alphabetical_index_mark_end}, 
		{_NS_style, _LN_writing_mode_automatic}, 
		{_NS_v11, _LN_MasterShortcut}, 
		{_NS_rtf, _LN_shpbottom}, 
		{_NS_a12, _LN_custGeom}, 
		{_NS_w12, _LN_evenAndOddHeaders}, 
		{_NS_table, _LN_buttons}, 
		{_NS_w12, _LN_paperSrc}, 
		{_NS_prj12, _LN_PeakUnits}, 
		{_NS_rtf, _LN_brdremboss}, 
		{_NS_rtf, _LN_pgbrdrhead}, 
		{_NS_v11, _LN_PageSheet}, 
		{_NS_a12, _LN_prstGeom}, 
		{_NS_rtf, _LN_shpbxcolumn}, 
		{_NS_style, _LN_snap_to_layout_grid}, 
		{_NS_text, _LN_alphabetical_index_mark}, 
		{_NS_w12, _LN_pgBorders}, 
		{_NS_rtf, _LN_pgbrdrl}, 
		{_NS_v11, _LN_Shapes}, 
		{_NS_w12, _LN_bookFoldRevPrinting}, 
		{_NS_rtf, _LN_brdrengrave}, 
		{_NS_table, _LN_show_filter_button}, 
		{_NS_style, _LN_page_number}, 
		{_NS_prj12, _LN_AvailableFrom}, 
		{_NS_text, _LN_key1}, 
		{_NS_rtf, _LN_shpbxignore}, 
		{_NS_v11, _LN_Icon}, 
		{_NS_w12, _LN_lnNumType}, 
		{_NS_rtf, _LN_pgbrdropt}, 
		{_NS_rtf, _LN_brdrframe}, 
		{_NS_text, _LN_key2}, 
		{_NS_prj12, _LN_AvailableTo}, 
		{_NS_w12, _LN_bookFoldPrinting}, 
		{_NS_v11, _LN_Connects}, 
		{_NS_rtf, _LN_shpbxmargin}, 
		{_NS_rtf, _LN_pgbrdrr}, 
		{_NS_w12, _LN_pgNumType}, 
		{_NS_style, _LN_background_transparency}, 
		{_NS_rtf, _LN_brdrhair}, 
		{_NS_table, _LN_drill_down_on_double_click}, 
		{_NS_prj12, _LN_CanLevel}, 
		{_NS_v11, _LN_Shape}, 
		{_NS_w12, _LN_cols}, 
		{_NS_rtf, _LN_shpbxpage}, 
		{_NS_rtf, _LN_pgbrdrsnap}, 
		{_NS_text, _LN_string_value_phonetic}, 
		{_NS_rtf, _LN_brdrinset}, 
		{_NS_w12, _LN_bookFoldPrintingSheets}, 
		{_NS_prj12, _LN_AccrueAt}, 
		{_NS_style, _LN_ruby_properties}, 
		{_NS_v11, _LN_Connect}, 
		{_NS_a12, _LN_round}, 
		{_NS_w12, _LN_formProt}, 
		{_NS_table, _LN_source_cell_range}, 
		{_NS_rtf, _LN_pgbrdrt}, 
		{_NS_rtf, _LN_shpbyignore}, 
		{_NS_rtf, _LN_brdrl}, 
		{_NS_v11, _LN_Page}, 
		{_NS_text, _LN_key1_phonetic}, 
		{_NS_a12, _LN_bevel}, 
		{_NS_w12, _LN_vAlign}, 
		{_NS_prj12, _LN_StandardRate}, 
		{_NS_style, _LN_ruby_position}, 
		{_NS_rtf, _LN_pghsxn}, 
		{_NS_rtf, _LN_brdrnil}, 
		{_NS_table, _LN_source_service}, 
		{_NS_v11, _LN_Window}, 
		{_NS_rtf, _LN_shpbymargin}, 
		{_NS_a12, _LN_miter}, 
		{_NS_w12, _LN_drawingGridHorizontalSpacing}, 
		{_NS_w12, _LN_noEndnote}, 
		{_NS_text, _LN_key2_phonetic}, 
		{_NS_rtf, _LN_pgnbidia}, 
		{_NS_style, _LN_ruby_align}, 
		{_NS_rtf, _LN_brdroutset}, 
		{_NS_prj12, _LN_StandardRateFormat}, 
		{_NS_rtf, _LN_shpbypage}, 
		{_NS_table, _LN_source_name}, 
		{_NS_a12, _LN_prstDash}, 
		{_NS_v11, _LN_StencilGroup}, 
		{_NS_w12, _LN_titlePg}, 
		{_NS_text, _LN_main_entry}, 
		{_NS_rtf, _LN_pgnbidib}, 
		{_NS_rtf, _LN_brdrr}, 
		{_NS_rtf, _LN_shpbypara}, 
		{_NS_a12, _LN_custDash}, 
		{_NS_table, _LN_object_name}, 
		{_NS_style, _LN_section_properties}, 
		{_NS_prj12, _LN_OvertimeRate}, 
		{_NS_w12, _LN_textFlow}, 
		{_NS_w12, _LN_drawingGridVerticalSpacing}, 
		{_NS_rtf, _LN_brdrs}, 
		{_NS_v11, _LN_StencilGroupPos}, 
		{_NS_rtf, _LN_pgnchosung}, 
		{_NS_a12, _LN_headEnd}, 
		{_NS_text, _LN_bibliography_mark}, 
		{_NS_style, _LN_columns}, 
		{_NS_rtf, _LN_shpfblwtxt}, 
		{_NS_table, _LN_user_name}, 
		{_NS_w12, _LN_rtlGutter}, 
		{_NS_rtf, _LN_brdrsh}, 
		{_NS_rtf, _LN_pgncnum}, 
		{_NS_v11, _LN_ShowRulers}, 
		{_NS_prj12, _LN_OvertimeRateFormat}, 
		{_NS_a12, _LN_tailEnd}, 
		{_NS_rtf, _LN_shpfhdr}, 
		{_NS_rtf, _LN_brdrt}, 
		{_NS_table, _LN_password}, 
		{_NS_w12, _LN_docGrid}, 
		{_NS_fo, _LN_column_count}, 
		{_NS_rtf, _LN_pgncont}, 
		{_NS_text, _LN_bibliography_type}, 
		{_NS_v11, _LN_ShowGrid}, 
		{_NS_w12, _LN_br}, 
		{_NS_w12, _LN_displayHorizontalDrawingGridEvery}, 
		{_NS_prj12, _LN_CostPerUse}, 
		{_NS_rtf, _LN_shpgrp}, 
		{_NS_rtf, _LN_brdrtbl}, 
		{_NS_w12, _LN_t}, 
		{_NS_fo, _LN_column_gap}, 
		{_NS_rtf, _LN_pgndbnum}, 
		{_NS_text, _LN_index_body}, 
		{_NS_table, _LN_data_pilot_field}, 
		{_NS_rtf, _LN_shpleft}, 
		{_NS_style, _LN_column}, 
		{_NS_v11, _LN_ShowPageBreaks}, 
		{_NS_rtf, _LN_brdrth}, 
		{_NS_prj12, _LN_CostVariance}, 
		{_NS_w12, _LN_delText}, 
		{_NS_rtf, _LN_pgndbnumd}, 
		{_NS_prj12, _LN_SV}, 
		{_NS_rtf, _LN_shplid}, 
		{_NS_text, _LN_index_title}, 
		{_NS_v11, _LN_ShowGuides}, 
		{_NS_rtf, _LN_brdrthtnlg}, 
		{_NS_fo, _LN_start_indent}, 
		{_NS_w12, _LN_displayVerticalDrawingGridEvery}, 
		{_NS_table, _LN_source_field_name}, 
		{_NS_w12, _LN_instrText}, 
		{_NS_rtf, _LN_pgndbnumk}, 
		{_NS_prj12, _LN_IsGeneric}, 
		{_NS_rtf, _LN_shplockanchor}, 
		{_NS_a12, _LN_ds}, 
		{_NS_rtf, _LN_brdrthtnmg}, 
		{_NS_fo, _LN_end_indent}, 
		{_NS_text, _LN_table_of_content}, 
		{_NS_w12, _LN_delInstrText}, 
		{_NS_table, _LN_selected_page}, 
		{_NS_rtf, _LN_pgndbnumt}, 
		{_NS_v11, _LN_ShowConnectionPoints}, 
		{_NS_prj12, _LN_IsInactive}, 
		{_NS_a12, _LN_noLine}, 
		{_NS_rtf, _LN_shppict}, 
		{_NS_rtf, _LN_brdrthtnsg}, 
		{_NS_w12, _LN_useMarginsForDrawingGridOrigin}, 
		{_NS_fo, _LN_space_before}, 
		{_NS_rtf, _LN_pgndec}, 
		{_NS_w12, _LN_noBreakHyphen}, 
		{_NS_rtf, _LN_shpright}, 
		{_NS_a12, _LN_solidLine}, 
		{_NS_v11, _LN_TabSplitterPos}, 
		{_NS_prj12, _LN_IsEnterprise}, 
		{_NS_text, _LN_table_of_content_source}, 
		{_NS_table, _LN_is_data_layout_field}, 
		{_NS_rtf, _LN_brdrtnthlg}, 
		{_NS_rtf, _LN_pgndecd}, 
		{_NS_fo, _LN_space_after}, 
		{_NS_w12, _LN_softHyphen}, 
		{_NS_rtf, _LN_shprslt}, 
		{_NS_a12, _LN_gradLine}, 
		{_NS_v11, _LN_EventItem}, 
		{_NS_prj12, _LN_BookingType}, 
		{_NS_table, _LN_used_hierarchy}, 
		{_NS_w12, _LN_drawingGridHorizontalOrigin}, 
		{_NS_rtf, _LN_brdrtnthmg}, 
		{_NS_rtf, _LN_pgnganada}, 
		{_NS_text, _LN_use_outline_level}, 
		{_NS_style, _LN_column_sep}, 
		{_NS_rtf, _LN_shptop}, 
		{_NS_w12, _LN_dayShort}, 
		{_NS_a12, _LN_pattLine}, 
		{_NS_style, _LN_height}, 
		{_NS_v11, _LN_HeaderMargin}, 
		{_NS_rtf, _LN_pgngbnum}, 
		{_NS_rtf, _LN_brdrtnthsg}, 
		{_NS_rtf, _LN_shptxt}, 
		{_NS_style, _LN_protect}, 
		{_NS_prj12, _LN_ActiveDirectoryGUID}, 
		{_NS_w12, _LN_monthShort}, 
		{_NS_table, _LN_data_pilot_level}, 
		{_NS_text, _LN_use_index_marks}, 
		{_NS_w12, _LN_drawingGridVerticalOrigin}, 
		{_NS_rtf, _LN_shpwrk}, 
		{_NS_rtf, _LN_pgngbnumd}, 
		{_NS_v11, _LN_FooterMargin}, 
		{_NS_rtf, _LN_brdrtnthtnlg}, 
		{_NS_w12, _LN_yearShort}, 
		{_NS_table, _LN_show_empty}, 
		{_NS_rtf, _LN_shpwr}, 
		{_NS_prj12, _LN_AvailabilityPeriods}, 
		{_NS_rtf, _LN_pgngbnumk}, 
		{_NS_text, _LN_dont_balance_text_columns}, 
		{_NS_v11, _LN_HeaderLeft}, 
		{_NS_w12, _LN_dayLong}, 
		{_NS_text, _LN_use_index_source_styles}, 
		{_NS_w12, _LN_doNotShadeFormData}, 
		{_NS_rtf, _LN_brdrtnthtnmg}, 
		{_NS_rtf, _LN_shpz}, 
		{_NS_rtf, _LN_pgngbnuml}, 
		{_NS_rtf, _LN_sl}, 
		{_NS_table, _LN_data_pilot_subtotals}, 
		{_NS_v11, _LN_HeaderCenter}, 
		{_NS_w12, _LN_monthLong}, 
		{_NS_text, _LN_index_scope}, 
		{_NS_style, _LN_table_properties}, 
		{_NS_prj12, _LN_AvailabilityPeriod}, 
		{_NS_rtf, _LN_brdrtnthtnsg}, 
		{_NS_table, _LN_align}, 
		{_NS_rtf, _LN_slmult}, 
		{_NS_w12, _LN_punctuationKerning}, 
		{_NS_rtf, _LN_pgnhindia}, 
		{_NS_w12, _LN_yearLong}, 
		{_NS_v11, _LN_HeaderRight}, 
		{_NS_table, _LN_data_pilot_subtotal}, 
		{_NS_rtf, _LN_brdrtriple}, 
		{_NS_prj12, _LN_AvailableUnits}, 
		{_NS_rtf, _LN_pgnhindib}, 
		{_NS_rtf, _LN_snaptogridincell}, 
		{_NS_text, _LN_relative_tab_stop_position}, 
		{_NS_v11, _LN_FooterLeft}, 
		{_NS_w12, _LN_annotationRef}, 
		{_NS_rtf, _LN_brdrw}, 
		{_NS_prj12, _LN_Rates}, 
		{_NS_style, _LN_may_break_between_rows}, 
		{_NS_w12, _LN_characterSpacingControl}, 
		{_NS_rtf, _LN_pgnhindic}, 
		{_NS_rtf, _LN_snext}, 
		{_NS_table, _LN_data_pilot_members}, 
		{_NS_prj12, _LN_Rate}, 
		{_NS_rtf, _LN_brdrwavy}, 
		{_NS_w12, _LN_footnoteRef}, 
		{_NS_v11, _LN_FooterCenter}, 
		{_NS_table, _LN_border_model}, 
		{_NS_w12, _LN_printTwoOnOne}, 
		{_NS_rtf, _LN_softcol}, 
		{_NS_rtf, _LN_pgnhindid}, 
		{_NS_prj12, _LN_RatesFrom}, 
		{_NS_rtf, _LN_brdrwavydb}, 
		{_NS_text, _LN_table_of_content_entry_template}, 
		{_NS_table, _LN_data_pilot_member}, 
		{_NS_w12, _LN_endnoteRef}, 
		{_NS_v11, _LN_FooterRight}, 
		{_NS_rtf, _LN_pgnhn}, 
		{_NS_rtf, _LN_softlheight}, 
		{_NS_prj12, _LN_RatesTo}, 
		{_NS_rtf, _LN_brkfrm}, 
		{_NS_style, _LN_table_column_properties}, 
		{_NS_w12, _LN_separator}, 
		{_NS_rtf, _LN_pgnhnsc}, 
		{_NS_w12, _LN_strictFirstAndLastChars}, 
		{_NS_table, _LN_show_details}, 
		{_NS_rtf, _LN_brsp}, 
		{_NS_rtf, _LN_softline}, 
		{_NS_text, _LN_illustration_index}, 
		{_NS_v11, _LN_HeaderFooterFont}, 
		{_NS_prj12, _LN_RateTable}, 
		{_NS_v11, _LN_DocumentProperties}, 
		{_NS_w12, _LN_footnote}, 
		{_NS_rtf, _LN_pgnhnsh}, 
		{_NS_style, _LN_column_width}, 
		{_NS_rtf, _LN_bullet}, 
		{_NS_rtf, _LN_softpage}, 
		{_NS_w12, _LN_noLineBreaksAfter}, 
		{_NS_a12, _LN_font}, 
		{_NS_prj12, _LN_Assignments}, 
		{_NS_w12, _LN_endnote}, 
		{_NS_rtf, _LN_pgnhnsm}, 
		{_NS_table, _LN_data_pilot_display_info}, 
		{_NS_rtfs, _LN_Symbol}, 
		{_NS_v11, _LN_DocumentSettings}, 
		{_NS_rtf, _LN_spersonal}, 
		{_NS_text, _LN_illustration_index_source}, 
		{_NS_style, _LN_rel_column_width}, 
		{_NS_prj12, _LN_Assignment}, 
		{_NS_rtf, _LN_pgnhnsn}, 
		{_NS_table, _LN_enabled}, 
		{_NS_rtf, _LN_buptim}, 
		{_NS_w12, _LN_noLineBreaksBefore}, 
		{_NS_v11, _LN_Colors}, 
		{_NS_rtf, _LN_splytwnine}, 
		{_NS_text, _LN_use_caption}, 
		{_NS_w12, _LN_continuationSeparator}, 
		{_NS_prj12, _LN_TaskUID}, 
		{_NS_rtf, _LN_bxe}, 
		{_NS_rtf, _LN_pgnhnsp}, 
		{_NS_table, _LN_data_field}, 
		{_NS_w12, _LN_sym}, 
		{_NS_rtf, _LN_caps}, 
		{_NS_v11, _LN_PrintSetup}, 
		{_NS_rtf, _LN_sprsbsp}, 
		{_NS_style, _LN_use_optimal_column_width}, 
		{_NS_w12, _LN_webPageEncoding}, 
		{_NS_rtf, _LN_pgnid}, 
		{_NS_rtf, _LN_category}, 
		{_NS_prj12, _LN_ResourceUID}, 
		{_NS_w12, _LN_pgNum}, 
		{_NS_v11, _LN_Fonts}, 
		{_NS_table, _LN_member_count}, 
		{_NS_rtf, _LN_cb}, 
		{_NS_text, _LN_caption_sequence_name}, 
		{_NS_rtf, _LN_sprslnsp}, 
		{_NS_w12, _LN_cr}, 
		{_NS_rtf, _LN_pgnlcltr}, 
		{_NS_prj12, _LN_Confirmed}, 
		{_NS_a12, _LN_txbPr}, 
		{_NS_w12, _LN_object}, 
		{_NS_rtf, _LN_cbpat}, 
		{_NS_w12, _LN_optimizeForBrowser}, 
		{_NS_style, _LN_table_row_properties}, 
		{_NS_v11, _LN_FaceNames}, 
		{_NS_rtf, _LN_sprsspbf}, 
		{_NS_rtf, _LN_pgnlcrm}, 
		{_NS_w12, _LN_pict}, 
		{_NS_rtf, _LN_cchs}, 
		{_NS_table, _LN_display_member_mode}, 
		{_NS_a12, _LN_txLstStyle}, 
		{_NS_w12, _LN_relyOnVML}, 
		{_NS_prj12, _LN_CostRateTable}, 
		{_NS_text, _LN_caption_sequence_format}, 
		{_NS_style, _LN_row_height}, 
		{_NS_rtf, _LN_cell}, 
		{_NS_rtf, _LN_sprstsm}, 
		{_NS_v11, _LN_StyleSheets}, 
		{_NS_rtf, _LN_pgnrestart}, 
		{_NS_w12, _LN_fldChar}, 
		{_NS_a12, _LN_spDef}, 
		{_NS_prj12, _LN_Delay}, 
		{_NS_rtf, _LN_cellx}, 
		{_NS_w12, _LN_allowPNG}, 
		{_NS_w12, _LN_ruby}, 
		{_NS_rtf, _LN_sprstsp}, 
		{_NS_table, _LN_data_pilot_sort_info}, 
		{_NS_rtf, _LN_pgnstart}, 
		{_NS_style, _LN_min_row_height}, 
		{_NS_a12, _LN_lnDef}, 
		{_NS_v11, _LN_DocumentSheet}, 
		{_NS_rtf, _LN_cf}, 
		{_NS_rtf, _LN_spv}, 
		{_NS_w12, _LN_doNotRelyOnCSS}, 
		{_NS_prj12, _LN_HasFixedRateUnits}, 
		{_NS_rtf, _LN_cfpat}, 
		{_NS_table, _LN_sort_mode}, 
		{_NS_a12, _LN_txbxDef}, 
		{_NS_v11, _LN_Masters}, 
		{_NS_rtf, _LN_pgnstarts}, 
		{_NS_text, _LN_illustration_index_entry_template}, 
		{_NS_w12, _LN_footnoteReference}, 
		{_NS_rtf, _LN_sreply}, 
		{_NS_rtf, _LN_cgrid}, 
		{_NS_style, _LN_use_optimal_row_height}, 
		{_NS_v11, _LN_Pages}, 
		{_NS_rtf, _LN_pgnthaia}, 
		{_NS_prj12, _LN_FixedMaterial}, 
		{_NS_text, _LN_table_index}, 
		{_NS_rtf, _LN_ssemihidden}, 
		{_NS_w12, _LN_endnoteReference}, 
		{_NS_v11, _LN_Windows}, 
		{_NS_rtf, _LN_charrsid}, 
		{_NS_table, _LN_data_pilot_layout_info}, 
		{_NS_rtf, _LN_pgnthaib}, 
		{_NS_w12, _LN_doNotSaveWebPagesAsSingleFile}, 
		{_NS_w12, _LN_oArt}, 
		{_NS_x11, _LN_ParameterValue}, 
		{_NS_prj12, _LN_LinkedFields}, 
		{_NS_rtf, _LN_staticval}, 
		{_NS_style, _LN_table_cell_properties}, 
		{_NS_text, _LN_table_index_source}, 
		{_NS_v11, _LN_EventList}, 
		{_NS_rtf, _LN_charscalex}, 
		{_NS_table, _LN_layout_mode}, 
		{_NS_rtf, _LN_pgnthaic}, 
		{_NS_w12, _LN_ptab}, 
		{_NS_x11, _LN_PromptString}, 
		{_NS_rtf, _LN_stextflow}, 
		{_NS_prj12, _LN_Overallocated}, 
		{_NS_rtf, _LN_chatn}, 
		{_NS_w12, _LN_doNotOrganizeInFolder}, 
		{_NS_w12, _LN_rStyle}, 
		{_NS_rtf, _LN_strike}, 
		{_NS_rtf, _LN_pgnucltr}, 
		{_NS_v11, _LN_HeaderFooter}, 
		{_NS_style, _LN_text_align_source}, 
		{_NS_table, _LN_add_empty_lines}, 
		{_NS_style, _LN_direction}, 
		{_NS_rtf, _LN_chbgbdiag}, 
		{_NS_w12, _LN_rFonts}, 
		{_NS_text, _LN_table_index_entry_template}, 
		{_NS_x11, _LN_NonDefaultName}, 
		{_NS_rtf, _LN_pgnucrm}, 
		{_NS_a12, _LN_masterClrMapping}, 
		{_NS_rtf, _LN_striked1}, 
		{_NS_prj12, _LN_ResponsePending}, 
		{_NS_w12, _LN_b}, 
		{_NS_v11, _LN_VBProjectData}, 
		{_NS_w12, _LN_doNotUseLongFileNames}, 
		{_NS_x11, _LN_SQLType}, 
		{_NS_rtf, _LN_chbgcross}, 
		{_NS_prj12, _LN_Units}, 
		{_NS_rtf, _LN_stshfbi}, 
		{_NS_rtf, _LN_pgnvieta}, 
		{_NS_w12, _LN_bCs}, 
		{_NS_text, _LN_object_index}, 
		{_NS_table, _LN_data_pilot_field_reference}, 
		{_NS_w12, _LN_i}, 
		{_NS_a12, _LN_overrideClrMapping}, 
		{_NS_style, _LN_glyph_orientation_vertical}, 
		{_NS_rtf, _LN_pgnx}, 
		{_NS_v11, _LN_EmailRoutingData}, 
		{_NS_w12, _LN_pixelsPerInch}, 
		{_NS_rtf, _LN_chbgdcross}, 
		{_NS_rtf, _LN_stshfdbch}, 
		{_NS_prj12, _LN_UpdateNeeded}, 
		{_NS_w12, _LN_iCs}, 
		{_NS_x11, _LN_RefreshOnChange}, 
		{_NS_rtf, _LN_pgny}, 
		{_NS_table, _LN_field_name}, 
		{_NS_a12, _LN_baseStyles}, 
		{_NS_prj12, _LN_VAC}, 
		{_NS_w12, _LN_caps}, 
		{_NS_style, _LN_diagonal_tl_br}, 
		{_NS_rtf, _LN_stshfhich}, 
		{_NS_rtf, _LN_chbgdkbdiag}, 
		{_NS_w12, _LN_targetScreenSz}, 
		{_NS_rtf, _LN_pgnzodiac}, 
		{_NS_table, _LN_member_type}, 
		{_NS_x11, _LN_DisplayEmptyMembers}, 
		{_NS_w12, _LN_smallCaps}, 
		{_NS_a12, _LN_objectDefaults}, 
		{_NS_prj12, _LN_WorkContour}, 
		{_NS_rtf, _LN_stshfloch}, 
		{_NS_rtf, _LN_chbgdkcross}, 
		{_NS_x11, _LN_Label}, 
		{_NS_rtf, _LN_pgnzodiacd}, 
		{_NS_style, _LN_diagonal_tl_br_widths}, 
		{_NS_x11, _LN_Caption}, 
		{_NS_w12, _LN_savePreviewPicture}, 
		{_NS_w12, _LN_strike}, 
		{_NS_table, _LN_member_name}, 
		{_NS_rtf, _LN_stylesheet}, 
		{_NS_table, _LN_type}, 
		{_NS_a12, _LN_masterClrScheme}, 
		{_NS_rtf, _LN_chbgdkdcross}, 
		{_NS_rtf, _LN_pgnzodiacl}, 
		{_NS_w12, _LN_dstrike}, 
		{_NS_x11, _LN_NotVisible}, 
		{_NS_style, _LN_diagonal_bl_tr}, 
		{_NS_rtf, _LN_styrsid}, 
		{_NS_w12, _LN_outline}, 
		{_NS_rtf, _LN_pgp}, 
		{_NS_w12, _LN_validateAgainstSchema}, 
		{_NS_rtf, _LN_sub}, 
		{_NS_rtf, _LN_chbgdkfdiag}, 
		{_NS_w12, _LN_shadow}, 
		{_NS_table, _LN_data_pilot_groups}, 
		{_NS_a12, _LN_overrideClrScheme}, 
		{_NS_x11, _LN_Orientation}, 
		{_NS_w12, _LN_emboss}, 
		{_NS_rtf, _LN_pgptbl}, 
		{_NS_rtf, _LN_subdocument}, 
		{_NS_style, _LN_diagonal_bl_tr_widths}, 
		{_NS_rtf, _LN_chbgdkhoriz}, 
		{_NS_w12, _LN_saveInvalidXML}, 
		{_NS_table, _LN_date_start}, 
		{_NS_w12, _LN_imprint}, 
		{_NS_x11, _LN_PivotCache}, 
		{_NS_rtf, _LN_pgwsxn}, 
		{_NS_a12, _LN_masterFontScheme}, 
		{_NS_table, _LN_start}, 
		{_NS_ip12, _LN_xDocumentClass}, 
		{_NS_c11, _LN_OWCVersion}, 
		{_NS_rtf, _LN_chbgdkvert}, 
		{_NS_fo, _LN_wrap_option}, 
		{_NS_rtf, _LN_phcol}, 
		{_NS_w12, _LN_noProof}, 
		{_NS_rtf, _LN_subfontbysize}, 
		{_NS_x11, _LN_CacheIndex}, 
		{_NS_table, _LN_date_end}, 
		{_NS_w12, _LN_ignoreMixedContent}, 
		{_NS_rtf, _LN_subject}, 
		{_NS_rtf, _LN_phmrg}, 
		{_NS_w12, _LN_vanish}, 
		{_NS_a12, _LN_overrideFontScheme}, 
		{_NS_c11, _LN_worksheetoptions}, 
		{_NS_rtf, _LN_chbgfdiag}, 
		{_NS_table, _LN_end}, 
		{_NS_style, _LN_rotation_angle}, 
		{_NS_rtf, _LN_super}, 
		{_NS_x11, _LN_PTPivotData}, 
		{_NS_ip12, _LN_schemaErrorMessages}, 
		{_NS_rtf, _LN_phpg}, 
		{_NS_table, _LN_step}, 
		{_NS_w12, _LN_webHidden}, 
		{_NS_rtf, _LN_chbghoriz}, 
		{_NS_rtf, _LN_swpbdr}, 
		{_NS_w12, _LN_color}, 
		{_NS_rtf, _LN_picbmp}, 
		{_NS_a12, _LN_masterStyleMatrix}, 
		{_NS_w12, _LN_alwaysShowPlaceholderText}, 
		{_NS_ip12, _LN_override}, 
		{_NS_style, _LN_rotation_align}, 
		{_NS_c11, _LN_displaycustomheaders}, 
		{_NS_rtf, _LN_tab}, 
		{_NS_x11, _LN_InvertedRowMember}, 
		{_NS_table, _LN_grouped_by}, 
		{_NS_w12, _LN_w}, 
		{_NS_rtf, _LN_chbgvert}, 
		{_NS_rtf, _LN_picbpp}, 
		{_NS_w12, _LN_kern}, 
		{_NS_rtf, _LN_tabsnoovrlp}, 
		{_NS_style, _LN_cell_protect}, 
		{_NS_rtf, _LN_chbrdr}, 
		{_NS_a12, _LN_overrideStyleMatrix}, 
		{_NS_c11, _LN_ComponentOptions}, 
		{_NS_w12, _LN_position}, 
		{_NS_ip12, _LN_applicationParameters}, 
		{_NS_rtf, _LN_piccropb}, 
		{_NS_w12, _LN_doNotUnderlineInvalidXML}, 
		{_NS_table, _LN_data_pilot_group}, 
		{_NS_x11, _LN_InvertedColumnMember}, 
		{_NS_rtf, _LN_taprtl}, 
		{_NS_rtf, _LN_chcbpat}, 
		{_NS_w12, _LN_sz}, 
		{_NS_a12, _LN_clrMap}, 
		{_NS_c11, _LN_Toolbar}, 
		{_NS_style, _LN_print_content}, 
		{_NS_rtf, _LN_piccropl}, 
		{_NS_rtf, _LN_tb}, 
		{_NS_style, _LN_decimal_places}, 
		{_NS_w12, _LN_szCs}, 
		{_NS_rtf, _LN_chcfpat}, 
		{_NS_ip12, _LN_solutionProperties}, 
		{_NS_x11, _LN_LeafRowMember}, 
		{_NS_w12, _LN_highlight}, 
		{_NS_w12, _LN_removeWordSchemaOnSave}, 
		{_NS_rtf, _LN_piccropr}, 
		{_NS_table, _LN_data_pilot_group_member}, 
		{_NS_c11, _LN_HideOfficeLogo}, 
		{_NS_w12, _LN_u}, 
		{_NS_rtf, _LN_tbllkbestfit}, 
		{_NS_rtf, _LN_chdate}, 
		{_NS_style, _LN_repeat_content}, 
		{_NS_w12, _LN_effect}, 
		{_NS_rtf, _LN_piccropt}, 
		{_NS_x11, _LN_LeafColumnMember}, 
		{_NS_rtf, _LN_chdpa}, 
		{_NS_ip12, _LN_featureRestrictions}, 
		{_NS_table, _LN_consolidation}, 
		{_NS_w12, _LN_useXSLTWhenSaving}, 
		{_NS_w12, _LN_bdr}, 
		{_NS_rtf, _LN_tbllkborder}, 
		{_NS_c11, _LN_SpreadsheetAutoFit}, 
		{_NS_rtf, _LN_pich}, 
		{_NS_x11, _LN_Item}, 
		{_NS_style, _LN_shrink_to_fit}, 
		{_NS_ip12, _LN_save}, 
		{_NS_rtf, _LN_chdpl}, 
		{_NS_c11, _LN_Label}, 
		{_NS_w12, _LN_fitText}, 
		{_NS_rtf, _LN_tbllkcolor}, 
		{_NS_rtf, _LN_pichgoal}, 
		{_NS_rtf, _LN_chftn}, 
		{_NS_w12, _LN_saveThroughXSLT}, 
		{_NS_a12, _LN_lnL}, 
		{_NS_x11, _LN_SourceName}, 
		{_NS_c11, _LN_Caption}, 
		{_NS_ip12, _LN_exportToWeb}, 
		{_NS_table, _LN_source_cell_range_addresses}, 
		{_NS_w12, _LN_vertAlign}, 
		{_NS_a12, _LN_lnR}, 
		{_NS_rtf, _LN_tbllkfont}, 
		{_NS_rtf, _LN_picprop}, 
		{_NS_style, _LN_list_level_properties}, 
		{_NS_rtf, _LN_chftnsep}, 
		{_NS_w12, _LN_showXMLTags}, 
		{_NS_x11, _LN_BoundField}, 
		{_NS_c11, _LN_MaxHeight}, 
		{_NS_w12, _LN_rtl}, 
		{_NS_text, _LN_space_before}, 
		{_NS_a12, _LN_lnT}, 
		{_NS_ip12, _LN_exportToExcel}, 
		{_NS_x11, _LN_DataType}, 
		{_NS_w12, _LN_cs}, 
		{_NS_rtf, _LN_picscaled}, 
		{_NS_rtf, _LN_chftnsepc}, 
		{_NS_rtf, _LN_tbllkhdrcols}, 
		{_NS_ip12, _LN_print}, 
		{_NS_a12, _LN_lnB}, 
		{_NS_table, _LN_target_cell_address}, 
		{_NS_c11, _LN_MaxWidth}, 
		{_NS_w12, _LN_em}, 
		{_NS_text, _LN_min_label_width}, 
		{_NS_rtf, _LN_chpgn}, 
		{_NS_rtf, _LN_picscalex}, 
		{_NS_x11, _LN_FilterCaption}, 
		{_NS_ip12, _LN_sendMail}, 
		{_NS_a12, _LN_lnTlToBr}, 
		{_NS_w12, _LN_alwaysMergeEmptyNamespace}, 
		{_NS_rtf, _LN_tbllkhdrrows}, 
		{_NS_w12, _LN_hyphen}, 
		{_NS_table, _LN_use_labels}, 
		{_NS_c11, _LN_NextSheetNumber}, 
		{_NS_rtf, _LN_chshdng}, 
		{_NS_rtf, _LN_picscaley}, 
		{_NS_w12, _LN_lang}, 
		{_NS_a12, _LN_lnBlToTr}, 
		{_NS_ip12, _LN_autoRecovery}, 
		{_NS_text, _LN_min_label_distance}, 
		{_NS_rtf, _LN_tbllklastcol}, 
		{_NS_rtf, _LN_pict}, 
		{_NS_x11, _LN_PLDataOrientation}, 
		{_NS_rtf, _LN_chtime}, 
		{_NS_ip12, _LN_query}, 
		{_NS_fo, _LN_width}, 
		{_NS_table, _LN_link_to_source_data}, 
		{_NS_a12, _LN_cell3D}, 
		{_NS_w12, _LN_doNotEmbedLinguisticData}, 
		{_NS_ip12, _LN_scripts}, 
		{_NS_w12, _LN_asianLayout}, 
		{_NS_rtf, _LN_picw}, 
		{_NS_fo, _LN_height}, 
		{_NS_x11, _LN_PLPosition}, 
		{_NS_ip12, _LN_script}, 
		{_NS_rtf, _LN_clbgbdiag}, 
		{_NS_rtf, _LN_tbllklastrow}, 
		{_NS_a12, _LN_gridCol}, 
		{_NS_table, _LN_dde_links}, 
		{_NS_x11, _LN_Position}, 
		{_NS_draw, _LN_stroke}, 
		{_NS_rtf, _LN_picwgoal}, 
		{_NS_w12, _LN_specVanish}, 
		{_NS_table, _LN_tracked_changes}, 
		{_NS_w12, _LN_hdrShapeDefaults}, 
		{_NS_a12, _LN_txbx}, 
		{_NS_rtf, _LN_clbgcross}, 
		{_NS_w12, _LN_oMath}, 
		{_NS_ip12, _LN_dataObjects}, 
		{_NS_rtf, _LN_tbllkshading}, 
		{_NS_table, _LN_track_changes}, 
		{_NS_rtf, _LN_plain}, 
		{_NS_x11, _LN_ShowAllItems}, 
		{_NS_table, _LN_insertion}, 
		{_NS_w12, _LN_compat}, 
		{_NS_a12, _LN_tcPr}, 
		{_NS_w12, _LN_lists}, 
		{_NS_draw, _LN_stroke_dash_names}, 
		{_NS_rtf, _LN_clbgdcross}, 
		{_NS_rtf, _LN_tblrsid}, 
		{_NS_table, _LN_position}, 
		{_NS_ip12, _LN_dataObject}, 
		{_NS_w12, _LN_styles}, 
		{_NS_a12, _LN_tc}, 
		{_NS_rtf, _LN_pmmetafile}, 
		{_NS_rtf, _LN_tc}, 
		{_NS_w12, _LN_docVars}, 
		{_NS_w12, _LN_fonts}, 
		{_NS_x11, _LN_NumberFormat}, 
		{_NS_table, _LN_count}, 
		{_NS_rtf, _LN_pn}, 
		{_NS_svg, _LN_stroke_width}, 
		{_NS_rtf, _LN_clbgdkbdiag}, 
		{_NS_ip12, _LN_adoAdapter}, 
		{_NS_w12, _LN_rsids}, 
		{_NS_rtf, _LN_tcelld}, 
		{_NS_a12, _LN_tableStyle}, 
		{_NS_w12, _LN_rubyAlign}, 
		{_NS_x11, _LN_CurrentPage}, 
		{_NS_rtf, _LN_pnacross}, 
		{_NS_rtf, _LN_tcf}, 
		{_NS_table, _LN_dependencies}, 
		{_NS_svg, _LN_stroke_color}, 
		{_NS_rtf, _LN_clbgdkcross}, 
		{_NS_w12, _LN_hps}, 
		{_NS_a12, _LN_tableStyleId}, 
		{_NS_ip12, _LN_webServiceAdapter}, 
		{_NS_rtf, _LN_tcl}, 
		{_NS_rtf, _LN_pnaiu}, 
		{_NS_w12, _LN_uiCompat97To2003}, 
		{_NS_table, _LN_dependency}, 
		{_NS_x11, _LN_NewItemsHidden}, 
		{_NS_a12, _LN_tblPr}, 
		{_NS_w12, _LN_hpsRaise}, 
		{_NS_rtf, _LN_tcn}, 
		{_NS_draw, _LN_marker_start}, 
		{_NS_rtf, _LN_clbgdkdcross}, 
		{_NS_rtf, _LN_pnaiud}, 
		{_NS_table, _LN_id}, 
		{_NS_w12, _LN_binData}, 
		{_NS_ip12, _LN_hwsAdapter}, 
		{_NS_x11, _LN_DataTable}, 
		{_NS_x11, _LN_Subtotal}, 
		{_NS_ip12, _LN_operation}, 
		{_NS_a12, _LN_tblGrid}, 
		{_NS_rtf, _LN_pnaiueo}, 
		{_NS_draw, _LN_marker_end}, 
		{_NS_w12, _LN_hpsBaseText}, 
		{_NS_table, _LN_deletions}, 
		{_NS_rtf, _LN_clbgdkfdiag}, 
		{_NS_rtf, _LN_tdfrmtxtBottom}, 
		{_NS_x11, _LN_Range}, 
		{_NS_a12, _LN_tr}, 
		{_NS_w12, _LN_sourceFileName}, 
		{_NS_w12, _LN_lid}, 
		{_NS_x11, _LN_PLSubtotal}, 
		{_NS_ip12, _LN_hwsOperation}, 
		{_NS_rtf, _LN_pnaiueod}, 
		{_NS_rtf, _LN_clbgdkhor}, 
		{_NS_w12, _LN_marW}, 
		{_NS_rtf, _LN_tdfrmtxtLeft}, 
		{_NS_x11, _LN_RowInputCell}, 
		{_NS_draw, _LN_marker_start_width}, 
		{_NS_w12, _LN_rubyPr}, 
		{_NS_rtf, _LN_pnb}, 
		{_NS_ip12, _LN_input}, 
		{_NS_table, _LN_cell_content_deletion}, 
		{_NS_x11, _LN_DetailFormat}, 
		{_NS_w12, _LN_marH}, 
		{_NS_w12, _LN_rt}, 
		{_NS_rtf, _LN_clbgdkvert}, 
		{_NS_rtf, _LN_pnbidia}, 
		{_NS_rtf, _LN_tdfrmtxtRight}, 
		{_NS_x11, _LN_ColumnInputCell}, 
		{_NS_draw, _LN_marker_end_width}, 
		{_NS_w12, _LN_scrollbar}, 
		{_NS_w12, _LN_rubyBase}, 
		{_NS_table, _LN_change_deletion}, 
		{_NS_x11, _LN_SubtotalFormat}, 
		{_NS_rtf, _LN_clbgfdiag}, 
		{_NS_rtf, _LN_pnbidib}, 
		{_NS_table, _LN_deletion}, 
		{_NS_rtf, _LN_tdfrmtxtTop}, 
		{_NS_x11, _LN_ExcelWorkbook}, 
		{_NS_w12, _LN_dateFormat}, 
		{_NS_rtf, _LN_template}, 
		{_NS_rtf, _LN_clbghoriz}, 
		{_NS_rtf, _LN_pncaps}, 
		{_NS_w12, _LN_noResizeAllowed}, 
		{_NS_x11, _LN_DetailWidth}, 
		{_NS_draw, _LN_marker_start_center}, 
		{_NS_rtf, _LN_time}, 
		{_NS_a12, _LN_unthemedFill}, 
		{_NS_rtf, _LN_title}, 
		{_NS_rtf, _LN_pncard}, 
		{_NS_w12, _LN_listItem}, 
		{_NS_rtf, _LN_clbgvert}, 
		{_NS_table, _LN_multi_deletion_spanned}, 
		{_NS_x11, _LN_PublishObjects}, 
		{_NS_w12, _LN_linkedToFile}, 
		{_NS_x11, _LN_GroupedWidth}, 
		{_NS_rtf, _LN_pncf}, 
		{_NS_draw, _LN_marker_end_center}, 
		{_NS_rtf, _LN_titlepg}, 
		{_NS_a12, _LN_themedFill}, 
		{_NS_rtf, _LN_clbrdrb}, 
		{_NS_x11, _LN_SupBook}, 
		{_NS_w12, _LN_docPartType}, 
		{_NS_table, _LN_cut_offs}, 
		{_NS_w12, _LN_noBorder}, 
		{_NS_x11, _LN_TotalWidth}, 
		{_NS_a12, _LN_ln}, 
		{_NS_rtf, _LN_tldot}, 
		{_NS_rtf, _LN_pnchosung}, 
		{_NS_rtf, _LN_clbrdrl}, 
		{_NS_svg, _LN_stroke_opacity}, 
		{_NS_rtf, _LN_tleq}, 
		{_NS_w12, _LN_flatBorders}, 
		{_NS_x11, _LN_SelectedSheets}, 
		{_NS_x11, _LN_PivotItem}, 
		{_NS_w12, _LN_docPartCategory}, 
		{_NS_rtf, _LN_pncnum}, 
		{_NS_table, _LN_insertion_cut_off}, 
		{_NS_a12, _LN_unthemedLine}, 
		{_NS_rtf, _LN_clbrdrr}, 
		{_NS_rtf, _LN_tlhyph}, 
		{_NS_x11, _LN_DataField}, 
		{_NS_rtf, _LN_pndbnum}, 
		{_NS_draw, _LN_stroke_linejoin}, 
		{_NS_x11, _LN_WindowHidden}, 
		{_NS_rtf, _LN_clbrdrt}, 
		{_NS_w12, _LN_docPartUnique}, 
		{_NS_a12, _LN_themedLine}, 
		{_NS_w12, _LN_framesetSplitbar}, 
		{_NS_draw, _LN_fill}, 
		{_NS_table, _LN_movement_cut_off}, 
		{_NS_rtf, _LN_tlmdot}, 
		{_NS_rtf, _LN_pndbnumd}, 
		{_NS_rtf, _LN_clcbpat}, 
		{_NS_x11, _LN_MapChildItems}, 
		{_NS_rtf, _LN_tlth}, 
		{_NS_w12, _LN_frameLayout}, 
		{_NS_a12, _LN_unthemedFonts}, 
		{_NS_draw, _LN_fill_color}, 
		{_NS_w12, _LN_pictureStorageType}, 
		{_NS_table, _LN_start_position}, 
		{_NS_rtf, _LN_pndbnumk}, 
		{_NS_x11, _LN_HideHorizontalScrollBar}, 
		{_NS_rtf, _LN_tlul}, 
		{_NS_rtf, _LN_clcbpatraw}, 
		{_NS_w12, _LN_frameset}, 
		{_NS_x11, _LN_AutoShowCount}, 
		{_NS_a12, _LN_themedFonts}, 
		{_NS_w12, _LN_docPart}, 
		{_NS_rtf, _LN_pndbnuml}, 
		{_NS_w12, _LN_frame}, 
		{_NS_table, _LN_end_position}, 
		{_NS_rtf, _LN_clcfpat}, 
		{_NS_draw, _LN_secondary_fill_color}, 
		{_NS_rtf, _LN_toplinepunct}, 
		{_NS_w12, _LN_start}, 
		{_NS_a12, _LN_fonts}, 
		{_NS_w12, _LN_alias}, 
		{_NS_x11, _LN_HideVerticalScrollBar}, 
		{_NS_x11, _LN_AutoShowField}, 
		{_NS_rtf, _LN_pndbnumt}, 
		{_NS_table, _LN_movement}, 
		{_NS_a12, _LN_b}, 
		{_NS_w12, _LN_nfc}, 
		{_NS_rtf, _LN_tphcol}, 
		{_NS_rtf, _LN_clcfpatraw}, 
		{_NS_a12, _LN_i}, 
		{_NS_rtf, _LN_pndec}, 
		{_NS_draw, _LN_fill_gradient_name}, 
		{_NS_w12, _LN_invalidIfBlank}, 
		{_NS_x11, _LN_AutoShowRange}, 
		{_NS_rtf, _LN_tphmrg}, 
		{_NS_rtf, _LN_cldgll}, 
		{_NS_x11, _LN_HideWorkbookTabs}, 
		{_NS_w12, _LN_lvlRestart}, 
		{_NS_a12, _LN_left}, 
		{_NS_rtf, _LN_pndecd}, 
		{_NS_w12, _LN_lock}, 
		{_NS_table, _LN_source_range_address}, 
		{_NS_rtf, _LN_tphpg}, 
		{_NS_rtf, _LN_cldglu}, 
		{_NS_w12, _LN_isLgl}, 
		{_NS_w12, _LN_placeholder}, 
		{_NS_a12, _LN_right}, 
		{_NS_x11, _LN_AutoShowType}, 
		{_NS_rtf, _LN_pnf}, 
		{_NS_x11, _LN_WindowHeight}, 
		{_NS_draw, _LN_gradient_step_count}, 
		{_NS_table, _LN_column}, 
		{_NS_w12, _LN_suff}, 
		{_NS_rtf, _LN_tposnegx}, 
		{_NS_a12, _LN_top}, 
		{_NS_rtf, _LN_clFitText}, 
		{_NS_rtf, _LN_pnfs}, 
		{_NS_table, _LN_row}, 
		{_NS_w12, _LN_showingPlcHdr}, 
		{_NS_x11, _LN_WindowWidth}, 
		{_NS_x11, _LN_AutoSortOrder}, 
		{_NS_w12, _LN_lvlText}, 
		{_NS_a12, _LN_bottom}, 
		{_NS_draw, _LN_fill_hatch_name}, 
		{_NS_rtf, _LN_tposnegy}, 
		{_NS_rtf, _LN_pnganada}, 
		{_NS_rtf, _LN_clftsWidth}, 
		{_NS_table, _LN_start_column}, 
		{_NS_x11, _LN_WindowTopX}, 
		{_NS_w12, _LN_edittingPlcHdr}, 
		{_NS_rtf, _LN_tposxc}, 
		{_NS_a12, _LN_insideH}, 
		{_NS_x11, _LN_AutoSortField}, 
		{_NS_rtf, _LN_clmgf}, 
		{_NS_rtf, _LN_pngblip}, 
		{_NS_w12, _LN_lvlPicBulletId}, 
		{_NS_draw, _LN_fill_hatch_solid}, 
		{_NS_rtf, _LN_tposxi}, 
		{_NS_table, _LN_start_row}, 
		{_NS_a12, _LN_insideV}, 
		{_NS_x11, _LN_WindowTopY}, 
		{_NS_rtf, _LN_clmrg}, 
		{_NS_w12, _LN_dataBinding}, 
		{_NS_x11, _LN_BaseField}, 
		{_NS_rtf, _LN_pngbnum}, 
		{_NS_w12, _LN_legacy}, 
		{_NS_rtf, _LN_tposxl}, 
		{_NS_a12, _LN_tl2br}, 
		{_NS_draw, _LN_fill_image_name}, 
		{_NS_rtf, _LN_cloWrap}, 
		{_NS_w12, _LN_lvlJc}, 
		{_NS_table, _LN_start_table}, 
		{_NS_w12, _LN_comboBox}, 
		{_NS_x11, _LN_BaseItem}, 
		{_NS_rtf, _LN_pngbnumd}, 
		{_NS_x11, _LN_ActiveSheet}, 
		{_NS_rtf, _LN_tposx}, 
		{_NS_a12, _LN_tr2bl}, 
		{_NS_w12, _LN_date}, 
		{_NS_w12, _LN_lsid}, 
		{_NS_rtf, _LN_clpadb}, 
		{_NS_rtf, _LN_pngbnumk}, 
		{_NS_table, _LN_end_column}, 
		{_NS_x11, _LN_ActiveChart}, 
		{_NS_draw, _LN_fill_image_width}, 
		{_NS_rtf, _LN_tposxo}, 
		{_NS_w12, _LN_plt}, 
		{_NS_a12, _LN_tcBorders}, 
		{_NS_w12, _LN_docPartObj}, 
		{_NS_x11, _LN_BlankLineAfterItems}, 
		{_NS_rtf, _LN_clpadfb}, 
		{_NS_w12, _LN_tmpl}, 
		{_NS_table, _LN_end_row}, 
		{_NS_rtf, _LN_pngbnuml}, 
		{_NS_rtf, _LN_tposxr}, 
		{_NS_a12, _LN_rPr}, 
		{_NS_rtf, _LN_clpadfl}, 
		{_NS_x11, _LN_FirstVisibleSheet}, 
		{_NS_w12, _LN_docPartList}, 
		{_NS_draw, _LN_fill_image_height}, 
		{_NS_w12, _LN_styleLink}, 
		{_NS_rtf, _LN_pnhang}, 
		{_NS_rtf, _LN_tposy}, 
		{_NS_table, _LN_end_table}, 
		{_NS_a12, _LN_tableBg}, 
		{_NS_x11, _LN_CantGetUniqueItems}, 
		{_NS_rtf, _LN_clpadfr}, 
		{_NS_rtf, _LN_pni}, 
		{_NS_rtf, _LN_tposyb}, 
		{_NS_w12, _LN_dropDownList}, 
		{_NS_w12, _LN_listStyleLink}, 
		{_NS_x11, _LN_GroupBy}, 
		{_NS_a12, _LN_wholeTable}, 
		{_NS_rtf, _LN_clpadft}, 
		{_NS_rtf, _LN_pnindent}, 
		{_NS_draw, _LN_fill_image_ref_point_x}, 
		{_NS_x11, _LN_HidePivotTableFieldList}, 
		{_NS_rtf, _LN_tposyc}, 
		{_NS_w12, _LN_lvl}, 
		{_NS_table, _LN_change_track_table_cell}, 
		{_NS_w12, _LN_picture}, 
		{_NS_x11, _LN_GroupStart}, 
		{_NS_a12, _LN_band1Horz}, 
		{_NS_rtf, _LN_clpadl}, 
		{_NS_rtf, _LN_pniroha}, 
		{_NS_rtf, _LN_tposyil}, 
		{_NS_w12, _LN_richText}, 
		{_NS_x11, _LN_ProtectStructure}, 
		{_NS_table, _LN_cell_address}, 
		{_NS_w12, _LN_startOverride}, 
		{_NS_rtf, _LN_clpadr}, 
		{_NS_x11, _LN_GroupEnd}, 
		{_NS_a12, _LN_band2Horz}, 
		{_NS_w12, _LN_text}, 
		{_NS_draw, _LN_fill_image_ref_point_y}, 
		{_NS_rtf, _LN_pnirohad}, 
		{_NS_rtf, _LN_tposyin}, 
		{_NS_w12, _LN_p}, 
		{_NS_w12, _LN_ilst}, 
		{_NS_rtf, _LN_clpadt}, 
		{_NS_a12, _LN_band1Vert}, 
		{_NS_x11, _LN_ProtectWindows}, 
		{_NS_rtf, _LN_pnlcltr}, 
		{_NS_table, _LN_matrix_covered}, 
		{_NS_w12, _LN_tbl}, 
		{_NS_x11, _LN_GroupStartAuto}, 
		{_NS_rtf, _LN_tposyoutv}, 
		{_NS_w12, _LN_tr}, 
		{_NS_rtf, _LN_clshdng}, 
		{_NS_w12, _LN_lvlOverride}, 
		{_NS_draw, _LN_fill_image_ref_point}, 
		{_NS_rtf, _LN_pnlcrm}, 
		{_NS_a12, _LN_band2Vert}, 
		{_NS_w12, _LN_tc}, 
		{_NS_rtf, _LN_tposyt}, 
		{_NS_x11, _LN_DisplayInkNotes}, 
		{_NS_x11, _LN_GroupEndAuto}, 
		{_NS_table, _LN_cell_content_change}, 
		{_NS_rtf, _LN_clshdngraw}, 
		{_NS_rtf, _LN_pnlvl}, 
		{_NS_w12, _LN_sdtPr}, 
		{_NS_w12, _LN_listPicBullet}, 
		{_NS_a12, _LN_lastCol}, 
		{_NS_rtf, _LN_tpvmrg}, 
		{_NS_draw, _LN_tile_repeat_offset}, 
		{_NS_x11, _LN_GroupType}, 
		{_NS_table, _LN_previous}, 
		{_NS_rtf, _LN_pnlvlblt}, 
		{_NS_w12, _LN_listDef}, 
		{_NS_rtf, _LN_clshdrawnil}, 
		{_NS_rtf, _LN_tpvpara}, 
		{_NS_a12, _LN_firstCol}, 
		{_NS_x11, _LN_EmbedSaveSmartTags}, 
		{_NS_w12, _LN_sdtContent}, 
		{_NS_w12, _LN_list}, 
		{_NS_w12, _LN_pPr}, 
		{_NS_draw, _LN_opacity_name}, 
		{_NS_rtf, _LN_tpvpg}, 
		{_NS_rtf, _LN_pnlvlbody}, 
		{_NS_x11, _LN_LayoutPageBreak}, 
		{_NS_rtf, _LN_cltxbtlr}, 
		{_NS_a12, _LN_lastRow}, 
		{_NS_x11, _LN_FutureVer}, 
		{_NS_table, _LN_acceptance_state}, 
		{_NS_w12, _LN_sdt}, 
		{_NS_rtf, _LN_tqc}, 
		{_NS_svg, _LN_fill_rule}, 
		{_NS_a12, _LN_seCell}, 
		{_NS_w12, _LN_ilfoMacAtCleanup}, 
		{_NS_rtf, _LN_pnlvlcont}, 
		{_NS_rtf, _LN_cltxlrtb}, 
		{_NS_x11, _LN_TabRatio}, 
		{_NS_w12, _LN_subDoc}, 
		{_NS_x11, _LN_ParentField}, 
		{_NS_rtf, _LN_tqdec}, 
		{_NS_w12, _LN_gridCol}, 
		{_NS_a12, _LN_swCell}, 
		{_NS_w12, _LN_aliases}, 
		{_NS_table, _LN_rejecting_change_id}, 
		{_NS_draw, _LN_symbol_color}, 
		{_NS_rtf, _LN_pnnumonce}, 
		{_NS_rtf, _LN_cltxlrtbv}, 
		{_NS_w12, _LN_insideH}, 
		{_NS_rtf, _LN_tqr}, 
		{_NS_x11, _LN_Function}, 
		{_NS_x11, _LN_WindowIconic}, 
		{_NS_w12, _LN_sti}, 
		{_NS_w12, _LN_insideV}, 
		{_NS_a12, _LN_firstRow}, 
		{_NS_rtf, _LN_pnord}, 
		{_NS_text, _LN_animation}, 
		{_NS_w12, _LN_tl2br}, 
		{_NS_rtf, _LN_cltxtbrl}, 
		{_NS_rtf, _LN_transmf}, 
		{_NS_style, _LN_handout_master}, 
		{_NS_x11, _LN_LayoutForm}, 
		{_NS_w12, _LN_basedOn}, 
		{_NS_w12, _LN_tr2bl}, 
		{_NS_a12, _LN_neCell}, 
		{_NS_rtf, _LN_pnordt}, 
		{_NS_rtf, _LN_trauth}, 
		{_NS_rtf, _LN_cltxtbrlv}, 
		{_NS_x11, _LN_DisplayDrawingObjects}, 
		{_NS_w12, _LN_tcW}, 
		{_NS_w12, _LN_next}, 
		{_NS_a12, _LN_nwCell}, 
		{_NS_rtf, _LN_pnprev}, 
		{_NS_text, _LN_animation_direction}, 
		{_NS_w12, _LN_link}, 
		{_NS_rtf, _LN_trautofit}, 
		{_NS_x11, _LN_LayoutSubtotalLocation}, 
		{_NS_rtf, _LN_clvertalb}, 
		{_NS_w12, _LN_gridSpan}, 
		{_NS_rtf, _LN_pnqc}, 
		{_NS_x11, _LN_CreateBackup}, 
		{_NS_pr, _LN_presentation_page_layout_name}, 
		{_NS_w12, _LN_priority}, 
		{_NS_rtf, _LN_trbgbdiag}, 
		{_NS_w12, _LN_hmerge}, 
		{_NS_rtf, _LN_pnql}, 
		{_NS_rtf, _LN_clvertalc}, 
		{_NS_x11, _LN_NoDragToRow}, 
		{_NS_text, _LN_animation_start_inside}, 
		{_NS_x11, _LN_Calculation}, 
		{_NS_rtf, _LN_pnqr}, 
		{_NS_style, _LN_page_layout_name}, 
		{_NS_w12, _LN_autoRedefine}, 
		{_NS_w12, _LN_vmerge}, 
		{_NS_rtf, _LN_trbgcross}, 
		{_NS_rtf, _LN_clvertalt}, 
		{_NS_draw, _LN_style_name}, 
		{_NS_w12, _LN_hidden}, 
		{_NS_w12, _LN_tcBorders}, 
		{_NS_x11, _LN_NoDragToColumn}, 
		{_NS_rtf, _LN_pnrauth}, 
		{_NS_rtf, _LN_clvmgf}, 
		{_NS_text, _LN_animation_stop_inside}, 
		{_NS_rtf, _LN_trbgdcross}, 
		{_NS_w12, _LN_noWrap}, 
		{_NS_draw, _LN_layer_set}, 
		{_NS_x11, _LN_DoNotCalculateBeforeSave}, 
		{_NS_w12, _LN_semiHidden}, 
		{_NS_rtf, _LN_pnrdate}, 
		{_NS_x11, _LN_NoDragToData}, 
		{_NS_rtf, _LN_clvmrg}, 
		{_NS_w12, _LN_tcMar}, 
		{_NS_a12, _LN_topBaseline}, 
		{_NS_draw, _LN_layer}, 
		{_NS_rtf, _LN_trbgdkbdiag}, 
		{_NS_w12, _LN_locked}, 
		{_NS_x11, _LN_Date1904}, 
		{_NS_text, _LN_animation_repeat}, 
		{_NS_draw, _LN_name}, 
		{_NS_rtf, _LN_pnrestart}, 
		{_NS_rtf, _LN_clwWidth}, 
		{_NS_x11, _LN_NoDragToHide}, 
		{_NS_w12, _LN_tcFitText}, 
		{_NS_draw, _LN_protected}, 
		{_NS_rtf, _LN_trbgdkcross}, 
		{_NS_a12, _LN_bottomBaseline}, 
		{_NS_x11, _LN_RefModeR1C1}, 
		{_NS_rtf, _LN_pnrnfc}, 
		{_NS_draw, _LN_display}, 
		{_NS_w12, _LN_unhideWhenUsed}, 
		{_NS_text, _LN_animation_delay}, 
		{_NS_rtf, _LN_collapsed}, 
		{_NS_draw, _LN_page}, 
		{_NS_w12, _LN_hideMark}, 
		{_NS_a12, _LN_pPr}, 
		{_NS_x11, _LN_NoDragToPage}, 
		{_NS_rtf, _LN_pnrnot}, 
		{_NS_draw, _LN_master_page_name}, 
		{_NS_w12, _LN_tcPr}, 
		{_NS_w12, _LN_qformat}, 
		{_NS_x11, _LN_Iteration}, 
		{_NS_rtf, _LN_trbgdkdcross}, 
		{_NS_rtf, _LN_colno}, 
		{_NS_draw, _LN_shape_id}, 
		{_NS_a12, _LN_endParaRPr}, 
		{_NS_text, _LN_animation_steps}, 
		{_NS_rtf, _LN_pnrpnbr}, 
		{_NS_x11, _LN_MemberProperty}, 
		{_NS_w12, _LN_personal}, 
		{_NS_w12, _LN_gridBefore}, 
		{_NS_rtf, _LN_colortbl}, 
		{_NS_rtf, _LN_trbgdkfdiag}, 
		{_NS_x11, _LN_MaxIterations}, 
		{_NS_pr, _LN_use_header_name}, 
		{_NS_pr, _LN_effect}, 
		{_NS_a12, _LN_defPPr}, 
		{_NS_rtf, _LN_cols}, 
		{_NS_rtf, _LN_pnrrgb}, 
		{_NS_pr, _LN_direction}, 
		{_NS_w12, _LN_gridAfter}, 
		{_NS_draw, _LN_auto_grow_width}, 
		{_NS_rtf, _LN_trbgdkhor}, 
		{_NS_x11, _LN_MaxChange}, 
		{_NS_w12, _LN_personalCompose}, 
		{_NS_rtf, _LN_colsr}, 
		{_NS_pr, _LN_speed}, 
		{_NS_rtf, _LN_pnrstart}, 
		{_NS_x11, _LN_MemberPropertiesOrder}, 
		{_NS_pr, _LN_use_footer_name}, 
		{_NS_a12, _LN_outline1pPr}, 
		{_NS_w12, _LN_wBefore}, 
		{_NS_pr, _LN_delay}, 
		{_NS_rtf, _LN_colsx}, 
		{_NS_rtf, _LN_trbgdkvert}, 
		{_NS_rtf, _LN_pnrstop}, 
		{_NS_draw, _LN_auto_grow_height}, 
		{_NS_w12, _LN_personalReply}, 
		{_NS_rtf, _LN_column}, 
		{_NS_a12, _LN_outline2pPr}, 
		{_NS_w12, _LN_wAfter}, 
		{_NS_x11, _LN_HideDropDowns}, 
		{_NS_x11, _LN_PrecisionAsDisplayed}, 
		{_NS_pr, _LN_start_scale}, 
		{_NS_pr, _LN_use_date_time_name}, 
		{_NS_rtf, _LN_trbgfdiag}, 
		{_NS_rtf, _LN_pnrxst}, 
		{_NS_rtf, _LN_colw}, 
		{_NS_draw, _LN_id}, 
		{_NS_draw, _LN_fit_to_size}, 
		{_NS_w12, _LN_tblStylePr}, 
		{_NS_a12, _LN_lvl1pPr}, 
		{_NS_w12, _LN_cantSplit}, 
		{_NS_rtf, _LN_comment}, 
		{_NS_pr, _LN_path_id}, 
		{_NS_x11, _LN_DrilledLevel}, 
		{_NS_rtf, _LN_pnscaps}, 
		{_NS_rtf, _LN_trbghoriz}, 
		{_NS_draw, _LN_rect}, 
		{_NS_x11, _LN_DoNotSaveLinkValues}, 
		{_NS_a12, _LN_lvl2pPr}, 
		{_NS_rtf, _LN_company}, 
		{_NS_w12, _LN_trHeight}, 
		{_NS_w12, _LN_lsdException}, 
		{_NS_draw, _LN_fit_to_contour}, 
		{_NS_pr, _LN_show_text}, 
		{_NS_rtf, _LN_pnseclvl}, 
		{_NS_rtf, _LN_trbgvert}, 
		{_NS_x11, _LN_ServerSortOrder}, 
		{_NS_rtf, _LN_cpg}, 
		{_NS_a12, _LN_lvl3pPr}, 
		{_NS_draw, _LN_corner_radius}, 
		{_NS_x11, _LN_NoAutoRecover}, 
		{_NS_w12, _LN_tblHeader}, 
		{_NS_rtf, _LN_pnsp}, 
		{_NS_pr, _LN_hide_shape}, 
		{_NS_rtf, _LN_trbrdrb}, 
		{_NS_rtf, _LN_crauth}, 
		{_NS_draw, _LN_line}, 
		{_NS_a12, _LN_lvl4pPr}, 
		{_NS_draw, _LN_textarea_vertical_align}, 
		{_NS_w12, _LN_versionOfBuiltInStylenames}, 
		{_NS_x11, _LN_IsMemberProperty}, 
		{_NS_rtf, _LN_pnstart}, 
		{_NS_svg, _LN_x1}, 
		{_NS_w12, _LN_tblCellSpacing}, 
		{_NS_rtf, _LN_crdate}, 
		{_NS_rtf, _LN_trbrdrh}, 
		{_NS_pr, _LN_hide_text}, 
		{_NS_a12, _LN_lvl5pPr}, 
		{_NS_x11, _LN_AcceptLabelsInFormulas}, 
		{_NS_svg, _LN_y1}, 
		{_NS_rtf, _LN_pnstrike}, 
		{_NS_pr, _LN_dim}, 
		{_NS_w12, _LN_latentStyles}, 
		{_NS_w12, _LN_tblPrEx}, 
		{_NS_x11, _LN_FilterMember}, 
		{_NS_rtf, _LN_creatim}, 
		{_NS_rtf, _LN_trbrdrl}, 
		{_NS_svg, _LN_x2}, 
		{_NS_a12, _LN_lvl6pPr}, 
		{_NS_w12, _LN_style}, 
		{_NS_x11, _LN_Uncalced}, 
		{_NS_rtf, _LN_cs}, 
		{_NS_draw, _LN_color}, 
		{_NS_rtf, _LN_pntext}, 
		{_NS_draw, _LN_textarea_horizontal_align}, 
		{_NS_w12, _LN_trPr}, 
		{_NS_svg, _LN_y2}, 
		{_NS_x11, _LN_Name}, 
		{_NS_rtf, _LN_trbrdrr}, 
		{_NS_x11, _LN_UniqueName}, 
		{_NS_a12, _LN_lvl7pPr}, 
		{_NS_rtf, _LN_ctrl}, 
		{_NS_pr, _LN_play}, 
		{_NS_w12, _LN_altName}, 
		{_NS_rtf, _LN_pntxta}, 
		{_NS_x11, _LN_Level}, 
		{_NS_w12, _LN_tblStyle}, 
		{_NS_draw, _LN_color_mode}, 
		{_NS_draw, _LN_polyline}, 
		{_NS_rtf, _LN_trbrdrt}, 
		{_NS_rtf, _LN_cts}, 
		{_NS_a12, _LN_lvl8pPr}, 
		{_NS_x11, _LN_DataValidation}, 
		{_NS_w12, _LN_panose1}, 
		{_NS_rtf, _LN_pntxtb}, 
		{_NS_w12, _LN_tblpPr}, 
		{_NS_rtf, _LN_cufi}, 
		{_NS_draw, _LN_points}, 
		{_NS_x11, _LN_ServerBased}, 
		{_NS_pr, _LN_animation_group}, 
		{_NS_rtf, _LN_trbrdrv}, 
		{_NS_a12, _LN_lvl9pPr}, 
		{_NS_draw, _LN_color_inversion}, 
		{_NS_w12, _LN_charset}, 
		{_NS_rtf, _LN_pnucltr}, 
		{_NS_x11, _LN_AutoFilter}, 
		{_NS_rtf, _LN_culi}, 
		{_NS_draw, _LN_polygon}, 
		{_NS_w12, _LN_family}, 
		{_NS_w12, _LN_tblOverlap}, 
		{_NS_rtf, _LN_trcbpat}, 
		{_NS_pr, _LN_node_type}, 
		{_NS_x11, _LN_SubtotalName}, 
		{_NS_a12, _LN_noAutofit}, 
		{_NS_rtf, _LN_curi}, 
		{_NS_rtf, _LN_pnucrm}, 
		{_NS_draw, _LN_luminance}, 
		{_NS_w12, _LN_tblRtl}, 
		{_NS_rtf, _LN_trcfpat}, 
		{_NS_pr, _LN_preset_id}, 
		{_NS_w12, _LN_notTrueType}, 
		{_NS_rtf, _LN_pnul}, 
		{_NS_rtf, _LN_cvmme}, 
		{_NS_draw, _LN_regular_polygon}, 
		{_NS_x11, _LN_PLTPivotItem}, 
		{_NS_a12, _LN_normAutofit}, 
		{_NS_draw, _LN_contrast}, 
		{_NS_x11, _LN_AutoFilterCondition}, 
		{_NS_rtf, _LN_trdate}, 
		{_NS_w12, _LN_pitch}, 
		{_NS_w12, _LN_bidiVisual}, 
		{_NS_rtf, _LN_pnuld}, 
		{_NS_draw, _LN_concave}, 
		{_NS_draw, _LN_gamma}, 
		{_NS_rtf, _LN_datafield}, 
		{_NS_pr, _LN_preset_sub_type}, 
		{_NS_x11, _LN_FormulaV10}, 
		{_NS_w12, _LN_sig}, 
		{_NS_a12, _LN_shapeAutofit}, 
		{_NS_rtf, _LN_date}, 
		{_NS_x11, _LN_AutoFilterOr}, 
		{_NS_rtf, _LN_trftsWidthA}, 
		{_NS_draw, _LN_red}, 
		{_NS_rtf, _LN_pnuldash}, 
		{_NS_draw, _LN_corners}, 
		{_NS_rtf, _LN_dbch}, 
		{_NS_w12, _LN_tblStyleRowBandSize}, 
		{_NS_a12, _LN_textWarp}, 
		{_NS_pr, _LN_preset_class}, 
		{_NS_draw, _LN_green}, 
		{_NS_w12, _LN_defaultFonts}, 
		{_NS_x11, _LN_ParseFormulaAsV10}, 
		{_NS_rtf, _LN_pnuldashd}, 
		{_NS_rtf, _LN_trftsWidthB}, 
		{_NS_draw, _LN_sharpness}, 
		{_NS_x11, _LN_AutoFilterAnd}, 
		{_NS_rtf, _LN_deff}, 
		{_NS_w12, _LN_font}, 
		{_NS_a12, _LN_prot}, 
		{_NS_draw, _LN_blue}, 
		{_NS_draw, _LN_path}, 
		{_NS_pr, _LN_master_element}, 
		{_NS_x11, _LN_PLCaption}, 
		{_NS_x11, _LN_Sorting}, 
		{_NS_rtf, _LN_pnuldashdd}, 
		{_NS_w12, _LN_tblStyleColBandSize}, 
		{_NS_rtf, _LN_trftsWidth}, 
		{_NS_a12, _LN_bodyPr}, 
		{_NS_rtf, _LN_defformat}, 
		{_NS_w12, _LN_blockQuote}, 
		{_NS_svg, _LN_d}, 
		{_NS_draw, _LN_image_opacity}, 
		{_NS_w12, _LN_tblW}, 
		{_NS_rtf, _LN_pnuldb}, 
		{_NS_pr, _LN_group_id}, 
		{_NS_rtf, _LN_trgaph}, 
		{_NS_x11, _LN_PLGroupType}, 
		{_NS_rtf, _LN_deflang}, 
		{_NS_a12, _LN_lstStyle}, 
		{_NS_draw, _LN_shadow}, 
		{_NS_w12, _LN_bodyDiv}, 
		{_NS_draw, _LN_circle}, 
		{_NS_a12, _LN_p}, 
		{_NS_w12, _LN_tblInd}, 
		{_NS_x11, _LN_ConditionalFormatting}, 
		{_NS_rtf, _LN_trhdr}, 
		{_NS_rtf, _LN_pnulhair}, 
		{_NS_svg, _LN_cx}, 
		{_NS_x11, _LN_PLName}, 
		{_NS_w12, _LN_marLeft}, 
		{_NS_rtf, _LN_deflangfe}, 
		{_NS_pr, _LN_event_listener}, 
		{_NS_svg, _LN_cy}, 
		{_NS_draw, _LN_shadow_offset_x}, 
		{_NS_rtf, _LN_trkeep}, 
		{_NS_w12, _LN_tblBorders}, 
		{_NS_x11, _LN_QueryTable}, 
		{_NS_rtf, _LN_pnulnone}, 
		{_NS_rtf, _LN_defshp}, 
		{_NS_svg, _LN_r}, 
		{_NS_w12, _LN_marRight}, 
		{_NS_script, _LN_event_name}, 
		{_NS_x11, _LN_SourceHierarchy}, 
		{_NS_rtf, _LN_trleft}, 
		{_NS_draw, _LN_kind}, 
		{_NS_rtf, _LN_pnulth}, 
		{_NS_w12, _LN_tblLayout}, 
		{_NS_rtf, _LN_deftab}, 
		{_NS_draw, _LN_shadow_offset_y}, 
		{_NS_x11, _LN_PivotTable}, 
		{_NS_w12, _LN_marTop}, 
		{_NS_pr, _LN_action}, 
		{_NS_rtf, _LN_trowd}, 
		{_NS_rtf, _LN_pnulw}, 
		{_NS_draw, _LN_start_angle}, 
		{_NS_rtf, _LN_deleted}, 
		{_NS_w12, _LN_tblCellMar}, 
		{_NS_pr, _LN_verb}, 
		{_NS_w12, _LN_marBottom}, 
		{_NS_x11, _LN_MemberPropertyParent}, 
		{_NS_draw, _LN_shadow_color}, 
		{_NS_rtf, _LN_trpaddb}, 
		{_NS_x11, _LN_WorksheetSource}, 
		{_NS_rtf, _LN_pnulwave}, 
		{_NS_rtf, _LN_delrsid}, 
		{_NS_draw, _LN_end_angle}, 
		{_NS_pr, _LN_header}, 
		{_NS_w12, _LN_tblLook}, 
		{_NS_w12, _LN_divBdr}, 
		{_NS_a12, _LN_buColorTx}, 
		{_NS_rtf, _LN_trpaddfb}, 
		{_NS_w12, _LN_tblPr}, 
		{_NS_draw, _LN_shadow_opacity}, 
		{_NS_rtf, _LN_pnzodiac}, 
		{_NS_rtf, _LN_dfrauth}, 
		{_NS_draw, _LN_ellipse}, 
		{_NS_pr, _LN_footer}, 
		{_NS_x11, _LN_SourceHierarchyLevel}, 
		{_NS_x11, _LN_WorksheetOptions}, 
		{_NS_w12, _LN_tblGrid}, 
		{_NS_w12, _LN_divsChild}, 
		{_NS_a12, _LN_buColor}, 
		{_NS_rtf, _LN_trpaddfl}, 
		{_NS_w12, _LN_pos}, 
		{_NS_svg, _LN_rx}, 
		{_NS_rtf, _LN_pnzodiacd}, 
		{_NS_rtf, _LN_dfrdate}, 
		{_NS_w12, _LN_div}, 
		{_NS_x11, _LN_Watches}, 
		{_NS_pr, _LN_date_time}, 
		{_NS_a12, _LN_buSzTx}, 
		{_NS_svg, _LN_ry}, 
		{_NS_x11, _LN_TotalAlignment}, 
		{_NS_w12, _LN_numFmt}, 
		{_NS_rtf, _LN_trpaddfr}, 
		{_NS_x11, _LN_Watch}, 
		{_NS_rtf, _LN_dfrmtxtx}, 
		{_NS_rtf, _LN_pnzodiacl}, 
		{_NS_draw, _LN_start_line_spacing_horizontal}, 
		{_NS_w12, _LN_txbxContent}, 
		{_NS_a12, _LN_buSzPct}, 
		{_NS_pr, _LN_header_decl}, 
		{_NS_draw, _LN_connector}, 
		{_NS_w12, _LN_numStart}, 
		{_NS_rtf, _LN_trpaddft}, 
		{_NS_pr, _LN_name}, 
		{_NS_x11, _LN_Source}, 
		{_NS_draw, _LN_type}, 
		{_NS_rtf, _LN_posnegx}, 
		{_NS_rtf, _LN_dfrmtxty}, 
		{_NS_w12, _LN_cfChunk}, 
		{_NS_a12, _LN_buSzPts}, 
		{_NS_x11, _LN_TotalCaptionAlignment}, 
		{_NS_rtf, _LN_trpaddl}, 
		{_NS_w12, _LN_numRestart}, 
		{_NS_rtf, _LN_posnegy}, 
		{_NS_x11, _LN_PageBreaks}, 
		{_NS_pr, _LN_footer_decl}, 
		{_NS_rtf, _LN_dfrstart}, 
		{_NS_draw, _LN_start_shape}, 
		{_NS_w12, _LN_active}, 
		{_NS_w12, _LN_proofErr}, 
		{_NS_a12, _LN_buFontTx}, 
		{_NS_draw, _LN_start_line_spacing_vertical}, 
		{_NS_x11, _LN_FormulaIndex}, 
		{_NS_rtf, _LN_trpaddr}, 
		{_NS_rtf, _LN_posx}, 
		{_NS_w12, _LN_hash}, 
		{_NS_rtf, _LN_dfrstop}, 
		{_NS_x11, _LN_ColBreaks}, 
		{_NS_a12, _LN_buFont}, 
		{_NS_w12, _LN_permStart}, 
		{_NS_pr, _LN_date_time_decl}, 
		{_NS_w12, _LN_column}, 
		{_NS_x11, _LN_Hidden}, 
		{_NS_rtf, _LN_trpaddt}, 
		{_NS_rtf, _LN_posxc}, 
		{_NS_draw, _LN_start_glue_point}, 
		{_NS_rtf, _LN_dfrxst}, 
		{_NS_a12, _LN_buBlip}, 
		{_NS_x11, _LN_ColBreak}, 
		{_NS_w12, _LN_permEnd}, 
		{_NS_pr, _LN_source}, 
		{_NS_rtf, _LN_trpat}, 
		{_NS_rtf, _LN_posxi}, 
		{_NS_draw, _LN_end_line_spacing_horizontal}, 
		{_NS_w12, _LN_uniqueTag}, 
		{_NS_x11, _LN_HideDetail}, 
		{_NS_draw, _LN_end_shape}, 
		{_NS_pr, _LN_settings}, 
		{_NS_x11, _LN_Column}, 
		{_NS_rtf, _LN_dghorigin}, 
		{_NS_a12, _LN_noBullet}, 
		{_NS_rtf, _LN_trqc}, 
		{_NS_rtf, _LN_posxl}, 
		{_NS_w12, _LN_comments}, 
		{_NS_w12, _LN_mappedName}, 
		{_NS_x11, _LN_CachePosition}, 
		{_NS_rtf, _LN_trql}, 
		{_NS_pr, _LN_start_page}, 
		{_NS_rtf, _LN_dghshow}, 
		{_NS_x11, _LN_RowBreaks}, 
		{_NS_rtf, _LN_posxo}, 
		{_NS_draw, _LN_end_glue_point}, 
		{_NS_w12, _LN_footnotes}, 
		{_NS_pr, _LN_show}, 
		{_NS_draw, _LN_end_line_spacing_vertical}, 
		{_NS_a12, _LN_autonumberBullet}, 
		{_NS_rtf, _LN_trqr}, 
		{_NS_x11, _LN_Member}, 
		{_NS_rtf, _LN_posxr}, 
		{_NS_rtf, _LN_dghspace}, 
		{_NS_w12, _LN_dynamicAddress}, 
		{_NS_x11, _LN_RowBreak}, 
		{_NS_draw, _LN_line_skew}, 
		{_NS_w12, _LN_endnotes}, 
		{_NS_rtf, _LN_trrh}, 
		{_NS_pr, _LN_full_screen}, 
		{_NS_rtf, _LN_posy}, 
		{_NS_a12, _LN_charBullet}, 
		{_NS_w12, _LN_udl}, 
		{_NS_draw, _LN_line_distance}, 
		{_NS_x11, _LN_Row}, 
		{_NS_rtf, _LN_dgmargin}, 
		{_NS_x11, _LN_DrilledMember}, 
		{_NS_draw, _LN_caption}, 
		{_NS_w12, _LN_table}, 
		{_NS_w12, _LN_category}, 
		{_NS_rtf, _LN_trshdng}, 
		{_NS_rtf, _LN_posyb}, 
		{_NS_pr, _LN_endless}, 
		{_NS_x11, _LN_ColFirst}, 
		{_NS_a12, _LN_blipBullet}, 
		{_NS_rtf, _LN_dgsnap}, 
		{_NS_w12, _LN_src}, 
		{_NS_draw, _LN_guide_overhang}, 
		{_NS_w12, _LN_types}, 
		{_NS_rtf, _LN_trspdb}, 
		{_NS_rtf, _LN_posyc}, 
		{_NS_pr, _LN_pause}, 
		{_NS_w12, _LN_filter}, 
		{_NS_draw, _LN_caption_point_x}, 
		{_NS_x11, _LN_DataValueEditing}, 
		{_NS_w12, _LN_description}, 
		{_NS_x11, _LN_ColLast}, 
		{_NS_w12, _LN_sort}, 
		{_NS_rtf, _LN_dgvorigin}, 
		{_NS_x11, _LN_Number}, 
		{_NS_rtf, _LN_trspdfb}, 
		{_NS_rtf, _LN_posyil}, 
		{_NS_pr, _LN_show_logo}, 
		{_NS_w12, _LN_guid}, 
		{_NS_draw, _LN_guide_distance}, 
		{_NS_x11, _LN_QTSource}, 
		{_NS_x11, _LN_Text}, 
		{_NS_rtf, _LN_dgvshow}, 
		{_NS_draw, _LN_caption_point_y}, 
		{_NS_w12, _LN_fieldMapData}, 
		{_NS_rtf, _LN_posyin}, 
		{_NS_rtf, _LN_trspdfl}, 
		{_NS_draw, _LN_measure}, 
		{_NS_w12, _LN_docPartPr}, 
		{_NS_x11, _LN_Boolean}, 
		{_NS_pr, _LN_force_manual}, 
		{_NS_draw, _LN_start_guide}, 
		{_NS_x11, _LN_ErrorString}, 
		{_NS_rtf, _LN_dgvspace}, 
		{_NS_rtf, _LN_posyout}, 
		{_NS_w12, _LN_colDelim}, 
		{_NS_rtf, _LN_trspdfr}, 
		{_NS_draw, _LN_control}, 
		{_NS_x11, _LN_Error}, 
		{_NS_w12, _LN_docPartBody}, 
		{_NS_draw, _LN_end_guide}, 
		{_NS_a12, _LN_shadow}, 
		{_NS_x11, _LN_NullString}, 
		{_NS_rtf, _LN_dibitmap}, 
		{_NS_rtf, _LN_posyt}, 
		{_NS_pr, _LN_mouse_visible}, 
		{_NS_rtf, _LN_trspdft}, 
		{_NS_w12, _LN_jdsoType}, 
		{_NS_w12, _LN_docPr}, 
		{_NS_rtf, _LN_dn}, 
		{_NS_x11, _LN_Horizontal}, 
		{_NS_draw, _LN_placing}, 
		{_NS_draw, _LN_page_thumbnail}, 
		{_NS_a12, _LN_outline}, 
		{_NS_rtf, _LN_prcolbl}, 
		{_NS_w12, _LN_fHdr}, 
		{_NS_rtf, _LN_trspdl}, 
		{_NS_w12, _LN_docParts}, 
		{_NS_draw, _LN_page_number}, 
		{_NS_pr, _LN_mouse_as_pen}, 
		{_NS_x11, _LN_PageFieldOrder}, 
		{_NS_x11, _LN_Rotation}, 
		{_NS_rtf, _LN_dntblnsbdb}, 
		{_NS_draw, _LN_parallel}, 
		{_NS_a12, _LN_engrave}, 
		{_NS_draw, _LN_g}, 
		{_NS_rtf, _LN_trspdr}, 
		{_NS_rtf, _LN_printdata}, 
		{_NS_w12, _LN_recipientData}, 
		{_NS_rtf, _LN_do}, 
		{_NS_svg, _LN_y}, 
		{_NS_w12, _LN_ignoreSubtree}, 
		{_NS_x11, _LN_Vertical}, 
		{_NS_a12, _LN_emboss}, 
		{_NS_rtf, _LN_trspdt}, 
		{_NS_x11, _LN_PageFieldWrapCount}, 
		{_NS_rtf, _LN_printim}, 
		{_NS_pr, _LN_start_with_navigator}, 
		{_NS_svg, _LN_x}, 
		{_NS_draw, _LN_measure_align}, 
		{_NS_rtf, _LN_dobxcolumn}, 
		{_NS_w12, _LN_mainDocumentType}, 
		{_NS_a12, _LN_highlight}, 
		{_NS_rtf, _LN_private}, 
		{_NS_svg, _LN_width}, 
		{_NS_w12, _LN_ignoreElements}, 
		{_NS_x11, _LN_ReadingOrder}, 
		{_NS_w12, _LN_body}, 
		{_NS_x11, _LN_DisableDrillDown}, 
		{_NS_rtf, _LN_truncatefontheight}, 
		{_NS_rtf, _LN_dobxmargin}, 
		{_NS_pr, _LN_transition_on_click}, 
		{_NS_x11, _LN_Path}, 
		{_NS_svg, _LN_height}, 
		{_NS_rtf, _LN_propname}, 
		{_NS_a12, _LN_formatting}, 
		{_NS_draw, _LN_measure_vertical_align}, 
		{_NS_w12, _LN_linkToQuery}, 
		{_NS_x11, _LN_Left}, 
		{_NS_rtf, _LN_trwWidthA}, 
		{_NS_a12, _LN_u}, 
		{_NS_w12, _LN_wordDocument}, 
		{_NS_rtf, _LN_dobxpage}, 
		{_NS_draw, _LN_unit}, 
		{_NS_draw, _LN_transform}, 
		{_NS_pr, _LN_stay_on_top}, 
		{_NS_rtf, _LN_proptype}, 
		{_NS_w12, _LN_dataType}, 
		{_NS_x11, _LN_DisableFieldDialog}, 
		{_NS_x11, _LN_Cell}, 
		{_NS_a12, _LN_ascii}, 
		{_NS_rtf, _LN_trwWidthB}, 
		{_NS_pr, _LN_pages}, 
		{_NS_draw, _LN_show_unit}, 
		{_NS_rtf, _LN_dobymargin}, 
		{_NS_svg, _LN_viewBox}, 
		{_NS_w12, _LN_wordTemplate}, 
		{_NS_x11, _LN_Top}, 
		{_NS_a12, _LN_ea}, 
		{_NS_w12, _LN_defaultSQL}, 
		{_NS_chart, _LN_chart}, 
		{_NS_draw, _LN_decimal_places}, 
		{_NS_draw, _LN_class_names}, 
		{_NS_x11, _LN_DisableWizard}, 
		{_NS_rtf, _LN_trwWidth}, 
		{_NS_a12, _LN_cs}, 
		{_NS_chart, _LN_class}, 
		{_NS_pr, _LN_style_name}, 
		{_NS_rtf, _LN_dobypage}, 
		{_NS_x11, _LN_NotInverted}, 
		{_NS_rtf, _LN_ts}, 
		{_NS_pr, _LN_class_names}, 
		{_NS_w12, _LN_glossaryDocument}, 
		{_NS_w12, _LN_connectString}, 
		{_NS_a12, _LN_hAnsi}, 
		{_NS_draw, _LN_caption_type}, 
		{_NS_x11, _LN_Id}, 
		{_NS_x11, _LN_DisplayErrorString}, 
		{_NS_chart, _LN_column_mapping}, 
		{_NS_a12, _LN_sym}, 
		{_NS_w12, _LN_query}, 
		{_NS_rtf, _LN_tsbgbdiag}, 
		{_NS_draw, _LN_text_style_name}, 
		{_NS_x11, _LN_Clipped}, 
		{_NS_a12, _LN_link}, 
		{_NS_draw, _LN_caption_angle_type}, 
		{_NS_chart, _LN_row_mapping}, 
		{_NS_w12, _LN_dataSource}, 
		{_NS_rtf, _LN_tsbgcross}, 
		{_NS_draw, _LN_z_index}, 
		{_NS_x11, _LN_NoPreserveFormatting}, 
		{_NS_chart, _LN_style_name}, 
		{_NS_x11, _LN_Descending}, 
		{_NS_chart, _LN_title}, 
		{_NS_draw, _LN_caption_angle}, 
		{_NS_rtf, _LN_tsbgdcross}, 
		{_NS_w12, _LN_headerSource}, 
		{_NS_x11, _LN_FillDown}, 
		{_NS_x11, _LN_HasNoAutoFormat}, 
		{_NS_table, _LN_end_cell_address}, 
		{_NS_table, _LN_cell_range}, 
		{_NS_wa11, _LN_sect}, 
		{_NS_draw, _LN_caption_gap}, 
		{_NS_rtf, _LN_tsbgdkbdiag}, 
		{_NS_table, _LN_end_x}, 
		{_NS_x11, _LN_RowNumbers}, 
		{_NS_chart, _LN_subtitle}, 
		{_NS_x11, _LN_SortKey}, 
		{_NS_w12, _LN_doNotSuppressBlankLines}, 
		{_NS_wa11, _LN_sub_section}, 
		{_NS_table, _LN_end_y}, 
		{_NS_chart, _LN_footer}, 
		{_NS_rtf, _LN_tsbgdkcross}, 
		{_NS_x11, _LN_User}, 
		{_NS_x11, _LN_SubtotalHiddenPageItems}, 
		{_NS_a12, _LN_acc}, 
		{_NS_draw, _LN_caption_escape_direction}, 
		{_NS_chart, _LN_legend}, 
		{_NS_w12, _LN_destination}, 
		{_NS_a12, _LN_bar}, 
		{_NS_x11, _LN_Condition}, 
		{_NS_rtf, _LN_tsbgdkdcross}, 
		{_NS_wa11, _LN_allowEmptyCollapse}, 
		{_NS_table, _LN_table_background}, 
		{_NS_x11, _LN_NoAutoFormatWidth}, 
		{_NS_wa11, _LN_font}, 
		{_NS_a12, _LN_box}, 
		{_NS_draw, _LN_caption_escape}, 
		{_NS_chart, _LN_legend_position}, 
		{_NS_wa11, _LN_sym}, 
		{_NS_x11, _LN_Qualifier}, 
		{_NS_a12, _LN_d}, 
		{_NS_w12, _LN_addressFieldName}, 
		{_NS_rtf, _LN_tsbgdkfdiag}, 
		{_NS_text, _LN_anchor_type}, 
		{_NS_wa11, _LN_top}, 
		{_NS_x11, _LN_NoRowGrand}, 
		{_NS_x11, _LN_Value1}, 
		{_NS_wa11, _LN_left}, 
		{_NS_a12, _LN_eqArray}, 
		{_NS_chart, _LN_legend_align}, 
		{_NS_w12, _LN_mailSubject}, 
		{_NS_rtf, _LN_tsbgdkhor}, 
		{_NS_wa11, _LN_bottom}, 
		{_NS_draw, _LN_caption_line_length}, 
		{_NS_a12, _LN_f}, 
		{_NS_x11, _LN_Value2}, 
		{_NS_x11, _LN_NoColumnGrand}, 
		{_NS_wa11, _LN_right}, 
		{_NS_text, _LN_anchor_page_number}, 
		{_NS_x11, _LN_Format}, 
		{_NS_a12, _LN_func}, 
		{_NS_wa11, _LN_between}, 
		{_NS_rtf, _LN_tsbgdkvert}, 
		{_NS_style, _LN_legend_expansion}, 
		{_NS_w12, _LN_mailAsAttachment}, 
		{_NS_wa11, _LN_bar}, 
		{_NS_draw, _LN_glue_point}, 
		{_NS_x11, _LN_FileName}, 
		{_NS_wa11, _LN_width}, 
		{_NS_a12, _LN_groupChr}, 
		{_NS_draw, _LN_caption_fit_line_length}, 
		{_NS_rtf, _LN_tsbgfdiag}, 
		{_NS_x11, _LN_NoDisplayNullString}, 
		{_NS_wa11, _LN_height}, 
		{_NS_draw, _LN_align}, 
		{_NS_w12, _LN_viewMergedData}, 
		{_NS_a12, _LN_lLow}, 
		{_NS_wa11, _LN_jc}, 
		{_NS_x11, _LN_Reference}, 
		{_NS_rtf, _LN_tsbghoriz}, 
		{_NS_x11, _LN_Type}, 
		{_NS_a12, _LN_lUp}, 
		{_NS_style, _LN_legend_expansion_aspect_ratio}, 
		{_NS_dr3d, _LN_horizontal_segments}, 
		{_NS_x11, _LN_NoPrintRepeatItems}, 
		{_NS_w12, _LN_activeRecord}, 
		{_NS_draw, _LN_escape_direction}, 
		{_NS_a12, _LN_m}, 
		{_NS_wa11, _LN_vertFromText}, 
		{_NS_rtf, _LN_tsbgvert}, 
		{_NS_chart, _LN_plot_area}, 
		{_NS_x11, _LN_CellRangeList}, 
		{_NS_x11, _LN_PivotView}, 
		{_NS_draw, _LN_frame}, 
		{_NS_a12, _LN_nary}, 
		{_NS_rtf, _LN_tsbrdrb}, 
		{_NS_w12, _LN_checkErrors}, 
		{_NS_dr3d, _LN_vertical_segments}, 
		{_NS_wa11, _LN_horizFromText}, 
		{_NS_a12, _LN_phant}, 
		{_NS_x11, _LN_UseBlank}, 
		{_NS_style, _LN_rel_width}, 
		{_NS_w12, _LN_odso}, 
		{_NS_rtf, _LN_tsbrdrdgl}, 
		{_NS_a12, _LN_rad}, 
		{_NS_wa11, _LN_borders}, 
		{_NS_x11, _LN_Min}, 
		{_NS_x11, _LN_AutoFormatAlignment}, 
		{_NS_chart, _LN_data_source_has_labels}, 
		{_NS_dr3d, _LN_edge_rounding}, 
		{_NS_wa11, _LN_margin_left}, 
		{_NS_style, _LN_rel_height}, 
		{_NS_a12, _LN_sPre}, 
		{_NS_x11, _LN_Max}, 
		{_NS_rtf, _LN_tsbrdrdgr}, 
		{_NS_chart, _LN_wall}, 
		{_NS_wa11, _LN_margin_right}, 
		{_NS_w12, _LN_origWordTableRules}, 
		{_NS_x11, _LN_Value}, 
		{_NS_a12, _LN_sSub}, 
		{_NS_x11, _LN_AutoFormatBorder}, 
		{_NS_draw, _LN_copy_of}, 
		{_NS_wa11, _LN_apo}, 
		{_NS_chart, _LN_floor}, 
		{_NS_dr3d, _LN_edge_rounding_mode}, 
		{_NS_rtf, _LN_tsbrdrh}, 
		{_NS_wa11, _LN_shd}, 
		{_NS_x11, _LN_ComboHide}, 
		{_NS_a12, _LN_sSubSup}, 
		{_NS_chart, _LN_axis}, 
		{_NS_w12, _LN_wpJustification}, 
		{_NS_draw, _LN_text_box}, 
		{_NS_rtf, _LN_tsbrdrl}, 
		{_NS_dr3d, _LN_back_scale}, 
		{_NS_x11, _LN_AutoFormatFont}, 
		{_NS_wa11, _LN_pBdrGroup}, 
		{_NS_a12, _LN_sSup}, 
		{_NS_x11, _LN_IMEMode}, 
		{_NS_chart, _LN_dimension}, 
		{_NS_dr3d, _LN_depth}, 
		{_NS_wa11, _LN_t}, 
		{_NS_rtf, _LN_tsbrdrr}, 
		{_NS_a12, _LN_r}, 
		{_NS_w12, _LN_noTabHangInd}, 
		{_NS_draw, _LN_chain_next_name}, 
		{_NS_chart, _LN_name}, 
		{_NS_x11, _LN_AutoFormatName}, 
		{_NS_x11, _LN_InputHide}, 
		{_NS_a12, _LN_chr}, 
		{_NS_wa11, _LN_uiName}, 
		{_NS_rtf, _LN_tsbrdrt}, 
		{_NS_chart, _LN_grid}, 
		{_NS_w12, _LN_noLeading}, 
		{_NS_fo, _LN_min_height}, 
		{_NS_dr3d, _LN_backface_culling}, 
		{_NS_a12, _LN_accPr}, 
		{_NS_x11, _LN_InputTitle}, 
		{_NS_rtf, _LN_tsbrdrv}, 
		{_NS_dr3d, _LN_end_angle}, 
		{_NS_chart, _LN_series}, 
		{_NS_x11, _LN_AutoFormatNumber}, 
		{_NS_a12, _LN_e}, 
		{_NS_w12, _LN_spaceForUL}, 
		{_NS_fo, _LN_min_width}, 
		{_NS_a12, _LN_barPr}, 
		{_NS_x11, _LN_InputMessage}, 
		{_NS_dr3d, _LN_close_front}, 
		{_NS_rtf, _LN_tscbandhorzeven}, 
		{_NS_x11, _LN_AutoFormatPattern}, 
		{_NS_fo, _LN_max_height}, 
		{_NS_w12, _LN_noColumnBalance}, 
		{_NS_a12, _LN_begChr}, 
		{_NS_chart, _LN_values_cell_range_address}, 
		{_NS_x11, _LN_ErrorHide}, 
		{_NS_dr3d, _LN_close_back}, 
		{_NS_rtf, _LN_tscbandhorzodd}, 
		{_NS_fo, _LN_max_width}, 
		{_NS_a12, _LN_sepChr}, 
		{_NS_x11, _LN_GrandTotalString}, 
		{_NS_draw, _LN_image}, 
		{_NS_x11, _LN_ErrorStyle}, 
		{_NS_dr3d, _LN_normals_kind}, 
		{_NS_chart, _LN_label_cell_address}, 
		{_NS_a12, _LN_endChr}, 
		{_NS_rtf, _LN_tscbandsh}, 
		{_NS_w12, _LN_balanceSingleByteDoubleByteWidth}, 
		{_NS_office, _LN_binary_data}, 
		{_NS_a12, _LN_grow}, 
		{_NS_x11, _LN_ErrorMessage}, 
		{_NS_x11, _LN_PrintSetTitles}, 
		{_NS_rtf, _LN_tscbandsv}, 
		{_NS_draw, _LN_filter_name}, 
		{_NS_chart, _LN_attached_axis}, 
		{_NS_dr3d, _LN_normals_direction}, 
		{_NS_a12, _LN_shp}, 
		{_NS_x11, _LN_ErrorTitle}, 
		{_NS_x11, _LN_MergeLabels}, 
		{_NS_draw, _LN_object}, 
		{_NS_chart, _LN_domain}, 
		{_NS_a12, _LN_dPr}, 
		{_NS_w12, _LN_transparentMetafiles}, 
		{_NS_x11, _LN_Formula}, 
		{_NS_rtf, _LN_tscbandverteven}, 
		{_NS_a12, _LN_baseJc}, 
		{_NS_draw, _LN_object_ole}, 
		{_NS_chart, _LN_categories}, 
		{_NS_x11, _LN_OleLink}, 
		{_NS_x11, _LN_PageFieldStyle}, 
		{_NS_dr3d, _LN_texture_generation_mode_x}, 
		{_NS_w12, _LN_noExtraLineSpacing}, 
		{_NS_rtf, _LN_tscbandvertodd}, 
		{_NS_a12, _LN_maxDist}, 
		{_NS_x11, _LN_PLExport}, 
		{_NS_chart, _LN_data_point}, 
		{_NS_x11, _LN_WantAdvise}, 
		{_NS_a12, _LN_objDist}, 
		{_NS_rtf, _LN_tscellcbpat}, 
		{_NS_draw, _LN_notify_on_update_of_ranges}, 
		{_NS_chart, _LN_repeated}, 
		{_NS_dr3d, _LN_texture_generation_mode_y}, 
		{_NS_x11, _LN_WantPict}, 
		{_NS_w12, _LN_doNotLeaveBackslashAlone}, 
		{_NS_a12, _LN_eqArrPr}, 
		{_NS_x11, _LN_ImmediateItemsOnDrop}, 
		{_NS_x11, _LN_Cf}, 
		{_NS_draw, _LN_class_id}, 
		{_NS_rtf, _LN_tscellcfpat}, 
		{_NS_a12, _LN_type}, 
		{_NS_chart, _LN_mean_value}, 
		{_NS_dr3d, _LN_texture_kind}, 
		{_NS_xldr12, _LN_cdrElemPr}, 
		{_NS_w12, _LN_ulTrailSpace}, 
		{_NS_draw, _LN_applet}, 
		{_NS_x11, _LN_FieldType}, 
		{_NS_a12, _LN_numJc}, 
		{_NS_xldr12, _LN_spPr}, 
		{_NS_rtf, _LN_tscellpaddb}, 
		{_NS_wb12, _LN_userBookViews}, 
		{_NS_xldr12, _LN_style}, 
		{_NS_draw, _LN_code}, 
		{_NS_chart, _LN_error_indicator}, 
		{_NS_dr3d, _LN_texture_filter}, 
		{_NS_a12, _LN_denJc}, 
		{_NS_x11, _LN_ShowPageMultipleItemLabel}, 
		{_NS_xldr12, _LN_txBody}, 
		{_NS_x11, _LN_FieldStart}, 
		{_NS_wb12, _LN_sheets}, 
		{_NS_rtf, _LN_tscellpaddfb}, 
		{_NS_a12, _LN_fPr}, 
		{_NS_xldr12, _LN_xfrm}, 
		{_NS_w12, _LN_doNotExpandShiftReturn}, 
		{_NS_draw, _LN_archive}, 
		{_NS_x11, _LN_SmallGrid}, 
		{_NS_wb12, _LN_wbPr}, 
		{_NS_dr3d, _LN_texture_mode}, 
		{_NS_xldr12, _LN_stCxn}, 
		{_NS_x11, _LN_Missing}, 
		{_NS_chart, _LN_regression_curve}, 
		{_NS_a12, _LN_num}, 
		{_NS_xldr12, _LN_endCxn}, 
		{_NS_rtf, _LN_tscellpaddfl}, 
		{_NS_draw, _LN_may_script}, 
		{_NS_wb12, _LN_webPub}, 
		{_NS_x11, _LN_SeqNum}, 
		{_NS_a12, _LN_den}, 
		{_NS_xldr12, _LN_graphic}, 
		{_NS_x11, _LN_TableStyle}, 
		{_NS_wb12, _LN_webPubItems}, 
		{_NS_w12, _LN_spacingInWholePoints}, 
		{_NS_dr3d, _LN_emissive_color}, 
		{_NS_xldr12, _LN_gspPr}, 
		{_NS_draw, _LN_plugin}, 
		{_NS_chart, _LN_stock_gain_marker}, 
		{_NS_x11, _LN_Tag}, 
		{_NS_a12, _LN_fName}, 
		{_NS_xldr12, _LN_sp}, 
		{_NS_x11, _LN_Expanded}, 
		{_NS_rtf, _LN_tscellpaddfr}, 
		{_NS_wb12, _LN_smartTag}, 
		{_NS_xldr12, _LN_gsp}, 
		{_NS_draw, _LN_mime_type}, 
		{_NS_dr3d, _LN_specular_color}, 
		{_NS_xldr12, _LN_graphicFrame}, 
		{_NS_x11, _LN_RowLast}, 
		{_NS_w12, _LN_lineWrapLikeWord6}, 
		{_NS_a12, _LN_groupChrPr}, 
		{_NS_x11, _LN_VacatedStyle}, 
		{_NS_draw, _LN_param}, 
		{_NS_rtf, _LN_tscellpaddft}, 
		{_NS_chart, _LN_stock_loss_marker}, 
		{_NS_xldr12, _LN_cxnsp}, 
		{_NS_a12, _LN_lim}, 
		{_NS_wb12, _LN_smartTagTypes}, 
		{_NS_x11, _LN_Pane}, 
		{_NS_draw, _LN_value}, 
		{_NS_dr3d, _LN_shininess}, 
		{_NS_xldr12, _LN_col}, 
		{_NS_a12, _LN_count}, 
		{_NS_dr3d, _LN_shadow}, 
		{_NS_rtf, _LN_tscellpaddl}, 
		{_NS_xldr12, _LN_colOff}, 
		{_NS_x11, _LN_ActiveRow}, 
		{_NS_x11, _LN_HideTotalsAnnotation}, 
		{_NS_chart, _LN_stock_range_line}, 
		{_NS_a12, _LN_mcJc}, 
		{_NS_wb12, _LN_fileRecovery}, 
		{_NS_w12, _LN_printBodyTextBeforeHeader}, 
		{_NS_draw, _LN_floating_frame}, 
		{_NS_xldr12, _LN_row}, 
		{_NS_office, _LN_forms}, 
		{_NS_rtf, _LN_tscellpaddr}, 
		{_NS_a12, _LN_mcPr}, 
		{_NS_style, _LN_horizontal_pos}, 
		{_NS_wb12, _LN_calcPr}, 
		{_NS_x11, _LN_ActiveCol}, 
		{_NS_xldr12, _LN_rowOff}, 
		{_NS_draw, _LN_frame_name}, 
		{_NS_w12, _LN_printColBlack}, 
		{_NS_a12, _LN_mc}, 
		{_NS_xldr12, _LN_from}, 
		{_NS_wb12, _LN_extRefs}, 
		{_NS_x11, _LN_ShowCellBackgroundFromOLAP}, 
		{_NS_rtf, _LN_tscellpaddt}, 
		{_NS_form, _LN_automatic_focus}, 
		{_NS_style, _LN_horizontal_rel}, 
		{_NS_xldr12, _LN_to}, 
		{_NS_x11, _LN_RangeSelection}, 
		{_NS_a12, _LN_plcHide}, 
		{_NS_w12, _LN_wpSpaceWidth}, 
		{_NS_draw, _LN_contour_polygon}, 
		{_NS_rtf, _LN_tscellpct}, 
		{_NS_wb12, _LN_namedRanges}, 
		{_NS_a12, _LN_mcs}, 
		{_NS_xldr12, _LN_clientData}, 
		{_NS_x11, _LN_Selection}, 
		{_NS_style, _LN_vertical_pos}, 
		{_NS_form, _LN_apply_design_mode}, 
		{_NS_x11, _LN_NoViewCalculatedMembers}, 
		{_NS_a12, _LN_mrJc}, 
		{_NS_draw, _LN_contour_path}, 
		{_NS_w12, _LN_showBreaksInFrames}, 
		{_NS_rtf, _LN_tscellwidth}, 
		{_NS_wb12, _LN_pivotCaches}, 
		{_NS_xldr12, _LN_extents}, 
		{_NS_x11, _LN_ExcelType}, 
		{_NS_form, _LN_form}, 
		{_NS_a12, _LN_mrPr}, 
		{_NS_style, _LN_vertical_rel}, 
		{_NS_xldr12, _LN_pos}, 
		{_NS_x11, _LN_VisualTotals}, 
		{_NS_wb12, _LN_docRoute}, 
		{_NS_a12, _LN_mPr}, 
		{_NS_draw, _LN_recreate_on_edit}, 
		{_NS_w12, _LN_subFontBySize}, 
		{_NS_rtf, _LN_tscellwidthfts}, 
		{_NS_style, _LN_editable}, 
		{_NS_office, _LN_target_frame}, 
		{_NS_x11, _LN_ParameterType}, 
		{_NS_a12, _LN_mr}, 
		{_NS_xldr12, _LN_twoCellAnchor}, 
		{_NS_x11, _LN_Location}, 
		{_NS_wb12, _LN_oleSize}, 
		{_NS_svg, _LN_desc}, 
		{_NS_style, _LN_wrap}, 
		{_NS_form, _LN_method}, 
		{_NS_draw, _LN_a}, 
		{_NS_rtf, _LN_tscfirstcol}, 
		{_NS_a12, _LN_limLoc}, 
		{_NS_w12, _LN_suppressBottomSpacing}, 
		{_NS_xldr12, _LN_oneCellAnchor}, 
		{_NS_x11, _LN_PLPivotField}, 
		{_NS_form, _LN_enctype}, 
		{_NS_wb12, _LN_bookProtection}, 
		{_NS_office, _LN_server_map}, 
		{_NS_a12, _LN_subHide}, 
		{_NS_rtf, _LN_tscfirstrow}, 
		{_NS_wb12, _LN_image}, 
		{_NS_style, _LN_wrap_dynamic_treshold}, 
		{_NS_x11, _LN_PLTotal}, 
		{_NS_wb12, _LN_fsb}, 
		{_NS_xldr12, _LN_absoluteAnchor}, 
		{_NS_a12, _LN_supHide}, 
		{_NS_form, _LN_allow_deletes}, 
		{_NS_w12, _LN_suppressTopSpacing}, 
		{_NS_draw, _LN_image_map}, 
		{_NS_rtf, _LN_tsclastcol}, 
		{_NS_x11, _LN_DataMember}, 
		{_NS_wb12, _LN_bookView}, 
		{_NS_a12, _LN_naryPr}, 
		{_NS_form, _LN_allow_inserts}, 
		{_NS_rtf, _LN_tsclastrow}, 
		{_NS_style, _LN_number_wrapped_paragraphs}, 
		{_NS_draw, _LN_area_rectangle}, 
		{_NS_a12, _LN_sub}, 
		{_NS_w12, _LN_suppressTopSpacingMac5}, 
		{_NS_x11, _LN_DataAxisEmpty}, 
		{_NS_wb12, _LN_userBookView}, 
		{_NS_a12, _LN_sup}, 
		{_NS_wb12, _LN_sheet}, 
		{_NS_rtf, _LN_tscnecell}, 
		{_NS_draw, _LN_area_circle}, 
		{_NS_form, _LN_allow_updates}, 
		{_NS_style, _LN_wrap_contour}, 
		{_NS_a12, _LN_width}, 
		{_NS_x11, _LN_DisplayFieldList}, 
		{_NS_a12, _LN_asc}, 
		{_NS_rtf, _LN_tscnwcell}, 
		{_NS_w12, _LN_suppressTopSpacingWP}, 
		{_NS_wb12, _LN_smartTagType}, 
		{_NS_draw, _LN_area_polygon}, 
		{_NS_form, _LN_apply_filter}, 
		{_NS_a12, _LN_desc}, 
		{_NS_wb12, _LN_namedRange}, 
		{_NS_style, _LN_wrap_contour_mode}, 
		{_NS_x11, _LN_NoAutoFit}, 
		{_NS_rtf, _LN_tscsecell}, 
		{_NS_wb12, _LN_name}, 
		{_NS_draw, _LN_nohref}, 
		{_NS_a12, _LN_phantPr}, 
		{_NS_x11, _LN_Height}, 
		{_NS_wb12, _LN_value}, 
		{_NS_form, _LN_command_type}, 
		{_NS_ec12, _LN_extConns}, 
		{_NS_style, _LN_run_through}, 
		{_NS_x11, _LN_Width}, 
		{_NS_w12, _LN_suppressSpBfAfterPgBrk}, 
		{_NS_wb12, _LN_comment}, 
		{_NS_dr3d, _LN_scene}, 
		{_NS_rtf, _LN_tscswcell}, 
		{_NS_a12, _LN_degHide}, 
		{_NS_x11, _LN_MaxHeight}, 
		{_NS_form, _LN_command}, 
		{_NS_ec12, _LN_extConn}, 
		{_NS_dr3d, _LN_vrp}, 
		{_NS_x11, _LN_MaxWidth}, 
		{_NS_rtf, _LN_tsd}, 
		{_NS_a12, _LN_radPr}, 
		{_NS_wb12, _LN_customMenu}, 
		{_NS_style, _LN_flow_with_text}, 
		{_NS_dr3d, _LN_vpn}, 
		{_NS_wb12, _LN_description}, 
		{_NS_ec12, _LN_dbProps}, 
		{_NS_form, _LN_datasource}, 
		{_NS_w12, _LN_swapBordersFacingPages}, 
		{_NS_rtf, _LN_tsnowrap}, 
		{_NS_a12, _LN_deg}, 
		{_NS_x11, _LN_MemberExpand}, 
		{_NS_dr3d, _LN_vup}, 
		{_NS_wb12, _LN_help}, 
		{_NS_x11, _LN_ConnectionString}, 
		{_NS_ec12, _LN_olapProps}, 
		{_NS_rtf, _LN_tsrowd}, 
		{_NS_style, _LN_overflow_behavior}, 
		{_NS_form, _LN_master_fields}, 
		{_NS_w12, _LN_convMailMergeEsc}, 
		{_NS_dr3d, _LN_projection}, 
		{_NS_wb12, _LN_statusBar}, 
		{_NS_style, _LN_mirror}, 
	};

	static const QNameTuple prefixTable[] = {
		{_LN_anium, _NS_anium},
		{_LN_math, _NS_math},
		{_LN_w10, _NS_w10},
		{_LN_chart, _NS_chart},
		{_LN_sc12, _NS_sc12},
		{_LN_odc12, _NS_odc12},
		{_LN_c11, _NS_c11},
		{_LN_rng, _NS_rng},
		{_LN_meta, _NS_meta},
		{_LN_udcs, _NS_udcs},
		{_LN_xlink, _NS_xlink},
		{_LN_style, _NS_style},
		{_LN_fo, _NS_fo},
		{_LN_xforms, _NS_xforms},
		{_LN_draw, _NS_draw},
		{_LN_prj12, _NS_prj12},
		{_LN_mdc12, _NS_mdc12},
		{_LN_table, _NS_table},
		{_LN_rs11, _NS_rs11},
		{_LN_sxr12, _NS_sxr12},
		{_LN_xldr12, _NS_xldr12},
		{_LN_m12, _NS_m12},
		{_LN_number, _NS_number},
		{_LN_p12, _NS_p12},
		{_LN_voldeps12, _NS_voldeps12},
		{_LN_pt12, _NS_pt12},
		{_LN_xml, _NS_xml},
		{_LN_w11, _NS_w11},
		{_LN_sl12, _NS_sl12},
		{_LN_s12, _NS_s12},
		{_LN_udc, _NS_udc},
		{_LN_dr3d, _NS_dr3d},
		{_LN_rdf, _NS_rdf},
		{_LN_sb12, _NS_sb12},
		{_LN_wb12, _NS_wb12},
		{_LN_RowsetSchema, _NS_RowsetSchema},
		{_LN_ec12, _NS_ec12},
		{_LN_qt12, _NS_qt12},
		{_LN_o12, _NS_o12},
		{_LN_s11, _NS_s11},
		{_LN_x11, _NS_x11},
		{_LN_ss11, _NS_ss11},
		{_LN_rtf, _NS_rtf},
		{_LN_smil, _NS_smil},
		{_LN_w12, _NS_w12},
		{_LN_rtfs, _NS_rtfs},
		{_LN_x2, _NS_x2},
		{_LN_ip12, _NS_ip12},
		{_LN_rel12, _NS_rel12},
		{QName::EMPTY_STRING, QName::EMPTY_STRING},
		{_LN_config, _NS_config},
		{_LN_text, _NS_text},
		{_LN_tbl12, _NS_tbl12},
		{_LN_pr, _NS_pr},
		{_LN_udcxf11, _NS_udcxf11},
		{_LN_office, _NS_office},
		{_LN_md12, _NS_md12},
		{_LN_pp12, _NS_pp12},
		{_LN_v11, _NS_v11},
		{_LN_sst12, _NS_sst12},
		{_LN_swb12, _NS_swb12},
		{_LN_pic12, _NS_pic12},
		{_LN_vml, _NS_vml},
		{_LN_ax12, _NS_ax12},
		{_LN_ws12, _NS_ws12},
		{_LN_wddr12, _NS_wddr12},
		{_LN_a12, _NS_a12},
		{_LN_svg, _NS_svg},
		{_LN_on12, _NS_on12},
		{_LN_form, _NS_form},
		{_LN_script, _NS_script},
		{_LN_aml12, _NS_aml12},
		{_LN_dc, _NS_dc},
		{_LN_wa11, _NS_wa11},
		{_LN_swbu12, _NS_swbu12},
		{_LN_c12, _NS_c12},
		{NULL, NULL}
	};


	struct HashSetHelper
	{
		inline size_t operator()(const QNameTuple *qn) const
		{
			return (size_t)qn->localName;
		}
		inline bool operator()(const QNameTuple *qn1, const QNameTuple *qn2) const
		{
			return (qn1->localName == qn2->localName) && (qn1->namespaceUri==qn2->namespaceUri);
		}

		inline size_t operator()(const sal_Char *str) const
		{
			return rtl_str_hashCode(str);
		}

		inline bool operator()(const sal_Char *str1, const sal_Char *str2) const
		{
			return rtl_str_compare(str1, str2)==0;
		}
	};


	struct StrPoolPage
	{
	public:
		StrPoolPage *previousPage;
		size_t bytesUsed;
		sal_Char data[10*1024];
		StrPoolPage(struct StrPoolPage *previousPage_) : previousPage(previousPage_), bytesUsed(0) 
		{ 
		};
	};


	class StrPool
	{
	private:
		StrPoolPage *lastPage;
		std::hash_set<const sal_Char *, HashSetHelper, HashSetHelper> refs;

		const PoolRef_t addStringToPool(const sal_Char *s) {
			for(;;) {
				const sal_Char *i=s;
				const sal_Char *id=lastPage->data+lastPage->bytesUsed;
				while(lastPage->bytesUsed<sizeof(lastPage->data) && *i!=(const sal_Char)0)
				{
					lastPage->data[lastPage->bytesUsed++]=*(i++);
				}
				if (lastPage->bytesUsed<sizeof(lastPage->data))
				{
					lastPage->data[lastPage->bytesUsed++]=0;
					assert(id>=lastPage->data && id < lastPage->data+sizeof(lastPage->data));
					return id;
				}
				else
				{
					lastPage=new StrPoolPage(lastPage);
				}
			}
		}


	public:
		StrPool() {
			lastPage = new StrPoolPage(NULL);
		}
		~StrPool() {
			StrPoolPage *page = lastPage;
			int pages=0;
			while(page!=NULL)
			{
				StrPoolPage *page_=page;
				page=page->previousPage;
				delete page_;
				pages++;
			}
		}

		inline void insertRef(PoolRef_t str)
		{
				refs.insert(str);
		}

		const PoolRef_t insert(const sal_Char *str)
		{
			std::hash_set<const sal_Char *, HashSetHelper, HashSetHelper>::iterator i;
			if ((i=refs.find(str))==refs.end())
			{
				const sal_Char *ref=addStringToPool(str);
				refs.insert(ref);
				return ref;
			}
			else return *i;
		}

	};


	struct HashMapHelper
	{
		inline size_t operator()(PoolRef_t ref) const
		{
			return (size_t)ref;
		}

		inline bool operator()(PoolRef_t ref1, PoolRef_t ref2) const
		{
			return (ref1==ref2); 
		}
	};

	const sal_Char* splitQName(const sal_Char *qname, sal_Char *prefix, int maxPrefixLen, int *prefixLen)
		{
			const sal_Char *_prefix=qname;
			int _prefix_len;
			const sal_Char *_local_name=qname;
			while(*_local_name!=0 && *_local_name!=':') _local_name++;
			if (*_local_name==':')
			{
				_prefix_len=(_local_name-qname);		
				_local_name++;
				if (_prefix_len+1<maxPrefixLen)
				{
					memcpy(prefix, _prefix, _prefix_len*sizeof(sal_Char));
					prefix[_prefix_len]=0;
				}
				else
				{
					throw std::logic_error(std::string("prefix too long!"));
				}
			}
			else
			{
				prefix[0]=0;
				_prefix_len=0;
				_local_name=qname;
			}
			*prefixLen=_prefix_len;
			return _local_name;
		}


	class QNameTokenizerContextImpl : public QNameTokenizerContext
	{
	private:
		StrPool &strPool;
		typedef std::pair< int, PoolRef_t > NSStackItem_t;
		typedef std::stack< NSStackItem_t >  NSStack_t;
		typedef std::hash_map< PoolRef_t, NSStack_t, HashSetHelper, HashMapHelper > NSMap_t;
		NSMap_t context;
		int level;

	public:
		QNameTokenizerContextImpl(StrPool &strPool_) :
		  strPool(strPool_), level(0)
		{
			_addBinding(QName::EMPTY_STRING, QName::EMPTY_STRING);
		}

        virtual ~QNameTokenizerContextImpl() {}

		void _addBinding(PoolRef_t prefix, PoolRef_t namespaceUri)
		{
			NSStack_t &binding=context[prefix];
			while(!binding.empty() && binding.top().first>=level) binding.pop();
			binding.push(NSStackItem_t(level, namespaceUri));
		}

		PoolRef_t resolvePrefix(const sal_Char *prefix)
		{
			PoolRef_t prefixref=strPool.insert(prefix);
			NSStack_t &binding=context[prefixref];
			while(!binding.empty() && binding.top().first>level) binding.pop();
			if (binding.empty())
			{
				throw std::logic_error(std::string("prefix not bound!"));				
			}
			else
			{
				return binding.top().second;
			}
		}

		void addBinding(const sal_Char *attrName, const sal_Char *attrValue)
		{
			sal_Char _prefix[50];
			int prefixLen;
			const sal_Char *_localName=splitQName(attrName, _prefix, sizeof(_prefix), &prefixLen);
			if (prefixLen==5 && _prefix[0]=='x' && _prefix[1]=='m' && _prefix[2]=='l' && _prefix[3]=='n' && _prefix[4]=='s')
			{
				PoolRef_t prefix=strPool.insert(_localName);
				PoolRef_t namespaceUri=strPool.insert(attrValue);
				_addBinding(prefix, namespaceUri);
			}
			else if (prefixLen==0 && _localName[0]=='x' && _localName[1]=='m' && _localName[2]=='l' && _localName[3]=='n' && _localName[4]=='s')
			{
				PoolRef_t prefix=strPool.insert(_prefix);
				PoolRef_t namespaceUri=strPool.insert(attrValue);
				_addBinding(prefix, namespaceUri);
			}
		}

		virtual void enterScope()
		{
			level++;
		}

		virtual void leaveScope()
		{
			level--;
		}
	};


	class QNameSerializerContextImpl : public QNameSerializerContext
	{
		StrPool &strPool;
		std::hash_map< PoolRef_t, PoolRef_t, HashSetHelper, HashMapHelper > map;
		int counter;
	public:
		QNameSerializerContextImpl(StrPool &strPool_) :
		  strPool(strPool_), counter(0)
		{
			const QNameTuple *t=prefixTable;
			while(t->namespaceUri!=NULL)
			{
				map[t->localName]=t->namespaceUri; 
				t++;
			}
		}
        virtual ~QNameSerializerContextImpl() {}
        

		virtual PoolRef_t addPrefix(const sal_Char *prefix, const sal_Char *namespaceUri)
		{
			PoolRef_t ns=strPool.insert(namespaceUri);
			PoolRef_t p=strPool.insert(prefix);
			map[ns]=p;
			return ns;
		}
		virtual PoolRef_t getPrefix(PoolRef_t namespaceUri)
		{
			PoolRef_t ns=strPool.insert(namespaceUri);
			std::hash_map< PoolRef_t, PoolRef_t, HashSetHelper, HashMapHelper >::iterator i=map.find(ns);
			if (i!=map.end())
			{
				PoolRef_t p=i->second;
				return p;
			}
			else
			{
				char buf[100];
				sprintf(buf, "_p%i", counter++);
				PoolRef_t p=strPool.insert(buf);
				map[ns]=p;
				return p;
			}
		}
	};


	class QNameTokenizerImpl : public QNameTokenizer
	{
	private:
		std::hash_set<const QNameTuple*, HashSetHelper, HashSetHelper> qNameDict;
		StrPool strPool;
		int qnames;

	public:
		static QNameTokenizerImpl tokenizer_;

	private:

		inline QName_t addPoolToken(PoolRef_t namespaceUriPoolToken, PoolRef_t localPoolNameToken)
		{
			static QNameTuple qt;
			QNameTuple *qnt=(qnames<QNAMETABLESIZE?&qNameTable[qnames]:&qt);
			qnt->localName=localPoolNameToken;
			qnt->namespaceUri=namespaceUriPoolToken;

			std::pair< std::hash_set<const QNameTuple*, HashSetHelper, HashSetHelper>::iterator, bool> ret=qNameDict.insert(qnt);
			if (ret.second)
			{
				if (qnames<QNAMETABLESIZE)
					this->qnames++;
				else
					throw std::logic_error(std::string("too many qnames!"));
			}
			int pos=(*ret.first)-qNameTable;
			return (QName_t)pos;
		}

	public:

		QNameTokenizerImpl()
		{
			strPool.insertRef(QName::EMPTY_STRING);
			qnames=QNAME_SIZE_STATIC;
			for(int i=0;i<qnames;i++)
			{
				strPool.insertRef(qNameTable[i].localName);
				strPool.insertRef(qNameTable[i].namespaceUri);
				qNameDict.insert(&qNameTable[i]);
			}
			const QNameTuple *t=prefixTable;
			while(t->namespaceUri!=NULL)
			{
				strPool.insertRef(t->localName);
				strPool.insertRef(t->namespaceUri);
				t++;
			}

		}
		virtual ~QNameTokenizerImpl()
		{
		}

		virtual QName_t insert(const sal_Char *namespaceUri, const sal_Char* localName)
		{
			return addPoolToken(strPool.insert(namespaceUri), strPool.insert(localName));
		}

		virtual bool insert(const sal_Char *qName, QNameTokenizerContext &nsContext, QName_t *qn, bool formDefault=false)
		{
			sal_Char _prefix[50];
			int prefixLen;
			const sal_Char *_localName=splitQName(qName, _prefix, sizeof(_prefix), &prefixLen);
			if (!((prefixLen==5 && _prefix[0]=='x' && _prefix[1]=='m' && _prefix[2]=='l' && _prefix[3]=='n' && _prefix[4]=='s')
				|| (prefixLen==0 && _localName[0]=='x' && _localName[1]=='m' && _localName[2]=='l' && _localName[3]=='n' && _localName[4]=='s')))
			{
				PoolRef_t localName=strPool.insert(_localName);
				if (prefixLen==0 && formDefault)
				{
					*qn=addPoolToken(QName::EMPTY_STRING, localName);
					return true;
				}
				else
				{
					PoolRef_t namespaceUri=nsContext.resolvePrefix(_prefix);
					*qn=addPoolToken(namespaceUri, localName);
					return true;
				}
			}
			else
			{
				return false;
			}
		}

		virtual PoolRef_t getNamespaceUri(QName_t qName) const 
		{
			return qNameTable[qName].namespaceUri;
		}
		virtual PoolRef_t getLocalName(QName_t qName) const
		{
			return qNameTable[qName].localName;
		}

		virtual int getQNames() const
		{
			return qnames;
		}


		virtual std::auto_ptr<QNameTokenizerContext> createQNameTokenizerContext()
		{
			return std::auto_ptr<QNameTokenizerContext>(new QNameTokenizerContextImpl(strPool));
		}

		virtual std::auto_ptr<QNameSerializerContext> createQNameSerializerContext()
		{
			return std::auto_ptr<QNameSerializerContext>(new QNameSerializerContextImpl(strPool));
		}
	};

	QNameTokenizerImpl QNameTokenizerImpl::tokenizer_;

	const QNameTokenizer& QName::serializer()
	{
		return QNameTokenizerImpl::tokenizer_;
	}

	QNameTokenizer& QName::tokenizer()
	{
		return QNameTokenizerImpl::tokenizer_;
	}

} /* namespace writerfilter */
